import splunk.admin as admin
import json
import sc_rest
from splunk.util import normalizeBoolean

from DataModelHandlerUtil import DataModelJSONResource

from datamodel.controller import loadModelFromJSON
from datamodel.model import Model
from datamodel.errormsgs import DMError, PivotError

class DataModelAccelerationHandler(sc_rest.BaseRestHandler):
    
    def setup(self):
        self.shouldAutoList = False

    def handleList(self, confInfo):

        #call endpoint to see what's on disc
        sc_rest.BaseRestHandler.handleList(self, confInfo)

        if 'default' in confInfo:
            del confInfo['default']

        #try loading each model for validation purposes, and get the description
        for modelName, modelData in confInfo.items():

            #fail fast if the model isn't accelerated
            if not 'acceleration' in modelData or normalizeBoolean(modelData['acceleration']) == False:
                del confInfo[modelName]
                continue
            
            if 'eai:data' in modelData:
                modelJSON = modelData['eai:data']
                del modelData['eai:data']
                
                appName = "-" 
                if 'eai:appName' in modelData:
                    appName = modelData['eai:appName']
            
                try:
                    modelObj = loadModelFromJSON(modelName, modelJSON, isAccelerated=True)
                except DMError as dmError:
                    confInfo.addErrorMsg(json.dumps(dmError.__str__()))
                    continue
                except PivotError as pivotError:
                    confInfo.addErrorMsg(PivotError.__str__())
                    continue
                except:
                    confInfo.addErrorMsg("Error in Datamodel controller")
                    continue

                search = modelObj.buildModelSearch()
                if len(search) == 0: #if we have an empty search, the acceleration is invalid and we shouldn't return anything for this model
                    continue
            
                confInfo[modelName].update({'search': search})

        return

admin.init(sc_rest.ResourceHandler(DataModelJSONResource, handler=DataModelAccelerationHandler), admin.CONTEXT_APP_AND_USER)
