import splunk.admin as admin
import json
import sc_rest
import logging
from splunk.util import normalizeBoolean

from DataModelHandlerUtil import DataModelJSONResource

from datamodel.controller import loadModelFromJSON, buildSearchWithModel
from datamodel.model import Model
from datamodel.errormsgs import DMError, PivotError

logger = logging.getLogger('splunk.datamodel')

#class DataModelEAIResource(DataModelJSONResource):
class DataModelEAIHandler(sc_rest.BaseRestHandler):

    def setup(self):
        self.supportedArgs.addOptArg('concise')

        if self.customAction == 'desc':            
            self.supportedArgs.addOptArg('object')
            self.customActionCap = "search"
        elif self.customAction == 'report':
            self.supportedArgs.addOptArg('pivot')        
            self.supportedArgs.addOptArg('namespace')
            self.customActionCap = "search"
        elif self.customAction == 'acl':
            self.supportedArgs.addOptArg('owner')
            self.supportedArgs.addOptArg('sharing')
            self.supportedArgs.addOptArg('perms.*')
            self.customActionCap = "search"
        elif self.requestedAction == admin.ACTION_CREATE:
            self.supportedArgs.addReqArg('description')
            self.supportedArgs.addOptArg('acceleration')
        elif self.requestedAction == admin.ACTION_EDIT:
            self.supportedArgs.addOptArg('description')
            self.supportedArgs.addOptArg('provisional')
            self.supportedArgs.addOptArg('acceleration')

    def handleList(self, confInfo):
        
        #get URL arguments before calling ewoo's endpoint
        conciseMode = False
        if 'concise' in self.callerArgs: 
            conciseMode = normalizeBoolean(self.callerArgs['concise'][0])
            
        self.maxCount = 0
        self.posOffset = 0

        #call endpoint to see what's on disc
        sc_rest.BaseRestHandler.handleList(self, confInfo)

        if 'default' in confInfo:
            del confInfo['default']

        #try loading each model for validation purposes, and get the description
        for modelName, modelData in confInfo.items():

            if 'eai:data' in modelData:
                modelJSON = modelData['eai:data']
                del modelData['eai:data'] #we're basically just replacing this with 'description'
            
                isAccelerated = False 
                if 'acceleration' in modelData and normalizeBoolean(modelData['acceleration']):
                    isAccelerated = True 
                try:
                    modelObj = loadModelFromJSON(modelName, modelJSON, isAccelerated)
                except DMError as dmError:
                    confInfo.addErrorMsg(dmError.__str__())
                    del confInfo[modelName]
                    continue
                except PivotError as pivotError:
                    confInfo.addErrorMsg(pivotError.__str__())
                    del confInfo[modelName]
                    continue
                except:
                    confInfo.addErrorMsg("Error in Datamodel controller for model: "+modelName)
                    del confInfo[modelName]
                    continue

                if conciseMode and self.callerArgs.id == None:
                    try:
                        outputJSON = modelObj.shortJSONdesc()
                    except DMError as dmError:
                        confInfo.addErrorMsg(dmError.__str__())
                        del confInfo[modelName]
                        continue
                    except PivotError as pivotError:
                        confInfo.addErrorMsg(pivotError.__str__())
                        del confInfo[modelName]
                        continue
                    except:
                        confInfo.addErrorMsg("Error in Datamodel controller for model: " + modelName)
                        del confInfo[modelName]
                        continue
                    outputStr = json.dumps(outputJSON)
                    confInfo[modelName].update({"description": outputStr})
                    confInfo[modelName].update({"displayName": outputJSON['displayName']})
                
                else:
                    try:
                        outputJSON = modelObj.JSONdesc(conciseMode)
                    except DMError as dmError:
                        confInfo.addErrorMsg(dmError.__str__())
                        del confInfo[modelName]
                        continue
                    except PivotError as pivotError:
                        confInfo.addErrorMsg(pivotError.__str__())
                        del confInfo[modelName]
                        continue
                    except:
                        confInfo.addErrorMsg("Error in Datamodel controller for model: "+modelName)
                        del confInfo[modelName]
                        continue
                    outputStr = json.dumps(outputJSON)
                    confInfo[modelName].update({"description": outputStr})
                    confInfo[modelName].update({"displayName": outputJSON['displayName']})


            #finally, add acceleration info
            
            accelerationDict = {}

            if 'acceleration' in modelData:
                if normalizeBoolean(modelData['acceleration']):
                    accelerationDict.update({"enabled": True})
                else:
                    accelerationDict.update({"enabled": False})
                if 'acceleration.earliest_time' in modelData:
                    accelerationDict.update({'earliest_time': modelData['acceleration.earliest_time']})
                    del modelData['acceleration.earliest_time']
                if 'acceleration.cron_schedule' in modelData:
                    accelerationDict.update({'cron_schedule': modelData['acceleration.cron_schedule']})
                    del modelData['acceleration.cron_schedule']

                confInfo[modelName].update({"acceleration": json.dumps(accelerationDict)})
                                           
            else:
                confInfo[modelName].update({'acceleration': "{\"enabled\": false}"})
        return
  

      

    def handleCustom(self, confInfo):
        conciseMode = False
        #check if we got concise=true
        if 'concise' in self.callerArgs and self.callerArgs['concise'][0] == 1:
            conciseMode = True;

        
        if self.customAction == "desc": 

            if 'object' in self.callerArgs:
                objectName = self.callerArgs['object'][0]
                modelName = self.callerArgs.id
                

                #call list endpoint to get model description
                listConfInfo = admin.ConfigInfo()
                self.requestedAction = admin.ACTION_LIST
                self.customAction = ""
                sc_rest.BaseRestHandler.handleList(self, listConfInfo)

                self.didFilter = True

                for modelName, modelData in listConfInfo.items():
                    if modelName == self.callerArgs.id:

                        isAccelerated = False 
                        if 'acceleration' in modelData and normalizeBoolean(modelData['acceleration']):
                            isAccelerated = True

                        modelJSON = modelData['eai:data']
                        del modelData['eai:data']
                        modelObj = loadModelFromJSON(modelName, modelJSON, isAccelerated)
                        if modelObj is None:
                            raise admin.BadProgrammerException("Model loading failed somehow, but failed to throw an error.")
                        continue #can break from loop if we've found the right model

                try:
                    #get the instantiated object
                    objectObj = modelObj.getObject(objectName) 
                except DMError as dmError:
                    raise admin.InternalException(dmError.__str__())
                except PivotError as pivotError:
                    raise admin.InternalException(pivotError.__str__())
                except:
                    raise admin.BadProgrammerException("Error in Datamodel controller")

                try:
                    objectDesc = json.dumps(objectObj.JSONdesc(conciseMode))
                except DMError as dmError:
                    confInfo.addErrorMsg(dmError.__str__())
                    return
                except PivotError as pivotError:
                    confInfo.addErrorMsg(pivotError.__str__())
                    return
                except:
                    confInfo.addErrorMsg("Error in Datamodel controller")
                    return
                objectConf = confInfo[self.callerArgs.id + '.' + objectName]
                objectConf["description"] = objectDesc 
            else:
                self.handleList(confInfo) #revert to model description if no object is specified

            
        elif self.customAction == "acl":
            self.shouldAutoList = False
            
            sc_rest.BaseRestHandler.handleCustom(self, confInfo)

        else:
            raise admin.NotFoundException("This endpoint does not support action: " + self.customAction)
        self.didFilter = True

    
    def handleCreate(self, confInfo):

        modelName = self.callerArgs.id

        #check for an existing model with the same name (possible with different case)
        tmpInfo = admin.ConfigInfo()
        self.callerArgs.id = None #need to make sure we check all models for possible duplicates with different case
        sc_rest.BaseRestHandler.handleList(self, tmpInfo)
        
        #set ID back after
        self.callerArgs.id = modelName

        for item, data in tmpInfo.items():
            if item.upper() == modelName.upper():
                raise admin.AlreadyExistsException('A datamodel with the name "%s" already exists' % modelName)

        #feed model through instantiation code to validate
        if 'description' in self.callerArgs.data:
            try:
                modelObject = loadModelFromJSON(modelName, self.callerArgs.data['description'][0])
            except DMError as dmError:
                raise admin.InternalException(dmError.__str__())
            except PivotError as pivotError:
                raise admin.InternalException(pivotError.__str__())
            except:
                raise admin.BadProgrammerException("Error in Datamodel controller")

            self.updateAccelerationSettings(confInfo, modelName)
            
            #if there's no exception, add with ewoo's endpoint
            del self.callerArgs.data['description']
            self.callerArgs['eai:data'] = [json.dumps(modelObject.JSONdesc(True), indent=4)]
            if 'acceleration' in self.callerArgs and self.callerArgs['acceleration'][0] == 1:
                if not modelObject.canAccelerate:
                    raise admin.UsageException("This model cannot be accelerated because it does not contain any event-based objects.")

            sc_rest.BaseRestHandler.handleCreate(self, confInfo)
                        
        else:
            raise admin.UsageException("The 'description' attribute is required.")


        self.didFilter = True
            

    def handleEdit(self, confInfo):
        self.supportedArgs.addOptArg('description')
        self.supportedArgs.addOptArg('acceleration')
        self.supportedArgs.addOptArg('provisional')

        modelName = self.callerArgs.id
        
        provisionalEdit = False
        if 'provisional' in self.callerArgs:
            provisionalEdit = normalizeBoolean(self.callerArgs['provisional'][0])

        conciseMode = False
        if 'concise' in self.callerArgs:
            conciseMode = normalizeBoolean(self.callerArgs['concise'][0])

        if not provisionalEdit:
            self.updateAccelerationSettings(confInfo, modelName)
        

        if 'acceleration' in self.callerArgs and self.callerArgs['acceleration'][0] == 0:
            #if the user is disabling acceleration, we can edit
            pass
        else:
            #load existing model data
            tmpInfo = admin.ConfigInfo()
            sc_rest.BaseRestHandler.handleList(self, tmpInfo)
        
            #check if model is accelerated
            for item, data in tmpInfo.items():
                if item == modelName and 'acceleration' in data and normalizeBoolean(data['acceleration']) == True and 'description' in self.callerArgs.data:
                    raise admin.BadActionException("Cannot edit accelerated model: %s. Please turn off acceleration before editing." % modelName)

        if 'description' in self.callerArgs.data:

            modelData = self.callerArgs.data['description'][0]
    
            try:
                modelObject = loadModelFromJSON(modelName, modelData)
            except DMError as dmError:
                raise admin.InternalException(dmError.__str__())
            except PivotError as pivotError:
                raise admin.InternalException(pivotError.__str__())
            except:
                raise admin.BadProgrammerException("Error in Datamodel controller")

            if not provisionalEdit:

                #use ewoo's endpoint here to write the model to the correct user/app location
                del self.callerArgs.data['description']
                modelDesc = json.dumps(modelObject.JSONdesc(True), indent=4) #should always write concise mode to disk
                self.callerArgs['eai:data'] = [modelDesc]
                if 'acceleration' in  self.callerArgs and self.callerArgs['acceleration'][0] == 1:
                    if not modelObject.canAccelerate:
                        raise admin.UsageException("This model cannot be accelerated because it does not contain any event-based objects.")

                sc_rest.BaseRestHandler.handleEdit(self, confInfo)

            else:
                self.shouldAutoList = False #does this work right?
                confInfo[modelName].update({"description": json.dumps(modelObject.JSONdesc(conciseMode), indent=4)})
                
        elif 'acceleration' in self.callerArgs.data and not provisionalEdit:
            if 'acceleration' in self.callerArgs and self.callerArgs['acceleration'][0] == 1:
                # need to load model here and check if it's something we can accelerate!
                tmpInfo = admin.ConfigInfo()
                if 'concise' in self.callerArgs.data:
                    del self.callerArgs.data['concise']
                sc_rest.BaseRestHandler.handleList(self, tmpInfo)
                for item, data in tmpInfo.items():
                    if item == modelName:
                        #load model
                        #raise admin.UsageException("data: %s" % data['eai:data'])
                        try:
                            modelObject = loadModelFromJSON(item, data['eai:data'])
                        except DMError as dmError:
                            raise admin.InternalException(dmError.__str__())
                        except PivotError as pivotError:
                            raise admin.InternalException(pivotError.__str__())
                        except:
                            raise admin.BadProgrammerException("Error in Datamodel controller")

                        if not modelObject.canAccelerate:
                            raise admin.UsageException("This model cannot be accelerated because it does not contain any event-based objects.")
                            
            sc_rest.BaseRestHandler.handleEdit(self,confInfo)

        self.didFilter = True        


    def handleRemove(self, confInfo):
        sc_rest.BaseRestHandler.handleRemove(self, confInfo)
        self.shouldAutoList = False
        #FIXME: this doesn't work for some reason


    #updates conf file with new acceleration settins and removes them from the callerArgs
    def updateAccelerationSettings(self, confInfo, modelName):
        settingsDict = {}
        if 'acceleration' in self.callerArgs: 
            try:
                accelerationDict = json.loads(self.callerArgs['acceleration'][0])
            except:
                raise admin.ArgValidationException("Could not parse acceleration JSON")
            
            if 'enabled' in accelerationDict:
                if normalizeBoolean(accelerationDict['enabled']):
                    self.callerArgs['acceleration'] = [1]
                else:
                    self.callerArgs['acceleration'] = [0]

            if 'earliest_time' in accelerationDict:
                if accelerationDict['earliest_time'] == None:
                    self.callerArgs['acceleration.earliest_time'] = [""]
                else:
                    self.callerArgs['acceleration.earliest_time'] = [accelerationDict['earliest_time']]

            if 'cron_schedule' in accelerationDict:
                if accelerationDict['cron_schedule'] == None:
                    self.callerArgs['acceleration.cron_schedule'] = [""]
                else:
                    self.callerArgs['acceleration.cron_schedule'] = [accelerationDict['cron_schedule']]

#admin.init(sc_rest.ResourceHandler(DataModelEAIResource), admin.CONTEXT_NONE)
admin.init(sc_rest.ResourceHandler(DataModelJSONResource, handler=DataModelEAIHandler), admin.CONTEXT_APP_AND_USER)


