import splunk.admin as admin
import json
import sc_rest
import logging
from splunk.util import normalizeBoolean

from DataModelHandlerUtil import DataModelJSONResource

from datamodel.controller import loadModelFromJSON, buildSearchWithModel, pivotSearchFromDict, pivotDictFromSearch
from datamodel.model import Model
from datamodel.errormsgs import DMError, PivotError

class DataModelReportHandler(sc_rest.BaseRestHandler):

    def setup(self):
        self.supportedArgs.addOptArg('pivot_json')
        self.supportedArgs.addOptArg('pivot_search')
        self.supportedArgs.addOptArg('namespace')
        self.supportedArgs.addOptArg('is_pivot_command')

    def handleList(self, confInfo): 

        if self.callerArgs.id == None:
            raise admin.UsageException("User must specify a model to pivot on.")

        #this first chunk takes in the arguments and translates to the appropriate format(s)

        #get model JSON            
        self.requestedAction = admin.ACTION_LIST
        self.customAction = ""
        sc_rest.BaseRestHandler.handleList(self, confInfo)
        
        modelObj = None
        
        if 'default' in confInfo:
            del confInfo['default']

        #are we coming from the pivot search command?
        isPivotCommand = False
        if 'is_pivot_command' in self.callerArgs:
            isPivotCommand = normalizeBoolean(self.callerArgs['is_pivot_command'][0])

        for modelName, modelData in confInfo.items():
            if modelName == self.callerArgs.id:

                if not 'eai:data' in modelData:
                    raise admin.UsageException("Model " + modelName + " has no data")

                modelJSON = modelData['eai:data']
                del modelData['eai:data']

                isAccelerated=False
                if 'acceleration' in modelData and normalizeBoolean(modelData['acceleration']):
                    isAccelerated=True

                modelObj = loadModelFromJSON(modelName, modelJSON, isAccelerated)

                if modelObj is None:
                    raise admin.BadProgrammerException("Model loading failed somehow, but failed to throw an error.")

                if 'namespace' in self.callerArgs:
                    pivotNamespace = self.callerArgs['namespace'][0]
                elif isAccelerated: 
                    pivotNamespace=modelName
                else: 
                    pivotNamespace=''

        if modelObj == None:
            raise admin.InternalException('Failed to load model "' + self.callerArgs.id + '"')

        if 'pivot_json' in self.callerArgs:
            self.shouldAutoList = False

            #setup
            pivotSpec = self.callerArgs['pivot_json'][0]

            try:
                pivotDict = json.loads(pivotSpec)
            except:
                raise admin.ArgValidationException("Failed to load pivot JSON")
            
            #get pivot search
            pivotSearch = pivotSearchFromDict(pivotDict, self.callerArgs.id)
            
        elif 'pivot_search' in self.callerArgs:
            pivotSearch = self.callerArgs['pivot_search'][0]
            pivotDict = pivotDictFromSearch(pivotSearch, modelObj)

        else:
            raise admin.UsageException("You must specify either pivot_json or pivot_search.")

                    
        #actually generate reporting search here
        try:
            reportDict = buildSearchWithModel(pivotDict, modelObj, pivotNamespace, useObjectNamespace=isPivotCommand)
        except PivotError as pivotError:
            raise admin.InternalException(pivotError.__str__())

        confInfo[self.callerArgs.id].update({"search": reportDict['searchString']})
        confInfo[self.callerArgs.id].update({"tstats_search": reportDict['tstatsSearchString']})
        confInfo[self.callerArgs.id].update({"open_in_search": reportDict['openInSearchString']})
        confInfo[self.callerArgs.id].update({"drilldown_search": reportDict['drilldownSearchString']})
        confInfo[self.callerArgs.id].update({"pivot_search": pivotSearch})
        confInfo[self.callerArgs.id].update({'pivot_json': json.dumps(pivotDict)})

        


admin.init(sc_rest.ResourceHandler(DataModelJSONResource, handler=DataModelReportHandler), admin.CONTEXT_APP_AND_USER)
