from base import Base
from stats import Stats
from top import Top
from field import Field
from datamodel.errormsgs import DMError

DEBUG = False

class BaseEventSet(Base):
    def __init__(self, objects, uniqueID=[]):
	Base.__init__(self)
        self.uniqueID = uniqueID
        #this event has all the filters that should apply to everything in this EventList
        self.objects = objects
        self.containedEvents = []
        self.notContainedEvents = []
	self.name = 'BaseEventSet'
	self.objectType = 'BaseEventSet'
	self.parentname=self.__class__.__bases__[0].__name__


    def contains(self, event):
        self.containedEvents = self.containedEvents + [event]

    def notcontains(self, event):
        self.notContainedEvents = self.notContainedEvents + [event]

    def getFieldList(self):
	fieldList = self.getFieldListBase()
	for object in self.objects:
	  fieldList = fieldList + object.getFieldList()

        return fieldList

class BaseTransaction(BaseEventSet): 
    def __init__(self, objectsToGroup, groupByFields=[], span = '', pause = '', mvlist = 't', keepevicted = 'f',  miscparams=''):
        BaseEventSet.__init__(self, objects=objectsToGroup, uniqueID=groupByFields)
        self.fields = [Field("eventcount",  type='number', owner=self.name, required=True, editable=False), Field("duration",  type='number', owner=self.name, required=True, editable=False), Field("_time",  type='timestamp', owner=self.name, required=True, editable=False)]
        self.span = span
        self.pause = pause
	self.mvlist = mvlist
	self.keepevicted = keepevicted
	self.miscparams = miscparams
	self.name = 'BaseTransaction'
	self.objectType = 'BaseTransaction'
	# check objectType on each event. 
	# if one event of baseTransaction type, one event of baseSearch type, 
	# or one+ events of baseEvent type, we can transact.
	# if we're mixing and matching base types, tough! fail gracefully
	if len(self.objects) > 1:
	  for object in self.objects:
	    if object.getObjectType() in ['BaseTransaction', 'BaseSearch', 'BaseInterface','BaseImplements']:
              raise DMError("OBJECT_BASETRANSACTION_MULTIPLE_BASEOBJECTS_NOT_BASEEVENT", {'objectName': self.name})
	  
	for object in self.objects:
	  # add fields from sample events and mark as multivalued
	  for field in object.getFields():
	     if self.hasField(field.getName())==False: 
		if field.getType() != "objectCount": # don't want to expose the event's objectcount
	          self.fields = self.fields+[Field(field.getName(), owner=self.name, multivalue=True)] 
	  # add calculated fields from sample events and mark as multivalued
	  for field in object.getCalculatedFields():
	     if self.hasField(field.getName())==False: 
		if field.getType() != "objectCount": # don't want to expose the event's objectcount
	          self.fields = self.fields+[Field(field.getName(), owner=self.name, multivalue=True)] 
        self.displayName = ''

    def search(self, mode='full'):
        retval = ''
	eventcount=1
	# need to get calculations separately because we can't do things like | lookup within the parentheses

	if self.tstatsNamespace == '':	
	  if len(self.objects) == 1 and (self.objects[0].getObjectType() in ['BaseEvent', 'BaseTransaction', 'BaseSearch', 'BaseInterface', 'BaseImplements']):
	    retval += self.objects[0].search(mode='preview') + ' | ' 
	
	  else:
	    # FIXME: renaming for multi-event transaction scenario. maybe use multisearch? 
	    eventcount = 0
	    for object in self.objects:
	      if eventcount==0:
              	retval += "| multisearch "
              retval += "[" + object.search(mode='preview')+"]"
	      eventcount = eventcount + 1
	    retval += ' | '	

	  # preprocess to add markers on events required to be contained in the transaction
	  markercount=1
	  for containedEvent in self.containedEvents:
            constraintSearch = containedEvent.getSearchComponents()["fields"] + containedEvent.getSearchComponents()["prefilter"] + containedEvent.getSearchComponents()["postfilter"]
	    retval += ' eval marker' + str(markercount) + '= if(searchmatch("' + constraintSearch+'"),1,"") | '
	    markercount=markercount+1
	
	  notContainedMarker = markercount

	  for notContainedEvent in self.notContainedEvents:
            constraintSearch = notContainedEvent.getSearchComponents()["fields"] + notContainedEvent.getSearchComponents()["prefilter"] + notContainedEvent.getSearchComponents["postfilter"]
	    retval += ' eval marker' + str(markercount) + '= if(searchmatch("' + constraintSearch+'"),1,"") | '
	    markercount=markercount+1

	  # transaction command	
	  if mode == 'preview':
	    retval += ' head 10000 | transaction '
	  else:
	    retval += 'transaction mvlist=' + self.mvlist + ' '
	  retval += 'keepevicted=' + self.keepevicted + ' '
          if self.span != '':
            retval += 'maxspan=' + self.span + ' '
          if self.pause != '':
            retval += 'maxpause=' + self.pause + ' '
	  if self.miscparams != '':
            retval += self.miscparams + ' ' 
          for groupbyfield in self.uniqueID:
            retval += groupbyfield + ' '


	  # post process after transacting
          retval += '| search *' 
	  for i in range(1,notContainedMarker):
	    retval += " " + 'marker' + str(i) + '!=""'	
	  for i in range(notContainedMarker,markercount):
	    retval += " " + 'marker' + str(i) + '==""'	

	retval += Base.search(self, mode)

        return retval


    def first(self, fieldname):
	self.evalField('first_'+fieldname,'mvindex(fieldname,0)')

    def last(self, fieldname):
	self.evalField('last_'+fieldname,'mvindex(fieldname,-1)')

    def duration(self):
	self.getField('duration')
	print 'get field duration'

    def eventcount(self):
	self.getField('eventcount')
	print 'get field eventcount'

    def JSONdesc(self, conciseMode):
        desc = BaseEventSet.JSONdesc(self, conciseMode)
        desc['groupByFields'] = self.uniqueID
        desc['transactionMaxTimeSpan'] = self.span
        desc['transactionMaxPause'] = self.pause
        desc['objectsToGroup'] = [object.getName() for object in self.objects]
        return desc


class BaseAggregate(BaseEventSet): 
    def __init__(self, objectsToGroup, uniqueID, aggregates):
        BaseEventSet.__init__(self, objects=objectsToGroup, uniqueID=uniqueID)
	self.name = 'BaseAggregate'
	self.objectType = 'BaseAggregate'
	self.aggregates = aggregates
	
    def JSONdesc(self, conciseMode):
        desc = BaseEventSet.JSONdesc(self, conciseMode)
        desc['uniqueID'] = self.uniqueID
        desc['objectsToGroup'] = [object.getName() for object in self.objects]
        desc['aggregates'] = self.aggregates
        return desc

    def search(self, mode="full"):
	retval=''

	retval += self.objects[0].search(mode="preview") #FIXME: simple case...tentative

	# FIXME: use multisearch if there are multiple objects to group

	# FIXME: tstats needs fixing. Need to also get searches for objectsToGroup first and append strings below
	if self.tstatsNamespace=='': #FIXME: this should be for the objects to group by
 	  retval+= Stats(self.aggregates, [{"field": self.uniqueID}]).search() 
	else: # object constraints as filters? FIXME: this is probably wrong...need to fix the logic
 	  retval+= Stats(self.aggregates, [{"field": self.uniqueID}]).tstatsSearch(self.tstatsNamespace, []) 
	return retval
