import json
import traceback
import logging

#note: this import is bad coding practice.  We should do something smarter.
from datamodel.model import Model
from datamodel.errormsgs import PivotError, DMError 
#startup

logger = logging.getLogger("splunk")

CELL_KEYWORDS = ['AS', 'SPARKLINE']
SPLIT_KEYWORDS = ['AS', 'RANGE', 'TRUELABEL', 'FALSELABEL', 'PERIOD']
SORT_KEYWORDS = ['TOP', 'BOTTOM', 'SORT']

MAX_ROWS = 1000000
MAX_COLS = 1000

def loadModelFromJSON(modelName, modelJSON, isAccelerated=False):
    try: 
        data = json.loads(modelJSON)
    except:
        raise DMError("MODEL_CANNOT_LOAD_JSON", {'modelName': modelName})
 
    modelObj = Model(data, modelName, isAccelerated)

    return modelObj


def escapeLabel(fieldlabel):
  if '*' in fieldlabel: raise PivotError('escapeLabel', 'asterix not allowed in field labels') 
  return fieldlabel.replace('\\','\\\\').strip("'").strip('"')

def escapeSearch(searchString):
  # need to escape the following characters in the search string: 
  return searchString.replace("\ ","\\ ").replace('"','\\\"')


# this function returns a search string for the pivot UI. 
# input: JSON that includes a datamodel name, object name, rows, columns, cell values and filters
# output: a search string
#
# Arguments:
# data - the dict of pivot report data
# modelObj - the instantiated model object
# pivotNamespace - the namespace supplied by whoever asked for the search (may be empty)
# useObjectNamespace - will be true when the call to buildSearch comes from the pivot search command
#    This allows us to use namespaces selectively for object that aren't accelerated (e.g. search-
#    based objects).
def buildSearchWithModel(data, modelObj, pivotNamespace, useObjectNamespace=False):
  baseClass = data['baseClass']
  rows = data['rows']
  columns = data['columns']
  filters = data['filters']
  cells = data['cells']
  if 'rowFormat' in data: 
    rowFormat = data['rowFormat']
  else: rowFormat = {}
  if 'colFormat' in data: 
    colFormat = data['colFormat']
  else: colFormat = {} 
  if 'showSummary' in rowFormat:
    rowSummary = rowFormat['showSummary']
  else: rowSummary = False
  if 'showSummary' in colFormat:
    colSummary = colFormat['showSummary']
  else: colSummary = False
  if 'showOther' in colFormat:
    showOther = colFormat['showOther']
  else: showOther = True 

  # should not have empty cells unless we have empty columns and non-empty rows
  if cells==[] and rows==[]:
    raise PivotError("buildSearchWithModel", "must have non-empty cells or non-empty rows ")
  if cells==[] and columns!=[]:
    raise PivotError("buildSearchWithModel", "cannot pivot with empty cells and non-empty columns ")

  #load object from the model we passed in
  myobject = modelObj.getObject(baseClass)

  if cells==[] and rows!=[]:
    # for scatter charts 
    # set cells to count and then remove count field at he end.
    myobject.count()

  if pivotNamespace=='': # when search strings should be non-tstats e.g. for open in search
    myobject.unsetTsidxNamespace() 

  elif myobject.getTsidxNamespace()=='' and useObjectNamespace == False:  
    # if model is accelerated, don't set it when object doesn't have a namespace e.g. BaseSearch
    # no model-wide tstats...use Simon's tstats 
    myobject.setTsidxNamespace(namespace=pivotNamespace)

  elif myobject.getTsidxNamespace()=='' and useObjectNamespace == True:  
    myobject.unsetTsidxNamespace() 

  elif myobject.getTsidxNamespace()!='' and myobject.getObjectType() not in ("BaseEvent") and useObjectNamespace == False: 
    myobject.setTsidxNamespace(namespace=pivotNamespace) # use Simon's tstats

  elif myobject.getTsidxNamespace()!='' and myobject.getObjectType() == "BaseEvent" and myobject.getLineage().split(".")[0] != modelObj.getFirstBaseEvent() and useObjectNamespace == False:
    # model-wide acceleration only works for first BaseEvent root object
    myobject.setTsidxNamespace(namespace=pivotNamespace) # use Simon's tstats

  else:  #using model-wide tstatsNamespace so just add a filter for the object we care about
    lineage = myobject.getLineage().split(".")
    for i in range(0,len(lineage)):
      myobject.filterwithname('nodename', '=', '"'+'.'.join(lineage[0:i+1])+'"')
 
  pivotNamespace = myobject.getTsidxNamespace()
  try:
    rawObjectSearch = myobject.search()
  except:
    logger.exception("Unknown error obtaining search. Traceback: "+traceback.format_exc())
    raise DMError("OBJECT_CANNOT_OBTAIN_SEARCH", {'objectName': baseClass})


  myobject.setRowSummary(rowSummary)
  if colSummary:
    myobject.setColSummary('show') # true = 'show', false = 'none'
  else:
    myobject.setColSummary('none') # true = 'show', false = 'none'
  myobject.setColOther(showOther)

  for k in cells:
	# escape quotes and backslash in label
        escapedlabel = escapeLabel(k['label'])

        if k['owner']=='':
          raise PivotError("buildSearchWithModel", "empty cell attribute owner")
	if pivotNamespace!='' and k['owner'][0:4]!="Base" and k['fieldName'] not in ['_time', 'host', 'source', 'sourcetype']: #FIXME: specialcase fieldNames in separate function
	  hierarchicalFieldName = "'"+k['owner']+'.'+k['fieldName'].strip('"').strip("'")+"'"
	else: 
	  hierarchicalFieldName = "'"+k['fieldName'].strip('"').strip("'")+"'"

 	if k['type'] == 'objectCount':
            myobject.count(label=escapedlabel)
 	elif k['type'] == 'childCount':
            myobject.statsTable('sum',hierarchicalFieldName,escapedlabel) 
        elif k['type'] == 'timestamp':
	    if k['value']=='duration': 
            	myobject.statsTable('range', hierarchicalFieldName, escapedlabel)
	    elif k['value'] in ['earliest', 'latest']:
            	myobject.statsTable(k['value'], hierarchicalFieldName, escapedlabel)
		myobject.reformatTimeValues(escapedlabel)
	elif k['type'] == 'number':
            myobject.statsTable(k['value'], hierarchicalFieldName, escapedlabel)
	elif k['type'] == 'string':
            myobject.statsTable(k['value'], hierarchicalFieldName, escapedlabel)
	elif k['type'] == 'ipv4':
            myobject.statsTable(k['value'], hierarchicalFieldName, escapedlabel)
        else:
            raise PivotError("buildSearchWithModel", "unknown cell field type "+k['type'])

	if "sparkline" in k and k["sparkline"] == True:
	  # FIXME: remove this once SPL-63308 is resolved
	  if tstats!='': 
	    raise PivotError("buildSearchWithModel", "Sparkline function not yet implemented in tsidxStats") 
          myobject.sparkline(k['value'], hierarchicalFieldName, escapedlabel)

  # constructing operation from bits and pieces of info the json dictionary
  for j in filters:
    j['fieldName'] = j['fieldName'].strip('"').strip("'")
    mytype = j['type']
    if "rule" in j:
      if mytype == 'string':
  	if j['rule']['comparator'] == 'is':
  	   operation = '='
      	   RHS = j['rule']['compareTo']
  	elif j['rule']['comparator'] == 'isNot':
  	   operation = '!='
      	   RHS = j['rule']['compareTo']
  	elif j['rule']['comparator'] == 'isNull':
  	   operation = '!='
      	   RHS = '*'
  	elif j['rule']['comparator'] == 'isNotNull':
  	   operation = '='
      	   RHS = '*'
  	elif j['rule']['comparator'] == 'contains':
  	   operation = '='
      	   RHS = '*'+j['rule']['compareTo']+'*'
  	elif j['rule']['comparator'] == 'doesNotContain':
  	   operation = '!='
      	   RHS = '*'+j['rule']['compareTo']+'*'
  	elif j['rule']['comparator'] == 'startsWith':
  	   operation = '='
      	   RHS = j['rule']['compareTo']+'*'
  	elif j['rule']['comparator'] == 'endsWith':
  	   operation = '='
      	   RHS = '*'+j['rule']['compareTo']
  	elif j['rule']['comparator'] == 'regex':
  	   operation = "regex"
  	   RHS = j['rule']['compareTo']
	else:
            raise PivotError("buildSearchWithModel", "invalid string filter rule comparator "+j['rule']['comparator'])
        #escape spaces and doublequotes in RHS # FIXME: need to escape other characters here too
        RHS = '"'+escapeSearch(RHS)+'"'

      elif mytype == 'number':
  	if j['rule']['comparator'] == 'isNull':
  	   operation = "!="
  	   RHS = "*"
  	elif j['rule']['comparator'] == 'isNotNull':
  	   operation = '='
      	   RHS = '*'
	else:
  	  operation = j['rule']['comparator']
      	  RHS = j['rule']['compareTo']
      elif mytype == 'ipv4': # FIXME: need to expand out scenarios here and add it to the frontend as well
  	# want to support things like 192.168.* and 192.*.1.1 and 192.168.1.1/*
  	if j['rule']['comparator'] == 'is':
  	   operation = '='
      	   RHS = j['rule']['compareTo']
  	elif j['rule']['comparator'] == 'isNot':
  	   operation = '!='
      	   RHS = j['rule']['compareTo']
  	elif j['rule']['comparator'] == 'isNull':
  	   operation = "!="
  	   RHS = "*"
  	elif j['rule']['comparator'] == 'isNotNull':
  	   operation = '='
      	   RHS = '*'
  	elif j['rule']['comparator'] == 'contains':
  	   operation = '='
      	   RHS = '*'+j['rule']['compareTo']+'*'
  	elif j['rule']['comparator'] == 'doesNotContain':
  	   operation = '!='
      	   RHS = '*'+j['rule']['compareTo']+'*'
  	elif j['rule']['comparator'] == 'startsWith':
  	   operation = '='
      	   RHS = j['rule']['compareTo']+'*'
	else:
            raise PivotError("buildSearchWithModel", "invalid ipv4 filter rule comparator "+j['rule']['comparator'])
      elif mytype == 'boolean':
        if 'comparator' not in j['rule'] or j['rule']['comparator'] == '=' or j['rule']['comparator'] == 'is':
            operation = '='
            RHS = j['rule']['compareTo']
  	elif j['rule']['comparator'] == 'isNull':
  	   operation = "!="
  	   RHS = "*"
  	elif j['rule']['comparator'] == 'isNotNull':
  	   operation = '='
      	   RHS = '*'
	else:
            raise PivotError("buildSearchWithModel", "invalid boolean filter rule comparator "+j['rule']['comparator'])
      # FIXME: add elif case for objectCount type? This is not exposed in the current UI but need to guard for it
      else:
        # catch all else clause that throws an exception
        raise PivotError("buildSearchWithModel", "invalid filter type "+mytype+" in filter")
  
      # split things up into LHS, comparator & RHS so we can construct
      # the search differently based on whether we're doing tstats or regular stats. 
      myobject.filterwithname(j['fieldName'], operation, RHS, j['owner'])

    # if limit option is used in filters, deconstruct into stats
    if "limit" in j:
      # FIXME:  check for non-empty fieldname, statsFn, attributeName etc. 
      if j['fieldName']=='' or j['owner']=='': 
        raise PivotError("buildSearchWithModel", "empty fieldName or owner field")
      if pivotNamespace!='' and j['owner'][0:4]!="Base" and j['fieldName'] not in ['_time', 'host', 'source', 'sourcetype']: #FIXME: specialcase fieldNames in separate function
        hierarchicalFieldName = j['owner']+'.'+j['fieldName'].strip('"').strip("'")
      else: 
        hierarchicalFieldName = j['fieldName'].strip('"').strip("'")
      if pivotNamespace!='' and 'attributeOwner' in j['limit'] and j['limit']['attributeOwner'][0:4]!="Base" and j['limit']['attributeName'] not in ['_time', 'host', 'source', 'sourcetype']: #FIXME: specialcase fieldNames in separate function
        hierarchicalAttributeName = j['limit']['attributeOwner']+'.'+j['limit']['attributeName'].strip('"').strip("'")
      elif j['limit']['attributeName']==j['fieldName']:
	hierarchicalAttributeName = hierarchicalFieldName
      else:
        hierarchicalAttributeName = j['limit']['attributeName'].strip('"').strip("'")

      if baseClass == j['limit']['attributeName']: # this is a count on an objectCount field
	# for objectCount, don't want to specify an attribute name...should just do count
        myobject.filterLimit(rawObjectSearch, hierarchicalFieldName, '', j['limit']['statsFn'], j['limit']['limitType'], j['limit']['limitAmount'])
      else: 
        myobject.filterLimit(rawObjectSearch, hierarchicalFieldName , hierarchicalAttributeName, j['limit']['statsFn'], j['limit']['limitType'], j['limit']['limitAmount'])

  # adding ability to bucket things here...
  for idx, i in enumerate(columns+rows): # note: go through columns before rows so we can get list of columns for sortLimit
    span = ''
    bins = ''
    start = ''
    end = ''
    # FIXME: check for empty fieldName, owner and type

    # escape quotes and backslash in label
    if 'label' in i:
      escapedlabel = escapeLabel(i['label'])
    else:
      escapedlabel = ''

    if pivotNamespace!='' and i['owner'][0:4]!="Base" and i['fieldName'] not in ['_time', 'host', 'source', 'sourcetype']: #FIXME: specialcase fieldNames in separate function
      hierarchicalFieldName = "'"+i['owner']+'.'+i['fieldName'].strip('"').strip("'")+"'"
    else: 
      hierarchicalFieldName = "'"+i['fieldName'].strip('"').strip("'")+"'"

    # e.g. bucket timestamp by Auto, Year, Month, Day, Hour, Minute and Second
    # | bucket _time span=5m
    if i['type']=='timestamp' and 'period' in i:
	if i['period']=='auto': span = ''
	elif i['period']=='year': span = '12mon'
	elif i['period']=='month': span = '1mon'
	elif i['period']=='week': span = '7d'
	elif i['period']=='day': span = '1d'
	elif i['period']=='hour': span = '1h'
	elif i['period']=='minute': span = '1m'
	elif i['period']=='second': span = '1s'
	if hierarchicalFieldName!="'_time'":
	  myobject.reformatTimeValues(hierarchicalFieldName.strip("'"))

    # e.g. bucket numbers by ranges. options are max # of ranges, range size, range start value and range end value
    # rangesize => span, max # of ranges => bins, start/end 
    if i['type']=='number':
      if 'display' in i and i['display']=='ranges':
        if 'maxNumberOf' in i['ranges']: bins = i['ranges']['maxNumberOf']
        if 'size' in i['ranges']: span = i['ranges']['size']
        if 'start' in i['ranges']: start = i['ranges']['start']
        if 'end' in i['ranges']: end = i['ranges']['end']

    if i['type']=='boolean':
	if  'trueLabel' in i and i['trueLabel']!='':
	  trueLabel = escapeLabel(i['trueLabel'])
	else: trueLabel=''
	if  'falseLabel' in i and i['falseLabel']!='':
	  falseLabel = escapeLabel(i['falseLabel'])
	else: falseLabel=''
	if trueLabel!='' or falseLabel!='':
	  myobject.replaceBoolValues(hierarchicalFieldName.replace('\'','\"'), trueLabel, falseLabel)

    if idx >= len(columns): #specify whether its a row or a column for formatting the table correctly
      myobject.splitby(hierarchicalFieldName, escapedlabel, 'row', span, bins, start, end) 
    else: # no labels for column splitbys
      myobject.splitby(hierarchicalFieldName, i['fieldName'], 'col', span, bins, start, end) 

  # filter/sort by row split	
  if 'limitAmount' in rowFormat and rowFormat['limitAmount']!='':
    limitAmt = int(rowFormat['limitAmount'])

  elif 'limitType' in rowFormat and 'limitType'!='':
    limitAmt = 0
  else: 
    limitAmt = ''

  if limitAmt !='':
    if 'limitFieldName' in rowFormat:
      hierarchicalFieldName = "'"+rowFormat['limitFieldName'].strip('"').strip("'")+"'"
      if pivotNamespace!='' and 'limitFieldOwner' in rowFormat and rowFormat['limitFieldOwner'][0:4]!="Base" and rowFormat['limitFieldName'] not in ['_time', 'host', 'source', 'sourcetype']: #FIXME: specialcase fieldNames in separate function
        hierarchicalFieldName = "'"+rowFormat['limitFieldOwner']+'.'+rowFormat['limitFieldName'].strip('"').strip("'")+"'"
    else: 
      hierarchicalFieldName = ''

    limitStatsFn=''
    if 'limitStatsFn' in rowFormat:
      limitStatsFn = rowFormat['limitStatsFn']

    limitType='default'
    if 'limitType' in rowFormat:
      limitType = rowFormat['limitType']
 	
    if 'limitByAttributeName' in rowFormat:
      hierarchicalAttributeName = rowFormat['limitByAttributeName'].strip('"').strip("'")
      if baseClass == rowFormat['limitByAttributeName']: # this is a count on an objectCount field
        # for objectCount, don't want to specify an attribute name...should just do count
        hierarchicalAttributeName = ''
      elif pivotNamespace!='' and 'limitByAttributeOwner' in rowFormat and rowFormat['limitByAttributeOwner'][0:4]!="Base" and rowFormat['limitByAttributeName'] not in ['_time', 'host', 'source', 'sourcetype']: #FIXME: specialcase fieldNames in separate function
        hierarchicalAttributeName = rowFormat['limitByAttributeOwner']+'.'+rowFormat['limitByAttributeName'].strip('"').strip("'")      
    else: 
        hierarchicalAttributeName = ''

    if limitType!='default' and not colSummary:
      myobject.setColSummary('hide') # true = 'show', false = 'none', calculate but don't show = 'hide'

    myobject.sortLimit(hierarchicalFieldName, hierarchicalAttributeName, limitStatsFn, limitType, limitAmt)

  if 'limitAmount' in colFormat and colFormat['limitAmount']!='': 
    myobject.setColLimit(colFormat['limitAmount'])

	  

  # return regular search as well as tstats search. send myobject.search() but also copy over myobject to myobject1 and set tstats and generate search.
  if pivotNamespace != '':
    try:
      tstatssearch = myobject.search()
      if cells==[] and rows!=[]:
        tstatssearch += " | fields - count"	
    except:
      logger.exception("error building tstats searchString. Traceback: " + traceback.format_exc())
      raise PivotError("buildSearchWithModel", "error building tstats searchString ")

    try:
      nontstats = buildSearchWithModel(data, modelObj, "")
      nontstatssearch = nontstats["searchString"]
      simpleSearch = nontstats["openInSearchString"]
      drilldownSearch = nontstats["drilldownSearchString"]

    except:
      logger.exception("error building non-tstats searchString. Traceback: " + traceback.format_exc())
      raise PivotError("buildSearchWithModel", "error building non-tstats searchString ")

  else: 
    tstatssearch = ''
    try:
      nontstatssearch = myobject.search() 
      simpleSearch = myobject.search(mode='simple') 
      drilldownSearch = myobject.search(mode='drilldown') 
      if cells==[] and rows!=[]:
        # for scatter charts 
        # set cells to count and then remove count field at he end.
        nontstatssearch += " | fields - count"	
        simpleSearch += " | fields - count"	
    except:
      logger.exception("error building non-tstats searchString. Traceback: "+ traceback.format_exc())
      raise PivotError("buildSearchWithModel", "error building non-tstats searchString ") 


  retval={"searchString": nontstatssearch, "tstatsSearchString": tstatssearch, "openInSearchString": simpleSearch, "drilldownSearchString": drilldownSearch}

  return retval



def pivotSearchFromDict(pivotDict, modelName): 

    searchTokens = ['|','pivot']

    #MODEL
    #Do we want to allow model names to be specified via the pivot dictionary? Useful for testing?
    if modelName == None or len(modelName) == 0:
        if 'modelName' in pivotDict:
            modelName = pivotDict['modelName']
        else:
            raise DMError("MODEL_NOT_SPECIFIED", {})

    searchTokens.append(modelName)

    #object
    if 'baseClass' in pivotDict:
        objectName = pivotDict['baseClass']
        searchTokens.append(objectName)
    else:
        raise DMError("OBJECT_NOT_SPECIFIED", {})

    #cell values
    for cell in pivotDict['cells']:
        addCellToSearch(cell, searchTokens, objectName)

    #split rows
    for row in pivotDict['rows']:
        addRowOrColToSearch(row, searchTokens, isRow=True)

    #split columns
    for col in pivotDict['columns']:
        addRowOrColToSearch(col, searchTokens, isRow=False)
        
    #filters
    for fil in pivotDict['filters']:
        addFilterToSearch(fil, searchTokens)
        
    #showrowsummary
    if 'rowFormat' in pivotDict:
	addRowFormat(pivotDict['rowFormat'], searchTokens)

    #showcolsummary
    if 'colFormat' in pivotDict:
	addColFormat(pivotDict['colFormat'], searchTokens)

    pivotSearch = ' '.join(searchTokens)

    return pivotSearch


def addRowFormat(rowFormat, searchTokens):
    if 'showSummary' in rowFormat and rowFormat['showSummary']:
        searchTokens.append('ROWSUMMARY')
        searchTokens.append('true')
    #limit attributes
    if 'limitType' in rowFormat:
        if rowFormat['limitType'] == 'ascending':
            searchTokens.append('BOTTOM')
        elif rowFormat['limitType'] == 'descending':
            searchTokens.append('TOP')
        elif rowFormat['limitType'] == 'default':
            searchTokens.append('SORT')
	else: 
            raise PivotError("addRowFormat", "invalid limitType");
        
        if 'limitAmount' in rowFormat:
            searchTokens.append(str(rowFormat['limitAmount']))
        else:
            searchTokens.append(str(0))
       
	if rowFormat['limitType'] == 'default': 
            if 'limitFieldName' not in rowFormat:
              raise PivotError("addRowFormat", "sort limit missing limitFieldName");
            searchTokens.append(rowFormat['limitFieldName'])
	else:
            if 'limitStatsFn' not in rowFormat:
              raise PivotError("addRowFormat", "sort limit missing limitStatsFn");
            if 'limitByAttributeName' not in rowFormat:
              raise PivotError("addRowFormat", "sort limit missing limitByAttributeName");                
            searchTokens.append(rowFormat['limitStatsFn'] + '(' + rowFormat['limitByAttributeName'] + ')')

def addColFormat(colFormat, searchTokens):
    if 'showSummary' in colFormat and colFormat['showSummary']:
        searchTokens.append('COLSUMMARY')
        searchTokens.append('true')
    if 'showOther' in colFormat and colFormat['showOther'] == False:
        searchTokens.append('SHOWOTHER')
        searchTokens.append('false') # default is true
    if 'limitAmount' in colFormat:
        searchTokens.append('NUMCOLS')
        searchTokens.append(str(colFormat['limitAmount']))

def addFilterToSearch(fil, searchTokens):
    if 'rule' in fil:
        searchTokens.append('FILTER')
        searchTokens.append(fil['fieldName'])
        if fil['type'] == 'boolean' and not 'comparator' in fil['rule']:
            searchTokens.append('is')
        else:
            searchTokens.append(fil['rule']['comparator'])

        #if the comparator is 'isNull' or 'isNotNull', there should be no compareTo value
        if fil['rule']['comparator'] == 'isNull' or fil['rule']['comparator'] == 'isNotNull':
            del fil['rule']['compareTo']

        if 'compareTo' in fil['rule'] and (not fil['type'] == 'boolean' or 'comparator' not in fil['rule'] or fil['rule']['comparator'] == 'is' or fil['rule']['comparator'] == '='):
            if ' ' in fil['rule']['compareTo']:
                searchTokens.append('"' + fil['rule']['compareTo'] + '"')
            else:
                searchTokens.append(fil['rule']['compareTo'])
    elif 'limit' in fil:
        searchTokens.append('LIMIT')
        searchTokens.append(fil['fieldName'])
        searchTokens.append('BY')
        if(fil['limit']['limitType'].upper() == 'HIGHEST'):
            searchTokens.append('top')
        else:
            searchTokens.append('bottom')
        searchTokens.append(str(fil['limit']['limitAmount']))
        
        limitby = fil['limit']['statsFn'] + '(' + fil['limit']['attributeName'] + ')'
        searchTokens.append(limitby)

def addRowOrColToSearch(split, searchTokens, isRow):
    if isRow:
        searchTokens.append('SPLITROW')
    else:
        searchTokens.append('SPLITCOL')

    #fieldName - a string, the name of the field the element operates on
    searchTokens.append(split['fieldName'])

    #label - a string, the label for the cell in the report
    if isRow and 'label' in split and len(split['label']) > 0:
        searchTokens.append('AS')
        searchTokens.append('"' + split['label'] + '"')

    #display - (only applicable for numbers) a string, available options are all, ranges
    #ranges - (optional, only applicable for numbers and when display is ranges) a RANGES
    if 'display' in split and split['display'].upper() == 'RANGES' and 'ranges' in split and split['ranges']!={}:
        addRangeToSearch(split['ranges'], searchTokens)

    #trueLabel - (only applicable for booleans) a string, the label for true values
    if 'trueLabel' in split:
        searchTokens.append('TRUELABEL')
        searchTokens.append('"' + split['trueLabel'] + '"')
    #falseLabel - (only applicable for booleans) a string, the label for false values
    if 'falseLabel' in split:
        searchTokens.append('FALSELABEL')
        searchTokens.append('"' + split['falseLabel'] + '"')
    #period - (only applicable for timestamps) a string, the period to bucket by, available options are auto, year, month, day, hour, minute and second
    if 'period' in split:
        searchTokens.append('PERIOD')
        searchTokens.append(split['period'])
    


def addRangeToSearch(rangeDict, searchTokens):
    rangeTokens = []
    if 'maxNumberOf' in rangeDict and rangeDict['maxNumberOf']!='':
        rangeTokens.append('max=' + rangeDict['maxNumberOf'])
    if 'size' in rangeDict and rangeDict['size']!='':
        rangeTokens.append('size=' + rangeDict['size'])
    if 'start' in rangeDict and rangeDict['start']!='':
        rangeTokens.append('start=' + rangeDict['start'])
    if 'end' in rangeDict and rangeDict['end']!='':
        rangeTokens.append('end=' + rangeDict['end'])
    if rangeTokens != []:
        searchTokens.append('RANGE')
	searchTokens += rangeTokens

def addCellToSearch(cell, searchTokens, objectName):
    #first figure out what function we're applying to the field
    if cell['type'] == 'boolean':
        if cell['value']=='countTrue':
            function = "countTrue"
            fieldName = cell['fieldName']
        elif cell['value']=='countFalse':
            function = "countFalse"
            fieldName = cell['fieldName']
	
    elif cell['type'] == 'objectCount':
        function = "count"
        fieldName = objectName

    elif cell['type'] == 'timestamp' or  cell['type'] == 'number' or cell['type'] == 'string' or cell['type'] == 'ipv4' or cell['type'] == 'childCount':
        function = cell['value']
        fieldName = cell['fieldName']
            #FIXME: may need to do something more sophisticated for childcount in the future
    else:
        raise PivotError("buildSearchWithModel", "unknown cell field type "+cell['type'])
        
    searchTokens.append(function + '(' + fieldName + ')')
        
    escapedlabel = escapeLabel(cell['label'])
    if len(escapedlabel) > 0:
        searchTokens.append('AS')
        searchTokens.append('"' + escapedlabel + '"')


def pivotDictFromSearch(pivotSearch, modelObj):
    #ignore commas
    pivotSearch.replace(',', ' ') #get rid of commas
    searchTokens = pivotSearch.split() #split on whitespace
    searchTokens.reverse()

    pivotDict = {}

    #basically just read tokens one at a time and process.  Some special stuff at the beginning for model/object names etc.
    searchTokens.pop() # get rid of '|'
    searchTokens.pop() # get rid of pivot
    #FIXME: possibly check these?

    #model
    modelName = searchTokens.pop()
    pivotDict['modelName'] = modelName

    #object    
    objectName = searchTokens.pop()
    pivotDict['baseClass'] = objectName

    pivotDict['cells'] = []
    pivotDict['rows'] = []
    pivotDict['columns'] = []
    pivotDict['filters'] = []
    pivotDict['rowFormat'] = {}
    pivotDict['colFormat'] = {}

    while len(searchTokens) > 0:
        next = searchTokens.pop()
        
        #split rows
        if next.upper() == 'SPLITROW':
            pivotDict['rows'].append(processSplitRowOrCol(searchTokens, isRow=True))

        elif next.upper() == 'SPLITCOL':
            pivotDict['columns'].append(processSplitRowOrCol(searchTokens, isRow=False))
            
        elif next.upper() == 'FILTER':
            pivotDict['filters'].append(processFilter(searchTokens))
        
        elif next.upper() == 'LIMIT':
            pivotDict['filters'].append(processLimit(searchTokens))
        
        elif next.upper() == 'ROWSUMMARY':
            next = searchTokens.pop()
            if next.upper() == 'TRUE':
                pivotDict['rowFormat']['showSummary'] = True
            else:
                pivotDict['rowFormat']['showSummary'] = False

        elif next.upper() == 'TOP' or next.upper() == 'BOTTOM' or next.upper() == 'SORT':
            pivotDict['rowFormat'].update(processSortLimit(next.upper(), searchTokens))

        elif next.upper() == 'COLSUMMARY':
            next = searchTokens.pop()
            if next.upper() == 'TRUE':
                pivotDict['colFormat']['showSummary'] = True
            else:
                pivotDict['colFormat']['showSummary'] = False

        elif next.upper() == 'SHOWOTHER':
            next = searchTokens.pop()
            if next.upper() == 'FALSE':
                pivotDict['colFormat']['showOther'] = False
            else:
                pivotDict['colFormat']['showOther'] = True

        elif next.upper() == 'NUMCOLS':
            pivotDict['colFormat']['limitAmount'] = int(float(searchTokens.pop()))
            if pivotDict['colFormat']['limitAmount'] > MAX_COLS:
                raise PivotError("pivotDictFromSearch", "The maximum number of columns allowed is " + str(MAX_COLS))

        #cell values
        else:
            pivotDict['cells'].append(processCell(next, searchTokens))

    return validatePivotDict(pivotDict, modelObj)

def processFnArg(searchTokens): #assumes the next part of the search tokens is of the form "function(argument)" and returns a pair (function, argument)
    tokens = searchTokens.pop().split('(')
    
    function = tokens[0]
    if len(tokens) < 2:
        raise PivotError("processFnArg", "expected a search component of the form <function>(<argument>)")
    
    next = tokens[1]
    argument = ""
    while not next.endswith(')'):
        argument = argument + next
        if len(searchTokens) > 0:
            next = searchTokens.pop()    
        else:
            raise PivotError("processFnArg", "Search parsing did not terminate.  Possible missing closing parenthesis.")
    argument = argument + next.strip(')')
    
    return (function, argument)

def processLimit(searchTokens):
    fil = {}
    fil['fieldName'] = searchTokens.pop()

    if not searchTokens.pop().upper() == 'BY':
        raise PivotError("processLimit", "expeced \"by\" after field name")

    limit = {}
    next = searchTokens.pop()
    if next.upper() == 'TOP':
        limit['limitType'] = 'highest'
    elif next.upper() == 'BOTTOM':
        limit['limitType'] = 'lowest'
    else:
        raise PivotError("processLimit", "must limit by 'top' or 'bottom' values")

    try:
        limit['limitAmount'] = int(float(searchTokens.pop()))
    except:
        raise PivotError("processLimit", "must limit by an integer value")

    if limit['limitAmount'] > MAX_ROWS:
        raise PivotError("processLimit", "max number of rows is " + str(MAX_ROWS))
    
    (limit['statsFn'], limit['attributeName']) = processFnArg(searchTokens) 
        
    fil['limit'] = limit
    return fil
        
def processFilter(searchTokens):
    fil = {}
    fil['fieldName'] = searchTokens.pop()
    
    rule = {}
    rule['comparator'] = searchTokens.pop()
    if not (rule['comparator'] == 'isNull' or rule['comparator'] == 'isNotNull'):
        rule['compareTo'] = processLabel(searchTokens)

    fil['rule'] = rule
    
    return fil


#need to pass "next" here because there's no keyword to indicate a cell value
def processCell(next, searchTokens): 
    #process first arg - this gives us fn and field
    cell = {}
    
    if not '(' in next:
        cell['fieldName'] = next
        cell['value'] = 'count'
    else:
        searchTokens.append(next) #put next back in the searchTokens
        (cell['value'], cell['fieldName']) = processFnArg(searchTokens)

    #process keywords
    while len(searchTokens) > 0 and searchTokens[len(searchTokens) - 1].upper() in CELL_KEYWORDS:
        keyword = searchTokens.pop().upper()
        if keyword == 'AS':
            cell['label'] = processLabel(searchTokens)
        elif keyword == 'SPARKLINE':
            cell['sparkline'] = True

    return cell

def processLabel(searchTokens):
    next = searchTokens.pop()
    if next.startswith('"'):
        label = next.lstrip('"')
        
        #check for the case where the label is a single word with quotation marks around it
        if label.endswith('"'):
            return label.rstrip('"')
    
        while not next.endswith('"'):
            next = searchTokens.pop()
            label = label + ' ' + next.rstrip('"')
        return label
    else:
        return next

def processSplitRowOrCol(searchTokens, isRow):
    split = {}
    split['fieldName'] = searchTokens.pop()
         
    while len(searchTokens) > 0 and searchTokens[len(searchTokens) - 1].upper() in SPLIT_KEYWORDS:
        keyword = searchTokens.pop().upper()

        if keyword == 'AS':
            if isRow:
                split['label'] = processLabel(searchTokens)
            else:
                raise PivotError("processSplitRowOrCol", "a SPLITCOL cannot have a label")
        elif keyword == 'RANGE':
            split['display'] = 'ranges'
            split['ranges'] = processRange(searchTokens)                    
        elif keyword == 'TRUELABEL':
            split['trueLabel'] = processLabel(searchTokens)
        elif keyword == 'FALSELABEL':
            split['falseLabel'] = processLabel(searchTokens)
        elif keyword == 'PERIOD':
            next = searchTokens.pop()
            if next.lower() in ['auto', 'year', 'month', 'day', 'hour', 'minute', 'second']:
                split['period'] = next.lower()

    return split

def processSortLimit(keyword, searchTokens):
    split = {}
    if keyword == 'TOP':
        split['limitType'] = 'descending'
    elif keyword == 'BOTTOM':
        split['limitType'] = 'ascending'
    elif keyword == 'SORT':
        split['limitType'] = 'default'

    split['limitAmount'] = int(float(searchTokens.pop()))
    if split['limitAmount'] > MAX_ROWS:
        raise PivotError("processSortLimit", "max number of rows is " + str(MAX_ROWS))

    if keyword == 'SORT':
        split['limitFieldName'] = searchTokens.pop()
    else:
        (split['limitStatsFn'], split['limitByAttributeName']) = processFnArg(searchTokens)

    return split


def processRange(searchTokens):
    #look for range settings until there are no more
    ranges = {}
    
    while len(searchTokens) > 0 and searchTokens[len(searchTokens) - 1].upper() not in SPLIT_KEYWORDS + ['SPLITCOL'] + ['SPLITROW'] + SORT_KEYWORDS:
        next = searchTokens.pop()
        if next.count('=') > 0:
            settingTokens = next.split('=')
        else:
            raise PivotError("processRange", "range setting must be of the form <setting>=<value>")

        if len(settingTokens) is not 2:
            raise PivotError("pivotDictFromPivotSearch", "invalid range setting for splitrow")
        
        if settingTokens[0].upper() == 'MAX':
            ranges['maxNumberOf'] = settingTokens[1]
        elif settingTokens[0].upper() == 'SIZE':
            ranges['size'] = settingTokens[1]
        elif settingTokens[0].upper() == 'START':
            ranges['start'] = settingTokens[1]
        elif settingTokens[0].upper() == 'END':
            ranges['end'] = settingTokens[1]

    return ranges

#this takes a dict generated by pivotDictFromSearch and fleshes it out to be compatible with buildSearchWithModel
def validatePivotDict(pivotDict, modelObj):

    #check model name if it's there

    if 'modelName' in pivotDict and pivotDict['modelName'] != modelObj.getName():
            raise PivotError("validatePivotDict", "model names do not match")

    #check that object is a valid object in this model
    try:
        myObject = modelObj.getObject(pivotDict['baseClass'])
    except:
        raise PivotError("validatePivotDict", "failed to retrieve object " + pivotDict['baseClass'] + " in model " + modelObj.getName())

    #for each cell, look up the type of the relevant field
    if 'cells' in pivotDict:
        for cell in pivotDict['cells']:
            validateCell(cell, myObject)
    else:
        raise PivotError("validatePivotDict", "cells array is missing")

    #stuff for rows
    if 'rows' in pivotDict:
        for row in pivotDict['rows']:
            validateRowOrCol(row, myObject)

    #stuff for columns
    if 'columns' in pivotDict:
        for col in pivotDict['columns']:
            validateRowOrCol(col, myObject)

    #filters
    if 'filters' in pivotDict:
        for fil in pivotDict['filters']:
            validateFilter(fil, myObject)

    #row formatting
    if 'rowFormat' in pivotDict:
      if not 'showSummary' in pivotDict['rowFormat']:
        pivotDict['rowFormat']['showSummary'] = False
	validateRowLimit(pivotDict['rowFormat'], myObject)

    #col formatting
    if 'colFormat' in pivotDict:
      if not 'showSummary' in pivotDict['colFormat']:
        pivotDict['colFormat']['showSummary'] = False
      if not 'showOther' in pivotDict['colFormat']:
        pivotDict['colFormat']['showOther'] = True
  
    return pivotDict
    
#fill in type and object for any dict that contains fieldName
def validateField(thing, myObject):
    fieldName = thing['fieldName']
    if myObject.hasField(fieldName):
        field = myObject.getField(fieldName)
    elif myObject.hasCalculatedField(fieldName):
        field = myObject.getCalculatedField(fieldName) 
    else:
        raise PivotError("validateField", "Field name " + fieldName + " was not found in object " + myObject.getName())       
    thing['type'] = field.getType()
    thing['owner'] = field.getOwner()


def validateFilter(fil, myObject):
    validateField(fil, myObject)
    if 'limit' in fil:
        dummy = {'fieldName': fil['limit']['attributeName']}
        validateField(dummy, myObject)
        fil['limit']['attributeOwner'] = dummy['owner']
        fil['limit']['attributeType'] = dummy['type']

def validateCell(cell, myObject):
    #fill in field data
    if cell['fieldName'] == myObject.getName():
        cell['type'] = 'objectCount'
        cell['owner'] = myObject.getLineage()
    else:
        validateField(cell, myObject)
    
    #fill in label
    if not 'label' in cell:
        cell['label'] = cell['fieldName']

def validateRowOrCol(split, myObject):
    #fill in field stuff
    validateField(split, myObject)

    #fill in label
    if not 'label' in split:
        split['label'] = split['fieldName']

def validateRowLimit(split, myObject):
    #fill in field stuff
    if 'limitFieldName' in split:
        dummy = {'fieldName': split['limitFieldName']}
        validateField(dummy, myObject)
        split['limitFieldOwner'] = dummy['owner']

    if 'limitByAttributeName' in split:
        dummy = {'fieldName': split['limitByAttributeName']}
        validateField(dummy, myObject)
        split['limitByAttributeOwner'] = dummy['owner']
