import glob
import logging, logging.handlers
import math
import os
import sys
import tempfile
import time
import json
import traceback


SPLUNK_HOME = os.environ['SPLUNK_HOME']


def getErrorMessageDetails(errorCode):
  #  Model Error:
  if errorCode=='MODEL_CANNOT_LOAD_JSON':
    errorMsg="Splunk cannot load the JSON for this data model."
  elif errorCode=='MODEL_EMPTY':
    errorMsg="The selected data model has no data."
  elif errorCode=='MODEL_OBJECT_ERROR':
    errorMsg = "The selected object has encountered an unknown error."
  elif errorCode=='MODEL_OBJECT_LOADING_ERROR':
    errorMsg="Splunk has encountered an unknown error loading this data model object."
  elif errorCode=='MODEL_NOT_SPECIFIED':
    errorMsg="Specify the name of the data model."
  #  Object Error:
  elif errorCode=='OBJECT_NOT_SPECIFIED':
    errorMsg="Specify an object."
  elif errorCode=='OBJECT_CANNOT_OBTAIN_SEARCH':
    errorMsg="The object cannot obtain the search for an unknown reason."
  elif errorCode=='OBJECT_BASETRANSACTION_MULTIPLE_BASEOBJECTS_NOT_BASEEVENT':
    errorMsg="When you include multiple Group Objects in a transaction object definition, they must all be event objects (root or child)."
  elif errorCode=='OBJECT_BASESEARCH_EMPTY_BASESEARCH':
    errorMsg="This root search object requires a search string."
  elif errorCode=='OBJECT_DUPLICATE_OBJECTNAME':
    errorMsg="You cannot create an object with the same Object Name as another object belonging to this data model."
  elif errorCode=='OBJECT_NONAME':
    errorMsg="This object requires an Object Name."
  elif errorCode=='OBJECT_INVALIDNAME':
    errorMsg="The Object Name can only contain letters, numbers and underscores."
  elif errorCode=='OBJECT_INTERNAL_OBJECT_NAME':
    errorMsg="The Object Name you have chosen is reserved. Please choose another."
  elif errorCode=='OBJECT_NULL_OBJECTFACTORY':
    errorMsg="The object factory is null for this object."
  elif errorCode=='OBJECT_CANNOT_ADD_CHILD':
    errorMsg="Splunk cannot add this child object to the parent object." 
  elif errorCode=='OBJECT_CANNOT_ADD_IMPLEMENTS_TO_INTERFACE':
    errorMsg="Splunk cannot add implements to the interface." 
  elif errorCode=='OBJECT_CANNOT_CREATE_PARENT':
    errorMsg="Splunk cannot create a parent object for this object."
  elif errorCode=='OBJECT_DUPLICATE_FIELDNAME':
    errorMsg="An object cannot have attributes with duplicate names." 
  elif errorCode=='OBJECT_DUPLICATE_DISPLAYNAME':
    errorMsg="An object cannot have attributes with duplicate display names." 
  elif errorCode=='OBJECT_INVALID_CALCULATION_TYPE':
    errorMsg="The data model includes an invalid calculation type. Valid calculation types include eval expressions, regular expressions, and lookups."
  elif errorCode=='OBJECT_PIPE_IN_CONSTRAINT':
    errorMsg="Object constraints cannot contain pipe characters."
  elif errorCode=='OBJECT_SUBSEARCH_IN_CONSTRAINT':
    errorMsg="Object constraints cannot contain subsearches."
  elif errorCode=='OBJECT_ERROR':
    errorMsg="The object is invalid for an unknown reason."
  elif errorCode=='OBJECT_CANNOT_OBTAIN_JSON_DESCRIPTION':
    errorMsg="The object cannot obtain its JSON description."
  elif errorCode=='OBJECT_BASETRANSACTION_MISSING_GROUPBY_SPAN_PAUSE':
    errorMsg="Root transaction objects require at least one Group by field, Max Pause value, or Max Span value."
  elif errorCode=='OBJECT_BASETRANSACTION_MISSING_OBJECTSTOGROUP':
    errorMsg="Root transaction objects require at least one Group Objects value."
  elif errorCode=='OBJECT_BASETRANSACTION_CANNOT_LOAD_BASEOBJECT':
    errorMsg="This root transaction object specifies an invalid Group Object."
  elif errorCode=='OBJECT_BASETRANSACTION_INVALID_GROUPBYFIELD':
    errorMsg="This root transaction object includes one or more invalid Group by fields."
  elif errorCode=='OBJECT_BASESEARCH_MISSING_BASESEARCH':
    errorMsg="This root search object requires a search string."
  elif errorCode=='OBJECT_BASEIMPLEMENTS_MISSING_IMPLEMENTSINTERFACE':
    errorMsg="This root implement object is missing a valid implements interface."
  elif errorCode=='OBJECT_BASEIMPLEMENTS_CANNOT_LOAD_IMPLEMENTSINTERFACE':
    errorMsg="This root implements object cannot load the provided implements interface. "
  elif errorCode=='MODEL_NONEXISTENT_OBJECT':
    errorMsg="The specified object does not exist in this data model."
  elif errorCode=='OBJECT_FIELD_EMPTY_FIELDNAME':
    errorMsg="Provide a Field Name for this attribute."
  elif errorCode=='OBJECT_CALCULATION_SPACE_IN_FIELDNAME':
    errorMsg="Calculations cannot have spaces in the field name."
  elif errorCode=='OBJECT_CALCULATION_DUPLICATE_NAME':
    errorMsg="Calculations cannot have multiple output fields with the same name."
  elif errorCode=='OBJECT_CALCULATION_DUPLICATE_DISPLAYNAME':
    errorMsg="Calculations cannot have multiple output fields with the same display name."
  elif errorCode=='OBJECT_CALCULATION_LOOKUP_EMPTY_LOOKUPNAME':
    errorMsg="Select a valid Lookup Table."
  elif errorCode=='OBJECT_CALCULATION_LOOKUP_EMPTY_INPUTFIELD':
    errorMsg="Provide a valid Input Attribute value."
  elif errorCode=='OBJECT_CALCULATION_LOOKUP_EMPTY_OUTPUTFIELDS':
    errorMsg="The Output Field in Lookup requires a valid value that matches the name of a field in the selected Lookup Table."
  elif errorCode=='OBJECT_CALCULATION_GEOIP_EMPTY_LOOKUPNAME':
    errorMsg="Select a valid IP value. This is the name of the IP lookup table you will be using."
  elif errorCode=='OBJECT_CALCULATION_GEOIP_EMPTY_INPUTFIELD':
    errorMsg="Indicate the name of the field you are matching in Splunk, such as clientip."
  elif errorCode=='OBJECT_CALCULATION_GEOIP_INVALID_OUTPUTFIELD':
    errorMsg="Provide a valid GeoIP output field, such as Latitude, Longitude, City, Country, or State/Province."
  elif errorCode=='OBJECT_CALCULATION_REX_EMPTY_FIELD':
    errorMsg="Field Name cannot be empty."
  elif errorCode=='OBJECT_CALCULATION_REX_EMPTY_OUTPUTFIELDS':
    errorMsg="No fields were extracted. Provide a working Regular Expression with capturing groups."
  elif errorCode=='OBJECT_CALCULATION_REX_EMPTY_EXPRESSION':
    errorMsg="Provide a Regular Expression for this attribute."
  elif errorCode=='OBJECT_CALCULATION_REX_INVALID_FIELD':
    errorMsg="The specified Regular Expression does not extract one or more of the listed fields."
  elif errorCode=='OBJECT_CALCULATION_EVAL_EMPTY_OUTPUTFIELD':
    errorMsg="Provide a Field Name for this attribute."
  elif errorCode=='OBJECT_CALCULATION_EVAL_EMPTY_VARVALUE':
    errorMsg="Provide a valid Eval Expression for this attribute."
  else: errorMsg=''
  return errorMsg

'''
Pivot Error:
./datamodel/model.py:		  raise PivotError("getObject", "Interface object doesn't have any implementations")
./controller.py:  if '*' in fieldlabel: raise PivotError('escapeLabel', 'asterix not allowed in field labels') 
./controller.py:    raise PivotError("buildSearchWithModel", "must have non-empty cells or non-empty rows ")
./controller.py:    raise PivotError("buildSearchWithModel", "cannot pivot with empty cells and non-empty columns ")
./controller.py:          raise PivotError("buildSearchWithModel", "empty cell attribute owner")
./controller.py:            raise PivotError("buildSearchWithModel", "unknown cell field type "+k['type'])
./controller.py:	    raise PivotError("buildSearchWithModel", "Sparkline function not yet implemented in tsidxStats") 
./controller.py:            raise PivotError("buildSearchWithModel", "unknown filter rule comparator "+j['rule']['comparator'])
./controller.py:        raise PivotError("buildSearchWithModel", "unknown type "+mytype+" in filter")
./controller.py:        raise PivotError("buildSearchWithModel", "empty fieldName or owner field")
./controller.py:      raise PivotError("buildSearchWithModel", "error building tstats searchString ")
./controller.py:      raise PivotError("buildSearchWithModel", "error building non-tstats searchString ")
./controller.py:      raise PivotError("buildSearchWithModel", "error building non-tstats searchString ") 
./datamodel/stats.py:	  raise PivotError("Stats::__init__", "must have at least one stats function ")
./datamodel/stats.py:	  	raise PivotError("Stats::__init__", "must specify a stats function ")
./datamodel/base.py:	  raise PivotError('setTsidxNamespace', 'must specify tsidx namespace')
./datamodel/base.py:	  raise PivotError("statsTable", "fieldname " + fieldname + " does not exist in object " + self.getName()) 
./datamodel/base.py:	  raise PivotError("statsTable", "unrecognized stats function " + statsfunc) 
./datamodel/base.py:	  raise PivotError("splitby", "must specify non-empty field name in splitby ") 
./datamodel/base.py:	  raise PivotError("splitby", "fieldname " + field + " does not exist in object " + self.getName()) 
'''


class PivotError(Exception):
    def __init__(self, fn, text):
        self.text = text
        self.fn = fn

    def __str__(self):
        return "Pivot Error in "+ self.fn + ": " + self.text

  
class DMError(Exception):
    def __init__(self, errMsg, parameters={}):
        self.errorMsg = errMsg
        self.objectName = object
	self.parameters = parameters

    def __str__(self):
	jsonblob={}
	jsonblob['parameters']=self.parameters
	jsonblob['errorCode']=self.errorMsg

	engMessage = ''
	if "modelName" in self.parameters and self.parameters["modelName"]!='':
	  engMessage += self.parameters["modelName"] + ' model' 
	if "objectName" in self.parameters and self.parameters["objectName"]!='':
	  if engMessage!='': engMessage+=', '
	  engMessage += self.parameters["objectName"] + ' object'
	if engMessage!='': engMessage+=': '
	jsonblob['errorMessageEnglish'] = engMessage + getErrorMessageDetails(self.errorMsg)
	return "~!~"+json.dumps(jsonblob)+"~!~"

    def __json__(self):
	jsonblob={}
	jsonblob['parameters']=self.parameters
	jsonblob['errorCode']=self.errorMsg
	jsonblob['errorMessageEnglish']=getErrorMessageDetails(self.errorMsg)
	return jsonblob


