import glob
import logging, logging.handlers
import math
import os
import sys
import tempfile
import time
import json
from errormsgs import DMError

SPLUNK_HOME = os.environ['SPLUNK_HOME']

class Field:
    def __init__(self, name, fieldSearch='', type='', owner='', required=False, multivalue=False, hidden=False, displayName='', comment='', lookupOutputFieldName='', editable=True):
 	if name=='':
          raise DMError("OBJECT_FIELD_EMPTY_FIELDNAME", {'objectName': owner})

	#if name contains spaces, replace with single quotes around it
	if " " in name:
          self.name = "'"+name+"'"
	else: self.name = name
	if displayName=='': self.displayName = name
	else: self.displayName = displayName
	self.constraints = []
        if fieldSearch!='': self.constraints.append(fieldSearch)
        if type!= '': self.type = type
	else: self.type='string'
	self.owner = owner
        self.isEditable = editable
        self.isRequired = required
        self.isMultivalue = multivalue
        self.isHidden = hidden
	self.comment=comment
	self.lookupOutputFieldName=lookupOutputFieldName  # FIXME: temporary solution for lookups

    def search(self):
        retval = ""
        if self.constraints == [] and self.isRequired:
            retval = retval + self.name + "=* "
	elif len(self.constraints) == 1:
            retval = retval + self.name + self.constraints[0]+" "
        else:
	    if self.constraints != []:	
	      retval = retval + ' ('
              for constraint in self.constraints:
	          if self.constraints.index(constraint) != 0:
	  	      retval = retval + ' OR ' 	
                  retval = retval + self.name + constraint
	      retval = retval + ')'
        return retval

    def constrain(self, value):
        self.constraints.append(value)
	
    def getType(self):
	if self.type!='':
	  return self.type
	else: return 'none'
	
    def getName(self):
	return self.name

    def getDisplayName(self):
	return self.displayName
	
    def setHidden(self,value):
	self.isHidden=value

    def getOwner(self):
	if self.owner != '':
	  return self.owner
	else: return 'none'

    def JSONdesc(self, conciseMode=False, inclOwner=False):
	name = self.name.strip('"').strip("'")
	lookupoutputname={}
	if self.lookupOutputFieldName!='': lookupoutputname={"lookupOutputFieldName": self.lookupOutputFieldName} 
	if conciseMode==True:
	  if inclOwner: #this case is used when we modify an inherited field...need to make sure owner is original owner
            retval= {"fieldName": name, "displayName": self.displayName, "type": self.getType(), "required": self.isRequired, "multivalue": self.isMultivalue, "hidden": self.isHidden, "comment": self.comment, "owner": self.getOwner()}
	  else:
            retval= {"fieldName": name, "displayName": self.displayName, "type": self.getType(), "required": self.isRequired, "multivalue": self.isMultivalue, "hidden": self.isHidden, "comment": self.comment}
	else:
          retval= {"fieldName": name, "displayName": self.displayName, "owner": self.getOwner(), "type": self.getType(), "fieldSearch": self.search(), "required": self.isRequired, "multivalue": self.isMultivalue, "hidden": self.isHidden, "comment": self.comment, 'editable': self.isEditable}
	retval.update(lookupoutputname)
	return retval
    
    def JSONconstraints(self):
        if(len(self.search()) > 0):
            return {"search": self.search(), "owner": self.getOwner()}
        else:
            return None
