define(
        ['jquery', 'underscore', 'jquery.history', 'splunk.util'],
        function($, _, History, splunkUtils) {
    var uri = {
        sync: function(method, model, options) {
            var resp = null;
            options = options || {};
            switch (method) {
                case "read":
                    var oldAttributes = model.toJSON(),
                        newAttributes = uri.parse(model.id);
                    delete oldAttributes["id"];
                    var oldAttrKeys = _.keys(oldAttributes),
                        newAttrKeys = _.keys(newAttributes),
                        attrDiff = _.difference(oldAttrKeys, newAttrKeys);

                    _.each(attrDiff, function(attr) {
                        model.unset(attr);
                    });

                    resp = newAttributes;
                    break;
                case "create":
                    resp = model.toJSON();
                    if (options.replaceState){
                        uri.replaceState(model.id, resp);
                    } else {
                        uri.pushState(model.id, resp);
                    }
                    break;
                case "update":
                    resp = model.toJSON();
                    if (options.replaceState){
                        uri.replaceState(model.id, resp);
                    } else {
                        uri.pushState(model.id, resp);
                    }
                    break;
                case "delete":
                    uri.pushState(model.id, null);
                    break;
                default: 
                    throw new Error('invalid method: ' + method);
            }
            if (resp) {
                options.success(model, resp, options);
            } else if(options.error) {
                options.error("Record not found");
            }
        },
        params: function() {
            var query = window.location.href.split("?")[1] || "";
            return splunkUtils.queryStringToProp(query);
        },
        parse: function(id) {
            var params = uri.params();
            if (params) {
                try {
                    return JSON.parse(decodeURIComponent(params[id]));
                } catch(e) {}
            }
            return {};
        },
        stringify: function(id, data) {
            var params = uri.params();
            if (!data){
                delete params[id];
            } else {
                delete data["id"];
                delete data["_suid"];
                params[id] = JSON.stringify(data);
            }
            return $.param(params).replace(/\+/g, '%20');
        },
        pushState: function(id, data) {
            var old_query = uri.params(),
                query_to_push = uri.stringify(id, data);
            History.pushState(old_query, window.title, "?" + query_to_push);
        },
        replaceState: function(id, data){
            History.replaceState(null, window.title,  "?" + uri.stringify(id, data));
        }
    };
    return uri;
});