define(function(require, exports, module) {
    var _ = require('underscore');
    var BaseChoiceView = require("./basechoiceview");

    var asArray = function(obj) {
        if (obj === undefined) {
            return [];
        }
        return _.isArray(obj) ? obj : [obj];
    };

    /**
     * Base class for choice arrays that can have multiple choice
     * values selected.  All controls based on this class always
     * take and return arrays of values.
     */
    var BaseMultiChoiceView = BaseChoiceView.extend({
        _hasValue: function() {
            var currentValue = this.val();
            if (_.isArray(currentValue)) {
                return currentValue.length > 0;
            }
            else {
                return BaseChoiceView.prototype._hasValue.apply(this, arguments);
            }
        },
        
        val: function(newValue) {
            var oldValue = this.settings.get('value');

            var oldValueAsArray = asArray(oldValue);
            var newValueAsArray = asArray(newValue);
            
            if (arguments.length === 0) {
                return oldValueAsArray;
            }
            
            var sortedOldValue = _.clone(oldValueAsArray).sort();
            var sortedNewValue = _.clone(newValueAsArray).sort();

            // Don't change the value if the new value is logically equal to the old value (ignoring order).
            // However, if the old value is undefined, we go ahead and set it anyway
            // to coerce it into an empty array.
            if (_.isEqual(sortedOldValue, sortedNewValue) && sortedOldValue.length > 0) {
                return oldValue;
            }

            this.settings.set('value', newValueAsArray);
            
            return newValueAsArray;
        }
    });
    
    return BaseMultiChoiceView;
});