define(function(require, exports, module) {

    var _ = require('underscore');
    var $ = require('jquery');
    var BaseMultiChoiceView = require("./basemultichoiceview");

    require("css!../css/checkbox.css");

    var asArray = function(obj) {
        if (obj === undefined) {
            return [];
        }
        return _.isArray(obj) ? obj : [obj];
    };

    /**
     * Displays a list of boolean checkboxes.  This control's value is
     * always the array of selected checkbox values.
     */
    var CheckboxGroupView = BaseMultiChoiceView.extend({
        moduleId: module.id,
        
        className: "splunk-checkboxgroup splunk-choice-input",
        
        options: {
            valueField: "",
            labelField: "",
            "default": undefined,
            choices: [],
            selectFirstChoice: false,
            value: undefined,
            disabled: false
        },

        events: {
            "change input:checkbox": "onDomChange"
        },
        
        initialize: function() {
            this.options = _.extend({}, BaseMultiChoiceView.prototype.options, this.options);
            BaseMultiChoiceView.prototype.initialize.apply(this, arguments);
        },

        _domVal: function() {
            return this.$('input:checkbox:checked').map(function(index, checkbox) {
                return $(checkbox).val();
            }).get();
        },

        onDomChange: function(event) {
            this.val(this._domVal());
        },

        updateDomVal: function(values) {
            this.$('input:checkbox:checked').prop('checked', false);
            _.each(values, function(val) { 
                this.$('input:checkbox[value="' + val + '"]').prop('checked', 'checked');
            });
        },
        
        createView: function() {
            this.$el.empty();
            return $("<div class='splunk-checkboxgroup-choices'/>").appendTo(this.el);
        },

        updateView: function(viz, data) {            
            viz.empty();

            // If there is no data, we don't want to just render a message,
            // because that will look odd. Instead, we render a single checkbox
            // that will subsequently get disabled (in BaseChoiceView), plus
            // the message. Finally, we also set the label to " " to make sure it
            // gets picked up.
            if (!data || data.length === 0) {
                data = [{value: "", label: " "}];
            }

            var that = this;
            var currentValues = this.settings.get("value");
            var id = this.id;

            _.each(data || [], function(entry, idx) {
                var itemId = id + idx;
                var input = $('<input type="checkbox" />')
                    .attr({name: id, value: entry.value, id: itemId});
                
                if (_.contains(currentValues, entry.value)) {
                    input.prop({'checked': 'checked'});
                }

                var choice = $('<div class="choice" />')
                    .append(input)
                    .append($('<label />')
                            .attr("for", itemId)
                            .text(entry.label || "(null)"));

                viz.append(choice);
            });

            return this;
        }
    });
    
    return CheckboxGroupView;
});
