define(
    [
         'underscore',
         'jquery',
         'backbone',
         'util/beacon',
         'controllers/BaseSearch',
         'views/timeline/Master'
    ],
    function(
        _,
        $,
        Backbone,
        beacon,
        BaseSearchController,
        TimelineMasterView
    ){
        return BaseSearchController.extend({
            initializeViewsAndEvents: function(){
                this.model.report.entry.content.set({
                    'disable_timerange_trigger': true
                });
                
                //views
                this.children.timelineMasterView = new TimelineMasterView({
                    model: {
                        result: this.model.result,
                        metaDataResult: this.model.metaDataResult,
                        summary: this.model.summaryV2,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        chartWorking: this.model.chartWorking,
                        resultJsonCols: this.model.resultJsonCols,
                        resultJsonRows: this.model.resultJsonRows,
                        searchResultsPreview: this.collection.searchResultsPreview,
                        application: this.model.application,
                        timeRange: this.model.timeRange,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    },
                    collection: {
                        times: this.collection.times,
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    showAutoPause: this.autoPause
                });

                //Events for the report model
                this.model.report.entry.content.on('change:display.prefs.events.offset change:display.events.fields change:display.events.maxLines change:display.events.raw.drilldown change:display.events.list.drilldown change:display.prefs.events.count change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.general.type change:display.events.table.sortColumn change:display.events.table.sortDirection', _.debounce(function() {
                    this.fetchResult();
                }, 0), this);
                
                this.model.result.on('forcefetch', function() {
                    this.fetchResult();
                },this);

                
                this.model.report.entry.content.on('change:display.events.type', function(m, value, options) {
                    if(m.previousAttributes()['display.events.type'] == 'table' || m.get('display.events.type') === 'table') {
                        this.fetchResult();
                    }
                }, this);
                
                this.collection.selectedFields.on('add remove reset', this.populateReportFromSelectedFields, this);
                
                this.model.summaryV2.on('change:fieldPickerOpen', this.fetchSummary, this);
                
                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime', function() {
                    this.model.report.entry.content.set('display.prefs.events.offset', '0');
                }, this);
                
                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.general.type', _.debounce(function() {
                    this.fetchSummary();
                }, 0), this);
                
                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.general.type', _.debounce(function() {
                    this.fetchTimeline();
                }, 0), this);
                
                this.model.report.entry.content.on('change:display.general.type', this.fetchResultJSONColumn, this);
                
                this.model.report.entry.content.on('change:display.prefs.events.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.events.count')){
                        this.model.report.entry.content.set({'display.prefs.events.offset': '0'});
                    }
                }, this);

                this.model.report.entry.content.on('change:display.prefs.statistics.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.statistics.count')){
                        this.model.report.entry.content.set({'display.prefs.statistics.offset': '0'});
                    }
                }, this);
                
                this.model.report.entry.content.on('change:display.general.enablePreview', function(){
                    var preview = this.model.report.entry.content.get('display.general.enablePreview');
                    if (preview === "1") {
                        this.model.searchJob.enablePreview();
                    } else {
                        this.model.searchJob.disablePreview();
                    }
                }, this);

                this.model.report.entry.content.on('change:display.statistics.sortColumn change:display.statistics.sortDirection change:display.prefs.statistics.count change:display.prefs.statistics.offset change:display.general.type', _.debounce(function() {
                    this.fetchResultJSONRows();
                }, 0), this);
                
                this.model.report.entry.content.on('applied', function() {
                    beacon.ping(this.model.application.get('root'), this.model.application.get('locale'), {event: 'ui.search.change'});
                }, this);
                
                //arbitrators to determine when to push on the history stack
                this.model.report.entry.content.on('change', function(){
                    var changed = $.extend(true, {}, this.model.report.entry.content.changedAttributes()),
                        jobPopulationDeferred = $.Deferred(),
                        uiPrefsPopulationDeferred = $.Deferred();
                    
                    this.populateJob(jobPopulationDeferred, changed);
                    this.populateUIPrefs(uiPrefsPopulationDeferred, changed);
                    
                    $.when(jobPopulationDeferred, uiPrefsPopulationDeferred).always(function() {
                        this.populateClassicUrl(changed);
                    }.bind(this));
                }, this);
                
                this.model.report.on('change:id', function(){
                    this.model.searchJob.set("cannotPauseOnRemove", true);
                    this.classicUrl.save(
                        {
                            's': this.model.report.id
                         },
                         {
                             trigger: true
                         }
                    );
                }, this);
                
                this.model.report.on("close", this.close, this);
                
                //Events on the searchJob model
                this.model.searchJob.on('destroy', function() {
                    this.classicUrl.save(
                        {
                            "display.prefs.events.offset": undefined,
                            "display.prefs.statistics.offset": undefined,
                            "display.events.timelineEarliestTime": undefined,
                            "display.events.timelineLatestTime": undefined,
                            "display.statistics.sortColumn": undefined,
                            "display.statistics.sortDirection": undefined,
                            "display.events.table.sortColumn": undefined,
                            "display.events.table.sortDirection": undefined,
                            auto_pause: undefined
                        },
                        {
                            replaceState: true
                        }
                    );
                }, this);
                
                this.model.searchJob.on("prepared", function(){
                    this.model.report.setDisplayType(this.model.searchJob.isReportSearch());
                    this.registerSearchJobFriends();
                }, this);
                
                this.model.searchJob.on("close", this.close, this);
            },
            setReplacementDOM: function() {
                this.replacementDOM = this.children.timelineMasterView.render().el;
            },
            fetchResult: function() {
                if (this.model.result.id && this.model.report.entry.content.get('display.general.type')==='events') {
                    BaseSearchController.prototype.fetchResult.apply(this);
                }
            },
            fetchResultJSONColumn: function() {
                if (this.model.resultJsonCols.id && this.model.report.entry.content.get('display.general.type')==='visualizations') {
                    BaseSearchController.prototype.fetchResultJSONColumn.apply(this);
                }
            },
            fetchResultJSONRows: function() {
                var type = this.model.report.entry.content.get('display.general.type');
                if (this.model.resultJsonRows.id && (type === 'visualizations' || type === 'statistics')) {
                    BaseSearchController.prototype.fetchResultJSONRows.apply(this);
                }
            },
            fetchSummary: function() {
                if (this.model.summaryV2.id && this.model.report.entry.content.get('display.general.type')==='events') { 
                    if (!this.model.summaryV2.get('fieldPickerOpen')) {
                        BaseSearchController.prototype.fetchSummary.apply(this);
                    }
                }
            },
            fetchTimeline: function() {
                if (this.model.timeline.id && this.model.report.entry.content.get('display.general.type')==='events') {
                    BaseSearchController.prototype.fetchTimeline.apply(this);
                }
            }
        });
    }
);
