(function ($) {
    var body = $('body');

    $.widget( "ui.scrollingpager", {
        options: {
            scrollTimeout: 100,
            container: 'body',
            updateThresholdPx: 250,
            autoLimit: true,
            upperThreshold: 20
        },
        _create: function() {
            this.grid = this.element.data("grid");
            this.container = this._container(); //TODO there are better ways to do this

            this._on({
                'gridrefresh': this.onRefresh
            });

            //todo investigate out why this scroll listener cant go in _bind above, and require $.proxy here
            this.container.bind("scroll", $.proxy( this.onScroll, this));

            this.grid.uiGridBodyTable.css({
                'position': 'absolute' //TODO this should be set in css
            });
        },
        _container: function(){
            if(this.options.container == 'body'){
                return $(window);
            }else{
                return $(this.options.container);
            }
        },
        onRefresh: function(){
            this.gridRowHeight = $(this.grid.uiGridBodyTable.find('tr')[0]).height(); //TODO issues here... can at least avoid the recast to jquery object using eq(0)
            this.totalHeight = this.grid.options.source.totalCount * this.gridRowHeight;
            this.limit = this.grid.options.source.option('paging.limit');
            this.grid.uiGridBody.css({
                height: this.totalHeight
            });
        },
        onScroll: function(event){
            var self = this;

            //need to use more efficient (read: metal) way of getting scrollTop TODO perf test
            var scrollTop = $(event.target).scrollTop();
            if(typeof this.lastScroll=='undefined'){
                this.lastScroll = scrollTop;
            }

            //TODO add tolerance for slow scrolling. eg if scroll is < 50px/second adjustPage
            //TODO use underscore debounce here
            if(this.scrollTimeoutInstance){
                clearTimeout(this.scrollTimeoutInstance);
            }
            this.scrollTimeoutInstance = setTimeout(function(){
                self.evaluatePage(event, scrollTop);
            }, this.options.scrollTimeout);
        },

        evaluatePage: function(event, scrollTop){
            console.log('\n---------- EVALUATING PAGINATION STATE '+ (new Date()).getTime());
            //check bounce. this is due to scroll "bounce" behaviour in chrome/safari on osx TODO investigate CSS that may disable behaviour
            //there is still a bounce case that will get past here when hitting the bottom. consider it a feature that allows user to forcefully trigger refresh :)
            if(scrollTop < 0 || this.lastScroll == scrollTop || this.lastScroll < 0){
                return;
            }

            //TODO need to put some more light weight calculations here that can tell us we can short circuit to avoid calculating all state
            var state = this.getState(scrollTop);

            this.page(state);
            this.lastScroll = state.scrollTop;
        },
        getState: function(scrollTop){
            state = {};
            state.scrollTop = scrollTop;
            state.containerHeight = this.container.height(); //TODO investigate using scrollHeight as the height of container will be misleading when container has a border/margin
            state.scrollBottom = state.containerHeight + state.scrollTop;
            state.topOfGridBody = this.grid.uiGridBody.offset().top;
            state.relativeScrollTop = Math.max(0, state.scrollTop - state.topOfGridBody);
            state.uiGridTableHeight = this.grid.uiGridBodyTable.height();
            state.topOfTable = this.grid.uiGridBodyTable.offset().top;
            state.relativeScrollPosition = state.relativeScrollTop / this.totalHeight;
            state.relativeRowPosition = state.relativeScrollPosition * this.grid.options.source.totalCount;
            state.bottomOfTable = state.topOfTable + state.uiGridTableHeight;
            state.isTopOfTableShowing = state.topOfTable > state.scrollTop && state.topOfTable < state.scrollBottom;
            state.isBottomOfTableShowing = state.bottomOfTable < state.scrollBottom && state.bottomOfTable > state.scrollTop;
            state.isTableAtRelativeTop = state.topOfTable + state.relativeScrollTop == state.topOfGridBody;
            state.isScrollPast = state.scrollTop > state.bottomOfTable;
            state.isScrollAbove = (state.topOfTable > state.scrollBottom) && !state.isTableAtRelativeTop;
            //var uiGridBodyHeight = this.grid.uiGridBody.height();
            //state.relativeRowPositionByRowHeight = Math.ceil( relativeScrollTop / this.gridRowHeight ); //todo make this safe with variable row height
            //var gridBodyTableMarginTop = parseInt( this.grid.uiGridBodyTable.css('marginTop') );
            //var direction = this.lastScroll < scrollTop ? 'down' : 'up';
            return state;
        },
        page: function(state){

            if(this.pageDfd && this.pageDfd.state() == 'pending'){

                console.log('need to abort inflight request...' + this.pageDfd.state());

                //console.log('//TODO should abort in flight request(s) and continue on here (not return). as of now aborting is not possible, so short circut to avoid race-condition');
                return;
            }
            //TODO ensure any existing inflight does not have what we need now. abort request if so.

            //append
            if(state.isBottomOfTableShowing){
                return this.pageDfd = this.appendPage();
            }

            //redraw
            if(state.isScrollPast || state.isScrollAbove){
                return this.pageDfd = this.redrawPage(state);
            }

            //prepend
            if(state.isTopOfTableShowing && !state.isTableAtRelativeTop){
                //TODO dont do anything if we are at start of record set
                //this.prependPage();
                //TODO unsure about prepending. maybe just redraw?
                return this.pageDfd = this.redrawPage(state);
            }

            return true;
        },
        redrawPage: function(state){
            console.log('----------------------- redraw');
            var self = this;
            var dfd = $.Deferred();
            var upperThresholdOffset = this.options.upperThreshold * this.gridRowHeight;
            var at = Math.max(0, state.relativeScrollTop - upperThresholdOffset);
            var offset = Math.max( 0, Math.floor(state.relativeRowPosition) - this.options.upperThreshold );
            //want to ensure we get at least the LIMIT number of records. so offset must be >= to limit
            //offset = Math.min( this.grid.options.source.totalCount-this.limit, offset );
            this.grid.options.source.option('paging.offset', Math.max(0, offset));
            this.grid.options.source.refresh(function(){

                //console.log('//TODO TODO check to ensure the results will be in view. if not, toss em, and call redrawPage again.');

                self.grid.uiGridBodyTable.css({
                    'marginTop': at
                });
                dfd.resolve();//TODO add reject in case of fail
            });
            return dfd.promise();
        },
        appendPage: function(){
            console.log('----------------------- append');
            var dfd = $.Deferred();
            var self = this;
            var lastRecordInSet = this.grid.options.source.options.paging.offset + this.grid.options.source.options.paging.limit;
            this.grid.options.source.option('paging.offset', lastRecordInSet);
            this.grid.options.renderMode = 'append'; //NEED TO IMPROVE GRID TODO
            this.grid.options.source.refresh(function(){
                self.grid.options.renderMode = null;
                dfd.resolve(); //TODO add reject in case of fail
            });
            return dfd.promise();
        },
        prependPage: function(){
            console.log('----------------------- prepend NOT IMPLEMENTED');
            /*
             this.grid.options.source.options.paging.offset = this.rowAtTop;
             this.grid.options.renderMode = 'prepend';
             this.grid.options.source.refresh(function(){
                 self.grid.options.renderMode = null;
                 self.grid.uiGridBodyTable.css({
                    'margin-top': scrollTop
                 });
             });
            */
        }
    });
})(jQuery);