define(['jquery', 'underscore', '../helpers/EventMixin'], function($, _, EventMixin) {

    var PanningScrollbar = function(properties){
        this.properties = properties || {};
    };

    PanningScrollbar.prototype = $.extend({}, EventMixin, {

        onChartLoad: function(chart){
            var parent = this.getParentElement(this.properties['container']);
            this.createPanningSlider(parent, chart);
            this.setSliderPosition(chart);
            this.addEventHandlers(chart);
        },

        onChartLoadOrResize: function(chart) {
            this.setSliderPosition(chart);
        },

        getParentElement: function(container){
            return $(container).parent();
        },

        createPanningSlider: function(parent){
            var $panning = $(".panning");
            if(!($panning.length > 0)) {
                $(parent).after('<div class="panning"></div>');
                $panning.append('<div id="panning-slider"></div>');
            }
        },

        addEventHandlers: function() {
            var that = this,
                panning = $('.panning'),
                delay= 50,
                onPanningScroll= function(){
                    that.setWindowToDisplay(that.pixelToWindowValue(panning));
                    that.trigger('scroll', that.getWindowToDisplay());
                };
            $(panning).bind('scroll', _.throttle(onPanningScroll, delay));
        },

        setSliderPosition: function(chart) {
            var pointsPlottedPerSeries = Math.ceil(this.properties.truncationLimit / (this.properties.numSeries)),
                ptsNotPlottedPerSeries = this.properties.seriesSize - pointsPlottedPerSeries;

            this.numWindows = Math.ceil(ptsNotPlottedPerSeries/pointsPlottedPerSeries);
            this.numberOfWindows = Math.ceil(ptsNotPlottedPerSeries - pointsPlottedPerSeries);

            $('.panning').css({
                'width': chart.plotWidth + 'px',
                'margin-left': chart.plotLeft + 'px',
                'margin-top': '2px',
                'overflow': 'scroll',
                'z-index': '100',
                'display': 'inline-block'
            });

            $('#panning-slider').css({
                'width': (this.numWindows*100) + '%',
                'height': '2px'
            });
        },

        pixelToWindowValue: function(panning){
            var currentPosition, scrollWidth,
                winLen;

            scrollWidth = $("#panning-slider").width();
            winLen = scrollWidth / this.numberOfWindows;
            currentPosition = $(panning).scrollLeft() + (winLen/2);
            return  Math.floor(currentPosition / winLen);
        },

        destroy: function() {
            this.off();
        },

        setWindowToDisplay: function(windowNumber){
            this.windowToDisplay = windowNumber;
        },

        getWindowToDisplay: function(){
            return this.windowToDisplay;
        }

    });

    return PanningScrollbar;

});