define([
            'jquery',
            'underscore',
            './MultiSeries',
            './ScatterSeries',
            '../util/lang_utils'
        ],
        function(
            $,
            _,
            MultiSeries,
            ScatterSeries,
            langUtils
        ) {

    var MultiScatterSeries = function(properties) {
        MultiSeries.call(this, properties);
        this.nestedSeriesList = [];
    };
    langUtils.inherit(MultiScatterSeries, MultiSeries);

    $.extend(MultiScatterSeries.prototype, {

        setData: function(inputData) {

            var filterDataByNameMatch = function(dataSeries, name) {
                return _(dataSeries).filter(function(point, i) {
                    return inputData.labels[i] === name;
                });
            };

            this.nestedSeriesList = _(inputData.labels).chain()
                .uniq()
                .filter(function(name) {
                    return !!name;
                })
                .map(function(name) {
                    var seriesProps = {
                        name: name,
                        type: 'scatter',
                        clickEnabled: this.properties['clickEnabled']
                    };
                    return new ScatterSeries(seriesProps);
                }, this)
                .value();

            _(this.nestedSeriesList).each(function(series) {
                series.setData({
                    x: filterDataByNameMatch(inputData.x, series.getName()),
                    y: filterDataByNameMatch(inputData.y, series.getName())
                });
            }, this);

            this.bindNestedSeries();
        }

    });

    return MultiScatterSeries;

});