/**
 * @author sfishel
 *
 * A mixin to be added to any view that wants to apply data-bindings between its model and its DOM elements
 */

define(['jquery', 'underscore', 'contrib/Backbone.ModelBinder'], function($, _, ModelBinder) {

    // WARNING: here be monkey patches...

    var SplunkModelBinder = function() {
        ModelBinder.call(this);
    };

    var F = function() { };
    F.prototype = ModelBinder.prototype;
    SplunkModelBinder.prototype = new F();
    SplunkModelBinder.prototype.constructor = ModelBinder;

    $.extend(SplunkModelBinder.prototype, {

        _setEl: function(el, elementBinding, convertedValue) {
            var newValue = convertedValue;
            // if we are in matchName mode, find the <option> whose name matches the convertedValue,
            // then call super with its text content
            if(el.is('select') && !elementBinding.elAttribute && elementBinding.matchName) {
                newValue = el.find('option[name="' + convertedValue + '"]').text();
            }
            ModelBinder.prototype._setEl.call(this, el, elementBinding, newValue);
        },

        _setElValue: function(el, convertedValue) {
            // if the convertedValue is an array, check the boxes whose name attributes are in the array,
            // un-check the ones that aren't
            if(el.is('[type="checkbox"]') && _(convertedValue).isArray()) {
                var value = el.val();
                if(_(convertedValue).indexOf(value) === -1) {
                    el.removeAttr('checked');
                }
                else {
                    el.attr('checked', 'checked');
                }
            }
            else {
                ModelBinder.prototype._setElValue.call(this, el, convertedValue);
            }
        },

        _getElValue: function(elementBinding, el) {
            // if we are getting the value from a checkbox group, return a list of the name attributes of the checked boxes
            if(el.is('[type="checkbox"]') && !elementBinding.elAttribute && elementBinding.boundEls.length > 1) {
                var values = [];
                $(elementBinding.boundEls).filter(':checked').each(function(i, boundEl) { values.push($(boundEl).val()); });
                return values;
            }
            // if we are in matchName mode, return the name instead of the text content
            else if(el.is('select') && !elementBinding.elAttribute && elementBinding.matchName) {
                return el.find('option:selected').attr('name');
            }
            else {
                return ModelBinder.prototype._getElValue.call(this, elementBinding, el);
            }
        }

    });

    /*
     * The public mixin traits, for documentation and usage, go here:  http://eswiki.splunk.com/Data_Binding_Mixin
     */

    var DataBinding = {

        applyDataBinding: function(bindings, options) {
            if(!this._dataBindings) {
                this._dataBindings = [];
            }
            options = options || {};
            var binder = new SplunkModelBinder(),
                model = options.model || this.model,
                rootEl = options.rootEl || this.el;

            binder.bind(model, rootEl, bindings);
            this._dataBindings.push(binder);
        },

        removeAllDataBindings: function() {
            _(this._dataBindings).invoke('unbind');
            this._dataBindings = [];
        }

    };

    return DataBinding;

});