define(
    [
        'jquery',
        'underscore',
        'models/Base',
        'splunk.util'
    ],
    function($, _, BaseModel, splunk_util) {
        return BaseModel.extend({
            initialize: function() {
                BaseModel.prototype.initialize.apply(this, arguments);
                // SPL-77030, if a line chart is set to log scale on the y-axis, clear any stack mode
                this.on('change:display.visualizations.charting.axisY.scale', function() {
                    if(this.get('display.visualizations.charting.chart') === 'line'
                                && this.get('display.visualizations.charting.axisY.scale') === 'log') {
                        this.set({ 'display.visualizations.charting.chart.stackMode': 'default' });
                    }
                }, this);
            },
            components: {
                'single': ['gen'],
                'line': ['gen', 'x', 'y', 'leg'],
                'area': ['gen', 'x', 'y', 'leg'],
                'column': ['gen', 'x', 'y', 'leg'],
                'bar': ['gen', 'x', 'y', 'leg'],
                'pie': ['gen'],
                'scatter':  ['gen', 'x', 'y', 'leg'],
                'radialGauge': ['gen', 'ranges'],
                'fillerGauge': ['gen', 'ranges'],
                'markerGauge': ['gen', 'ranges'],
                'statistics': ['gen'],
                'events': ['gen']
            },
            defaults: {
                'display.visualizations.charting.axisY.scale': 'linear',
                'display.visualizations.charting.axisX.scale': 'linear',
                'display.visualizations.charting.axisX.minimumNumber': '',
                'display.visualizations.charting.axisX.maximumNumber': '',
                'display.visualizations.charting.axisY.minimumNumber': '',
                'display.visualizations.charting.axisY.maximumNumber': '',
                'display.visualizations.charting.axisTitleX.text': '',
                'display.visualizations.charting.axisTitleY.text': '',
                'display.visualizations.charting.axisLabelsX.majorUnit': '',
                'display.visualizations.charting.axisLabelsY.majorUnit': '',
                'display.visualizations.charting.legend.placement': 'right',
                'display.visualizations.charting.legend.labelStyle.overflowMode': 'ellipsisMiddle',
                'display.visualizations.charting.chart.stackMode':	'default',
                'display.visualizations.charting.chart.nullValueMode': 'zero',
                'display.visualizations.charting.chart.sliceCollapsingThreshold': 0.01,
                'display.visualizations.charting.chart.rangeValues': '["0","30","70","100"]',
                'display.visualizations.charting.chart.style': 'shiny',
                'display.visualizations.charting.gaugeColors': [0x84E900,0xFFE800,0xBF3030],
                'display.prefs.events.count': '10',
                'display.prefs.statistics.count': '10'
            },
            validation: {
                'display.visualizations.charting.axisX.minimumNumber': [
                    {
                        fn: 'validateNumberOrAuto',
                        required: false
                    },
                    {
                        fn: 'validateXAxisExtremes',
                        required: false
                    }
                ],
                'display.visualizations.charting.axisX.maximumNumber': [
                    {
                        fn: 'validateNumberOrAuto',
                        required: false
                    },
                    {
                        fn: 'validateXAxisExtremes',
                        required: false
                    }
                ],
                'display.visualizations.charting.axisY.minimumNumber': [
                    {
                        fn: 'validateNumberOrAuto',
                        required: false
                    },
                    {
                        fn: 'validateYAxisExtremes',
                        required: false
                    }
                ],
                'display.visualizations.charting.axisY.maximumNumber': [
                    {
                        fn: 'validateNumberOrAuto',
                        required: false
                    },
                    {
                        fn: 'validateYAxisExtremes',
                        required: false
                    }
                ],
                'display.visualizations.charting.axisLabelsX.majorUnit': [
                    {
                        fn: 'validateNumberOrAuto',
                        required: false
                    }
                ],
                'display.visualizations.charting.axisLabelsY.majorUnit': [ 
                    {
                        fn: 'validateNumberOrAuto',
                        required: false
                    }
                ],
                'display.visualizations.charting.axisY.scale': {
                    fn: 'validateYScaleAndStacking',
                    required: false
                },
                'display.visualizations.charting.chart.stackMode': {
                    fn: 'validateYScaleAndStacking',
                    required: false
                },
                'display.visualizations.charting.chart.sliceCollapsingThreshold': {
                    pattern: 'number',
                    min: 0,
                    msg: _('Minimum Size must be a non-negative number.').t(),
                    required: false
                },
                'display.visualizations.charting.chart.rangeValues': {
                    fn: 'validateRangeValues',
                    required: false
                },
                'display.prefs.events.count': {
                    pattern: 'digits',
                    min: 1,
                    msg: _('Rows Per Page must be a positive number.').t(),
                    required: true
                },
                'display.prefs.statistics.count': {
                    pattern: 'digits',
                    min: 1,
                    msg: _('Rows Per Page must be a positive number.').t(),
                    required: true
                }
            },

            validateXAxisExtremes: function(value, attr, computedState) {
                var min = attr === 'display.visualizations.charting.axisX.minimumNumber' ? parseFloat(value) :
                                        parseFloat(computedState['display.visualizations.charting.axisX.minimumNumber']),
                    max = attr === 'display.visualizations.charting.axisX.maximumNumber' ? parseFloat(value) :
                                        parseFloat(computedState['display.visualizations.charting.axisX.maximumNumber']);

                if(!_.isNaN(min) && !_.isNaN(max) && max <= min) {
                    return _('The X-Axis Min Value must be less than the Max Value.').t();
                }
            },

            validateYAxisExtremes: function(value, attr, computedState) {
                var min = attr === 'display.visualizations.charting.axisY.minimumNumber' ? parseFloat(value) :
                                        parseFloat(computedState['display.visualizations.charting.axisY.minimumNumber']),
                    max = attr === 'display.visualizations.charting.axisY.maximumNumber' ? parseFloat(value) :
                                        parseFloat(computedState['display.visualizations.charting.axisY.maximumNumber']);

                if(!_.isNaN(min) && !_.isNaN(max) && max <= min) {
                    return _('The Y-Axis Min Value must be less than the Max Value.').t();
                }
            },

            validateYScaleAndStacking: function(value, attr, computedState) {
                // SPL-77030, since line charts ignore stack mode, this validation rule does not apply
                if(computedState['display.visualizations.charting.chart'] === 'line') {
                    return false;
                }
                var yAxisScale = attr === 'display.visualizations.charting.axisY.scale' ? value :
                                                computedState['display.visualizations.charting.axisY.scale'],
                    stackMode = attr === 'display.visualizations.charting.chart.stackMode' ? value :
                                                computedState['display.visualizations.charting.chart.stackMode'];

                if(yAxisScale === 'log' && stackMode !== 'default') {
                    return _('Log scale and stacking cannot be enabled at the same time.').t();
                }
            },

            validateRangeValues: function(value) {
                var ranges = _(value ? JSON.parse(value) : []).map(parseFloat);
                if(_(ranges).any(_.isNaN)) {
                    return _('All color ranges must be valid numbers.').t();
                }

                var dedupedRanges = _.uniq(ranges),
                    sortedRanges = $.extend([], ranges).sort(function(a, b) { return a - b; });

                if(!_.isEqual(ranges, dedupedRanges) || !_.isEqual(ranges, sortedRanges)) {
                    return _('Color ranges must be entered from lowest to highest.').t();
                }
            },
            
            validateNumberOrAuto: function(value, attr, computedState) {
                var num = parseFloat(value),
                    title;
                switch(attr){
                case 'display.visualizations.charting.axisLabelsX.majorUnit':
                    title = 'X-Axis Interval';
                    break;
                case 'display.visualizations.charting.axisLabelsY.majorUnit':
                    title = 'Y-Axis Interval';
                    break;
                case 'display.visualizations.charting.axisX.minimumNumber':
                    title = 'X-Axis Min Value';
                    break;
                case 'display.visualizations.charting.axisY.minimumNumber':
                    title = 'Y-Axis Min Value';
                    break;
                case 'display.visualizations.charting.axisX.maximumNumber':
                    title = 'X-Axis Max Value';
                    break;
                case 'display.visualizations.charting.axisY.maximumNumber':
                    title = 'Y-Axis Max Value';
                    break;
                default:
                    title = attr;
                    break;
                }
                if(attr === 'display.visualizations.charting.axisLabelsX.majorUnit' || attr === 'display.visualizations.charting.axisLabelsY.majorUnit'){
                    if(value && (_.isNaN(num) || num <= 0) && !value.match(/^(auto)$/)){
                        return splunk_util.sprintf(
                            _('%s must be a positive number or "auto".').t(),
                            title
                        );
                    }
                } else {
                    if(value && _.isNaN(num) && !value.match(/^(auto)$/)){
                        return splunk_util.sprintf(
                            _('%s must be a number or "auto".').t(),
                            title
                        );
                    }
                }
            },

            attrToArray: function(attr) {
                var value = this.get(attr);
                if(!value){
                    return [];
                }
                return _.values(JSON.parse(value));
            },

            rangesValuesToArray: function() {
                return this.attrToArray('display.visualizations.charting.chart.rangeValues');
            },

            deserializeGaugeColorArray: function() {
                var arrayStr =  this.get('display.visualizations.charting.gaugeColors');
                if(arrayStr.charAt(0) !== '[' || arrayStr.charAt(arrayStr.length - 1) !== ']') {
                    return false; //need to find a different way to bail 
                }
                return splunk_util.stringToFieldList(arrayStr.substring(1, arrayStr.length - 1));
            },

            // use auto mode only if ranges and colors are both not defined
            gaugeIsInAutoMode: function() {
                return !this.get('display.visualizations.charting.chart.rangeValues') && !this.get('display.visualizations.charting.gaugeColors');
            }
        },
        {
            CHARTING_ATTRS_FILTER: ['^display.visualizations.charting...*'],
            SINGLE_VALUE_ATTRS_FILTER: ['^singlevalue\..*']
        });
    }
);
