define(
    [
        'jquery',
        'underscore',
        'splunk.util',
        'backbone',
        'models/Base',
        'util/router_utils'
    ],
    function($, _, util, Backbone, BaseModel, routerUtils) {
        var Model = BaseModel.extend({
            id: 'classicURL',
            initialize: function() {
                BaseModel.prototype.initialize.apply(this, arguments);
            },
            sync: function(method, model, options) {
                var resp,
                    state,
                    dfd = $.Deferred();
                options = options || {};

                var silentClear = options.silentClear;
                var replaceState = options.replaceState;

                delete options.silentClear;
                delete options.replaceState;

                if (silentClear) {
                    model.clear({silent: true});
                }

                if (method==='read') {
                    resp = model.currentQueryString();
                } else if (method==='create' || method==='update') {
                    state = model.toURLEncodedQueryString(options);
                    if (replaceState){
                        model.replaceState(state, options);
                    } else {
                        model.pushState(state, options);
                    }
                    resp = this.currentQueryString();
                } else if (method==='delete') {
                    this.pushState('', options);
                    resp = this.currentQueryString();
                } else {
                    throw new Error('invalid method: ' + method);
                }
                options.success(model, resp, options);
                return dfd.resolve(model, resp, options).promise();
            },
            parse: function(response) {
                // Parse "a=1&b=2" -> {'a': '1', 'b': '2'}
                var urlQueryParameters = util.queryStringToProp(response);
                
                /* 
                 * Insert undefined values for each old attribute that
                 * is not specified in the new query parameters.
                 * 
                 * That way, attributes missing from the new URL will
                 * be blanked out appropriately.
                 */
                var newAttributesWithBlanks = _.clone(this.attributes);
                _.each(newAttributesWithBlanks, function(value, key) {
                    newAttributesWithBlanks[key] = undefined;
                });
                _.extend(newAttributesWithBlanks, urlQueryParameters);
                
                return newAttributesWithBlanks;
            },
            currentQueryString: function() {
                var fragment;
                if (Backbone.history._hasPushState) {
                    return window.location.href.split('?')[1] || '';//not safe to read directly
                }
                fragment = Backbone.history.getFragment(window.location.href.split('#')[1] || '');//not safe to read directly
                return fragment.split('?')[1] || '';
            },
            pushState: function(data, options) {
                if (data){
                    data = '?' + data;
                }
                Backbone.history.navigate(this.root() + data, $.extend(true, {}, options, {replace: false}));
            },
            replaceState: function(data, options) {
                if (data) {
                    data = '?' + data;
                }
                Backbone.history.navigate(this.root() + data, $.extend(true, {}, options, {replace: true}));
            },
            root: function() {
                return Backbone.history._hasPushState ? window.location.pathname :
                        '#' + routerUtils.strip_route(window.location.pathname);
            }
        });
        return new Model();
    }
);
