/**
 * A reusable model encapsulating the fetch data for "results-like" endpoints.
 *
 * Adds special handling of the "sortKey" and "sortDirection" attributes, which are mapped to a trailing "| sort" in the
 * post-process search, and the "filter" attribute, which is mapped from a dictionary of string pairs
 * to keyword filters in the post-process search string.
 */

define([
            'underscore',
            'models/Base'
        ],
        function(
            _,
            Base
        ) {

    return Base.extend({

        defaults: {
            filter: {}
        },

        toJSON: function() {
            var json = Base.prototype.toJSON.call(this),
                sortKey = json.sortKey,
                sortDirection = json.sortDirection,
                search = [];
            if(_(json.filter).size() > 0) {
                _(json.filter).each(function(match, key) {
                    search.push('(' + key + '=*' + match + '*' + ')');
                });
                search = ['search ' + search.join(' OR ')];
            }
            delete json.filter;

            if(sortKey) {
                search.push('| sort ' + (sortDirection === 'desc' ? '-' : '') + '"' + sortKey + '"');
            }
            delete json.sortKey;
            delete json.sortDirection;

            if(search.length > 0) {
                json.search = search.join(' ');
            }
            return json;
        }

    });

});
