/**
 * @author sfishel
 *
 * A basic model to encapsulate the metadata associated with rendering a grid view.
 *
 * Currently a work-in-progress, for up to date documentation see:
 * http://eswiki.splunk.com/Bubbles_Grid_and_Table_Implementation#The_Grid_Source:_Metadata_Object
 */

define(['jquery', 'models/Base'], function($, BaseModel) {

    return BaseModel.extend({

        /**
         * Attributes:
         *
         * offset {Integer} the offset index of the first grid item to display
         * count {Integer} the number of items to display
         * totalCount {Integer} the total number of items
         * sortKey {String} a key to sort results on
         * sortDirection {String} the sort direction
         * filters {Object} a dictionary of field-name to filter-string pairs
         */

        defaults: {
            offset: 0,
            count: 20,
            sortKey: '',
            sortDirection: 'desc',
            filter: {}
        },

        /**
         * (To be optionally extended by sub-classes)
         *
         * Returns a JSON representation of the metadata, which will be passes as fetch parameters whenever the grid
         * data object is refreshed.
         *
         * @return {Object}
         */

        toJSON: function() {
            return ({
                offset: this.get('offset'),
                count: this.get('count'),
                sortKey: this.get('sortKey'),
                sortDirection: this.get('sortDirection'),
                filter: $.extend(true, {}, this.get('filter'))
            });
        },

        // stub out the save method so this model can fill in for a setting model
        save: function() { }

    });

});