/**
 * @author sfishel
 *
 * A model for an individual number column pivot element.
 */

define([
            'jquery',
            'underscore',
            'splunk.util',
            '../BaseElement'
        ],
        function(
            $,
            _,
            splunkUtils,
            BaseElement
        ) {

    return BaseElement.extend({

        /**
         * Attributes
         *
         * (See models/pivot/elements/BaseElement.js for inherited attributes)
         *
         * display {String} the display mode, allowed options: all, ranges
         * maxNumRanges {String} the maximum number of ranges to display, must parse to a positive integer
         * rangeSize {String} the size of each range, must parse to a positive number
         * rangeStart {String} the start value for the ranges, must parse to a number
         * rangeEnd {String} the end value for the ranges, must parse to a number
         * limitAmount {String, should parse to an integer} the number of results to limit to
         */

        defaults: $.extend({}, BaseElement.prototype.defaults, {

            elementType: 'column',
            display: 'ranges'

        }),

        validation: {
            maxNumRanges: {
                pattern: 'digits',
                min: 2,
                required: false,
                msg: _('Max # of Ranges must be an integer greater than 1.').t()
            },
            rangeSize: {
                pattern: 'number',
                min: Number.MIN_VALUE,
                required: false,
                msg: _('Range Size must be a positive number.').t()
            },
            rangeStart: {
                pattern: 'number',
                required: false,
                msg: _('Range Start must be a number.').t()
            },
            rangeEnd: {
                pattern: 'number',
                required: false,
                msg: _('Range End must be a number.').t()
            }
        },

        parse: function(response) {
            response = $.extend(true, {}, response);
            if(response.ranges) {
                response.maxNumRanges = response.ranges.maxNumberOf;
                response.rangeSize = response.ranges.size;
                response.rangeStart = response.ranges.start;
                response.rangeEnd = response.ranges.end;
                delete response.ranges;
            }
            return response;
        },

        toJSON: function() {
            var json = BaseElement.prototype.toJSON.call(this);
            return $.extend(json, {
                display: this.get('display'),
                ranges: {
                    maxNumberOf: this.get('maxNumRanges'),
                    size: this.get('rangeSize'),
                    start: this.get('rangeStart'),
                    end: this.get('rangeEnd')
                }
            });
        }

    });

});
