/**
 * A replacement for the <models/SplunkDWhiteList> in <models/services/datamodel/DataModel>.
 *
 * The services/datamodel endpoint does not yet support _new, so we hard-code the required and optional fields here.
 */

define([
            'jquery',
            'models/SplunkDWhiteList'
        ],
        function(
            $,
            SplunkDWhiteListV2
        ) {

    var CREATE_DATA_MODEL_WHITELIST = {
        entry: [
            {
                fields: {
                    required: [
                        'name'
                    ],
                    optional: [
                        'description'
                    ],
                    wildcard: []
                }
            }
        ]
    };

    var UPDATE_DATA_MODEL_WHITELIST = {
        entry: [
            {
                fields: {
                    required: [],
                    optional: [
                        'acceleration',
                        'description'
                    ],
                    wildcard: []
                }
            }
        ]
    };

    var UPDATE_NO_ACCELERATION_WHITELIST = {
        entry: [
            {
                fields: {
                    required: [],
                    optional: [
                        'description'
                    ],
                    wildcard: []
                }
            }
        ]
    };

    var DataModelWhiteList = SplunkDWhiteListV2.extend({

        sync: function(method, model, options) {
            if(method !== 'read') {
                throw new Error('Method not supported: ' + method);
            }
            var response;
            if(options.url.indexOf(model.url + '/_new') > -1) {
                response = CREATE_DATA_MODEL_WHITELIST;
            }
            else {
                response = DataModelWhiteList.canEditAcceleration ? UPDATE_DATA_MODEL_WHITELIST : UPDATE_NO_ACCELERATION_WHITELIST;
            }
            options.success(model, response, options);
            model.trigger('sync', model, response, options);
            return $.Deferred().resolve(model, response, options);
        }

    },
    {
        canEditAcceleration: true
    });

    return DataModelWhiteList;
});