define([
        'underscore', 
        'backbone', 
        'models/SplunkDBase',
        'models/Application',
        'models/services/AppLocal',
        'models/config',
        'models/services/server/ServerInfo',
        'views/shared/Page',
        'util/csrf_protection',
        'util/ajax_no_cache',
        'util/ajax_logging',
        'splunk.util',
        'util/console',
        'splunk.error'
    ], 
    function(
        _, 
        Backbone,
        SplunkDBaseModel,
        ApplicationModel,
        AppLocalModel,
        configModel,
        ServerInfoModel,
        PageView,
        undefined,
        ajaxNoCache,
        ajaxLogging,
        splunkUtils,
        console,
        splunkError
    ) {
    return Backbone.Router.extend({
        routes: {
            ':locale/app/:app/:page': 'page',
            ':locale/app/:app/:page/': 'page',
            ':locale/app/:app/:page/*splat': 'page',
            ':locale/manager/:app/:page': 'page',
            ':locale/manager/:app/:page/': 'page',
            ':locale/manager/:app/:page/*splat': 'page',
            '*root/:locale/app/:app/:page': 'pageRooted',
            '*root/:locale/app/:app/:page/': 'pageRooted',
            '*root/:locale/app/:app/:page/*splat': 'pageRooted',
            '*root/:locale/manager/:app/:page': 'pageRooted',
            '*root/:locale/manager/:app/:page/': 'pageRooted',
            '*root/:locale/manager/:app/:page/*splat': 'pageRooted',
            '*splat': 'notFound' 
        },
        initialize: function() {
            //add __splunk__ to global namespace
            if (window.location.href.indexOf('debug=1')!=-1) {
                (function(exports) {
                    this.__splunk__ = exports;
                })(this);
            }
            this.enableAppBar = true;
            this.enablePageView = true;
            this.loadingMessage = '';
            this.model = {};
            this.model.config = configModel;
            this.model.application = new ApplicationModel({owner: this.model.config.get('USERNAME')});
            this.collection = {};
            this.deferreds = {};
            this.history = {};
            _.each(this.routes, function(value) {
                this.on('route:' + value, function() {
                    this.history[window.location.pathname] = true;
                }, this);
            }, this);
        },
        page: function(locale, app, page) {
            this.shouldRender = !this.history[window.location.pathname];
            this.model.application.set({
                locale: decodeURIComponent(locale),
                app: decodeURIComponent(app),
                page: decodeURIComponent(page.split('?')[0])
            });
            var appNavPartialData = __splunkd_partials__['/servicesNS/' + encodeURIComponent(this.model.application.get('owner')) + '/' + encodeURIComponent(this.model.application.get('app')) + '/apps/nav'];
            if (appNavPartialData && !this.model.appNavPartial) {
                this.model.appNavPartial = new SplunkDBaseModel({}, {splunkDPayload: appNavPartialData});
            }
            var appsLocalPartialData = __splunkd_partials__['/servicesNS/nobody/system/apps/local/' + encodeURIComponent(this.model.application.get('app'))];
            if (appsLocalPartialData && !this.model.appsLocalPartial) {
                this.model.appsLocalPartial = new AppLocalModel({}, {splunkDPayload: appsLocalPartialData});
            }
            var serverInfoPartialData = __splunkd_partials__['/services/server/info'];
            if (serverInfoPartialData && !this.model.serverInfoPartial) {
                this.model.serverInfoPartial = new ServerInfoModel({}, {splunkDPayload: serverInfoPartialData});
            }
            if (this.enablePageView && !this.pageView) {
                this.pageView = new PageView({
                    splunkBar: true,
                    footer: true,
                    section: this.model.application.get('page'),
                    loadingMessage: this.loadingMessage,
                    model: {
                        application: this.model.application,
                        appNav: this.enableAppBar ? this.model.appNavPartial : undefined
                    }
                });
                this.pageView.render();
            }
        },
        pageRooted: function(root, locale, app, page) {
            this.model.application.set({
                root: decodeURIComponent(root)
            }, {silent: true});
            this.page(locale, app, page);
        },
        notFound: function() {
            console.log('Page not found.');
        },
        // convenience method for subclasses to update the page <title>
        // as browser compatibility issues arise, they can be encapsulated here
        setPageTitle: function(title) {
            document.title = splunkUtils.sprintf(_('%s | Splunk').t(), title);
        }
    });
});
