define(
    [
        'jquery',
        'underscore',
        'backbone',
        'routers/Base',
        'helpers/polling_manager',
        'models/services/cluster/master/Control',
        'models/services/cluster/master/Info',
        'models/services/cluster/master/Generation',
        'collections/services/cluster/master/Peers',
        'views/clustering/push/Master',
        'util/console'
    ],
    function(
        $,
        _,
        Backbone,
        BaseRouter,
        PollingManager,
        MasterControlModel,
        MasterInfoModel,
        MasterGenerationModel,
        MasterPeersCollection,
        ClusteringPushView,
        console
        ){
        return BaseRouter.extend({
            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.setPageTitle(_('Distribute Configuration Bundle').t());
                var that = this;
                this.VALIDATION_SUCCESS = 'Validation successful';
                this.VALIDATION_FAIL = 'Validation failed';

                this.pushModel = new Backbone.Model();
                this.masterControl = new MasterControlModel();
                this.masterInfo = new MasterInfoModel();
                this.masterPeers = new MasterPeersCollection();
                this.masterGeneration = new MasterGenerationModel();
                this.masterGeneration.fetch();

                this.masterActiveBundle = null;
                this.masterLatestBundle = null;
                this.errors = {};
                this.validationProgress = [];
                this.restartProgress = [];

                this.clusteringPushView = new ClusteringPushView({
                    model: {
                        application: this.model.application,
                        pushModel: this.pushModel,
                        masterInfo: this.masterInfo,
                        masterControl: this.masterControl,
                        masterGeneration: this.masterGeneration
                    }
                });

                // initial setup
                this.pushModel.set('inProgress', false); // if push is in progress at this moment
                this.firstRun = true; // need it to stop the initial page load
                that.pushModel.set('peersValidated', 0);
                that.pushModel.set('peersRestarted', 0);
                this.masterPeers.on('reset change', function() {
                    /*
                    Here we're going through all peers, counting as done those that have no errors and active=latest=master
                     */
                    if (!that.masterActiveBundle || !that.masterLatestBundle) {
                        return;
                    }
                    var i;
                    // initial reset of validationProgress array
                    if (that.validationProgress.length == 0) {
                        for (i=0; i<that.masterPeers.length; i++) {
                            that.validationProgress[i] = false;
                        }
                    }
                    if (that.restartProgress.length == 0) {
                        for (i=0; i<that.masterPeers.length; i++) {
                            that.restartProgress[i] = false;
                        }
                    }

                    if (that.pushModel.get('state') == 'validation') {
                        // while validation is on, repopulate the errors from zero
                        that.errors = {};
                    }

                    console.log('peers state:'+ that.pushModel.get('state'));
                    console.log(that.masterPeers.length);
                    i = 0;
                    that.masterPeers.each(function(peer) {
                        var validationErrors = peer.entry.content.get('bundle_validation_errors'),
                            label = peer.entry.content.get('label'),
                            bundleStatus = peer.entry.content.get('bundle_status'),
                            status = peer.entry.content.get('status');

                        if (!that.isValidationComplete() || that.firstRun) {
                            console.log(label + ' bundle_status:', bundleStatus, 'status:', status, 'master-valid-comp:', that.bundle_validation_on_master_succeeded);
                            if (validationErrors) {
                                console.log(validationErrors);
                                that.errors[label] = validationErrors;
                            }
                            if (bundleStatus == that.VALIDATION_SUCCESS ||
                                bundleStatus == that.VALIDATION_FAIL ||
                                status !== 'Up' ||
                                that.firstRun) {

                                that.validationProgress[i] = true;
                            }
                            console.log('validationProgress:',that.validationProgress);
                        }

                        if (that.pushModel.get('state') == 'restart') {
                            console.log(label + ': ' + status);
                            if (status == 'Restarting' || status == 'Down') {
                                that.restartProgress[i] = true;
                            }
                        }

                        i+=1;

                    });
                    that.firstRun = false;

                    var peersValidated = that.getValidatedCount();
                    var peersRestarted = that.getRestartedCount();
                    console.log('peersValidated', peersValidated);
                    if (peersValidated >= that.pushModel.get('peersValidated')) {
                        that.pushModel.set('peersValidated', peersValidated);
                    }
                    if (peersRestarted >= that.pushModel.get('peersRestarted')) {
                        that.pushModel.set('peersRestarted', peersRestarted);
                    }
                    that.pushModel.set('peersTotal', that.masterPeers.length);
                    console.log('errors:', that.errors);
                    that.pushModel.set('errors', that.errors);
                    that.pushModel.trigger('tick');


                    console.log('validation complete: '+ that.isValidationComplete());
                    if (that.pushModel.get('state') == 'idle' && that.isValidationComplete()) {

                        console.log('master done');

                        PollingManager.stop(that.masterPeers);

                        that.pushModel.set('lastPushSuccess', that.isPushSuccessful());
                        that.pushModel.set('inProgress', false);
                                    console.log('all peers updated: polling stopped!');
                    }
                });

                this.masterInfo.on('reset change', function() {
                    that.masterActiveBundle = that.masterInfo.entry.content.get('active_bundle').checksum;
                    that.masterLatestBundle = that.masterInfo.entry.content.get('latest_bundle').checksum;
                    that.masterValidationSuccess = that.masterInfo.entry.content.get('bundle_validation_on_master_succeeded');
                    that.masterValidationErrors = that.masterInfo.entry.content.get('bundle_validation_errors_on_master');

                    var validation = that.masterInfo.entry.content.get('bundle_validation_in_progress'),
                        reload = that.masterInfo.entry.content.get('reload_bundle_issued'),
                        restart = that.masterInfo.entry.content.get('rolling_restart_flag'),
                        label = that.masterInfo.entry.content.get('label');

                    if (that.masterValidationErrors && that.masterValidationErrors.length) {
                        PollingManager.stop(that.masterPeers);
                        var errors = {};
                        errors[label] = that.masterValidationErrors;
                        that.pushModel.set('lastPushSuccess', that.isPushSuccessful());
                        that.pushModel.set('inProgress', false);
                        that.pushModel.set('errors', errors);
                        that.pushModel.trigger('tick');
                    }
                    console.log('master-valid:',that.masterValidationSuccess,that.masterValidationErrors);
                    var state = validation? 'validation': reload? 'reload': restart? 'restart': 'idle';
                    that.pushModel.set('state', state);

                    if (state == 'idle') {
                        PollingManager.stop(that.masterInfo);
                    }
                    console.log('[Master] state:' + state, 'a:'+that.masterActiveBundle, 'l:'+that.masterLatestBundle);
                });


                this.pushModel.on('confirmed', function() {
                    /*
                    On user confirm, issue an apply command and start polling status endpoints until all peers
                    receive the bundle or report an error
                     */
                    that.pushModel.set('peersValidated', 0);
                    that.pushModel.set('errors', {});
                    that.validationProgress.splice(0);
                    this.masterControl.save()
                        .done(function(){
                            console.log('     ----    ');
                            console.log('push command went successful');
                            that.monitorPushStatus();
                        })
                        .fail(function() {
                            that.masterInfo.fetch();
                        });
                }, this);

                // Initial status check
                this.monitorPushStatus();

            },

            getValidatedCount: function() {
                return _.reduce(this.validationProgress, function(memo, item) {
                    return item ? memo+1 : memo;
                }, 0);
            },

            getRestartedCount: function() {
                return _.reduce(this.restartProgress, function(memo, item) {
                    return item ? memo+1 : memo;
                }, 0);
            },

            isValidationComplete: function() {
                return _.reduce(this.validationProgress, function(memo, item) {
                    return memo && item;
                }, true);
            },

            isPushSuccessful: function() {
                return this.masterValidationSuccess && (this.masterActiveBundle == this.masterLatestBundle) && (_.isEmpty(this.errors));
            },

            monitorPushStatus: function() {
                this.pushModel.set('state', 'validation');
                this.pushModel.set('inProgress', true);
                PollingManager.start(this.masterInfo, {
                    delay: 1000
                });
                PollingManager.start(this.masterPeers, {
                    delay: 500
                });
                console.log('polling started!');
            },

            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                $('.preload').replaceWith(this.pageView.render().el);
                this.pageView.$('.main-section-body').append(this.clusteringPushView.render().el);
            }
        });
    }
);

