define(
    [
        'jquery',
        'underscore',
        'collections/services/datamodel/DataModels',
        'models/classicurl',
        'views/data_models/DataModels',
        'routers/Base',
        'splunk.util'
    ],
    function(
        $,
        _,
        DataModelCollection,
        classicUrl,
        DataModels,
        BaseRouter,
        splunkUtils
    ) {
        return BaseRouter.extend({

            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.setPageTitle(_('Select a Data Model').t());

                this.dmCollection = new DataModelCollection();
                // set the collection to sort ascending by displayName
                this.dmCollection.fetchData.set({ sortKey: 'displayName', sortDirection: 'asc', count: 0 }, { silent: true });
                
                //views
                this.dataModelsView = new DataModels({
                    collection: this.dmCollection,
                    model: {
                        application: this.model.application
                    }
                });
            },

            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                if (this.shouldRender) {
                    $('.preload').replaceWith(this.pageView.el);
                    this.pageView.$('.main-section-body').append($('<div class="data-models-placeholder"></div>'));
                }
                
                $('.data-models-placeholder').replaceWith(this.dataModelsView.render().el);

                var that = this,
                    fetchData = {
                        concise: true,  
                        app: this.model.application.get('app'),
                        owner: this.model.application.get('owner')
                    };

                $.when(classicUrl.fetch(), this.dmCollection.fetch({ data: fetchData })).done(function() {
                    var nonEmptyDataModels = that.dmCollection.filter(function(dataModel) { return !dataModel.isEmpty(); });
                    // If there is only one data model, redirect immediately to the data model explorer, unless the "noredirect" flag is set
                    if(nonEmptyDataModels.length === 1 && !classicUrl.get('noredirect')) {
                        // set the "noredirect" flag before redirecting to avoid a back-button loop
                        classicUrl.save({ noredirect: true }, { replaceState: true });
                        var firstModel = nonEmptyDataModels[0];
                        splunkUtils.redirect_to('app/' + encodeURIComponent(that.model.application.get('app')) + '/data_model_explorer', { model: firstModel.get('id') });
                    }
                });
            }
        });
    }
);
