define(
    [
        'underscore',
        'jquery',
        'routers/Base',
        'models/services/server/ServerInfo',
        'util/splunkd_utils', 
        'views/deploymentserver/DeploymentServer',
        'views/deploymentserver/shared/ErrorMessage',
        'views/shared/Paywall', 
        'models/services/AppLocal',
        'collections/services/DeploymentServerClients',
        'collections/services/DeploymentServerClasses',
        'collections/services/DeploymentServer',
        'collections/services/DeploymentApplications',
        'collections/services/deploymentserver/ConfigViolations',
        'bootstrap.tab'
    ],
    function(
        _,
        $,
        BaseRouter,
        ServerInfo, 
        splunkd_utils, 
        DeploymentServerView,
        ErrorMessageView, 
        Paywall, 
        AppLocalModel, 
        DeploymentServerClientsCollection,
        DeploymentServerClassesCollection,
        DeploymentServerCollection,
        DeploymentAppsCollection,
        ConfigViolationsCollection,
        undefined
    ){
        return BaseRouter.extend({
            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.setPageTitle(_('Forwarder Management').t());
                //Models
                this.appLocalModel = new AppLocalModel();
                
                this.enableAppBar = false;
            },
            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                $('.preload').replaceWith(this.pageView.el);

                //collections
                var serverClientsCollection = new DeploymentServerClientsCollection();
                var clientsWithUnknownStatus = new DeploymentServerClientsCollection();
                var serverClassesCollection = new DeploymentServerClassesCollection();
                var deploymentAppsCollection = new DeploymentAppsCollection(); 
                var deploymentServerCollection = new DeploymentServerCollection();  
                var configViolations = new ConfigViolationsCollection();  

                this.deferred = {}; 
                this.deferred.configViolations = configViolations.fetch(); 
                this.deferred.clients = serverClientsCollection.fetch({data:{count: 10, offset: 0}}); 
                this.deferred.clientsWithUnknownStatus = clientsWithUnknownStatus.fetch({data:{count: 1, offset: 0, search:'Unknown'}}); 
                this.deferred.serverclasses = serverClassesCollection.fetch({data:{count: 10, offset: 0}}); 
                this.deferred.apps = deploymentAppsCollection.fetch({data:{count: 10, offset: 0}}); 

          
                this.deferred.config = deploymentServerCollection.fetch({data:{app:'system', owner:'nobody'}, 
                   error: function() {
                        //There was an error loading the UI.  Two possibilities: User has a free license or there is a syntax error in his serverclass.conf        
                        var serverInfoModel = new ServerInfo();
                        serverInfoModel.fetch({
                            success: function() {
                                if (serverInfoModel.entry.content.get('isFree') == true) {  // Case 1: User has a free license
                                    this.payWallView = new Paywall({model:this.model, title: "Forwarder management"}); 
                                    this.pageView.$('.main-section-body').append(this.payWallView.render().el);
                                } else {   // Case 2: User has a syntax error in serverclass.conf
                                     this.errorView = new ErrorMessageView();  
                                     this.pageView.$('.main-section-body').append(this.errorView.render().el);
                                 }
                            }.bind(this)
                       });
                  }.bind(this)
              }); 

/*
                if (!this.deferred.appLocalDeferred) {
                    this.deferred.appLocalDeferred = this.appLocalModel.fetch({
                        url: splunkd_utils.fullpath(this.appLocalModel.url + "/" + this.model.application.get("app")),
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner")
                        }
                    });
                }
*/


                $.when(                    
                    this.deferred.clients, 
                    this.deferred.clientsWithUnknownStatus, 
                    this.deferred.serverclasses, 
                    this.deferred.apps, 
                    this.deferred.config, 
                    //this.deferred.appLocalDeferred, 
                    this.deferred.configViolations
                ).then(function(){
                    this.deploymentServerView = new DeploymentServerView({
                            collection: { 
                                serverClients: serverClientsCollection, 
                                serverClasses: serverClassesCollection,
                                deploymentApps: deploymentAppsCollection,
                                deploymentServers: deploymentServerCollection, 
                                clientsWithUnknownStatus: clientsWithUnknownStatus 
                            }, 
                            isReadOnly: configViolations.length > 0, 
                            application: this.model.application 
                        });  
                        this.pageView.$('.main-section-body').append(this.deploymentServerView.render().el);
                }.bind(this));
            }
        });
    }
);

