define([
            'jquery',
            'underscore',
            'routers/Base',
            'controllers/PivotController',
            'models/services/datamodel/DataModel',
            'models/pivot/PivotReport',
            'models/services/AppLocal',
            'models/services/authentication/User',
            'models/classicurl',
            'collections/services/data/ui/Times',
            'collections/services/datamodel/DataModels',
            'views/shared/ResponsiveHeaderPage',
            'views/shared/FlashMessages',
            'helpers/pivot/ReportHistoryManager',
            'uri/route',
            'util/splunkd_utils'
        ],
        function(
            $,
            _,
            Base,
            PivotController,
            DataModel,
            PivotReport,
            AppLocal,
            User,
            classicurl,
            Times,
            DataModels,
            ResponsiveHeaderPage,
            FlashMessages,
            ReportHistoryManager,
            route,
            splunkdUtils
        ) {

    return Base.extend({

        initialize: function() {
            Base.prototype.initialize.apply(this, arguments);
            this.setPageTitle(_('Pivot').t());
            this.classicurl = classicurl;
            this.appLocal = new AppLocal();
            this.user = new User();
            this.timePresets = new Times();
            this.dataModels = new DataModels();
            this.reportDefaults = new PivotReport();
            this.reportHistoryManager = new ReportHistoryManager();
            this.enablePageView = false;
        },

        page: function(locale, app, page) {
            Base.prototype.page.apply(this, arguments);
            if (this.shouldRender) {
                this.pageView = new ResponsiveHeaderPage({
                    splunkBar: true,
                    footer: false,
                    section: 'pivot',
                    model: {
                        application: this.model.application,
                        appNav: this.model.appNavPartial
                    }
                });
                this.pageView.render();
                // TODO: this should be handled with urlRoot
                this.appLocal.id = this.appLocal.url + '/' + encodeURIComponent(this.model.application.get('app'));
                // create a "ready" deferred that will be passed down to the controller
                var appOwner = { app: this.model.application.get('app'), owner: this.model.application.get('owner') };
                this.readyDfd = $.when(
                    this.timePresets.fetch({ data: appOwner }),
                    this.dataModels.fetch({ data: $.extend({ concise: true }, appOwner) }),
                    this.appLocal.fetch({ data: appOwner }),
                    this.user.fetch({
                        url: splunkdUtils.fullpath(this.user.url + '/' + encodeURIComponent(this.model.application.get('owner'))),
                        data: appOwner
                    })
                );
            }
            if(this.flashMessages) {
                this.flashMessages.remove();
            }
            this.flashMessages = new FlashMessages({
                model: {
                    appLocal: this.appLocal,
                    user: this.user
                },
                collection: {
                    timePresets: this.timePresets
                },
                helperOptions: {
                    postProcess: function(messages) {
                        // remove the "Could not find object id" messages, they are confusing because the word object
                        // has special meaning in the data model context, the associated 404 message is more informative
                        return _(messages).filter(function(message) {
                            return !/Could not find object id=/.test(message.get('html'));
                        });
                    }
                }
            });
            var flashMessagesHelper = this.flashMessages.flashMsgHelper;
            this.classicurl.clear();
            this.dataModel = this.dataModel ? this.dataModel.clone() : new DataModel();
            flashMessagesHelper.register(this.dataModel);

            if(this.report) {
                this.reportHistoryManager.unregister(this.report);
            }
            this.report = this.report ? this.report.clone() : new PivotReport();
            flashMessagesHelper.register(this.report);
            // we leave it up to the controller to register the report with the reportHistoryManager at the correct time

            var pivotController = new PivotController({
                model: {
                    application: this.model.application,
                    classicurl: this.classicurl,
                    dataModel: this.dataModel,
                    report: this.report,
                    reportDefaults: this.reportDefaults,
                    appLocal: this.appLocal,
                    user: this.user
                },
                collection: {
                    timePresets: this.timePresets,
                    dataModels: this.dataModels
                },
                reportHistoryManager: this.reportHistoryManager,
                routerReadyDfd: this.readyDfd,
                flashMessages: this.flashMessages
            });
            // expose this here for unit testing
            this.collectJob = pivotController.model.collectJob;
            $('.main-section-body .load-failure-messages').remove();
            pivotController.on('ready', function() {
                if (this.pivotController) {
                    // TODO [sff] this is pretty hacky, but it prevents the viz config menu from losing its scroll position
                    pivotController.masterView.vizConfigMenuOffset = this.pivotController.masterView.vizConfigMenuOffset;
                    this.pivotController.off(null, null, this);
                    this.pivotController.remove();
                }
                this.pivotController = pivotController;
                var $preloadPlaceholder = $('.preload');
                if($preloadPlaceholder.length > 0) {
                    this.pageView.replaceAll($preloadPlaceholder);
                }
                this.pivotController.masterView.render().appendTo($('.main-section-body'));
            }, this);
            pivotController.on('failed', function(errorType, redirectUrl) {
                // if we have already rendered the page, update the errors in place
                if(this.pivotController) {
                    pivotController.off(null, null, this);
                    pivotController.remove();
                    this.pivotController.masterView.renderErrors(this.flashMessages);
                }
                // otherwise handle the page load failure
                else {
                    this.handleLoadFailure(errorType, redirectUrl);
                }
            }, this);
        },

        handleLoadFailure: function(errorType, redirectUrl) {
            if(errorType === PivotController.URL_ERROR) {
                window.location.href = redirectUrl;
                return;
            }
            var $preloadPlaceholder = $('.preload');
            if($preloadPlaceholder.length > 0) {
                this.pageView.replaceAll($preloadPlaceholder);
            }

            var $failMessagesHolder = $('<div class="load-failure-messages"></div>').appendTo($('.main-section-body')),
                $linksHolder = $('<div class="links-holder"></div>').appendTo($failMessagesHolder),
                pageRouter = route.getContextualPageRouter(this.model.application);

            var addLink = function(text, href) {
                var $anchor = $('<a></a>').text(text).attr('href', href);
                $linksHolder.append($anchor);
            };

            this.flashMessages.prependTo($failMessagesHolder);
            if(errorType === PivotController.DATA_MODEL_ERROR) {
                addLink(_('Select New Data Model').t(), pageRouter.data_models());
                addLink(_('Manage Data Models').t(), pageRouter.data_model_manager());
                // SPL-71881, special handling for the case where a saved report is loading but can't load its data model
                if(!this.report.isNew()) {
                    this.flashMessages.flashMsgHelper.addGeneralMessage(_.uniqueId('pivot-search-error'), {
                    type: splunkdUtils.ERROR,
                    html: _('The data model used by this report either does not exist or does not have the correct permissions.').t()
                });
                }
            }
            else if(errorType === PivotController.OBJECT_ERROR) {
                var dataModelId = this.dataModel.id;
                addLink(_('Select New Object').t(), pageRouter.data_model_explorer({ data: { model: dataModelId } }));
                addLink(_('Edit Objects').t(), pageRouter.data_model_editor({ data: { model: dataModelId } }));
            }
            else if(errorType === PivotController.REPORT_ERROR) {
                addLink(_('Manage Reports').t(), pageRouter.reports());
            }
            else if(errorType === PivotController.PIVOT_SEARCH_ERROR) {
                this.flashMessages.flashMsgHelper.addGeneralMessage(_.uniqueId('pivot-search-error'), {
                    type: splunkdUtils.ERROR,
                    html: _('The search string could not be parsed into a valid pivot configuration.').t()
                });
                var reportContent = this.report.entry.content,
                    latestTime = this.classicurl.get('latest') || reportContent.get('dispatch.latest_time'),
                    searchParams = {
                        q: this.classicurl.get('q') || reportContent.get('s'),
                        earliest: this.classicurl.get('earliest') || reportContent.get('dispatch.earliest_time') || 0
                    };

                if(latestTime) {
                    searchParams.latest = latestTime;
                }
                addLink(_('Open In Search').t(), pageRouter.search({ data: searchParams }));
                if(this.dataModel.id) {
                    addLink(_('Select New Object').t(), pageRouter.data_model_explorer({ data: { model: this.dataModel.id } }));
                }
                else {
                     addLink(_('Select New Data Model').t(), pageRouter.data_models());
                }
            }
        }

    });

});