define(
    [
        'underscore',
        'jquery',
        'backbone',
        'routers/Base',
        'controllers/Search',
        'models/classicurl',
        'models/services/AppLocal',
        'models/services/authentication/User',
        'models/services/data/ui/Pref',
        'collections/services/data/ui/Times',
        'collections/services/authorization/Roles',
        'collections/services/configs/EventRenderers',
        'collections/services/data/ui/WorkflowActions',
        'util/splunkd_utils'
    ],
    function(
        _,
        $,
        Backbone,
        BaseRouter,
        SearchController,
        classicurl,
        AppLocalModel,
        UserModel,
        UIPrefsModel,
        TimesCollection,
        RolesCollection,
        EventRenderers,
        WorkflowActionsCollection,
        splunkd_utils
    ) {
        return BaseRouter.extend({
            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.setPageTitle(_('Search').t());
                this.loadingMessage = _('Loading...').t();
                
                //controllers
                this.timelineController;
                
                //models
                this.model.appLocal = new AppLocalModel();
                this.model.user = new UserModel();
                this.model.uiPrefs = new UIPrefsModel();
                
                //collections
                this.collection.times = new TimesCollection();
                this.collection.roles = new RolesCollection();
                this.collection.eventRenderers = new EventRenderers();
                this.collection.workflowActions = new WorkflowActionsCollection();
                
                //deferreds
                this.deferreds.roles = this.collection.roles.fetch();
                
                $(window).resize(_.debounce(function(){
                    this.setBodyMinHeight();
                }.bind(this), 0));
            },
            //Our only Action method
            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                
                //models
                if (!this.deferreds.uiPrefs) {
                    this.deferreds.uiPrefs = $.Deferred();
                    this.model.uiPrefs.bootstrap(this.deferreds.uiPrefs, this.model.application.get("page"), this.model.application.get("app"), this.model.application.get("owner"));
                }
                
                if (!this.deferreds.appLocal) {
                    if (this.model.appsLocalPartial) {
                        this.model.appLocal.setFromSplunkD(this.model.appsLocalPartial.toSplunkD());
                        this.deferreds.appLocal = $.Deferred();
                        this.deferreds.appLocal.resolve();
                    } else {
                        this.deferreds.appLocal = this.model.appLocal.fetch({
                            url: splunkd_utils.fullpath(this.model.appLocal.url + "/" + encodeURIComponent(this.model.application.get("app"))),
                            data: {
                                app: this.model.application.get("app"),
                                owner: this.model.application.get("owner")
                            }
                        });
                    }
                }
                
                if (!this.deferreds.user) {
                    this.deferreds.user = this.model.user.fetch({
                        url: splunkd_utils.fullpath(this.model.user.url + "/" + encodeURIComponent(this.model.application.get("owner"))),
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner")
                        }
                    });
                }

                //collections
                if (!this.deferreds.times) {
                    this.deferreds.times = this.collection.times.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner")
                        }
                    });
                }
                
                if (!this.deferreds.eventRenderers) {
                    this.deferreds.eventRenderers = this.collection.eventRenderers.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner"),
                            sort_key: 'priority',
                            sort_dir: 'desc',
                            count: -1
                        }
                    });
                }

                if (!this.deferreds.workflowActions) {
                    this.deferreds.workflowActions = this.collection.workflowActions.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner"),
                            count: -1,
                            sort_key: "name",
                            search: 'disabled=false'
                        }
                    });
                }
                
                if (this.shouldRender) {
                    this.pageView.$('.section-padded').remove();//remove once all pages migrated to Page view class correctly
                    $('.preload').replaceWith(this.pageView.el);
                }
                
                //this is a longlived singleton object and we have to clear it each time the route is hit
                classicurl.clear();
                
                this.setBodyMinHeight();
                
                var searchController = new SearchController({
                    model: {
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        uiPrefs: this.model.uiPrefs
                    },
                    collection: {
                        times: this.collection.times,
                        roles: this.collection.roles,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    deferreds: {
                        appLocal: this.deferreds.appLocal,
                        user: this.deferreds.user,
                        times: this.deferreds.times,
                        roles: this.deferreds.roles,
                        eventRenderers: this.deferreds.eventRenderers,
                        workflowActions: this.deferreds.workflowActions,
                        uiPrefs: this.deferreds.uiPrefs
                    }
                });

                searchController.on('ready', function() {
                    //perform your replace
                    if (this.searchController) {
                        var resultsContainer = this.searchController.children.timelineMasterView.children.resultsContainerView;
                        if (resultsContainer) {
                            resultsContainer.children.eventsPane.children.timeline.remove();
                        }
                        this.searchController.debouncedRemove({detach: true});
                    }
                    this.searchController = searchController;
                    $('.main-section-body').html(this.searchController.replacementDOM);
                    this.searchController.reflow();
                    
                    //replace with application level event view:append
                    $(document).trigger("rendered");
                }, this);
            },
            setBodyMinHeight: function() {
                this.minHeight = this.pageView.$('header').height() + $(window).height();
                $('body').css('min-height', this.minHeight);
            }
        });
    }
);
