define(
    [
        'jquery',
        'underscore',
        'models/classicurl',
        'models/services/data/vix/IndexActions',
        'collections/services/data/vix/Providers',
        'collections/services/data/vix/Indexes',
        'collections/services/configs/Indexes',
        'views/virtual_indexes/Master',
        'views/virtual_indexes/NoLicense',
        'routers/Base'
    ],
    function(
        $,
        _,
        classicurlModel,
        IndexActions,
        ProviderCollection,
        IndexCollection,
        IndexesConfCollection,
        VixMasterView,
        NoLicenseView,
        BaseRouter
        ) {
        return BaseRouter.extend({

            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.setPageTitle(_('Virtual indexes').t());
                this.enableAppBar = false;
                var that = this;
                this.pageReadydfd = $.Deferred();
                this.indexesConfCollection = new IndexesConfCollection();
                this.indexesConfCollection.fetch({
                    data: {
                        search: 'name=provider-family*',
                        f: 'name'
                    }
                }).done(function() {
                    if (that.indexesConfCollection.length > 0) {
                        that.loadMainPage();
                    } else {
                        that.loadNoLicense();
                    }
                    that.pageReadydfd.resolve();
                });
            },

            loadMainPage: function() {
                this.providers = new ProviderCollection();
                this.indexes = new IndexCollection();
                this.providersAll = new ProviderCollection();
                this.indexesAll = new IndexCollection();
                this.indexActions = new IndexActions();

                this.providers.fetchData.set('count', 10);
                this.indexes.fetchData.set('count', 10);
                this.indexes.fetch();
                this.providers.fetch();

                this.indexesAll.fetchData.set('count', -1);
                this.indexesAll.fetchData.set('f', 'name');
                this.providersAll.fetchData.set('count', -1);
                this.providersAll.fetchData.set('f', 'name');
                this.indexesAll.fetch();
                this.providersAll.fetch();

                this.providers.on('deleteApproved', function(victimModel) {
                    var that = this;
                    victimModel.destroy().done(function() {
                        that.providers.fetch();
                        that.providersAll.fetch();
                    });
                }, this);
                this.indexes.on('deleteApproved', function(victimModel) {
                    var that = this;
                    victimModel.destroy().done(function() {
                        that.providers.fetch();
                        that.indexes.fetch();
                        that.indexesAll.fetch();
                    });
                }, this);
                this.indexes.on('disableRequest', function(victimModel) {
                    var that = this,
                        links = victimModel.entry.links;
                    this.indexActions.set('id', links.get('disable'));
                    this.indexActions.save().done(function() {
                        that.providers.fetch();
                        that.indexes.fetch();
                        that.indexesAll.fetch();
                    });
                }, this);
                this.indexes.on('enableRequest', function(victimModel) {
                    var that = this,
                        links = victimModel.entry.links;
                    this.indexActions.set('id', links.get('enable'));
                    this.indexActions.save().done(function() {
                        that.providers.fetch();
                        that.indexes.fetch();
                        that.indexesAll.fetch();
                    });
                }, this);

                this.masterView = new VixMasterView({
                    model: {
                        application: this.model.application
                    },
                    collection: {
                        providers: this.providers,
                        indexes: this.indexes,
                        providersAll: this.providersAll,
                        indexesAll: this.indexesAll
                    }
                });
            },

            loadNoLicense: function() {
                this.masterView = new NoLicenseView({
                    model: {
                        application: this.model.application
                    }
                });
            },


            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                $('.preload').replaceWith(this.pageView.el);
                var that = this;
                $.when(that.pageReadydfd).done(function() {
                    that.pageView.$('.main-section-body').append(that.masterView.render().el);
                });


            }
        });
    }
);
