define(function(require, exports, module) {
    var _ = require('underscore');
    var $ = require('jquery');
    var SimpleSplunkView = require("./simplesplunkview");

    /**
     * Private abstract base class for form input views.  Provides
     * a render() option that draws if static content is available
     * and there is no dynamic manager provided.
     */
    var BaseInputView = SimpleSplunkView.extend({
        output_mode: 'json',
        
        options: {
            disabled: false,
            managerid: null,
            data: "preview"
        },

        initialize: function(options) {
            SimpleSplunkView.prototype.initialize.apply(this, arguments);
            
            // The 'value' setting is always pushed
            this.settings.enablePush("value");
            
            // Handle the default value
            this._onDefaultChange();
            
            this.settings.on("change:disabled", this._onDisable, this);
            this.settings.on("change:default", this._onDefaultChange, this);
        },
        
        _onDefaultChange: function(model, value, options) {   
            var oldDefaultValue = this.settings.previous("default");
                 
            // Initialize value with default, if provided.
            var defaultValue = this.settings.get("default");
            var calledFromConstructor = !model;
            if (defaultValue === undefined && calledFromConstructor) {
                var supportsSeedSetting = _.contains(
                    _.keys(this.constructor.prototype.options), "seed");
                
                if (supportsSeedSetting) {
                    // The `seed` option has been deprecated and replaced by the
                    // `default` option, which is supported on all inputs. If a text
                    // input has both, the `default` wins.
                    defaultValue = this.settings.get("seed");
                }
            }
            
            var currentValue = this.settings.get('value');
            if (defaultValue !== undefined &&
                (currentValue === oldDefaultValue || currentValue === undefined))
            {
                this.val(defaultValue);
            }
        },

        _onChange: function(e) {
            this.trigger("change", this.val(), this);
        },

        _onDisable: function(e) {
            var state = this.settings.get("disabled");
            this._disable(state);
        },
        
        _disable: function(state) {
            this.$('input').prop("disabled", state);
        },

        render: function() {
            this._updateView(this._viz, this._data || []);
            return this;
        },

        // Skip the empty data check.  Empty data is acceptable for
        // form objects.

        _updateView: function() {
            var data = this._data || [];
 
            if (!this._viz) {
                this._createView(data); 
            }

            if (!this._viz) {
                return; // Couldn't create the visualization
            }

            this.updateView(this._viz, data);
            this._onDisable();
        },

        createView: function() {
            // Must return true to tell view that a valid
            // visualization exists.
            return true;  
        },
        
        // Get or set the current inputs value.
        val: function(value, selector) {
            selector = selector || "input";
            var input = $(selector, this.$el);
            if (value) {
                return input.val(value);
            }
            return input.val();
        }
    });
    
    return BaseInputView;
});
