define(function (require) {
    var _ = require('underscore'),
        $ = require('jquery'),
        utils = require('./utils'),
        splunkUtils = require('splunk.util'),
        route = require('uri/route'),
        console = require('util/console'),
        util = require('splunk.util'),
        shared_models = require('./sharedmodels'),
        IntentionsParser = require('models/services/search/IntentionsParser'),
        general_utils = require('util/general_utils');

    var Drilldown = {

        intentionsParser: new IntentionsParser(),
        applyPivotDrilldownIntention: function(clickInfo, drilldownType, manager) {
            var intentionDFD, field, value;
            var applicationModel = shared_models.get('app');
            var query = manager.query.resolve({ qualified: true });
            var dfd = $.Deferred();
            var earliest = manager.job.properties().searchEarliestTime,
                latest = manager.job.properties().searchLatestTime,
                params = { q: query, namespace: utils.getCurrentApp() },
                values = [];

            if (!earliest && manager.job.properties().earliestTime){
                earliest = util.getEpochTimeFromISO(manager.job.properties().earliestTime);
            }
            if (!latest && manager.job.properties().latestTime){
                latest = util.getEpochTimeFromISO(manager.job.properties().latestTime);
            }
            Drilldown.intentionsParser.clear({silent: true});
            if(clickInfo.type === 'legendClick') {
                // if the legend click has a name and value (this happens for scatter charts), use them
                if(clickInfo.hasOwnProperty('name') && clickInfo.hasOwnProperty('value')) {
                    intentionDFD = Drilldown.intentionsParser.fetch({
                        data: {
                            q: query,
                            action: 'fieldvalue',
                            field: clickInfo.name,
                            value: clickInfo.value,
                            fieldMetaData: JSON.stringify(manager.job.properties().fieldMetadataResults),
                            app: applicationModel.get('app'),
                            owner: applicationModel.get('owner')
                        }
                    });
                }
                // otherwise do a keyword search based on name2
                else {
                    intentionDFD = Drilldown.intentionsParser.fetch({
                        data: {
                            q: query,
                            action: 'keyword',
                            field: clickInfo.name2,
                            fieldMetaData: JSON.stringify(manager.job.properties().fieldMetadataResults),
                            app: applicationModel.get('app'),
                            owner: applicationModel.get('owner')
                        }
                    });
                }
            }
            else {
                if(clickInfo._span) {
                    Drilldown.intentionsParser.set({
                        'dispatch.earliest_time': clickInfo.value,
                        'dispatch.latest_time': JSON.stringify(parseFloat(clickInfo.value) + clickInfo._span)
                    }, {silent: true});
                }
                if(drilldownType === 'row') {
                    field = _(clickInfo.rowContext).keys().map(function(f) { return f.replace('row.', ''); });
                    value = _(clickInfo.rowContext).values();
                } else if(clickInfo.name === '_time') {
                    field = clickInfo.name2;
                    value = clickInfo.value2;
                } else {
                    if(clickInfo.name && clickInfo.name2){
                        field = [clickInfo.name, clickInfo.name2];
                        value = [clickInfo.value, clickInfo.value2];
                    } else if(clickInfo.name) {
                        field = clickInfo.name;
                        value = clickInfo.value;
                    } else {
                        field = clickInfo.name2;
                        value = clickInfo.value2;
                    }
                }

                intentionDFD = Drilldown.intentionsParser.fetch({
                    data: {
                        q: query,
                        action: 'fieldvalue',
                        field: field || '',
                        value: value || '',
                        fieldMetaData: JSON.stringify(manager.job.properties().fieldMetadataResults),
                        app: applicationModel.get('app'),
                        owner: applicationModel.get('owner')
                    }
                });

            }
            intentionDFD.done(function(){
                dfd.resolve({
                    'q': Drilldown.intentionsParser.get('fullSearch'),
                    'earliest': Drilldown.intentionsParser.get('dispatch.earliest_time') || earliest || 0,
                    'latest': Drilldown.intentionsParser.get('dispatch.latest_time') || latest || 'now'
                });
            });
            return dfd;

        },

        applyDrilldownIntention: function (clickInfo, drilldownType, manager) {
            var query = manager.query.resolve({ qualified: true });
            if (general_utils.isValidPivotSearch(query)){
                return Drilldown.applyPivotDrilldownIntention(clickInfo, drilldownType, manager);
            }
            var dfd = $.Deferred();
            var earliest = manager.job.properties().searchEarliestTime,
                latest = manager.job.properties().searchLatestTime,
                params = { q: query, namespace: utils.getCurrentApp() },
                values = [];

            if (!earliest && manager.job.properties().earliestTime){
                earliest = util.getEpochTimeFromISO(manager.job.properties().earliestTime);
            }
            if (!latest && manager.job.properties().latestTime){
                latest = util.getEpochTimeFromISO(manager.job.properties().latestTime);
            }

            if (clickInfo.name === '_time') {
                if (clickInfo.hasOwnProperty('_span')) {
                    earliest = parseInt(clickInfo.value, 10);
                    latest = earliest + parseInt(clickInfo._span, 10);
                } else {
                    earliest = parseInt(clickInfo.value, 10);
                    latest = earliest + 1;
                }
                values.push(['_time', [earliest, latest].join('-')]);
            } else {
                values.push([clickInfo.name, clickInfo.value]);
            }

            if ((drilldownType === 'all' || drilldownType === 'cell') && clickInfo.hasOwnProperty('name2')) {
                values.push([clickInfo.name2, clickInfo.value2]);
            } else {
                values.push(values[0]);
            }

            params.intentions = JSON.stringify([
                {
                    arg: { vals: values},
                    flags: ['keepevents'],
                    name: 'drilldown'
                }
            ]);
            // Apply the intention using the legacy search parser
            // TODO-Sigi: Replace this with the new intentions parser
            $.ajax({
                url: splunkUtils.make_full_url('parser/parse'),
                data: params,
                type: 'post',
                dataType: 'json',
                async: false,
                success: function (data) {
                    var result = { q: splunkUtils.stripLeadingSearchCommand(data.search) || params.q };
                    if(earliest !== undefined) {
                        result.earliest = earliest;
                    }
                    if(latest !== undefined) {
                        result.latest = latest;
                    }
                    dfd.resolve(result);
                },
                error: function (xhr, status, error) {
                    dfd.reject(error);
                }
            });
            return dfd.promise();
        },

        handleDrilldown: function(click, drilldownType, manager, target) {
            Drilldown.applyDrilldownIntention(click, drilldownType, manager).done(function(params){
                var pageInfo = utils.getPageInfo(), url = route.search(pageInfo.root, pageInfo.locale, pageInfo.app, {
                    data: params
                });
                utils.redirect(url, click.modifierKey, target);
            });
        }

    };

    return Drilldown;

});
