define(function (require, exports, module) {
    var _ = require("underscore"),
        $ = require('jquery'),
        mvc = require('./mvc'),
        BaseSplunkView = require("./basesplunkview"),
        GlobalNav = require('views/shared/splunkbar/Master'),
        AppNav = require('views/shared/appbar/Master'),
        sharedModels = require('./sharedmodels'),
        userAgent = require('helpers/user_agent');


    var HeaderView = BaseSplunkView.extend({
        moduleId: module.id,
        
        className: 'splunk-header',

        options: {
            appbar: true,
            acceleratedAppNav: false
        },

        initialize: function() {
            this.configure();
            this.model = this.model || {};
            this.model.application= sharedModels.get("app");
            this.model.user= sharedModels.get("user");
            this.model.appLocal= sharedModels.get("appLocal");
            this.model.serverInfo= sharedModels.get("serverInfo");

            var acceleratedAppNav = this.settings.get('acceleratedAppNav') && userAgent.isIE7();
            this.collection = this.collection || {};
            this.collection.appLocals = sharedModels.get("appLocals");

            this.dfd = $.when.apply($, [
                this.model.application.dfd,
                this.model.user.dfd,
                this.model.appLocal.dfd,
                this.model.serverInfo.dfd,
                this.collection.appLocals.dfd
            ]);

            this.dfd.done(_.bind(function(){

                this.globalNav = GlobalNav.create({
                    model: {
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user,
                        serverInfo: this.model.serverInfo
                    },
                    collection: {
                        apps: this.collection.appLocals
                    }
                });

                if(this.settings.get('appbar')){
                    this.appNav = AppNav.create({
                        model: {
                            application: this.model.application,
                            app: this.model.appLocal
                        },
                        appServerUrl: acceleratedAppNav
                    });
                }
            }, this));
        },
        render: function() {
            this.dfd.done(_.bind(function(){
                this.$el.empty()
                    .append(this.globalNav.render().el);
                if(this.settings.get('appbar')){
                    this.$el.append(this.appNav.render().el);
                }
            }, this));
            return this;
        }
    });

    return HeaderView;
});
