define(function(require) {
    var _ = require('underscore');
    var $ = require('jquery');
    var BaseSplunkView = require('../../basesplunkview');
    var TokenUtils = require('../../tokenutils');

    var BaseInput = BaseSplunkView.extend({
        options: {
            submitOnChange: false
        },
        initialize: function(options) {
            this.configure();
            this.settings.enablePush('value');
            
            // Update self when settings change
            this.settings.on('change:label', this.renderLabel, this);
            
            options = options || {};
            this.inputId = options.inputId || _.uniqueId((this.id || 'input') + '_');
        },
        handleChange: function() {
            // Update MVC token
            this.settings.set('value', this.getValue());
        },
        submit: function() {
            // Notify listeners
            this.trigger('change', this);
        },
        hasValue: function() {
            var value = this.settings.get("value");
            var defaultValue = this.settings.get("default");
            // We have a value if one of these conditions is true:
            // 1. Value is not null nor undefined
            // 2. We have no default value
            // 3. Our value and default value are the same
            return (value !== undefined && value !== null) || defaultValue === undefined || value === defaultValue;
        },
        setValue: function(v) {
            throw 'setValue not implemented';
        },
        getValue: function() {
            throw 'getValue not implemented';
        },
        renderLabel: function() {
            var label = this.$('label');
            if(!label.length) {
                label = $('<label></label>').appendTo(this.el);
            }
            label.attr('for', this.inputId);
            if(this.settings.has('label')) {
                label.text(this.settings.get('label'));
            } else {
                var v = label.text();
                if(v) {
                    this.settings.set('label', v);
                }
            }
        },
        // For API compatibility with MVC controls.
        val: function(newValue) {
            // NOTE: Ignore parameters beyond the first one.
            if (arguments.length >= 1) {
                this.setValue(newValue);
            }
            return this.getValue();
        },
        on: function() {
            return this._applyToWrappedView('on', arguments);
        },
        off: function() {
            return this._applyToWrappedView('off', arguments);
        },
        trigger: function() {
            return this._applyToWrappedView('trigger', arguments);
        },
        _applyToWrappedView: function(funcName, args) {
            var wrappedView = this._getWrappedView();
            if (wrappedView === this) {
                return BaseSplunkView.prototype[funcName].apply(this, args);
            } else {
                return wrappedView[funcName].apply(wrappedView, args);
            }
        },
        /*
         * Subclasses should override if they delegate functionality
         * to an underlying view.
         */
        _getWrappedView: function() {
            return this;
        }
    });
    
    return BaseInput;
});
