define(function(require, exports, module) {
    var _ = require('underscore');
    var $ = require('jquery');
    var BaseView = require('views/Base');
    var AddPanelDialog = require('../dialog/addpanel');
    var pdfUtils = require('util/pdf_utils');
    var DashboardModel = require('models/Dashboard');
    var Dashboard = require('../controller');
    var console = require('util/console');
    var mvc = require('../../../mvc');
    var utils = require('../../utils');
    var route = require('uri/route');
    var TimeRangePickerInput = require('../../simpleform/input/timerange');
    var config = require('splunk.config');
    var splunkDUtils = require('util/splunkd_utils');
    var EditMenu = require('./editmenu');
    var MoreInfoMenu = require('./moreinfomenu');

    var MenuView = BaseView.extend({
        moduleId: module.id,
        className: 'edit-menu',
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);
            this.model.state.on('change:edit', this.onEditModeChange, this);
            this.model.state.on('change:editable change:pdf_available', this.render, this);
            this.model.state.on('change:default_timerange', this.handleDefaultTimeRangeChange, this);
            this.model.state.user.on("change", this.render, this);
            this._ready = false;
            var that = this;
            _.defer(function(){
                that._ready = true;
            });
        },
        events: {
            'click a.edit-btn': function(e) {
                e.preventDefault();
                var $target = $(e.currentTarget);
                if (this.children.editMenu && this.children.editMenu.shown) {
                    this.children.editMenu.hide();
                    return;
                }
                if (this.children.moreInfoMenu && this.children.moreInfoMenu.shown) {
                    this.children.moreInfoMenu.hide();
                }
                $target.addClass('active');

                this.children.editMenu = new EditMenu({
                    model: {
                        application: this.model.application,
                        dashboard: this.model.dashboard,
                        state: this.model.state, 
                        scheduledView: this.model.scheduledView
                    },
                    collection: this.collection,
                    showOpenActions: this.options.showOpenActions,
                    deleteRedirect: this.options.deleteRedirect,
                    onHiddenRemove: true
                });
                $('body').append(this.children.editMenu.render().el);
                this.children.editMenu.show($target);
                this.children.editMenu.on('hide', function() {
                    $target.removeClass('active');
                }, this);
            },
            'click a.more-info-btn': function(e) {
                e.preventDefault();
                var $target = $(e.currentTarget);
                if (this.children.moreInfoMenu && this.children.moreInfoMenu.shown) {
                    this.children.moreInfoMenu.hide();
                    return;
                }
                if (this.children.editMenu && this.children.editMenu.shown) {
                    this.children.editMenu.hide();
                }
                $target.addClass('active');
                this.children.moreInfoMenu= new MoreInfoMenu({
                    model: {
                        application: this.model.application,
                        dashboard: this.model.dashboard,
                        state: this.model.state,
                        scheduledView: this.model.scheduledView
                    },
                    collection: this.collection,
                    onHiddenRemove: true
                });
                
                $('body').append(this.children.moreInfoMenu.render().el);
                this.children.moreInfoMenu.show($target);
                this.children.moreInfoMenu.on('hide', function() {
                    $target.removeClass('active');
                }, this);
            },
            'click a.edit-done': function(e){
                e.preventDefault();
                this.model.state.set('edit', false);
            },
            'click a.add-panel': function(e) {
                e.preventDefault();
                this.children.addPanelDialog = new AddPanelDialog({
                    controller: this.options.controller
                });
                this.children.addPanelDialog.render().appendTo($("body")).show();
            },
            'click a.add-trp': function(e) {
                e.preventDefault();

                var $trpEl = $('<div class="input input-timerangepicker" id="field3"><label>&nbsp;</label></div>');
                var fieldset = $('body>.dashboard-body>.fieldset');

                var submitButton = fieldset.find('.form-submit');
                if(submitButton.length) {
                    $trpEl.insertBefore(submitButton);
                } else {
                    $trpEl.appendTo(fieldset);
                }

                var id = "timerange", seq = 1;
                while(mvc.Components.has(id)) {
                    id = "timerange" + (seq++);
                }

                var trp = new TimeRangePickerInput({
                    id: "timerange",
                    isGlobal: true,
                    earliest_time: "$earliest$",
                    latest_time: "$latest$",
                    el: $trpEl
                }, {tokens: true}).render();
                
                // Simulate submitOnChange=true
                trp.on('change', function() {
                    var defaultTokenModel = mvc.Components.getInstance('default');
                    var submittedTokenModel = mvc.Components.getInstance('submitted');
                    
                    // Simulate submitTokens()
                    submittedTokenModel.set(defaultTokenModel.toJSON());
                });

                Dashboard.trigger('formupdate');
            },
            'click a.print-dashboard': function(e){
                e.preventDefault();
                window.print();
            },
            'click a.generate-pdf': function(e){
                e.preventDefault();
                var view = this.model.dashboard.entry.get('name'),
                    app = this.model.dashboard.entry.acl.get('app'),
                    params = {}, idx = 0;

                // Collect SIDs for search jobs on the dashboard
                _.map(mvc.Components.get('dashboard').getElementIds(), function(id){
                    var element = mvc.Components.get(id);
                    if(element && element.getExportParams) {
                        _.extend(params, element.getExportParams('sid_'+idx));
                    }
                    idx++;
                });

                pdfUtils.isPdfServiceAvailable().done(function(available, type){
                    if(type === 'pdfgen') {
                        var xml = Dashboard.model.view.getFlattenedXML({ useLoadjob: false, indent: false });
                        if(console.DEBUG_ENABLED) {
                            console.log(xml);
                        }
                        pdfUtils.downloadReportFromXML(xml, app, params);
                    } else if(type === 'deprecated') {
                        pdfUtils.getRenderURL(view, app, params).done(function(url){
                            window.open(url);
                        });
                    }
                });
            }
        },
        onEditModeChange: function() {
            var edit = this.model.state.get('edit');
            this.$('.dashboard-view-controls')[edit ? 'hide' : 'show']();
            this.$('.dashboard-edit-controls')[edit ? 'show' : 'hide']();
        },
        handleDefaultTimeRangeChange: function(m) {
            if(this._ready && this.model.state.get('edit')) {
                var isForm = $('body>.dashboard-body>.fieldset').children().length > 0,
                    trp = m.get('default_timerange'), defaultValue;
                if(trp) {
                    defaultValue = trp;
                }
                this.model.dashboard.saveFormSettings(isForm, trp, defaultValue);
            }
            this.render();
        },
        render: function() {
            var app = this.model.application.toJSON();
            var renderModel = {
                dashboard: this.model.dashboard.isDashboard(),
                editLinkViewMode: route.manager(app.root, app.locale, app.app, ['data','ui','views', app.page], {
                            data: {
                                action: 'edit',
                                ns: app.app,
                                redirect_override: route.page(app.root, app.locale, app.app, app.page)
                            }
                        }),
                editLinkEditMode: route.manager(app.root, app.locale, app.app, ['data','ui','views', app.page], {
                            data: {
                                action: 'edit',
                                ns: app.app,
                                redirect_override: route.page(app.root, app.locale, app.app, app.page) + '/edit'
                            }
                        }),
                dashboardType: this.model.dashboard.getViewType(),
                editable: this.model.state.get('editable'),
                canWrite: this.model.dashboard.entry.acl.canWrite(),
                removable: this.model.dashboard.entry.links.get('remove') ? true : false,
                isSimpleXML: this.model.dashboard.isSimpleXML(),
                isHTML: this.model.dashboard.isHTML(),
                isPdfServiceAvailable: this.model.state.get('pdf_available'),
                showAddTRP: !this.model.state.get('default_timerange'),
                _: _
            };

            this.$el.html(this.compiledTemplate(renderModel));

            this.$('.generate-pdf').tooltip({ animation:false, title: _("Export PDF").t() });
            this.$('.print-dashboard').tooltip({ animation:false, title: _("Print").t() });
            this.onEditModeChange();
            return this;
        },
        template: '\
            <span class="dashboard-view-controls">\
                <% if(canWrite) { %>\
                    <div class="btn-group">\
                        <a class="btn edit-btn" href="#"><%- _("Edit").t() %> <span class="caret"></span></a>\
                        <a class="btn more-info-btn" href="#"><%- _("More Info").t() %> <span class="caret"></span></a>\
                    </div>\
                <% }else{ %>\
                    <div class="btn-group">\
                        <a class="btn edit-btn" href="#"><%- _("Edit").t() %> <span class="caret"></span></a>\
                    </div>\
                <% } %>\
                <div class="btn-group">\
                    <% if(isSimpleXML && isPdfServiceAvailable) { %>\
                        <a class="btn generate-pdf" href="#"><i class="icon-export icon-large"></i></a>\
                    <% } %>\
                        <a class="btn print-dashboard" href="#"><i class="icon-print icon-large"></i></a>\
                </div>\
            </span>\
            <span class="dashboard-edit-controls" style="display:none;">\
                <div class="btn-group">\
                    <a class="btn add-panel" href="#"><i class="icon-plus"></i> <%- _("Add Panel").t() %></a> \
                    <% if(showAddTRP) { %>\
                        <a class="btn add-trp" href="#"><i class="icon-clock"></i> <%- _("Add Time Range Picker").t() %></a> \
                    <% } %>\
                    <a class="btn edit-source" href="<%- editLinkEditMode %>"><i class="icon-code"></i> <%- _("Edit Source").t() %></a> \
                </div>\
                <a class="btn btn-primary edit-done" href="#"><%- _("Done").t() %></a>\
            </span>\
        '
    });
    
    return MenuView;
}); 
