define(function(require, exports, module) {
    var BaseTokenModel = require('../basetokenmodel');
    var classicurl = require('models/classicurl');
    var DashboardController = require("./controller");
    var _ = require('underscore');
    var general_utils = require('util/general_utils');

    /**
     * Automatically mirrors the current URL query parameters.
     */
    var UrlTokenModel = BaseTokenModel.extend({
        moduleId: module.id,
        
        initialize: function() {
            classicurl.on('change', function(model, options) {
                this.set(model.toJSON());
            }, this);

            this.set(classicurl.toJSON());

            DashboardController.router.on('route', function() {
                this.trigger("url:navigate");
            }, this);
        },
        /** Saves this model's current attributes to the URL. */
        save: function(attributes, options) {
            var simpleAttributes = {};
            this.set(attributes);
            _.each(this.toJSON(), function(value, key) {
                // Don't try to persist complex values to the URL
                if (!_.isObject(value)) {
                    simpleAttributes[key] = value;
                }
            });

            classicurl.save(simpleAttributes, options);
        },
        saveOnlyWithPrefix: function(prefix, attributes, options){
            var filter =["^"+prefix+".*", "^earliest$", "^latest$"];
            this.save(general_utils.filterObjectByRegexes(attributes, filter,  { allowEmpty: true } ), options);
        }
    });
    
    return UrlTokenModel;
});