/**
 * Util package for working with views.
 */

define(['jquery', 'backbone'], function($, Backbone) {

    /**
     * @author sfishel
     *
     * DEPRECATED: use delegate view instead
     *
     * When called on a DOM element, mouse scroll events inside the element will
     * be prevented from bubbling up to the browser window.
     *
     * @param element {DOM Element} - the container element in which to restrict scrolling
     *
     * FIXME: this currently only works in Chrome, should expand to all browsers
     * TODO: should this be a jquery plugin?
     */

    var addModalScrolling = function(element) {
        $(element).on('mousewheel DOMMouseScroll', function(jqEvent) {
            // this seems to only work for Chrome, Firefox reverses the sign of the delta
            var e = jqEvent.originalEvent;
            var delta = e.wheelDelta || e.detail;
            this.scrollTop -= delta;
            e.preventDefault();
        });
    };

    var parseViews =  function(viewsCollection) {
        var views = new Backbone.Collection();
        viewsCollection.each(function (view, index) {
            var xml = $($.parseXML(view.get('eai:data'))),
                viewLabel = xml.find('label'),
                name = view.get('name'),
                app = view.get('app'),
                owner = view.get('author');
            views.add({
                id: name,
                label: viewLabel ? viewLabel.text() : name,
                app: app,
                owner: owner
            });
        });
        return views;
    };

    return ({
        addModalScrolling: addModalScrolling,
        parseViews: parseViews
    });

});