define(['jquery', 'underscore'], function($, _) {

    function $node(str) {
        var document = $.parseXML(str);
        return $(document.childNodes[0]);
    }

    function cdata(str) {
        var document = $.parseXML('<x></x>');
        return document.createCDATASection(str);
    }

    function stripEmptyTextNodes(n) {
        //IE <9 does not have TEXT_NODE
        var TEXT_NODE = n.TEXT_NODE || 3,
            i, child, childNodes = n.childNodes;
        for(i = childNodes.length - 1; i >= 0 ; i--) {
            child = childNodes[i];
            if(child !== undefined) {
                if(child.nodeType === TEXT_NODE) {
                    if(/^\s*$/.test(child.nodeValue)) {
                        n.removeChild(child);
                    } else {
                        child.nodeValue = $.trim(child.nodeValue);
                    }
                }
            }
        }
        childNodes = n.childNodes;
        for(i = childNodes.length - 1; i >= 0 ; i--) {
            child = childNodes[i];
            if(child.nodeType === n.ELEMENT_NODE || child.nodeType === n.DOCUMENT_NODE) {
                stripEmptyTextNodes(child);
            }
        }
    }

    var INDENT = '  ';

    function indentTxt(level) {
        var txt = ['\n'], x = level;
        while(x--) {
            txt.push(INDENT);
        }
        return txt.join('');
    }

    function indent($xml, n, level) {
        if(!n || !n.childNodes) {
            return;
        }

        //IE <9 does not have TEXT_NODE
        var TEXT_NODE = n.TEXT_NODE || 3;

        for(var i = 0; i <= n.childNodes.length; i++) {
            var child1 = n.childNodes[i - 1], child2 = n.childNodes[i];
            if(child2 && child2.nodeType !== TEXT_NODE && ((!child1) || child1.nodeType !== TEXT_NODE)) {
                n.insertBefore($xml.createTextNode(indentTxt(level)), child2);
            }
            if(i === n.childNodes.length && child1 && child1.nodeType !== TEXT_NODE) {
                n.appendChild($xml.createTextNode(indentTxt(level - 1)));
            }
        }
        _.chain(n.childNodes).filter(function(c) {
            return c.nodeType === n.ELEMENT_NODE;
        }).each(function(c) {
                    indent($xml, c, level + 1);
                });
    }

    function formatXMLDocument($xml) {
        stripEmptyTextNodes($xml[0]);
        indent($xml[0], $xml[0].childNodes[0], 1);
        return $xml;
    }

    function serialize(xml) {
        if(xml instanceof $) {
            xml = xml[0];
        }
        return Object.prototype.hasOwnProperty.call(xml, 'xml') ? xml.xml : new XMLSerializer().serializeToString(xml);
    }

    return {
        $node: $node,
        cdata: cdata,
        stripEmptyTextNodes: stripEmptyTextNodes,
        formatXMLDocument: formatXMLDocument,
        serialize: serialize
    };
});