define([
    'underscore',
    'views/Base',
    'module',
    'views/shared/alertcontrols/details/Master'
    ],
    function(
        _,
        BaseView,
        module,
        DetialsView
    ) {
    return BaseView.extend({
        moduleId: module.id,
        /**
         * @param {Object} options {
         *      model: {
         *         savedAlert: <models.Report>,
         *         application: <models.Application>,
         *         appLocal: <models.services.AppLocal>,
         *         user: <models.services.admin.User>
         *     },
         *     collection: roles: <collections.services.authorization.Roles>
         * }
         */
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.children.detailsView = new DetialsView({
                model: {
                    savedAlert: this.model.savedAlert,
                    application: this.model.application,
                    appLocal: this.model.appLocal,
                    user: this.model.user
                },
                collection: {
                    roles: this.collection.roles
                },
                twoColumn: true,
                displayApp: true
            });
            this.model.savedAlert.entry.on('change:name', this.render, this);
            this.model.savedAlert.entry.content.on('change:description', this.render, this);
        },
        render: function() {
            this.el.innerHTML = this.compiledTemplate({
                name: this.model.savedAlert.entry.get('name'),
                description: this.model.savedAlert.entry.content.get('description')
            });
            this.$el.append(this.children.detailsView.render().el);
            return this;
        },
        template: '\
            <h2 class="section-title"><%- name %></h2>\
            <% if(description) { %>\
                <p class="section-description">\
                    <%- description %>\
                </p>\
            <% } %>\
        '
    });
});
