define([
    'underscore',
    'module',
    'views/Base',
    'views/clustering/master/IndexesGrid',
    'views/shared/CollectionPaginator',
    'views/shared/tablecaption/Input',
    'views/shared/controls/SyntheticSelectControl'
],
function(
    _,
    module,
    BaseView,
    IndexesGrid,
    Paginator,
    Filter,
    SyntheticSelectControl
){
    return BaseView.extend({
        moduleId: module.id,
        initialize: function(options){
            BaseView.prototype.initialize.call(this, options);

            this.children.filter = new Filter({
                model: {
                    state: this.collection.masterIndexes.fetchData
                }
            });

            // Select the number of results per page
            this.children.pageCount = new SyntheticSelectControl({
                menuWidth: "narrow",
                className: "btn-group pull-left",
                items: [
                    {value: 10, label: _('10 per page').t()},
                    {value: 20, label: _('20 per page').t()},
                    {value: 50, label: _('50 per page').t()},
                    {value: 100, label: _('100 per page').t()}
                ],
                model: this.collection.masterIndexes.fetchData,
                modelAttribute: 'count',
                toggleClassName: 'btn-pill'
            });

            this.children.paginator = new Paginator({
                collection: this.collection.masterIndexes
            });

            this.children.indexesGrid = new IndexesGrid({
                collection: this.collection,
                model: this.model
            });

            this.render();
        },
        render: function(){
            var html = this.compiledTemplate();
            this.$el.html(html);
            this.$el.find('.section-controls').append(this.children.filter.render().el);
            this.$el.find('.section-controls').append(this.children.pageCount.render().el);
            this.$el.find('.section-controls').append(this.children.paginator.render().el);
            this.$el.append(this.children.indexesGrid.el);
            return this;
        },
        template: '\
            <div class="clearfix section-controls"></div>\
        '
    });
});
