define([
    'jquery',
    'underscore',
    'module',
    'models/classicurl',
    'views/Base',
    'views/clustering/EditMenu',
    'views/clustering/master/StatusSummary',
    'views/clustering/master/PeersTab',
    'views/clustering/master/IndexesTab',
    'views/clustering/master/SearchheadsTab',
    'views/clustering/master/MasterNodeInfo',
    'uri/route',
    'contrib/text!views/clustering/master/MasterNode.html'
],
function(
    $,
    _,
    module,
    classicurlModel,
    BaseView,
    EditMenu,
    StatusSummaryView,
    PeersTabView,
    IndexesTabView,
    SearchheadsTabView,
    MasterNodeInfoView,
    route,
    MasterNodeTemplate
){
    return BaseView.extend({
        moduleId: module.id,
        template: MasterNodeTemplate,
        initialize: function(options){
            BaseView.prototype.initialize.call(this, options);

            this.lastPeerCount = -1;

            this.children.statusSummary = new StatusSummaryView({
                model: this.model,
                collection: this.collection
            });

            this.children.editMenu = new EditMenu({
                model: this.model,
                collection: this.collection
            });

            this.children.peersTab = new PeersTabView({
                collection: this.collection,
                model: this.model
            });

            this.children.indexesTabView = new IndexesTabView({
                collection: this.collection,
                model: this.model
            });

            this.children.searchheadsTabView = new SearchheadsTabView({
                collection: this.collection,
                model: this.model
            });

            this.children.masterNodeInfo = new MasterNodeInfoView({
                collection: this.collection,
                model: this.model
            });

            this.model.peersStatusSummary.on('change reset', function(){
                this.checkNoPeers();
                this.updatePeersCount(this.$el);
            }, this);

            this.model.indexesStatusSummary.on('change reset', function(){
                this.updateIndexesCount(this.$el);
            }, this);

            this.collection.masterSearchheadsTotal.on('change reset', function(){
                this.updateSearchheadsCount(this.$el);
            }, this);
        },
        checkNoPeers: function(){
            var count = this.model.peersStatusSummary.get('totalCount');
            // re-render when last peer goes down or first one appears
            if (this.lastPeerCount == -1 || (count > 0 && this.lastPeerCount == 0) || (count == 0 && this.lastPeerCount > 0)){
                this.lastPeerCount = count;
                this.render();
            }
            return count == 0;
        },
        updatePeersCount: function($html){
            var el = $html.find('.peers_count'),
                count = this.model.peersStatusSummary.get('totalCount');
            if (el) {
                el.text(count || '');
            }
        },
        updateIndexesCount: function($html){
            var el = $html.find('.indexes_count'),
                count = this.model.indexesStatusSummary.get('totalCount');
            if (el) {
                el.text(count);
            }
        },
        updateSearchheadsCount: function($html){
            var el = $html.find('.searchheads_count');
            if (el) {
                el.text(this.collection.masterSearchheadsTotal.length);
            }
        },
        events: {
            'click #tab-peers a': function() {
                classicurlModel.save({t: 'peers'});
            },
            'click #tab-indexes a': function() {
                classicurlModel.save({t: 'indexes'});
            },
            'click #tab-searchheads a': function() {
                classicurlModel.save({t: 'searchheads'});
            }
        },
        render: function(){
            var urlDfd = $.Deferred();
            classicurlModel.fetch({
                success: function(model, response) {
                    var t = classicurlModel.get('t');
                    urlDfd.resolve({t:t});
                }
            });
            var link = route.docHelp(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'manager.clustering.dashboard'
                ),
                peersCount = this.model.peersStatusSummary.get('totalCount'),
                html = this.compiledTemplate({
                    peers_count: peersCount,
                    indexes_count: this.collection.masterIndexes.length,
                    searchheads_count: this.collection.masterSearchheads.length,
                    clusterStatus: this.model.clusterStatus,
                    model: this.model,
                    docLink: link
                }),
                $html = $(html);
            var learnMoreLink = route.docHelp(
                this.model.application.get('root'),
                this.model.application.get('locale'),
                'manager.clustering.configureClustering'
            );
            this.updatePeersCount($html);
            if (peersCount){
                this.updateIndexesCount($html);
                this.updateSearchheadsCount($html);
            }

            if (peersCount){
                $html.filter('.statusSummary').html(this.children.statusSummary.render().el);
            } else {
                $html.filter('.statusSummary').html(_.template(this.noPeersTemplate)({link: learnMoreLink}));
            }

            var menuContent = this.$('.section-header .btn-group').detach();
            if (menuContent.length){
                $html.find('.btn-group').replaceWith(menuContent);
            } else {
                $html.find('.editMenu').replaceWith(this.children.editMenu.render().el);
                $html.find('.info').replaceWith(this.children.masterNodeInfo.render().el);
            }

            // open tab requested by the url
            $.when(urlDfd).done(function(args) {
                var currentTab = $html.find('#clustering_'+args.t).length ? args.t : 'peers';
                if (peersCount) {
                    $html.find('#clustering_peers').append(this.children.peersTab.el);
                    $html.find('#clustering_indexes').append(this.children.indexesTabView.el);
                }
                $html.find('#clustering_searchheads').append(this.children.searchheadsTabView.el);

                if (!peersCount){
                    $html.find('.nav-tabs a[href="#clustering_searchheads"]').tab('show');
                    $html.find('#tab-searchheads').addClass('active');
                } else {
                    $html.filter('.nav-tabs').find('a[href="#clustering_'+currentTab+'"]').tab('show');
                    $html.find('#clustering_'+currentTab).addClass('active');
                }
            }.bind(this));

            this.$el.html($html);

            return this;
        },
        noPeersTemplate: '\
            <div class="searchable-summary no-peers">\
                <div class="single-value">\
                    <h3 class="single-result"><%= _("No Peers Configured").t() %></h3>\
                    <p class=""><%= _("To learn how to configure peer nodes, refer to the documentation.").t() %> \
                    <a href="<%= link %>" class="external" target="_blank"><%= _("Learn More").t() %></a></p>\
                </div>\
            </div>'
    });
});
