define([
    'jquery',
    'underscore',
    'module',
    'views/Base',
    'views/shared/delegates/RowExpandCollapse',
    'views/shared/delegates/ColumnSort',
    'helpers/grid/RowIterator',
    'contrib/text!views/clustering/searchhead/SearchHeadGrid.html',
    'contrib/text!views/clustering/searchhead/SearchHeadGridExpandedRow.html'
],
function(
    $,
    _,
    module,
    Base,
    RowExpandCollapse,
    ColumnSort,
    RowIterator,
    SearchHeadGridTemplate,
    ExpandedRowTemplate
){
    return Base.extend({

        moduleId: module.id,
        template: SearchHeadGridTemplate,
        events: {},

        initialize: function(options) {
            Base.prototype.initialize.call(this, options);

            this.collection.searchheadConfigs.on('sync', function() {
                this.debouncedRender();
            }, this);

            this.collection.searchheadGenerations.on('sync', function() {
                this.debouncedRender();
            }, this);

            this.children.rowExpandCollapse = new RowExpandCollapse({
                el: this.el,
                autoUpdate: false
            });

            this.children.columnSort = new ColumnSort({
                el: this.el,
                model: this.collection.searchheadConfigs.fetchData,
                autoUpdate: true
            });

            this.children.rowExpandCollapse.on(RowExpandCollapse.ROW_EXPAND, this.rowExpand, this);
            this.children.rowExpandCollapse.on(RowExpandCollapse.ROW_COLLAPSE, this.rowCollapse, this);

        },
        rowExpand: function(id, expandedIds, event) {
            this.expandedIds = expandedIds;

            var target = $(event.target);
            var tr = target.closest('tr');
            var td = target.closest('td');
            tr.addClass(RowExpandCollapse.EXPANDED_ROW_CLASS);
            td.html(RowExpandCollapse.EXPANDED_CELL_MARKUP);

            var generationModel = this.collection.searchheadGenerations.get('/services/cluster/searchhead/generation/'+id);
            var numPeers = 0;
            if(generationModel.entry && generationModel.entry.content){
                var genPeers = generationModel.entry.content.get('generation_peers');
                for(var i in genPeers){
                    numPeers++;
                }
            }

            var newRow = _.template(ExpandedRowTemplate)({
                generationModel: generationModel,
                numPeers: numPeers
            });
            tr.after(newRow);
        },
        rowCollapse: function(id, expandedIds, event, tr) {
            this.expandedIds = expandedIds;
            var td = tr.children('.expands');
            td.html(RowExpandCollapse.COLLAPSED_CELL_MARKUP);
            tr.removeClass(RowExpandCollapse.EXPANDED_ROW_CLASS);
            tr.next().remove();
        },
        render: function() {
            var rowIterator = new RowIterator({ });
            var html = this.compiledTemplate({
                collection: this.collection,
                eachRow: rowIterator.eachRow,
                toggleCellClass: RowExpandCollapse.TOGGLE_CELL_CLASS,
                headerCellClass: RowExpandCollapse.HEADER_CELL_CLASS,
                rowIdAttribute: RowExpandCollapse.ROW_ID_ATTR,
                sortKeyAttribute: ColumnSort.SORT_KEY_ATTR
            });
            var $html = $(html);

            this.children.columnSort.update($html);
            this.children.rowExpandCollapse.update($html);

            this.$el.html($html);

            return this;
        }

    });

});
