define(function(require, module, exports) {
    var _ = require('underscore'),
        BaseView = require('views/Base');
    var ControlGroup = require('views/shared/controls/ControlGroup');
    var console = require('util/console');
    var route = require('uri/route');
    var time_utils = require('util/time_utils');
    var token_utils = require('splunkjs/mvc/tokenutils');
    require('bootstrap.tooltip');

    return BaseView.extend({
        moduleId: module.id,
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.model.timeRange.on('applied', function() {
                this.updateReportTime();
                this.updateTime();
            }, this);

            var hasTokens = token_utils.hasToken(this.model.report.get("dispatch.earliest_time"));
            var useTimeFrom = hasTokens ? 'dashboard' : 'search';
            this.model.timeRange.set({useTimeFrom: useTimeFrom});

            if (this.model.state.get('default_timerange')){
                this.children.timeScope = new ControlGroup({
                    label: _("Time Range Scope").t(),
                    controlType: 'SyntheticRadio',
                    controlOptions: {
                        className: 'btn-group btn-group-2 time-range-scope-select',
                        items: [
                            {value: 'dashboard', label: _('Dashboard').t(),tooltip: _('Use the time range set by the dashboard time range picker.').t()},
                            {value: 'search', label: _('Search').t(),tooltip: _('Override the dashboard time range picker by setting a time range for this Search.').t()}
                        ],
                        model: this.model.timeRange,
                        modelAttribute: 'useTimeFrom',
                        elastic: true
                    }
                });
            }
            else {
                this.model.timeRange.set('useTimeFrom', 'search');
            }

            this.listenTo(this.model.timeRange, 'change:useTimeFrom', this.updateTokens, this);
        },
        updateReportTime: function(){
            this.model.report.set({
                'dispatch.earliest_time': this.model.timeRange.get('earliest'),
                'dispatch.latest_time':this.model.timeRange.get('latest')
            }, {tokens: true});
        },
        updateTokens: function(){
            this.toggleTimeRangePicker();
            this.updateTime();
        },
        toggleTimeRangePicker: function() {
            if (this.model.timeRange.get('useTimeFrom') == "dashboard"){
                this.$('.timerange').hide();
            }
            else{
                this.$('.timerange').show();
            }
        },
        updateTime: function() {
            var timeLabel = this.model.timeRange.generateLabel(this.collection);
            this.$el.find("span.time-label").text(timeLabel);
        },
        render: function() {
            if (this.children.timeScope){
                this.children.timeScope.render().appendTo(this.el);
            }

            this.$el.append('<div class="timerange" style="display: block;"><label class="control-label">' + _('Time Range').t() + '</label></div>');
            this.$('div.timerange').append('<div class="controls"><a href="#" class="btn timerange-control"><span class="time-label"></span><span class="icon-triangle-right-small"></span></a></div>');
            this.toggleTimeRangePicker();
            this.updateTime();


            return this;
        }
    });

});