define(
        [
            'module',
            'jquery',
            'views/Base',
            'views/shared/delegates/TableRowToggle',
            'views/dashboards/table/controls/EditMenu',
            'uri/route',
            'util/splunkd_utils'
        ],
        function(module, $, BaseView, TableRowToggleView, EditMenu, route, splunkd_utils) {
            return BaseView.extend({
                moduleId: module.id,
                tagName: 'tr',
                className: 'expand',
                /**
                 * @param {Object} options {
             *     model: {
             *          state: <models.State>
             *          application: <models.Application>,
             *          dashboard: <models.services.data.ui.View>,
             *          scheduledView: <models.services.ScheduledView>
             *     },
             *     collection: {
             *          roles: <collections.services.authorization.Roles>,
             *          dashboards: <collections.Dashboards>
             *     }
             * }
                 */
                initialize: function() {
                    BaseView.prototype.initialize.apply(this, arguments);
                    this.$el.addClass((this.options.index % 2) ? 'odd' : 'even');
                    this.children.editmenu = new EditMenu({
                        model: this.model,
                        collection: this.collection,
                        user: this.model.user
                    });

                    this.model.dashboard.on('updateCollection', function() {
                        this.model.state.trigger('change:search');
                    }, this);

                    this.model.dashboard.meta.on('change:label', this.updateLabel, this);
                    this.compiledLabelTemplate = this.compileTemplate(this.labelTemplate);
                    this.model.dashboard.entry.acl.on('change:sharing', this.updateSharing, this);
                },
                updateLabel: function() {
                    var app = this.model.application, dashboard = this.model.dashboard;
                    this.$('td.title').html(this.compiledLabelTemplate({
                        label: dashboard.meta.get('label'),
                        link: route.page(app.get("root"), app.get("locale"), app.get("app"), dashboard.entry.get('name'))
                    }));
                },
                updateSharing: function () {
                    this.$('td.sharing').text(splunkd_utils.getSharingLabel(this.model.dashboard.entry.acl.get('sharing')));
                },
                render: function() {
                    this.$el.html(this.compiledTemplate({
                        app: this.model.dashboard.entry.acl.get('app'),
                        owner: this.model.dashboard.entry.acl.get('owner'),
                        index: this.options.index
                    }));
                    this.$('.actions').append(this.children.editmenu.render().el);
                    this.updateLabel();
                    this.updateSharing();
                    return this;
                },
                template: '\
                <td class="expands">\
                    <a href="#"><i class="icon-triangle-right-small"></i></a>\
                </td>\
                <td class="title">\
                </td>\
                <td class="actions">\
                </td>\
                <td class="owner"><%= owner %></td>\
                <td class="app"><%= app %></td>\
                <td class="sharing"></td>\
            ',
                labelTemplate: '<a href="<%= link %>"><%- label %></a>'
            });
        }
);


