/**
 * @author jszeto
 * @date 11/20/2012
 *
 * A view for editing the output field of a Lookup calculation.
 *
 * Input:
 *      model {models/services/datamodel/private/Field} - The output field to edit
 *      flashMessagesHelper {helpers/FlashMessagesHelper} - Used to register validating models
 *
 * @fires LookupOutputFieldView#action:removeOutputField
 */
define(
    [
        'underscore',
        'views/ValidatingView',
        'views/shared/controls/ControlGroup',
        'views/data_model_editor/form_components/SelectFieldType',
        'views/data_model_editor/form_components/SelectFieldFlags',
        'module'
    ],
    function(
        _,
        ValidatingView,
        ControlGroup,
        SelectFieldType,
        SelectFieldFlags,
        module
        )
    {

        return ValidatingView.extend({
            moduleId: module.id,
            className: 'lookup-attribute-controls',
            tagName: 'tr',

            events: {
                'click .remove-button': function(e) {
                    e.preventDefault();
                    /**
                     * Remove the output field
                     *
                     * @event LookupOutputFieldView#action:removeOutputField
                     * @param {string} - ID of the output field
                     */
                    this.trigger("action:removeOutputField", this.model.cid);
                }
            },

            // TODO [JCS] Remove
            modelToControlGroupMap: {
                fieldName: "textFieldName",
                displayName: "textDisplayName"
            },

            initialize: function(options) {
                ValidatingView.prototype.initialize.call(this, options);
                options = options || {};
                this.$el.addClass(options.className);

                this.children.textLookupOutputFieldName = new ControlGroup({controlType: "Text",
                                                                controlOptions:
                                                                    {model:this.model,
                                                                     modelAttribute:"lookupOutputFieldName"}});

                this.children.textFieldName = new ControlGroup({controlType: "Text",
                                                                controlOptions:
                                                                    {model:this.model,
                                                                     modelAttribute:"fieldName"}});
                this.children.textDisplayName = new ControlGroup({controlType: "Text",
                                                                  controlOptions:
                                                                  {model:this.model,
                                                                   modelAttribute:"displayName"}});

                this.children.selectFieldType = new SelectFieldType({model:this.model});
                this.children.selectFieldFlags= new SelectFieldFlags({model:this.model});

                options.flashMessagesHelper.register(this.model);
            },

            render: function() {
                var html = _(this.template).template({ });
                this.$el.html(html);

                this.$(".lookup-output-field-name-placeholder").append(this.children.textLookupOutputFieldName.render().el);
                this.$(".field-name-placeholder").append(this.children.textFieldName.render().el);
                this.$(".display-name-placeholder").append(this.children.textDisplayName.render().el);
                this.$(".field-type-placeholder").append(this.children.selectFieldType.render().el);
                this.$(".field-flags-placeholder").append(this.children.selectFieldFlags.render().el);

                return this;
            },

            template: '\
                <td class="lookup-output-field-name-placeholder"></td>\
                <td class="field-name-placeholder"></td>\
                <td class="display-name-placeholder"></td>\
                <td class="field-type-placeholder"></td>\
                <td class="field-flags-placeholder"></td>\
                <td><a href="#" class="remove-button"><%- _("Remove").t() %></a></td>\
            '

        });

    });
