/**
 * @author jszeto
 * @date 1/2/13
 *
 * Dialog that allows the user to edit an extracted field of an Object.
 *
 * Inputs:
 *      model {models/services/datamodel/private/Field} - The extracted field to edit
 *      parent {View} - The view that created this dialog
 *
 * @fires EditExtractedFieldDialog#action:saveExtractedField
 */
define(
    [
        'jquery',
        'underscore',
        'backbone',
        'views/shared/dialogs/DialogBase',
        'views/shared/controls/ControlGroup',
        'views/data_model_editor/form_components/SelectFieldFlags',
        'views/data_model_editor/form_components/SelectFieldType',
        'module'
    ],
    function(
        $,
        _,
        Backbone,
        DialogBase,
        ControlGroup,
        SelectFieldFlags,
        SelectFieldType,
        module
        )
    {
        return DialogBase.extend({
                moduleId: module.id,

                initialize: function(options) {
                    DialogBase.prototype.initialize.call(this, options);

                    this.settings.set("primaryButtonLabel", _("Save").t());
                    this.settings.set("cancelButtonLabel", _("Cancel").t());
                    this.settings.set("titleLabel", _("Edit Auto-Extracted Attribute").t());

                    this.children.labelFieldName = new ControlGroup({label:_("Field").t(),
                                                                     controlType: "Label",
                                                                     controlOptions: {modelAttribute:"fieldName",
                                                                                      model:this.model}});

                    this.children.textDisplayName = new ControlGroup({label:_("Rename").t(),
                                                                      controlType: "Text",
                                                                      controlOptions: {modelAttribute:"displayName",
                                                                                       model:this.model}});

                    this.children.selectFieldType = new ControlGroup({label: _("Type").t(),
                                                                      controls: new SelectFieldType({model:this.model,
                                                                          popdownOptions: {
                                                                              attachDialogTo: '.modal:visible',
                                                                              scrollContainer: '.modal:visible .modal-body:visible'
                                                                          }})});
                    this.children.selectFieldFlags= new ControlGroup({label: _("Flags").t(),
                                                                      controls: new SelectFieldFlags({model:this.model,
                                                                          popdownOptions: {
                                                                              attachDialogTo: '.modal:visible',
                                                                              scrollContainer: '.modal:visible .modal-body:visible'
                                                                          }})});
                },

                primaryButtonClicked: function() {
                    DialogBase.prototype.primaryButtonClicked.call(this, arguments);

                    // The form can't put the field into an invalid state, so no need to perform validation
                    /**
                     * Save the Data Model
                     *
                     * @event EditExtractedFieldDialog#action:saveExtractedField
                     * @param {{models/services/datamodel/private/Field} Field that has been edited.
                     */
                    this.trigger("action:saveExtractedField", this.model);
                    this.hide();
                },

                renderBody : function($el) {
                    var html = _(this.bodyTemplate).template({});
                    $el.html(html);
                    $el.find(".form").append(this.children.labelFieldName.render().el);
                    $el.find(".form").append(this.children.textDisplayName.render().el);
                    $el.find(".form").append(this.children.selectFieldType.render().el);
                    $el.find(".form").append(this.children.selectFieldFlags.render().el);
                },

                bodyTemplate: '\
                    <form class="form form-horizontal"></form>\
                '
            }
        );}
);
