define([
    'jquery',
    'underscore',
    'module',
    'views/Base',
    'views/shared/searchbar/Master',
    'views/shared/whatsnewdialog/Master',
    './InfoBar',
    './AppList',
    'models/services/data/ui/Pref',
    'contrib/text!views/home/Home.html',
    'splunk.util',
    'uri/route'
],
function(
    $,
    _,
    module,
    BaseView,
    SearchBar,
    WhatsNewDialogView,
    InfoBar,
    AppList,
    UIPrefsModel,
    TemplateHome,
    splunk_util,
    route
){
    return BaseView.extend({
        moduleId: module.id,
        initialize: function() {
            BaseView.prototype.initialize.call(this, arguments);
            this.isRenderedDfd = $.Deferred();

            $.when(this.options.uiPrefsDfd, this.options.userDfd).done(function() {
                this.children.searchBar = new SearchBar({
                    collection: {
                        apps: this.collection.navs,
                        times: this.collection.times
                    },
                    model: {
                        state: this.model.setting,
                        user: this.model.user,
                        timeRangeWorking: this.model.timeRangeWorking,
                        timeRange: this.model.timeRange,
                        application: this.model.application,
                        appLocal: this.model.appLocal
                    }
                });
            }.bind(this));

            this.children.appList = new AppList({
                collection: this.collection,
                model: this.model,
                canManageRemoteAppsDfd: this.options.canManageRemoteAppsDfd
            });

            this.children.infoBar = new InfoBar({
                model: {
                    serverInfo: this.model.serverInfo,
                    application: this.model.application,
                    metaDataResult: this.model.metaDataResult

                },
                canEditMonitorDfd: this.options.canEditMonitorDfd
            });

            this.model.serverInfo.on('change reset', function() {
                this.updateLogo();
            }, this);

        },
        template: TemplateHome,
        updateLogo: function() {
            $.when(this.isRenderedDfd).done(function() {
                var logoUrl = this.getLogoUrl();
                if (logoUrl) {
                    this.$('img.productLogo').attr('src', logoUrl).show();
                }
            }.bind(this));
        },

        getLogoUrl: function() {
            var logoFile = this.model.serverInfo.getBranding() == 'hunk'? 'splunk_logo-hunk.png' :
                           this.model.serverInfo.getBranding() == 'enterprise'? 'splunk_logo-enterprise.png' : null;
            if (logoFile) {
                return route.img(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    '@' + Math.random(),
                    'skins/default/' + logoFile
                );
            } else {
                return null;
            }
        },

        render: function(){
            var self = this;
            this.$el.append(this.compiledTemplate());
            $.when(this.options.uiPrefsDfd, this.options.userDfd).done(function() {
                this.$('.searchBarPlaceHolder').append(this.children.searchBar.render().el);
            }.bind(this));
            this.$('.appListPlaceHolder').append(this.children.appList.el);
            this.$('.infoBarPlaceHolder').append(this.children.infoBar.render().el);

            //show whats new
            this.model.userPref.on('change reset', function(){
                if(self.model.userPref && self.model.userPref.entry && self.model.userPref.entry.content
                    && splunk_util.normalizeBoolean(self.model.userPref.entry.content.get('showWhatsNew'))){
                    self.children.whatsNewDialog = new WhatsNewDialogView({
                        model: self.model
                    }).show();
                }
            });

            this.isRenderedDfd.resolve();
            return this;
        }
    });
});
