define([
            'underscore',
            'module',
            'models/pivot/PivotJob',
            'views/Base',
            'views/shared/jobstatus/Master',
            'views/shared/jobstatus/controls/Reload',
            'uri/route',
            'bootstrap.tooltip'
        ],
        function(
            _,
            module,
            PivotJob,
            Base,
            JobstatusMaster,
            Reload,
            route
            //tooltip
        ) {

    return Base.extend({

        moduleId: module.id,

        events: {
            'click a.disabled': function(e) {
                e.preventDefault();
            },
            'click .export-button:not(.disabled)': function(e) {
                e.preventDefault();
                this.model.searchJob.trigger('export');
            },
            'click .print-button:not(.disabled)': function(e) {
                e.preventDefault();
                this.model.searchJob.trigger('print');
            },
            'click .share-button:not(.disabled)': function(e) {
                e.preventDefault();
                this.model.searchJob.trigger('share');
            },
            'click .inspect-button:not(.disabled)': function(e) {
                this.model.searchJob.trigger('inspect');
                e.preventDefault();
            }
        },

        /**
         * @constructor
         * @param options {Object} {
         *     model: {
         *         application: <models/Application> the application model
         *         report: <models/pivot/PivotReport> the pivot report
         *         searchJob: <models/pivot/PivotJob> the current pivot job
                   appLocal <models.services.AppLocal> the local splunk app
         *     }
         */

        initialize: function() {
            Base.prototype.initialize.apply(this, arguments);

            this.children.jobstatus = new JobstatusMaster({
                enableSearchMode: false,
                // reload button is only enabled for managed acceleration
                enableReload: this.model.searchJob.getAccelerationType() === PivotJob.MANAGED_ACCELERATION,
                showJobButtons: false,
                showJobMenu: false,
                model: {
                    application: this.model.application,
                    state: this.model.report.entry.content,
                    searchJob: this.model.searchJob,
                    appLocal: this.model.appLocal
                }
            });

            this.model.report.entry.content.on(
                'change:search change:dispatch.earliest_time change:dispatch.latest_time',
                _.debounce(this.updateOpenInSearch, 0),
                this
            );
            // our usage of the job status bar sometimes leaves space for a hidden spinner, fix that here
            this.model.searchJob.entry.content.on('change', function(jobContent) {
                if(jobContent.get('isDone') || jobContent.get('isPaused')) {
                    this.$('.shared-jobstatus-spinner').hide();
                }
                if(jobContent.get('isDone')) {
                    this.$('.export-button').removeClass('disabled');
                }
            }, this);
        },

        render: function() {
            this.$el.html(this.compiledTemplate({
                showInspectButton: this.model.searchJob.getAccelerationType() === PivotJob.MANAGED_ACCELERATION,
                enableExport: this.model.searchJob.entry.content.get('isDone')
            }));
            this.children.jobstatus.render().appendTo(this.$('.jobstatus-wrapper'));
            this.$('> .btn-group > a').tooltip({animation:false, container: 'body'});
            
            this.updateOpenInSearch();
            return this;
        },

        remove: function() {
            this.$('> .btn-group > a').tooltip({animation:false, container: 'body'});
            return Base.prototype.remove.apply(this, arguments);
        },

        // hackery here just to get things to look right...
        showJobCancelled: function() {
            this.children.jobstatus.children.spinner.$el.hide();
            this.children.jobstatus.children.count.$el.hide();
            this.children.jobstatus.sleep();
            this.$('.action-bar .btn').addClass('disabled');
            this.$('> .btn-group > a').tooltip('destroy');
            var jobStatusControls = this.children.jobstatus.children.controls;
            jobStatusControls.$('.progress-bar').empty().prepend('<span class="progress-label">' + _('Cancelled').t() + '</span>');
            jobStatusControls.$('a').remove();
            this.children.reloadButton = new Reload({ model: this.model.searchJob });
            this.children.reloadButton.render().appendTo(jobStatusControls.el);
        },

        updateOpenInSearch: function() {
            var $openInSearchButton = this.$('.open-in-search-button'),
                root = this.model.application.get('root'),
                locale = this.model.application.get('locale'),
                app = this.model.application.get('app'),
                reportContent = this.model.report.entry.content;

            if(!reportContent.has('search')) {
                return;
            }

            var qsArgs = { q: reportContent.get('search') };
            $openInSearchButton.removeClass('disabled');
            qsArgs.earliest = reportContent.get('dispatch.earliest_time') || 0;
            if(reportContent.has('dispatch.latest_time')) {
                qsArgs.latest = reportContent.get('dispatch.latest_time');
            }
            var openInSearchUrl = route.search(root, locale, app, { data: qsArgs });

            $openInSearchButton.attr('href', openInSearchUrl);
        },

        template: '\
            <div class="jobstatus-wrapper <%- showInspectButton ? \'with-inspect-button\' : \'\' %>"></div>\
            <div class="action-bar btn-group pull-right">\
                <% if(showInspectButton) { %>\
                    <a href="#" class="inspect-button btn btn-small btn-square" title="<%- _(\'Inspect\').t() %>">\
                        <i class="icon-info"></i><span class="hide-text"><%- _("Inspect").t() %></span>\
                    </a>\
                <% } %>\
                <a href="#" class="share-button btn btn-small btn-square" title="<%- _(\'Share\').t() %>">\
                    <i class="icon-share"></i><span class="hide-text"><%- _("Share").t() %></span>\
                </a>\
                <a href="#" class="export-button btn btn-small btn-square <%= enableExport ? \'\' : \'disabled\' %>" title="<%- _(\'Export\').t() %>">\
                    <i class="icon-export"></i><span class="hide-text"><%- _("Export").t() %></span>\
                </a>\
                <a href="#" class="print-button btn btn-small btn-square" title="<%- _(\'Print\').t() %>">\
                    <i class="icon-print"></i><span class="hide-text"><%- _("Print").t() %></span>\
                </a>\
                <a href="#" class="open-in-search-button btn btn-small btn-square disabled" title="<%- _(\'Open in Search\').t() %>">\
                    <i class="icon-search"></i><span class="hide-text"><%- _("Open in Search").t() %></span>\
                </a>\
            </div>\
        '

    });

});