define([
            'jquery',
            'underscore',
            'module',
            'views/Base',
            'views/pivot/custom_controls/FieldPickerControlGroup',
            'views/shared/controls/ControlGroup',
            'helpers/pivot/PivotVisualizationManager'
        ],
        function(
            $,
            _,
            module,
            Base,
            FieldPickerControlGroup,
            ControlGroup,
            pivotVizManager
        ) {

    return Base.extend({

        moduleId: module.id,
        tagName: 'form',
        className: 'form-horizontal',

        /**
         * @constructor
         * @param options {
         *     model: {
         *         report <models/pivot/PivotReport> the current report
         *         dataModel <models/services/datamodel/DataModel> the current dataModel,
         *         element {Model} the report element the subpanel represents
         *                         must be subclasses of <models/pivot/elements/BaseElement>
         *         application: <models/Application> the application state model
         *         appLocal <models.services.AppLocal> the local splunk app
         *         user <models.services/admin.User> the current user
         *     }
         *     collection {
         *         timePresets <collections/services/data/ui/Times> the current user's time presets
         *     }
         *     panel {Object} a panel configuration object,
         *                    see helpers/pivot/PivotVisualization for full documentation
         * }
         */

        initialize: function() {
            Base.prototype.initialize.apply(this, arguments);
            var rerenderAttributes = this.determineRerenderAttributes(this.options.panel.controlGroups);
            if(this.model.element) {
                this.model.element.on('change', function() {
                    var shouldRerender = _(rerenderAttributes).any(function(attribute) {
                        return this.model.element.hasChanged(attribute);
                    }, this);

                    if(shouldRerender) {
                        this.debouncedRender();
                    }
                }, this);
            }
        },

        render: function() {
            _(this.children).invoke('remove');
            this.children = {};
            var panelOptions = this.options.panel;
            if(this.model.element && !this.options.hideFieldPicker) {
                var fieldPicker = this.children.fieldPicker = new FieldPickerControlGroup({
                    model: this.model.element,
                    report: this.model.report,
                    dataModel: this.model.dataModel,
                    dataTypes: this.options.panel.dataTypes,
                    showRemoveButton: !this.options.panel.required
                });
                fieldPicker.render().appendTo(this.$el);
            }
            _(panelOptions.controlGroups).each(function(controlGroup, i) {
                if(!this.groupIsVisible(controlGroup)) {
                    return;
                }
                var child = this.children['control_group_' + i] = new controlGroup.view($.extend({}, controlGroup.options, {
                    model: (controlGroup.type === pivotVizManager.REPORT_CONTROL ? this.model.element : this.model.report.entry.content),
                    report: this.model.report,
                    dataModel: this.model.dataModel,
                    collection: this.collection.timePresets,
                    application: this.model.application,
                    appLocal: this.model.appLocal,
                    user: this.model.user
                }));
                child.render().appendTo(this.$el);
            }, this);
            return this;
        },

        getAllControlGroups: function() {
            return _(this.children).filter(function(child) { return child instanceof ControlGroup; });
        },

        determineRerenderAttributes: function(controlGroups) {
            return _(controlGroups)
                .chain()
                .pluck('visibility')
                .reduce(function(keys, visibility) {
                    if(visibility) {
                        return _.union(keys, _(visibility).keys());
                    }
                    return keys;
                }, [])
                .value();
        },

        groupIsVisible: function(controlGroup) {
            return _(controlGroup.visibility).every(function(allowedValues, attribute) {
                return _(allowedValues).contains(this.model.element.get(attribute));
            }, this);
        }

    });

});