define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/reportcontrols/editmenu/Master',
        'views/shared/reportcontrols/dialogs/dashboardpanel/Master',
        'views/shared/reportcontrols/details/Master',
        'views/shared/delegates/Popdown'
    ],
    function(
        _,
        module,
        BaseView,
        EditButtonView,
        DashboardDialog,
        DetailsView,
        Popdown
    ) {
    return BaseView.extend({
        moduleId: module.id,
        className: 'btn-group pull-right',
         /**
         * @param {Object} options {
         *      model: {
         *          application: <models.Application>,
         *          report: <models.Report>,
         *          searchJob: <models.services.search.Job>,
         *          appLocal: <models.services.AppLocal>,
         *          user: <models.services.admin.User>
         *      },
         *      collection: <collections.services.authorization.Roles>
         * }
         */
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.children.editButtonView = new EditButtonView({
                model: {
                    report: this.model.report,
                    application: this.model.application,
                    searchJob: this.model.searchJob,
                    user: this.model.user,
                    appLocal: this.model.appLocal
                },
                collection: this.collection,
                deleteRedirect: true,
                className: 'btn-combo'
            });
            
            this.children.detailsView = new DetailsView({
                model: {
                    report: this.model.report,
                    application: this.model.application,
                    searchJob: this.model.searchJob,
                    user: this.model.user,
                    appLocal: this.model.appLocal
                },
                collection: this.collection
            });

            this.model.searchJob.on("prepared", function() {
                this.$('a.addtodashboard').removeClass('disabled');
            }, this);
        },
        events: {
            'click a.addtodashboard': function(e) {
                this.children.dashboardDialog = new DashboardDialog({
                    model:  {
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    onHiddenRemove: true
                });

                $("body").append(this.children.dashboardDialog.render().el);
                this.children.dashboardDialog.show();

                e.preventDefault();
            }
        },
        render: function() {
            this.$el.html(this.compiledTemplate({
                _: _
            }));
            this.$el.prepend(this.children.editButtonView.render().el);
            this.$('.more-info .popdown-dialog-body').prepend(this.children.detailsView.render().el);

            if (this.model.searchJob.isPreparing()) {
                this.$('a.addtodashboard').addClass('disabled');
            }

            this.children.popdownDelegate = new Popdown({el: this.$('.more-info')});
            return this;
        },
        template: '\
            <div class="btn-combo more-info"><a href=# class="popdown-toggle btn"><%- _("More Info").t() %><span class="caret"></span></a><div class="popdown-dialog"><div class="arrow"></div><div class="popdown-dialog-body"></div></div></div>\
            <a href=# class="addtodashboard btn"><%- _("Add to Dashboard").t() %></a>\
        '
    });
});
