define([
        'underscore',
        'backbone',
        'module',
        'models/Base',
        'models/services/search/IntentionsParser',
        'views/Base',
        'views/shared/FlashMessages',
        'views/report/Title',
        'views/shared/jobstatus/Master',
        'views/report/ActionBar',
        'views/shared/timerangepicker/Master',
        'views/shared/JSChart',
        'views/shared/SingleValue',
        'views/shared/results_table/ResultsTableMaster',
        'views/shared/eventsviewer/Master',
        'views/shared/JobDispatchState',
        'views/report/tablecontrols/Master',
        'util/splunkd_utils',
        'splunk.util',
        'uri/route'
    ],
    function(
        _,
        Backbone,
        module,
        BaseModel,
        IntentionsParser,
        BaseView,
        FlashMessageView,
        TitleView,
        JobStatusView,
        ActionBarView,
        TimeRangePickerView,
        JSChartView,
        SingleValueView,
        ResultsTableView,
        EventsViewer,
        JobDispatchStateView,
        TableControlsView,
        splunkd_utils,
        splunkUtil,
        route
    ){
        return BaseView.extend({
            moduleId: module.id,
            className: 'view-report-page',
             /**
             * @param {Object} options {
             *      model: {
             *          result: <models.services.search.job.ResultsV2>,
             *          summary: summary: <model.services.search.job.SummaryV2>,
             *          timeline: <model.services.search.job.TimelineV2>,
             *          searchJob: <models.services.search.Job>,
             *          searchResultsColumn: <models.services.search.jobs.ResultJsonCols>,
             *          searchResultsRow: <models.services.search.jobs.ResultJsonRows>,
             *          report: <models.Report>,
             *          reportPristine: <model.reportPristine>,
             *          application: <models.Application>,
             *          timeRange: <models.TimeRange>,
             *          appLocal: <models.services.AppLocal>,
             *          user: <models.services.admin.User>
             *      },
             *      collection: {
             *          times: <collections.services.data.ui.TimesV2>,
             *          selectedFields: <collections.SelectedFields>,
             *          roles: <collections.services.authorization.Roles>,
             *          workflowActions: <collections.services.data.ui.WorkflowActions>
             *       }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.errorTypes = [splunkd_utils.FATAL, splunkd_utils.ERROR];

                this.isError = splunkd_utils.messagesContainsOneOfTypes(this.model.report.error.get("messages"), this.errorTypes);
                this.resultCountAttribute;
                this.isReportSearch = false;

                this.children.flashMessageView = new FlashMessageView({
                    className: 'report-message',
                    model: {
                        searchJob: this.model.searchJob,
                        searchJobControl: this.model.searchJob.control,
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        result: this.model.result,
                        summary: this.model.summary,
                        searchResultsColumn: this.model.searchResultsColumn,
                        searchResultsRow: this.model.searchResultsRow
                    },
                    whitelist: this.errorTypes
                });

                // views
                if (this.model.report.isNew()) {
                    if (!this.isError) {
                        var noReportIdError = splunkd_utils.createSplunkDMessage(
                            splunkd_utils.FATAL,
                            _("No report was specified.").t());
                        this.model.report.trigger('error', this.model.report, noReportIdError);
                    }
                } else {
                    this.model.state = new BaseModel();
                    this.model.drilldownIntentionsParser = new IntentionsParser();

                    this.children.jobDispatchStateView = new JobDispatchStateView({
                        mode: 'auto',
                        model: this.model.searchJob
                    });

                    this.children.titleView = new TitleView({model: this.model.reportPristine});

                    this.children.jobStatusView = new JobStatusView({
                        model: {
                            searchJob: this.model.searchJob,
                            state: this.model.report.entry.content,
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            report: this.model.report
                        },
                        enableSearchMode: false,
                        allowDelete: false
                    });

                    this.children.actionBarView = new ActionBarView({
                        model: {
                            searchJob: this.model.searchJob,
                            report: this.model.reportPristine,
                            application: this.model.application,
                            timeRange: this.model.timeRange,
                            user: this.model.user,
                            appLocal: this.model.appLocal
                        },
                        collection: this.collection.roles
                    });

                    this.children.timeRangePickerView = new TimeRangePickerView({
                        className: 'controls',
                        model: {
                            state: this.model.report.entry.content,
                            timeRange: this.model.timeRange,
                            user: this.model.user,
                            appLocal: this.model.appLocal,
                            application: this.model.application
                        },
                        collection: this.collection.times,
                        timerangeClassName: 'btn btn-primary pull-left'
                    });

                    this.children.jsChartView = new JSChartView({
                        model: {
                            searchResultsColumn: this.model.searchResultsColumn,
                            state: this.model.report.entry.content
                        },
                        width: '100%',
                        height: this.model.report.entry.content.get('display.visualizations.chartHeight')
                    });

                    this.children.singleValueView = new SingleValueView({
                        model: {
                            searchResultsColumn: this.model.searchResultsColumn,
                            state: this.model.report.entry.content
                        },
                        width: 800,
                        height: 300
                    });

                    this.children.resultsTableView = new ResultsTableView({
                        model: {
                            results: this.model.searchResultsRow,
                            metadata: this.model.report.entry.content,
                            format: this.model.report.entry.content
                        },
                        sortKeyAttr: 'display.statistics.sortColumn',
                        sortDirAttr: 'display.statistics.sortDirection',
                        countAttr: 'display.prefs.statistics.count',
                        offsetAttr: 'display.prefs.statistics.offset',
                        tableDockOffset: 36
                    });

                    this.children.eventsViewer = new EventsViewer({
                        model: {
                            result: this.model.result,
                            summary: this.model.summary,
                            searchJob: this.model.searchJob,
                            report: this.model.report,
                            application: this.model.application,
                            state: this.model.state
                        },
                        collection: {
                            eventRenderers: this.collection.eventRenderers,
                            selectedFields: this.collection.selectedFields,
                            workflowActions: this.collection.workflowActions
                        },
                        selectableFields: false,
                        headerMode: 'dock'
                    });

                    //listeners
                    this.model.drilldownIntentionsParser.on('change', function() {
                        var routeString = route.search(
                                this.model.application.get('root'),
                                this.model.application.get('locale'),
                                this.model.application.get('app'),
                                {data: {
                                    earliest: this.model.drilldownIntentionsParser.get('dispatch.earliest_time') || this.model.report.entry.content.get('dispatch.earliest_time'),
                                    latest: this.model.drilldownIntentionsParser.get('dispatch.latest_time') || this.model.report.entry.content.get('dispatch.latest_time'),
                                    q: this.model.drilldownIntentionsParser.get('fullSearch')
                                }}
                            );
                        window.location = routeString;
                    }, this);

                    this.children.resultsTableView.on('cellClick', function(clickMeta) {
                        var field, value;
                        this.model.drilldownIntentionsParser.clear({silent: true});
                        if(clickMeta._span) {
                            this.model.drilldownIntentionsParser.set({
                                'dispatch.earliest_time': clickMeta.value,
                                'dispatch.latest_time': JSON.stringify(parseFloat(clickMeta.value) + clickMeta._span)
                            }, {silent: true});
                            delete clickMeta.name;
                            delete clickMeta.value;
                            delete clickMeta.rowContext['row._time'];
                        }

                        if(this.model.report.entry.content.get('display.statistics.drilldown') === 'row') {
                            field = _(clickMeta.rowContext).keys().map(function(f) { return f.replace('row.', ''); });
                            value = _(clickMeta.rowContext).values();
                        } else if(this.model.report.entry.content.get('display.statistics.drilldown') === 'cell') {
                            if(clickMeta.name && clickMeta.name2){
                                field = [clickMeta.name, clickMeta.name2];
                                value = [clickMeta.value, clickMeta.value2];
                            } else if(clickMeta.name) {
                                field = clickMeta.name;
                                value = clickMeta.value;
                            } else {
                                field = clickMeta.name2;
                                value = clickMeta.value2;
                            }
                        }

                        this.model.drilldownIntentionsParser.fetch({
                             data: {
                                q: this.model.report.entry.content.get('search'),
                                action: 'fieldvalue',
                                field: field || '',
                                value: value || '',
                                fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                app: this.model.application.get('app'),
                                owner: this.model.application.get('owner')
                            }
                        });
                    },this);

                    this.children.jsChartView.on('chartClick', function(eventInfo) {
                        var field, value;
                        this.model.drilldownIntentionsParser.clear({silent: true});

                        if(eventInfo.type === 'legendClick') {
                            // if the legend click has a name and value (this happens for scatter charts), use them
                            if(eventInfo.hasOwnProperty('name') && eventInfo.hasOwnProperty('value')) {
                                this.model.drilldownIntentionsParser.fetch({
                                    data: {
                                        q: this.model.report.entry.content.get('search'),
                                        action: 'fieldvalue',
                                        field: eventInfo.name,
                                        value: eventInfo.value,
                                        fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                        app: this.model.application.get('app'),
                                        owner: this.model.application.get('owner')
                                    }
                                });
                            }
                            // otherwise do a keyword search based on name2
                            else {
                                this.model.drilldownIntentionsParser.fetch({
                                    data: {
                                        q: this.model.report.entry.content.get('search'),
                                        action: 'keyword',
                                        field: eventInfo.name2,
                                        fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                        app: this.model.application.get('app'),
                                        owner: this.model.application.get('owner')
                                    }
                                });
                            }
                            return;
                        }
                        
                        if(eventInfo._span) {
                            this.model.drilldownIntentionsParser.set({
                                'dispatch.earliest_time': eventInfo.value,
                                'dispatch.latest_time': JSON.stringify(parseFloat(eventInfo.value) + eventInfo._span)
                            }, {silent: true});
                        }
                        if(eventInfo.name === '_time') {
                            field = eventInfo.name2;
                            value = eventInfo.value2;
                        } else {
                            field = [eventInfo.name, eventInfo.name2];
                            value = [eventInfo.value, eventInfo.value2];
                        }

                        this.model.drilldownIntentionsParser.fetch({
                            data: {
                                q: this.model.report.entry.content.get('search'),
                                action: 'fieldvalue',
                                field: field,
                                value: value,
                                fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                app: this.model.application.get('app'),
                                owner: this.model.application.get('owner')
                            }
                        });
                    },this);

                    var visibility = _.debounce(this.visibility, 0);
                    this.model.searchJob.entry.content.on("change:dispatchState change:resultCount change:resultPreviewCount change:eventCount change:eventAvailableCount", visibility, this);

                    this.model.reportPristine.entry.content.on('change:dispatch.earliest_time change:dispatch.latest_time', _.debounce(function() {
                        this.model.report.entry.content.set({
                            'dispatch.earliest_time': this.model.reportPristine.entry.content.get('dispatch.earliest_time'),
                            'dispatch.latest_time': this.model.reportPristine.entry.content.get('dispatch.latest_time')
                        });
                    }, 0), this);

                    this.model.reportPristine.entry.content.on('change:display.page.search.mode', _.debounce(function() {
                        this.model.report.entry.content.set({
                            'display.page.search.mode': this.model.reportPristine.entry.content.get('display.page.search.mode')
                        },
                        {silent:true});
                    }, 0), this);

                    this.debouncedWindowResize =  _(_.debounce(this.children.jsChartView.reflow, 50)).bind(this);
                    $(window).on('resize', this.debouncedWindowResize);
                }
            },
            remove: function() {
                BaseView.prototype.remove.apply(this, arguments);
                $(window).off('resize', this.debouncedWindowResize);
            },
            visibility: function() {
                if (this.model.searchJob.isPreparing()) {
                    this.hideResults();
                    this.hideEvents();
                    return;
                }

                this.isReportSearch = this.model.searchJob.isReportSearch();
                var dispatchState = this.model.searchJob.entry.content.get("dispatchState");

                if (!this.resultCountAttribute) {
                    if (this.isReportSearch) {
                        if (this.model.searchJob.entry.content.get('isPreviewEnabled')) {
                            this.resultCountAttribute = 'resultPreviewCount';
                        } else {
                            this.resultCountAttribute = 'resultCount';
                        }
                    } else {
                        this.resultCountAttribute = 'eventAvailableCount';
                    }
                }

                var resultCount = this.model.searchJob.entry.content.get(this.resultCountAttribute);

                if (dispatchState === 'RUNNING' && (resultCount === 0)) {
                    if (this.isReportSearch && this.model.searchJob.isRealtime()) {
                        this.realTimeNoStats();
                    } else {
                        this.hideContents();
                    }
                } else if (!dispatchState ||
                        (dispatchState === 'FINALIZING' && (resultCount === 0)) ||
                        (dispatchState === 'DONE' && (resultCount === 0))) {
                    this.hideContents();
                } else {
                    this.showContents();
                }
            },
            showContents: function() {
                this.isReportSearch ? this.showResults() : this.showEvents();
            },
            hideContents: function() {
                this.isReportSearch ? this.hideResults() : this.hideEvents();
            },
            showResults: function() {
                this.children.jobDispatchStateView.sleep().$el.hide();

                var hasVisualization = this.model.report.entry.content.get('display.visualizations.show') === "1",
                    displayType = this.model.report.entry.content.get('display.visualizations.type');
                if (!this.children.statisticsControlsView) {
                    if (hasVisualization) {
                        if (displayType === "charting") {
                            this.$('.chart-viewer').append(this.children.jsChartView.render().el);
                        } else {
                            this.$('.chart-viewer').append(this.children.singleValueView.render().el);
                        }
                    } else {
                        // No visualization
                        this.$('.chart-viewer').remove();
                    }

                    this.children.statisticsControlsView = new TableControlsView({
                        model: {
                            report: this.model.report,
                            searchJob: this.model.searchJob,
                            state: this.model.report.entry.content
                        },
                        mode: this.model.searchJob.entry.content.get('isPreviewEnabled') ? 'results_preview' : 'results',
                        offsetKey: "display.prefs.statistics.offset",
                        countKey: "display.prefs.statistics.count"
                    });

                    this.$('.events-viewer-wrapper').append(this.children.resultsTableView.render().el);
                    this.$('.events-viewer-wrapper').before(this.children.statisticsControlsView.render().el);
                }

                if (hasVisualization) {
                    if(displayType === 'charting') {
                        this.children.jsChartView.wake().$el.show();
                    } else {
                        this.children.singleValueView.wake().$el.show();
                    }
                }
                this.children.statisticsControlsView.wake().$el.show();
                this.children.resultsTableView.wake().$el.show();
            },
            hideResults: function() {
                this.children.jobDispatchStateView.wake().$el.show();
                var hasVisualization = this.model.report.entry.content.get('display.visualizations.show') === "1";

                if (hasVisualization) {
                    if (this.model.report.entry.content.get('display.visualizations.type') === 'charting') {
                        this.children.jsChartView.sleep().$el.hide();
                    } else {
                        this.children.singleValueView.sleep().$el.hide();
                    }
                }

                if (this.children.statisticsControlsView) {
                    this.children.statisticsControlsView.sleep().$el.hide();
                }

                this.children.resultsTableView.sleep().$el.hide();
            },
            showEvents: function() {
                this.children.jobDispatchStateView.sleep().$el.hide();

                if (!this.children.eventsControlsView) {
                    this.children.eventsControlsView = new TableControlsView({
                        model: {
                            report: this.model.report,
                            searchJob: this.model.searchJob,
                            state: this.model.state,
                            timeline: this.model.timeline
                        },
                        mode: 'events',
                        offsetKey: "display.prefs.events.offset",
                        countKey: "display.prefs.events.count"
                    });
                    this.$('.events-viewer-wrapper').append(this.children.eventsViewer.render().el);
                    this.$('.events-viewer-wrapper').before(this.children.eventsControlsView.render().el);
                    this.$('.chart-viewer').remove();
                }

                this.children.eventsControlsView.wake().$el.show();
                this.children.eventsViewer.$el.show();
            },
            hideEvents: function() {
                this.children.jobDispatchStateView.wake().$el.show();

                if (this.children.eventsControlsView) {
                    this.children.eventsControlsView.sleep().$el.hide();
                }

                this.children.eventsViewer.$el.hide();
            },
            realTimeNoStats: function() {
                this.children.jobDispatchStateView.wake().$el.show();
                var hasVisualization = this.model.report.entry.content.get('display.visualizations.show') === "1";

                if (hasVisualization) {
                    if (this.model.report.entry.content.get('display.visualizations.type') === 'charting') {
                        this.children.jsChartView.wake().$el.show();
                    } else {
                        this.children.singleValueView.wake().$el.show();
                    }
                }

                if (this.children.statisticsControlsView) {
                    this.children.statisticsControlsView.sleep().$el.hide();
                }

                if (this.children.resultsTableView) {
                    this.children.resultsTableView.sleep().$el.hide();
                }
            },
            render: function() {
                if (this.model.report.isNew()) {
                    this.$el.append('<div class="section-padded section-header"></div>');
                    this.$('.section-header').append(this.children.flashMessageView.render().el);
                } else {
                    this.$el.html(this.template);
                    var $pageOrientation = this.$('.section-header'),
                        $resultsWrapper = this.$('.results-wrapper');

                    //Header and Job status views
                    $pageOrientation.append(this.children.actionBarView.render().el);
                    $pageOrientation.append(this.children.titleView.render().el);

                    if (splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.general.timeRangePicker.show'))) {
                        $pageOrientation.append(this.children.timeRangePickerView.render().el);
                    }

                    $pageOrientation.append(this.children.flashMessageView.render().el);

                    // messages
                    $resultsWrapper.append(this.children.jobDispatchStateView.render().el);

                    if (!(this.model.report.isRealTime() && !this.model.user.canRTSearch())) {
                        this.$('.job-bar').replaceWith(this.children.jobStatusView.render().el);

                        this.visibility();
                    } else {
                        $resultsWrapper.remove();
                    }
                }

                return this;
            },
            template: '\
                <div class="section-padded section-header"></div>\
                <div class="job-bar"></div>\
                <div class="results-wrapper">\
                <div class="chart-viewer"></div>\
                <div class="events-viewer-wrapper"></div>\
                </div>\
            '
        });
    }
);
