define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/report/tablecontrols/ResultsCount',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/SearchResultsPaginator',
        'views/shared/delegates/Dock'
    ],
    function(_, module, Base, ResultsCount, SyntheticSelectControl, SearchResultsPaginator, Dock) {
        return Base.extend({
            moduleId: module.id,
            tagName: 'div',
            className: 'table-caption',
             /**
             * @param {Object} options {
             *      model: {
             *          report: <models.Report>,
             *          searchJob: <models.services.search.Job>,
             *          state: <models.Base>
             *          timeline: <model.services.search.job.TimelineV2> (required only for mode: events)
             *      },
             *      mode: events || results || results_preview,
             *      countKey: attr to change count on
             *      offsetKey: attr to change offset on
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.children.resultsCount = new ResultsCount({
                    model: {
                        searchJob: this.model.searchJob
                    }
                });
                var items = [
                    {value: '10', label: _('10 per page').t()},
                    {value: '20', label: _('20 per page').t()},
                    {value: '50', label: _('50 per page').t()}
                ];
                if(this.options.mode !== 'events') {
                    items.push({value: '100', label: _('100 per page').t()});
                }
                this.children.count = new SyntheticSelectControl({
                    menuWidth: "narrow",
                    className: "btn-group pull-left",
                    items: items,
                    model: this.model.report.entry.content,
                    modelAttribute: this.options.countKey,
                    toggleClassName: 'btn-pill'
                });

                this.children.searchResultsPaginator = new SearchResultsPaginator({
                    mode: this.options.mode,
                    model: {
                        state: this.model.report.entry.content,
                        searchJob: this.model.searchJob,
                        timeline: this.model.timeline
                    },
                    collection: this.collection,
                    countKey: this.options.countKey,
                    offsetKey: this.options.offsetKey
                });

                this.model.state.on('change:modalize', function() {
                    var tableDock = this.children.dock;
                    if (tableDock) {
                        (this.model.state.get('modalize') >= 0) ?
                            tableDock.disable():
                            tableDock.enable();
                    }
                },this);
            },
            render: function() {
                this.$el.append(this.template);
                if (this.options.mode === 'results' || this.options.mode === 'results_preview') {
                    this.$('.table-caption-inner').append(this.children.resultsCount.render().el);
                }
                this.$('.table-caption-inner').append(this.children.count.render().el);
                this.$('.table-caption-inner').append(this.children.searchResultsPaginator.render().el);

                this.children.dock = new Dock({ el: this.el, affix: '.table-caption-inner' });
                return this;
            },
            template: '\
                <div class="table-caption-inner"></div>\
            '
        });
    }
);
