define(
    [
        'module',
        'jquery',
        'underscore',
        'util/splunkd_utils',
        'views/Base',
        'views/shared/tablecaption/Master',
        'views/shared/TableHead',
        'views/shared/delegates/TableRowToggle',
        'views/reports/table/TableRow',
        'views/reports/table/MoreInfo',
        'views/shared/delegates/TableDock'
    ],
    function(
        module,
        $,
        _,
        splunkDUtils,
        BaseView,
        CaptionView,
        TableHeadView,
        TableRowToggleView,
        TableRow,
        MoreInfo,
        TableDock
    ){
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model:
             *          state: <Backbone.Model>
             *          application: <models.Application>,
             *          uiPrefs: <models.services.admin.UIPrefs>
             *     }
             *     collection: {
             *         reports: <collections.Reports>,
             *         roles: <collections.services.authorization.Roles>
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.children.tableRowToggle = new TableRowToggleView({el: this.el, collapseOthers: true });
                this.children.caption = new CaptionView({
                    countLabel: _('Reports').t(),
                    model: {
                        state: this.model.state,
                        application: this.model.application,
                        uiPrefs: this.model.uiPrefs
                    },
                    collection: this.collection.reports
                });
                this.children.head = new TableHeadView({
                    model: this.model.state,
                    columns: [
                        { label: _('i').t(), className: 'col-info', html: '<i class="icon-info"></i>'},
                        { label: _('Title').t(), sortKey: 'name'},
                        { label: _('Actions').t(), className: 'col-actions', colSpan: 2},
                        { label: _('Owner').t(), sortKey: 'eai:acl.owner,name', className: 'col-owner'},
                        { label: _('App').t(), sortKey: 'eai:acl.app,name', className: 'col-app'},
                        { label: _('Sharing').t(), sortKey: 'eai:acl.sharing,name', className: 'col-sharing'}
                    ]
                });
                this.children.rows = this.rowsFromCollection();
                this.collection.reports.on('reset', this.renderRows, this);

                this.children.tableDock = new TableDock({ el: this.el, offset: 36, dockScrollBar: false, defaultLayout: 'fixed', flexWidthColumn: 1 });
            },
            rowsFromCollection: function() {
                return _.flatten(
                    this.collection.reports.map(function(model, i) {
                        return [
                            new TableRow({
                                model: {
                                    report: model,
                                    application: this.model.application,
                                    state: this.model.state,
                                    user: this.model.user,
                                    appLocal: this.model.appLocal
                                },
                                collection: this.collection.roles,
                                index: i
                            }),
                            new MoreInfo({
                                model: {
                                    report: model,
                                    application: this.model.application,
                                    user: this.model.user,
                                    appLocal: this.model.appLocal
                                },
                                collection: this.collection.roles,
                                index: i
                            })
                        ];
                    }, this)
                );
            },
            _render: function() {
                _(this.children.rows).each(function(row){
                    this.$('.report-listings').append(row.render().el);
                }, this);
                this.children.tableDock.update();
            },
            renderRows: function() {
                 _(this.children.rows).each(function(row){ row.remove(); }, this);
                 this.children.rows = this.rowsFromCollection();
                 this._render();
            },
            render: function() {
                if (!this.el.innerHTML) {
                    this.$el.append(this.children.caption.render().el);
                    this.$el.append(this.compiledTemplate({}));
                    this.$('> .table-chrome').prepend(this.children.head.render().el);
                }
                this._render();
                return this;
            },
            template: '\
                <table class="table table-chrome table-striped table-row-expanding table-listing">\
                <tbody class="report-listings"></tbody>\
                </table>\
            '
        });
    }
);
