define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/reportcontrols/editmenu/Master',
        'views/shared/delegates/Popdown',
        'views/shared/delegates/TableRowToggle',
        'uri/route',
        'util/splunkd_utils'
    ],
    function(
        _,
        module,
        BaseView,
        EditDropDown,
        PopdownView,
        TableRowToggleView,
        route,
        splunkd_utils
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            className: 'expand',
            /**
             * @param {Object} options {
             *     model: {
             *          report: <models.Report>,
             *          application: <models.Application>,
             *          state: <Backbone.Model>,
             *          appLocal: <models.services.AppLocal>,
             *          user: <models.service.admin.user>
             *     },
             *     collections: <collections.services.authorization.Roles>
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.$el.addClass((this.options.index % 2) ? 'odd' : 'even');
                this.editdropdown = new EditDropDown({
                    model: {
                        application: this.model.application,
                        report: this.model.report,
                        user: this.model.user,
                        appLocal: this.model.appLocal
                    },
                    collection: this.collection,
                    button: false,
                    showOpenActions: false
                });

                this.model.report.on('updateCollection', function() {
                    this.model.state.trigger('change:search');
                }, this);

                this.model.report.entry.acl.on('change:sharing', this.updateSharing, this);
            },
            updateSharing: function () {
                this.$('td.sharing').text(splunkd_utils.getSharingLabel(this.model.report.entry.acl.get('sharing')));
            },
            render: function() {
                var openInView = this.model.report.openInView(),
                    reportName   = this.model.report.entry.get('name'),
                    reportLink   = route.report(
                                    this.model.application.get("root"),
                                    this.model.application.get("locale"),
                                    this.model.application.get("app"),
                                    { data: { s: this.model.report.id}}),
                    openLink = route[openInView](
                                    this.model.application.get("root"),
                                    this.model.application.get("locale"),
                                    this.model.application.get("app"),
                                    { data: { s: this.model.report.id }});

                this.$el.html(this.compiledTemplate({
                    reportName: reportName,
                    openInText: openInView === 'pivot' ? _('Open in Pivot').t() : _('Open in Search').t(),
                    reportLink: reportLink,
                    link: openLink,
                    app: this.model.report.entry.acl.get('app'),
                    owner: this.model.report.entry.acl.get('owner'),
                    index: this.options.index
                }));
                this.updateSharing();
                this.$('.actions-edit').append(this.editdropdown.render().el);

                return this;
            },
            template: '\
                <td class="expands">\
                    <a href="#"><i class="icon-triangle-right-small"></i></a>\
                </td>\
                <td class="title">\
                    <a href="<%= reportLink %>" title="<%- reportName %>"><%- reportName %></a>\
                </td>\
                <td class="actions">\
                    <a class="openInLink" href="<%= link %>"><%- openInText %></a>\
                </td>\
                <td class="actions actions-edit"></td>\
                <td class="owner"><%- owner %></td>\
                <td class="app"><%- app %></td>\
                <td class="sharing"></td>\
            '
        });
    }
);

