define(
    [
        'underscore',
        'jquery',
        'backbone',
        'module',
        'keyboard/SearchModifier',
        'views/shared/SearchJobRemoteRender',
        'splunk.util',
        'views/shared/fieldworkflowaction/Master',
        'views/shared/EventWorkflowAction',
        'views/shared/delegates/Popdown',
        'collections/UIWorkflowActions',
        'models/services/search/IntentionsParser'
	],
    function(_, $, Backbone, module, KeyboardSearchModifier, SearchJobRemoteRender, splunk_util, FieldActions, EventActions, Popdown, WorkFlowActions, IntentionsParser){
        var EventsViewer = SearchJobRemoteRender.extend({
            tagName: 'ul',
            moduleId: module.id,
            initialize: function(options) {
                SearchJobRemoteRender.prototype.initialize.call(this, arguments);
                this.keyboardSearchModifier = new KeyboardSearchModifier();
                this.options.url =  '/en-US/module/system/Splunk.Module.EventsViewer/render';
                this.model.intentionsParser = new IntentionsParser();
                this.model.workflowActions = new Backbone.Model();
                
                this.model.state.on('change:display.events.fields change:display.prefs.events.offset change:display.prefs.events.count change:display.events.rowNumbers change:display.events.list.maxLines change:display.events.list.drilldown change:display.events.timelineEarliestTime change:display.events.timelineLatestTime', this.debouncedRender, this);
                this.model.state.on('change:display.events.wrap', this.wrap, this);
                this.collection.on('reset', this.render, this);
                this.model.intentionsParser.on('change', function() {
                    this.model.state.set('search', this.model.intentionsParser.fullSearch());
                }, this);

                this.children.fieldActions = new FieldActions({
                    model: {
                        workflowActions: this.model.workflowActions,
                        application: this.model.application
                    },
                    collection: new WorkFlowActions()
                });
    
                this.children.eventActions = new EventActions({
                    model: this.model.workflowActions,
                    collection: new WorkFlowActions()
                });

                this.fieldActionsPopdown = new Popdown({
                    el: this.el,
                    toggle: '.fm',
                    dialog: '.field-actions'
                });
                this.eventActionsPopdown = new Popdown({
                    el: this.el,
                    toggle: '.actions',
                    dialog: '.event-actions'
                });
            },
            postRender: function(){
                this.$el.append(this.children.fieldActions.render().el);
                this.$el.append(this.children.eventActions.render().el);
            },
            wrap: function(){
                var className = 'wrap';
                if (this.model.state.get('display.events.wrap') === "1") {
                    this.$el.addClass(className);
                } else {
                    this.$el.removeClass(className);
                }
            },
            events: {
                'mouseover .t': function(e) {
                    var $element = this.getSegmentParent(e.target);
                    $element.addClass('h');
                },
                'click .t': function(e) {
                    var segment = this.getSegmentParent(e.target).text();
                    this.model.intentionsParser.clear({silent: true});
                    this.model.intentionsParser.fetch({
                        data: {
                            q: this.keyboardSearchModifier.isReplacement(e) ? '*' : this.model.state.get('search'), //revert to search job
                            action: 'addterm',
                            value: segment,
                            negate: this.keyboardSearchModifier.isNegation(e),
                            app: this.model.application.get('app'),
                            owner: this.model.application.get('owner')
                        }
                    });
                    return false;
                },
                'mouseout .t': function(e) {
                    var $element = this.getSegmentParent(e.target);
                    $element.removeClass('h');
                },
                'click .actions': function(e) {
                    this.children.eventActions.collection.reset();
                    this.children.eventActions.collection.fetch({ data: this.workflowParams() });
                    e.preventDefault();
                },
                'click .fm': function(e) {
                    var fieldData = this.extractAndSetFieldData($(e.target)),
                    data = _.extend(fieldData, this.workflowParams());
                    this.children.fieldActions.collection.reset();
                    this.children.fieldActions.collection.fetch({ data: data });
                    e.preventDefault();
                },
                'click .v': function(e) {
                    var $el = $(e.target),
                        field = $el.siblings('.k:first').text(),
                        value = $el.text();
                    this.model.intentionsParser.clear({silent: true});
                    this.model.intentionsParser.fetch({
                        data: {
                            q: this.keyboardSearchModifier.isReplacement(e) ? '*' : this.model.state.get('search'), //revert to search job
                            action: 'fieldvalue',
                            field: field,
                            value: value,
                            negate: this.keyboardSearchModifier.isNegation(e)
                        }
                    });
                },
                'click a.jsexpand': function(e) {
                    var $activator = $(e.target),
                        $sibling = $activator.next();
                    if (!$sibling.length) {
                        e.preventDefault();
                        return;
                    }
                    if ($sibling.css('display')=='none') {
                        $sibling.css('display', 'inline');
                        $activator.html('[-]');
                    } else {
                        $sibling.css('display', 'none');
                        $activator.html('[+]');
                    }
                    e.preventDefault();
                 },
                 'click a.showinline': function(e) {
                     var $activator = $(e.target);
                     var $parent = $activator.closest('li.item');
                     $activator.html('Getting search results...');
                     $.ajax({
                         url: $activator.attr('href'),
                         complete: function(data, textStatus){
                             if (data.status==200) {
                                 $parent.html(data.responseText);
                             } else {
                                 $activator.html('Could not retrieve updated lines, try again?');
                             }
                         }
                     });
                     e.preventDefault();
                 }
            },
            //remap legacy class names
            extractAndSetFieldData: function($element){
                var data = {},
                map = { k: 'field_name', v: 'field_value'};
                $element.parent().children().each(function(i, child){
                    if(map.hasOwnProperty($(child).attr('class'))){
                        data[map[$(child).attr('class')]] =  $(child).text();
                    }
                });
                this.model.workflowActions.set({
                    fieldName: data.field_name,
                    fieldValue: data.field_value,
                    nameAndValue: data.field_name + '=' + data.field_value
                });
                return data;
            },
            workflowParams: function() {
                var params = {};
                return $.extend(params, {
                    offset: this.model.state.get('display.prefs.events.offset') || 0,
                    entity_name: 'events',
                    sid: this.model.searchJob.id,
                    client_app: 'search',
                    max_lines: this.model.state.get('display.events.list.maxLines') || 10
                });
            },
            params: function() {
                var maxLines = this.model.state.get('display.events.list.maxLines') || 10;
                var data = {
                        field_list: splunk_util.fieldListToString(this.collection.pluck("name")),
                        offset: this.model.state.get('display.prefs.events.offset') || 0,
                        count: this.model.state.get('display.prefs.events.count') || 20,
                        display_row_numbers: this.model.state.get('display.events.rowNumbers'),
                        entity_name: 'events',
                        enable_event_actions: 1,
                        enable_field_actions: 1,
                        segmentation: this.model.state.get('display.events.list.drilldown') || 'full',
                        sid: this.model.searchJob.id,
                        min_lines: maxLines,
                        max_lines: maxLines,
                        max_lines_constraint: 500,
                        client_app: 'search'
                };
                var earliest_time = this.model.state.get('display.events.timelineEarliestTime'),
                    latest_time = this.model.state.get('display.events.timelineLatestTime');

                if (this.model.searchJob.entry.content.get('isRealTimeSearch')) {
                    data.offset = -(data.offset) - data.count;
                    data.reverse_order = 1;
                }

                if (earliest_time) {
                    data.earliest_time = earliest_time;
                }
                if (latest_time) {
                    data.latest_time = latest_time;
                }

                return data;
            },
            render: function() {
                this.children.fieldActions.$el.detach();
                this.children.eventActions.$el.detach();
                this.wrap(); // apply wrapping
                return SearchJobRemoteRender.prototype.render.call(this, arguments);
            },
            getSegmentParent: function(element){
                var parent = element.parentNode;
                if (parent.childNodes[parent.childNodes.length-1]==element && $(parent).hasClass('t')) {
                    element = parent;
                }
                return $(element);
            }
        },
        {
            TERM_FILTER_EVENT: "1"
        });

        return EventsViewer;
    }
);
