define(
    [
        'jquery',
        'underscore',
        'module',
        'views/Base'
    ],
    function($, _, module, BaseView) {
        return BaseView.extend({
            moduleId: module.id,
            className: "json-tree",
            /**
             * @param {Object} options {
             *     json: <json stringified object>
             *     isValidJSON: <a flag allowing opt-out of safe set json routine>
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                
                var defaults = {
                        isValidJSON: false,
                        json: '{}'
                };
                this.options = $.extend(true, defaults, this.options);
                
                this.setJSON(this.options.json, this.options.isValidJSON);
            },
            setJSON: function(json, optout) {
                var parsed;
                if(optout) {
                    return !(this._json = json);
                }

                if(typeof json !== 'string') {
                    json = JSON.stringify(json);
                } 

                try{
                    parsed = JSON.parse(json);
                } catch(e) {}
                
                this._json = parsed;
            }, 
            isJSON: function() {
                return !!this._json;
            },
            events: {
                'click .jsexpands': function(e) {
                    var $target = $(e.currentTarget);
                    this.trigger('interaction');
                    this.interacted = true;
                    $target.removeClass('jsexpands').addClass('jscollapse').text('[-]');
                    $target.next().show();
                    e.preventDefault();
                },
                'click .jscollapse': function(e) {
                    var $target = $(e.currentTarget);
                    this.trigger('interaction');
                    this.interacted = true;
                    $target.removeClass('jscollapse').addClass('jsexpands').text('[+]');
                    $target.next().hide();
                    e.preventDefault();
                }
            },
            printJSON: function(obj, indent, view) {
                var newIndent   = indent + 2,
                    returnValue = [],
                    level       = newIndent/2,
                    type        = typeof obj;
                
                if(!_.isObject(obj)) {   
                    return [view._type({ type: type, obj: obj})];               
                } else {
                    var isObj = !(obj instanceof Array),
                        list  = (isObj) ? _.keys(obj).sort() : obj.sort();

                    returnValue.push(view._bracket({isObj: isObj})); 
                    (list.length>0) && returnValue.push(view._state({expands: !(level<2), level: level }).trim());          
                    returnValue.push((level!==1)? '<span style="display: none;">': '<span>');
                    
                    for(var i=0; i<list.length; i++){
                        returnValue.push(view._indent({newIndent: newIndent}));
                        (isObj) ?
                            returnValue.push(
                                view._key({ 
                                    val: list[i], 
                                    level:level,
                                    pjson: view.printJSON,
                                    obj: obj,
                                    nIndent: newIndent,
                                    view: view
                                })):
                            returnValue.push(view.printJSON(list[i], newIndent, view));
                    }
                    returnValue.push(view._closeIndent({closingIndent: newIndent-1}), view._closeBracket({isObj: isObj}));
                }
                return _.flatten(returnValue).join('');                
            },
            render: function() {
                if(!this.interacted) {
                   this.$el.html(this.printJSON(this._json, 0, this));
                }
                return this;
            },
            _type: _.template('<span class="t <%- type %>"><%- obj %></span>'),
            _bracket: _.template('<span><%if(isObj){%>{<%}else{%>[<%}%></span>'),
            _state: _.template('\
                <% if(expands) { %>\
                    <a class="jsexpands level-<%-level%>">[+]</a>\
                <% } else { %>\
                    <a class="jscollapse level-<%-level%>">[-]</a>\
                <% } %>\
            '),
            _indent: _.template('<br><span><%-Array(newIndent+1).join(" ")%></span>'),
            _key: _.template('<span class="key level-<%-level%>"><span class="key-name"><%-val%></span>:<%= pjson(obj[val], nIndent, view) %></span>'),
            _closeIndent: _.template('</span><br><span class="level-<%-closingIndent%>"><%-Array(closingIndent).join(" ")%>'),
            _closeBracket: _.template('<%if(isObj){ %><span>}</span><%}else{%><span>]</span><%}%></span>')
        });
    }
 );
