define(['underscore', 'views/Base', 'splunk.util'], function(_, Base, splunkutil) {
    var SearchJobRemoteRender = Base.extend({
        initialize: function(options) {
        	Base.prototype.initialize.call(this, arguments);
            this.reset();
            this.options.url = options.url || '';
            this.options.params = options.params || {};
            this.options.retry = options.retry || 3;
            this.mode = options.mode || 'events';//events || results || results_preview
            switch (this.mode) {
                case 'results_preview':
                    this.model.searchJob.entry.content.on('change:resultPreviewCount change:scanCount change:dispatchState', this.debouncedRender, this);
                    break;
                case 'results':
                    this.model.searchJob.entry.content.on('change:resultCount change:scanCount change:dispatchState', this.debouncedRender, this);
                    break;
                case 'events':
                    this.model.searchJob.entry.content.on('change:eventAvailableCount change:scanCount change:dispatchState', this.debouncedRender, this);
                    break;
                default:
                    throw 'invalid mode';
            }
            this.model.searchJob.on('destroy', function() {
                this.$el.empty();
                this.reset();
            }, this);
        },
        reset: function() {
            this.counter = 0;
            this.xhr = null;
        },
        abort: function() {
            if (this.xhr && this.xhr.readyState < 4) {
                this.xhr.abort();
            }
        },
        params: function() {
            return {};
        },
        render: function() {
            if (this.model.searchJob.isNew()) {
                this.$el.html('');
                return this;
            }
            if (this.xhr) {
                if (this.xhr.readyState < 4) {
                    if (this.counter < this.options.retry) {
                        this.counter++;
                        return;
                    } else {
                        this.abort(); 
                        this.reset();
                    }
                } else {
                    this.reset();
                }
            }
            var el = this.$el,
                self = this,
                url = this.options.url + '?' + splunkutil.propToQueryString(this.params()); //in jquery space characters are encoded as '+' rather than the '%20'
            this.xhr = $.ajax({
                url: url, 
                success: function(responseText) {
                    el.html(responseText);
                    self.postRender();
                }
            });
            return this;
        },
        postRender: function() {
            //implement for post render routine
        }
    });
    return SearchJobRemoteRender;
});
