define([
            'jquery',
            'underscore',
            'contrib/text!views/shared/Paginator.html',
            'splunk.paginator',
            'module',
            'views/Base',
            'bootstrap.tooltip'
        ],
        function(
            $,
            _,
            template,
            splunkPaginator,
            module,
            BaseView,
            undefined
        ) {

    /**
     * Paginate twice it's a long way to the bay!
     */
    return BaseView.extend({
        className: 'pagination pull-right',
        template: template,
        moduleId: module.id,
        initialize: function(options) {
            BaseView.prototype.initialize.apply(this, arguments);
            this.maxPerPage = this.options.maxPerPage || 10;
            this.maxPages = this.options.maxPages || 10;
            this.offset = this.options.offset || 0;
            this.offsetKey = this.options.offsetKey || 'offset';
            this.countKey = this.options.countKey || 'count';
            this.timelineEarliestTimeKey = this.options.timelineEarliestTimeKey || 'display.events.timelineEarliestTime';
            this.timelineLatestTimeKey = this.options.timelineLatestTimeKey || 'display.events.timelineLatestTime';
            this.mode = this.options.mode || 'events'; //events || results || results_preview
            this.model.state.on('change:' + this.offsetKey +
                                ' change:' + this.countKey, this.debouncedRender, this);
            switch (this.mode) {
                case 'results_preview':
                    this.model.searchJob.entry.content.on('change:resultPreviewCount', this.render, this);
                    break;
                case 'results':
                    this.model.searchJob.entry.content.on('change:resultCount', this.render, this);
                    break;
                case 'events':
                    (this.collection || this.model.timeline.buckets).on('reset', this.render, this);
                    this.model.state.on('change:' + this.timelineEarliestTimeKey +
                                        ' change:' + this.timelineLatestTimeKey, this.debouncedRender, this);
                    this.model.searchJob.entry.content.on('change:eventAvailableCount change:eventCount', this.render, this);
                    break;
                default:
                    throw 'invalid mode';
            }
        },
        events: {
            'click li:not(.disabled) a': function(e) {
                this.model.state.set(this.offsetKey, $(e.currentTarget).attr('data-offset'));
                e.preventDefault();
            }
        },
        render: function() {
            var length, timelineEarliestTime, timelineLatestTime, truncated;
            if (this.mode==='events') {
                timelineEarliestTime = this.model.state.get(this.timelineEarliestTimeKey);
                timelineLatestTime = this.model.state.get(this.timelineLatestTimeKey);
                if (this.model.searchJob.entry.content.get('statusBuckets') > 0 && timelineEarliestTime && timelineLatestTime) {
                    var ac_info = (this.collection || this.model.timeline).availableCount(timelineEarliestTime, timelineLatestTime);
                    length = ac_info.length;
                    truncated = ac_info.trunc;
                } else {
                    truncated = this.model.searchJob.entry.content.get('eventAvailableCount') < this.model.searchJob.entry.content.get('eventCount');
                    length = this.model.searchJob.entry.content.get('eventAvailableCount');
                }
            }
            if (this.mode==='results') {
                length = this.model.searchJob.entry.content.get('resultCount');
            }
            if (this.mode=='results_preview') {
                length = this.model.searchJob.entry.content.get('resultPreviewCount');
            }
            var options = {
                    max_items_page: this.model.state.get(this.countKey) || this.maxPerPage,
                    max_pages: this.maxPages,
                    item_offset: this.model.state.get(this.offsetKey) || this.offset
                },
                paginator = new splunkPaginator.Google(length || 0, options),
                template = this.compiledTemplate({
                    paginator: paginator,
                    truncated: truncated,
                    _: _
                });
            this.$el.html(template);
            this.$('.max-events-per-bucket span').tooltip({animation: false, title: 'Currently displaying the most recent ' +  length + ' events in the selected range.  Select a narrower range or zoom in to see more events.'});
            return this;
        }
    });

});
