define(
    [
        'jquery',
        'underscore',
        'module',
        'views/Base',
        'splunk.util',
        'uri/route'
    ],
    function($, _, module, Base, splunkUtil, route) {


        var getStringRepresentation = function(label, formatted) {
            if (_.isFunction(label)) {
                try {
                    label = label(formatted);
                } catch (e) {
                    return '';
                }
            }

            return label;
        };

        return Base.extend({
            moduleId: module.id,
            className: "single-value",
            initialize: function(options) {
                Base.prototype.initialize.apply(this, arguments);
                this.model.state.on('change:display.visualizations.singlevalue.beforeLabel change:display.visualizations.singlevalue.afterLabel change:display.visualizations.singlevalue.underLabel change:display.visualizations.singlevalue.additionalClass', this.debouncedRender, this);
                this.model.searchResultsColumn.on('change:fields', this.render, this);
            },
            getResultField: function(field) {
                var columns = $.extend(true, [], this.model.searchResultsColumn.get('columns')),
                    fields  = this.getFieldNames();
                _(columns).each(function(column, i) { column.push(fields[i]); });
                fields = _(fields).filter(function(_field) {
                    return (_field === '_time') || (_field === '_raw') || (_field.charAt(0) != '_');
                });
                columns = _(columns).filter(function(column) {
                    return (_.indexOf(fields, column[column.length-1]) >= 0);
                });
                if(!field) {
                    // If the result field is unspecified - use the first of the filtered fields
                    field = fields[0];
                    if (columns.length && columns[0].length){
                        return columns[0][0];
                    } else {
                        return _('N/A').t();
                    }
                }
                for (var i = 0; i < columns.length; i++) {
                    if (columns[i][columns[i].length - 1]===field){
                        return columns[i][0];
                    }
                }
                return _('N/A').t();
            },
            // fields can either be a list of strings or a list of dictionaries each with a 'name' entry
            // depending on whether 'show_metadata' is enabled
            getFieldNames: function() {
                var fields = this.model.searchResultsColumn.get('fields');
                if(!fields || fields.length === 0) {
                    return [];
                }
                if(_.isObject(fields[0])) {
                    return _(fields).pluck('name');
                }
                return $.extend([], fields);
            },
            setSeverity: function() {
                if(this.model.searchResultsColumn.get('fields')) {
                    var fields = _(this.model.searchResultsColumn.get('fields')).pluck('name'),
                        i = fields.indexOf('range');
                    if(i > -1) {
                        this.$('span.single-result').addClass(this.model.searchResultsColumn.get('columns')[i][0]);
                    }
                }
            },
            render: function() {
                var extracted = this.getResultField(this.model.state.get("display.visualizations.singlevalue.field")),
                    html = this.compiledTemplate({
                        extracted: extracted,
                        beforeLabel: getStringRepresentation(this.model.state.get("display.visualizations.singlevalue.beforeLabel") || "", extracted),
                        afterLabel: getStringRepresentation(this.model.state.get("display.visualizations.singlevalue.afterLabel") || "", extracted),
                        underLabel: getStringRepresentation(this.model.state.get("display.visualizations.singlevalue.underLabel") || "", extracted)
                    }),
                    cls = [this.model.state.get('display.visualizations.singlevalue.additionalClass'),
                           this.getResultField(this.model.state.get('display.visualizations.singlevalue.classField'))];
                this.$el.addClass(cls.join(' '));
                this.$el.html(html);
                this.wrapLinks();
                this.setSeverity();
                return this;
            },
            wrapLinks: function(){
                var linkFields = this.model.state.get('display.visualizations.singlevalue.linkFields') || "",
                    linkView = this.model.state.get('display.visualizations.singlevalue.linkView'),
                    linkSearch = this.model.state.get('display.visualizations.singlevalue.linkSearch'),
                    link, url;
                if(linkView && linkSearch) {
                    // For links like "/app/foo/bar" generate the absolute URL
                    if(linkView.charAt(0) === '/') {
                        url = splunkUtil.make_full_url(linkView, { q: linkSearch });
                    } else {
                        // Treat anything else as a simple view name
                        if(this.model.application) {
                            // If the application model is there, use uri.route to create an absolute URL to the target view
                            url = route.page(this.model.application.get('root'), this.model.application.get('locale'), this.model.application.get('app'), linkView, {
                                data: {
                                    q: linkSearch
                                }
                            });
                        } else {
                            // Fallback: Create a relative link to the target view
                            url = [ linkView, splunkUtil.propToQueryString({ q: linkSearch }) ].join('?');
                        }
                    }
                    link = $('<a />').attr('href',url);
                    _.each({'beforelabel': 'before-label', 'afterlabel': 'after-label', 'underlabel': 'under-label', 'result': 'single-result'}, function(cls, key){
                        if (linkFields.indexOf(key) > -1 && linkView){
                            this.$('.'+cls).wrap(link);
                        }
                    }, this);
                }
            },
            template: '\
                <span class="before-label">\
                    <%- beforeLabel %>\
                </span>\
                <span class="single-result">\
                    <%- extracted %>\
                </span>\
                <span class="after-label">\
                    <%- afterLabel %>\
                </span>\
                <br>\
                <span class="under-label">\
                    <%- underLabel %>\
                </span>\
            '
        });
    }
 );
