define(
    [
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/controls/ControlGroup',
        'splunk.util',
        'uri/route'
    ],
    function(_, Backbone, module, Base, ControlGroup, splunkUtil, route) {
        return Base.extend({
            moduleId: module.id,
            className: 'enable-actions-view',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.children.showTriggeredAlerts = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticCheckbox',
                    controlOptions: {
                        modelAttribute: 'ui.showtriggeredalerts',
                        model: this.model.alert
                    },
                    label: _('List in Triggered Alerts').t(),
                    help: _('Triggered Alerts is available in the activity menu.').t()
                });
                this.children.severity = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticSelect',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.severity',
                        model: this.model.alert,
                        items: [
                            { label: _('Info').t(), value: 1 },
                            { label: _('Low').t(), value: 2 },
                            { label: _('Medium').t(), value: 3 },
                            { label: _('High').t(), value: 4 },
                            { label: _('Critical').t(), value: 5 }
                        ],
                        toggleClassName: 'btn',
                        labelPosition: 'outside',
                        elastic: true,
                        popdownOptions: {
                            attachDialogTo: '.modal:visible',
                            scrollContainer: '.modal:visible .modal-body:visible'
                        }
                    },
                    label: _('Severity').t()
                });
                var configEmailHelpLink = route.docHelp(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        'learnmore.alert.email'
                );

                this.children.sendEmail = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticCheckbox',
                    controlOptions: {
                        modelAttribute: 'ui.email',
                        model: this.model.alert
                    },
                    label: _('Send Email').t(),
                    help: splunkUtil.sprintf(_('Email must be configured in System&nbsp;Settings > Alert&nbsp;Email&nbsp;Settings. %s').t(), ' <a href="' + configEmailHelpLink + '" target="_blank">' + _("Learn More").t() + ' <i class="icon-external"></i></a>')
                });
                this.children.emailSubject = new ControlGroup({
                    className: 'alert-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.emailsubject',
                        model: this.model.alert,
                        placeholder: _('Splunk Alert: $name$').t()
                    },
                    label: _('Subject').t()
                });
                 this.children.emailAddresses = new ControlGroup({
                    className: 'alert-name control-group',
                    controlType: 'Textarea',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.emailaddresses',
                        model: this.model.alert
                    },
                    label: _('Email addresses').t(),
                    help: _('Comma separated list.').t()
                });

                 var formatItems = [
                    {
                        label: _('None').t(),
                        value: 'none'
                    },
                    {
                        label: _('Inline').t(),
                        value: 'inline'
                    },
                    {
                        label: _('CSV').t(),
                        value: 'csv'
                    }
                ];

                if (this.options.pdfAvailable) {
                    formatItems.push({
                        label: _('PDF').t(),
                        value: 'pdf'
                    });
                }

                this.children.resultsFormat = new ControlGroup({
                    className: 'alert-type control-group',
                    controlType: 'SyntheticRadio',
                    controlClass: formatItems.length == 4 ?'controls-quarterblock' : 'controls-thirdblock' ,
                    controlOptions: {
                        modelAttribute: 'ui.emailattachmentformat',
                        model: this.model.alert,
                        items: formatItems
                    },
                    label: _('Include results').t()
                });

                this.children.runScript = new ControlGroup({
                    className: 'control-group',
                    controlType: 'SyntheticCheckbox',
                    controlOptions: {
                        modelAttribute: 'ui.script',
                        model: this.model.alert
                    },
                    label: _('Run a Script').t()
                });
                this.children.scriptName = new ControlGroup({
                    className: 'alert-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.scriptfilename',
                        model: this.model.alert
                    },
                    label: _('Filename').t(),
                    help: _('Located in $SPLUNK_HOME/bin/scripts').t()
                });

                this.model.alert.on('change:ui.showtriggeredalerts', function(){
                    this.toggleShowTriggeredAlerts();
                },this);

                this.model.alert.on('change:ui.email', function(){
                    this.toggleEmail();
                },this);

                this.model.alert.on('change:ui.script', function(){
                    this.toggleScript();
                },this);
            },
            toggleShowTriggeredAlerts: function() {
                if (this.model.alert.get('ui.showtriggeredalerts')) {
                    this.children.severity.$el.show();
                    this.$el.find('.alert-options').addClass('outline');
                } else {
                    this.children.severity.$el.hide();
                    this.$el.find('.alert-options').removeClass('outline');
                }
            },
            toggleEmail: function() {
                if (this.model.alert.get('ui.email')){
                    this.children.emailSubject.$el.show();
                    this.children.emailAddresses.$el.show();
                    this.children.resultsFormat.$el.show();
                    this.$el.find('.email-options').addClass('outline');

                } else {
                    this.children.emailSubject.$el.hide();
                    this.children.emailAddresses.$el.hide();
                    this.children.resultsFormat.$el.hide();
                    this.$el.find('.email-options').removeClass('outline');
                }
            },
            toggleScript: function() {
                if (this.model.alert.get('ui.script')) {
                    this.children.scriptName.$el.show();
                    this.$el.find('.script-options').addClass('outline');
                } else {
                    this.children.scriptName.$el.hide();
                    this.$el.find('.script-options').removeClass('outline');
                }
            },
            toggleAll: function() {
                this.toggleShowTriggeredAlerts();
                this.toggleEmail();
                this.toggleScript();
            },
            render: function()  {
                // add some structure
                this.$el.append('<fieldset class="alert-options"></fieldset>');
                this.$el.find('.alert-options').append(this.children.showTriggeredAlerts.render().el);
                this.$el.find('.alert-options').append(this.children.severity.render().el);

                this.$el.append('<fieldset class="email-options"></fieldset>');
                this.$el.find('.email-options').append(this.children.sendEmail.render().el);
                this.$el.find('.email-options').append(this.children.emailSubject.render().el);
                this.$el.find('.email-options').append(this.children.emailAddresses.render().el);
                this.$el.find('.email-options').append(this.children.resultsFormat.render().el);

                this.$el.append('<fieldset class="script-options"></fieldset>');
                this.$el.find('.script-options').append(this.children.runScript.render().el);
                this.$el.find('.script-options').append(this.children.scriptName.render().el);

                this.children.severity.$el.hide();
                this.children.emailSubject.$el.hide();
                this.children.emailAddresses.$el.hide();
                this.children.resultsFormat.$el.hide();
                this.children.scriptName.$el.hide();

                this.toggleAll();
                return this;
            }
        });
});
