define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/Modal',
        'views/shared/controls/ControlGroup',
        'views/shared/FlashMessages',
        'views/shared/alertcontrols/dialogs/shared/actions/EnableActions',
        'views/shared/alertcontrols/dialogs/shared/actions/ActionsOptions',
        'util/splunkd_utils',
        'util/pdf_utils'
     ],
     function(
        _,
        module,
        BaseView,
        Modal,
        ControlGroup,
        FlashMessagesView,
        EnableActions,
        ActionsOptions,
        splunkd_utils,
        pdfUtils
    ){
        return BaseView.extend({
            moduleId: module.id,
             /**
             * @param {Object} options {
             *     model: {
             *         alert: <models.Alert>,
             *         inmem: <models.Report>,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>
             *     },
             *     {string} dialogType: edit|new  and indicates if it is a save or edit dialog.
             *     {string} dialogTitle: Text used for the dialogs title.
             *     {boolean} mainScreen (Optional): Whether this view will be the first or only screen in a dialog. Default false.
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                
                //deferrs
                this.deferredPdfAvailable = pdfUtils.isPdfServiceAvailable();
                
                this.options.mainScreen = this.options.mainScreen || false;

                if (this.options.mainScreen) {
                    this.model.alert.set('ui.step', 'actions');
                }

                this.children.flashMessages = new FlashMessagesView({ model: this.model.alert});

                this.children.name = new ControlGroup({
                    className: 'alert-name control-group',
                    controlType: 'Label',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'ui.name',
                        model: this.model.alert,
                        save: false
                    },
                    label: _('Alert').t()
                });

                $.when(this.deferredPdfAvailable).then(function(pdfAvailable) {
                    this.children.enableActions = new EnableActions({
                        model: {
                            alert: this.model.alert,
                            application: this.model.application,
                            appLocal: this.model.appLocal
                        },
                        pdfAvailable: pdfAvailable
                    });

                }.bind(this));

                this.children.actionOptions = new ActionsOptions({ model: this.model.alert });
                this.children.sharing = new ControlGroup({
                    className: 'alert-type control-group',
                    controlType: 'SyntheticRadio',
                    controlOptions: {
                        modelAttribute: 'ui.permissions',
                        model: this.model.alert,
                        items: [
                            { label: _('Private').t(), value: splunkd_utils.USER },
                            { label: _('Shared in App').t(), value: splunkd_utils.APP }
                        ],
                        save: false
                    },
                    label: _('Permissions').t()
                });
            },
            events: {
                'click .save' : function(e) {
                    this.model.alert.entry.content.trigger('save');
                    e.preventDefault();
                },
                'click .back' : function(e) {
                    this.model.alert.entry.content.trigger('back');
                    e.preventDefault();
                }
            },
            render: function() {
                $.when(this.deferredPdfAvailable).then(function() {
                    this.$el.html(Modal.TEMPLATE);
                    this.$(Modal.HEADER_TITLE_SELECTOR).html(this.options.dialogTitle);
                    this.$(Modal.BODY_SELECTOR).addClass('modal-body-scrolling');
                    this.$(Modal.BODY_SELECTOR).append(Modal.FORM_HORIZONTAL_COMPLEX);

                    this.$(Modal.BODY_FORM_SELECTOR).append(this.children.flashMessages.render().el);

                    if (this.options.mainScreen) {
                        this.$(Modal.BODY_FORM_SELECTOR).append(this.children.name.render().el);
                    }

                    this.$(Modal.BODY_FORM_SELECTOR).append('<p class="control-heading">' + _("Enable Actions").t() + '</p>');
                    this.$(Modal.BODY_FORM_SELECTOR).append(this.children.enableActions.render().el);
                    this.$(Modal.BODY_FORM_SELECTOR).append('<p class="control-heading">' + _("Action Options").t() + '</p>');
                    this.$(Modal.BODY_FORM_SELECTOR).append(this.children.actionOptions.render().el);

                    if (this.options.dialogType === 'new' && this.model.inmem.entry.acl.get("can_share_app")) {
                        this.$(Modal.BODY_FORM_SELECTOR).append('<p class="control-heading">' + _("Sharing").t() + '</p>');
                        this.$(Modal.BODY_FORM_SELECTOR).append(this.children.sharing.render().el);
                    }

                    this.$(Modal.BODY_FORM_SELECTOR).show();
                    this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);

                    if (!this.options.mainScreen) {
                        this.$(Modal.FOOTER_SELECTOR).append('<a href="#" class="btn back">' + _("Back").t() + '</a>');
                    }

                    this.$(Modal.FOOTER_SELECTOR).append($(Modal.BUTTON_SAVE).addClass('save'));
                    return this;
                }.bind(this));
            }
        }
    );
});
