define([
    'jquery',
    'underscore',
    'backbone',
    'module',
    'views/Base',
    'views/shared/appbar/NavItem',
    'views/shared/appbar/AppNav',
    'views/shared/appbar/AppLabel',
    'models/Application',
    'models/services/data/ui/Nav',
    'models/services/AppLocal',
    'collections/services/data/ui/Views',
    'collections/services/SavedSearches',
    'contrib/text!views/shared/appbar/Master.html',
    'helpers/AppNav',
    'uri/route',
    'util/color_utils'
],
function(
    $,
    _,
    Backbone,
    module,
    BaseView,
    NavItem,
    AppNavView,
    AppLabelView,
    ApplicationModel,
    NavModel,
    AppModel,
    ViewsCollection,
    SavedSearchesCollection,
    templateMaster,
    appNavParser,
    route,
    color_utils
){
    var View = BaseView.extend({
        moduleId: module.id,
        className: 'app-bar',
        initialize: function() {
            BaseView.prototype.initialize.apply(this, arguments);

            this.children.appNav = new AppNavView({
                model: this.model,
                collection: this.collection
            });

            this.children.appLabel = new AppLabelView({
                model: this.model,
                collection: this.collection
            });

            this.setBannerColor();
            this.model.appNav.on('change:color', this.setBannerColor, this);

            this.debouncedRender();
        },
        render: function() {
            var html = _.template(templateMaster);
            this.$el.html(html);
            this.$el.find('.nav').html(this.children.appNav.el);
            this.$el.find('.app-name').html(this.children.appLabel.render().el);

            return this;
        },
        setBannerColor: function(){
            if(!this.model.appNav){return false;}

            var navColor = this.model.appNav.get('color');
            if(!navColor){return false;}

            this.$el.css('background-color', navColor);

            var navColorNorm = color_utils.normalizeHexString(navColor);
            var darkerColorEnd = color_utils.modifyLuminosityOfHexString(navColor, View.END_GRADIENT_LUMINOSITY);

            var gradients = color_utils.generateGradientStyles(navColorNorm, darkerColorEnd);

            for (var i=0;i<gradients.length;i++){
                this.$el.css('background-image', gradients[i]);
                // FIXME: ghetto hack for IE since it uses filter instead of bg-img
                // BG filters cause clipping issues so removing gradients for IE < 9
                // this.$el.css('filter', gradients[i]);
            }
        }
    },
    {
        END_GRADIENT_LUMINOSITY: 0.90,
        createWithAppNavUrl: function(options){
            var self = this;
            var url = route.appNavUrl(options.model.application.get('root'), options.model.application.get('locale'), options.model.application.get('app'));
            $.ajax({
                url: url,
                dataType:'json'
            }).done(function(data){
                var appLink, appIcon, appLogo, reportRoute;
                appLink = route.page(
                    options.model.application.get('root'),
                    options.model.application.get('locale') || '',
                    options.model.application.get('app')
                );

                appIcon = route.appIcon(
                    options.model.application.get('root'),
                    options.model.application.get('locale') || '',
                    options.model.application.get('owner'),
                    options.model.application.get('app')
                );

                appLogo = route.appLogo(
                    options.model.application.get('root'),
                    options.model.application.get('locale') || '',
                    options.model.application.get('owner'),
                    options.model.application.get('app')
                );

                reportRoute = route.page(
                    options.model.application.get('root'),
                    options.model.application.get('locale') || '',
                    options.model.application.get('app'),
                    'report');

                options.model.appNav.set({
                    nav: data.nav,
                    color: data.color,
                    label: data.label,
                    icon: appIcon,
                    logo: appLogo,
                    link: appLink,
                    defaultView: data.defaultView
                    //searchView: 'not implemented TODO'
                });
            }).fail(function(){
                self.createWithBackbone(options);
            });
        },
        createWithAppNavModel: function(options) {
            var appLink, appIcon, appLogo, reportRoute;

            appLink = route.page(
                options.model.application.get('root'),
                options.model.application.get('locale') || '',
                options.model.application.get('app')
            );

            appIcon = route.appIcon(
                options.model.application.get('root'),
                options.model.application.get('locale') || '',
                options.model.application.get('owner'),
                options.model.application.get('app')
            );

            appLogo = route.appLogo(
                options.model.application.get('root'),
                options.model.application.get('locale') || '',
                options.model.application.get('owner'),
                options.model.application.get('app')
            );

            reportRoute = route.page(
                options.model.application.get('root'),
                options.model.application.get('locale') || '',
                options.model.application.get('app'),
                'report');

            options.model.appNav.set({
                nav: options.model.appNav.entry.content.get('nav'),
                color: options.model.appNav.entry.content.get('color'),
                label: options.model.appNav.entry.content.get('label'),
                icon: appIcon,
                logo: appLogo,
                link: appLink,
                defaultView: options.model.appNav.entry.content.get('defaultView')
                //searchView: 'not implemented TODO'
            });
        },
        createWithBackbone: function(options){
            var applicationDfd = $.Deferred();
            var appNavDfd = $.Deferred();
            var viewsDfd = $.Deferred();
            var savedSearchesDfd = $.Deferred();
            function updateNavData(){
                if(appNavDfd.state() === 'resolved' && viewsDfd.state() === 'resolved' && savedSearchesDfd.state() === 'resolved'){
                    var data = appNavParser.parseNavModel(options.model.splunkDappNav, options.collection.views, options.collection.savedSearches, options.model.application.get('root'));

                    var appLink, appIcon, appLabel, appLogo;
                    appLink = route.page(
                        options.model.application.get('root'),
                        options.model.application.get('locale') || '',
                        options.model.application.get('app')
                    );

                    appIcon = route.appIcon(
                        options.model.application.get('root'),
                        options.model.application.get('locale') || '',
                        options.model.application.get('owner'),
                        options.model.application.get('app')
                    );

                    appLogo = route.appLogo(
                        options.model.application.get('root'),
                        options.model.application.get('locale') || '',
                        options.model.application.get('owner'),
                        options.model.application.get('app')
                    );

                    if(options.model.app && options.model.app.entry && options.model.app.entry.content && options.model.app.entry.content.get('label')){
                        appLabel = options.model.app.entry.content.get('label') || '';
                    }

                    options.model.appNav.set({
                        nav: data.nav,
                        color: data.color,
                        label: appLabel,
                        icon: appIcon,
                        link: appLink,
                        logo: appLogo,
                        searchView: data.searchView
                        //defaultView: 'not implemented TODO'
                    });
                }
            }

            var app = options.model.application.get('app');
            var owner = options.model.application.get('owner');
            if(app && owner){
                applicationDfd.resolve(app, owner);
            }

            options.model.application.on('change', function(){
                var app = options.model.application.get('app');
                var owner = options.model.application.get('owner');
                if(app && owner){
                    applicationDfd.resolve(app, owner);
                }
            });

            if(!options.model.app){
                options.model.app = new AppModel();
                applicationDfd.done(function(app, owner){
                    options.model.app.set({id:'apps/local/' + app});
                    options.model.app.fetch({data: {app: app, owner: owner}});
                });
            }

            if(!options.collection.views){
                options.collection.views = new ViewsCollection();
                applicationDfd.done(function(app, owner){
                    options.collection.views.fetch({data: {app: app, owner: owner, count: -1, digest: 1}});
                });
            }

            if(!options.collection.savedSearches){
                options.collection.savedSearches = new SavedSearchesCollection();
                applicationDfd.done(function(app, owner){
                    options.collection.savedSearches.fetch({data: {app: app, owner: '-', search:'is_visible=1 AND disabled=0', count:-1}}); //TODO: _with_new=1?
                });
            }

            if(!options.model.splunkDappNav){
                options.model.splunkDappNav = new NavModel();
                applicationDfd.done(function(app, owner){
                    options.model.splunkDappNav.fetch({data: {app: app, owner: owner}});
                });
            }

            options.collection.views.on('reset', function(){
                viewsDfd.resolve();
                updateNavData();
            });

            options.collection.savedSearches.on('reset', function(){
                savedSearchesDfd.resolve();
                updateNavData();
            });

            options.model.splunkDappNav.on('change', function(){
                appNavDfd.resolve();
                updateNavData();
            });

        },
        create: function(options){
            options = options || {};
            options.collection = options.collection || {};
            options.model = options.model || {};
            if (options.model.appNav) {
                this.createWithAppNavModel(options);
            } else {
                options.model.appNav = new (Backbone.Model.extend())();
                if(options.appServerUrl){
                    this.createWithAppNavUrl(options);
                }else{
                    this.createWithBackbone(options);
                }
            }
            return new View(options);
        }
    });
    return View;
});
