/**
 *   views/shared/delegates/Base
 *
 *   Desc:
 *     This a base class for other delegates.

 *   @param {Object} (Optional) options An optional object literal having one settings.
 *
 *    Usage:
 *       var p = new DelegateBase({el: {el}})
 *
 *    Options:
 *        el (required): The event delegate
 */


define([
    'backbone'
],function(
    Backbone
){
    return Backbone.View.extend({
        dispose: function() {
            this.stopListening();
            return this;
        },
        
        // need to override remove, remove should only operate on DOM elements that the view owns,
        // delegate views do not own their root el so letting the superclass get called is not correct
        remove: function() {
            this.dispose();
            return this;
        }
    });
});