define([
    'jquery',
    'underscore',
    'views/shared/delegates/Base'
],function(
    $,
    _,
    DelegateBase
    
){
    return DelegateBase.extend({
      awake: true,
      initialize: function(options) {
        var defaults = {
          offset: 0,
          affix: '> div',
          affixTopClassName: 'affix-top' 
        };
        _.defaults(options, defaults);
        DelegateBase.prototype.initialize.call(arguments);
        
        this.$container = this.$el.parent();
        this.$window = $(window);
        this.windowHeight = this.$window.height();
        
        this.bindListeners();
        this.checkPosition();
      },
      bindListeners: function () {
          this.$window 
            .on('scroll.affix.' + this.cid, this.checkPosition.bind(this))
            .on('resize.affix.' + this.cid, this.resizeWindow.bind(this))
            .on('click.affix.' + this.cid,  function() {
                _.defer(this.checkPosition.bind(this));
            }.bind(this));
      },
      unbindListeners: function () {
          this.$window.off('.' + this.cid);
      },
      resizeWindow:function() {
          this.windowHeight = this.$window.height();
          this.checkPosition();
      },
      checkPosition: function () {
        if (!this.$el.is(':visible')) return;
    
        var scrollTop = this.$window.scrollTop(),
            containerTop = this.$el.offset().top,
            containerHeight = this.$el.height(),
            affix;
    
        affix = scrollTop > containerTop - this.options.offset;
        
        if (this.affixed === affix) return;
    
        this.affixed = affix;
        this.$(this.options.affix)[affix ? 'addClass' :  'removeClass'](this.options.affixTopClassName);
      },
      disable: function() {
        this.$(this.options.affix).addClass('disabled');
      },
      enable: function() {
        this.$(this.options.affix).removeClass('disabled');
      },
      wake: function() {
          this.awake = true;
          this.bindListeners();
          
          this.resizeWindow(); //also calls checkPosition();
          return this;
      },
      sleep: function() {
          this.awake = false;
          this.unbindListeners();
          return this;
      },
      remove: function() {
          $(window).off('scroll.affix.' + this.cid);
          $(window).off('resize.affix.' + this.cid);
          $(window).off('click.affix.' + this.cid);
          return this;
      }
    });
});
