define(
    [
        'jquery',
        'underscore',
        'module',
        'models/Base',
        'models/services/search/jobs/Result',
        'views/Base',
        'views/shared/eventsviewer/raw/Master',
        'views/shared/eventsviewer/list/Master',
        'views/shared/eventsviewer/table/Master'
    ],
    function($, _, module, BaseModel, ResultModel, BaseView, EventsRawView, EventsListView, EventsTableView){
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.jobs.ResultsV2>,
             *         summary: <model.services.search.jobs.SummaryV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>,
             *         state: <models.BaseV2> (optional)
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>,
             *     },
             *     selectableFields: true|false,
             *     headerMode: dock|none (eventually this will have static mode),
             *     allowRowExpand: true|false
             * }
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                var defaults = {
                        selectableFields: true,
                        headerMode: 'dock',
                        allowRowExpand: true,
                        scrollToTopOnPagination: false
                };
                this.options = $.extend(true, defaults, this.options);
                
                this.rendered = {
                    list: false,
                    table: false,
                    raw: false
                };

                //CLONE RESULTS 
                this.model._result = new ResultModel();
                
                this.model._result.setFromSplunkD(this.model.result.responseText ? JSON.parse(this.model.result.responseText) : {});
                this.model.result.results.on('reset', function() {
                    if(!this.model._result.stopReset) {
                        this.model._result.setFromSplunkD(JSON.parse(this.model.result.responseText), {clone: true});
                    }
                },this);
                
                this.model.state = this.model.state || new BaseModel();
                this.model.listState =  new BaseModel();
                this.model.tableState =  new BaseModel();
                this.model.rawState =  new BaseModel();
                this.model.resultWorking = new ResultModel();
                
                this.model._result.results.on('reset', this.wake, this); 
                
                this.children.list = new EventsListView({
                    model: { 
                        result: this.model._result,
                        summary: this.model.summary,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        application: this.model.application,
                        state: this.model.listState
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: this.options.selectableFields,
                    headerMode: this.options.headerMode,
                    allowRowExpand: this.options.allowRowExpand
                });
                
                this.children.raw = new EventsRawView({
                    model: { 
                        result: this.model._result,
                        summary: this.model.summary,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        application: this.model.application,
                        state: this.model.rawState
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: false,
                    allowRowExpand: this.options.allowRowExpand
                });
                 
                this.children.table = new EventsTableView({
                    model: { 
                        result: this.model._result,
                        summary: this.model.summary,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        application: this.model.application,
                        state: this.model.tableState
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: this.options.selectableFields,
                    headerMode: this.options.headerMode,
                    allowRowExpand: this.options.allowRowExpand
                });
                this.model.report.entry.content.on('change:display.events.type', function(m, val, options) {
                    if(m.previousAttributes()['display.events.type'] !== 'table' &&  m.get('display.events.type') !== 'table'){
                        this.wake();
                    }
                }, this);

                
                this.model.listState.on('change:modalize', function(model) {
                    this.model.state.set('modalize', model.get('modalize'));  
                }, this);
                this.model.rawState.on('change:modalize', function(model) {
                    this.model.state.set('modalize', model.get('modalize'));  
                }, this);
                this.model.tableState.on('change:modalize', function(model) {
                    this.model.state.set('modalize', model.get('modalize'));  
                }, this);
                this.model.state.on('unmodalize', function(idx) {
                    this.model.listState.trigger('unmodalize'+idx);
                    this.model.rawState.trigger('unmodalize'+idx);
                    this.model.tableState.trigger('unmodalize'+idx);
                },this);
                this.model.state.on('change:fieldpicker', function() { 
                    this.children.table && this.children.table.updateDock();
                },this);
                
                this.model.report.entry.content.on('change:display.page.search.showFields', function() {
                    this.children.table.updateDock();
                    this.children.list.deferUpdateDock();
                },this);
                    
                this.model._result.on('swap-single-event', function(options) {
                    this.fetchResultWithAllFields(options);
                }, this);

                this.model.report.entry.content.on('change:display.prefs.events.offset', function() {
                    if(this.options.scrollToTopOnPagination) {
                        var containerTop = this.$el.offset().top,
                            currentScrollPos = $(document).scrollTop(),
                            headerHeight = this.$el.children(':visible').find('thead:visible').height(),
                            eventControlsHeight = $('.events-controls-inner').height();
                        if(currentScrollPos > containerTop)
                            $(document).scrollTop(containerTop - (headerHeight + eventControlsHeight));
                    }
                },this);
            
            },
            events: {
                'click .header-table-docked.disabled': function(e) {
                    this.model.state.trigger('unmodalize', this.model.state.get('modalize'));
                    this.model.state.unset('modalize');
                    e.preventDefault();
                }
            },
            wake: function() {
                var type = this.model.report.entry.content.get('display.events.type');
                
                if(!this.rendered[type]) {
                    this._render(type);
                }

                if (_.isNumber(this.model.state.get('modalize'))) {
                    return this;
                }
                switch (type) {
                    case 'raw':
                        this.children.list.sleep().$el.hide();
                        this.children.raw.wake().$el.show();
                        this.children.table.sleep().$el.hide(); 
                        break;
                    case 'list':
                        this.children.list.wake().$el.show();
                        this.children.raw.sleep().$el.hide();
                        this.children.table.sleep().$el.hide();
                        break;
                    case 'table':
                        this.children.list.sleep().$el.hide();
                        this.children.raw.sleep().$el.hide();
                        this.children.table.wake().$el.show();
                        break;
                    default: 
                        this.children.list.sleep().$el.hide();
                        this.children.raw.sleep().$el.hide();
                        this.children.table.sleep().$el.hide();
                        break;
                }
                return this;
            },
            fetchResultWithAllFields: function(options) {
                var offset,
                    search,
                    dispatchState = this.model.searchJob.entry.content.get('dispatchState'),
                    isRealtime = this.model.searchJob.isRealtime(),
                    eventModel = this.model._result.results.at((isRealtime) ? options.rtindex : options.index);
                
                if(isRealtime) {
                    search = ['search _serial=', eventModel.get('_serial')[0], ' AND ', 'splunk_server=', eventModel.get('splunk_server')[0], ' | head 1'].join('');
                } else {
                    offset = this.model._result.offset(options.index);
                    search = this.model.report.getSortingSearch();
                }

                this.model.resultWorking.id = this.model.searchJob.entry.links.get('events');
                this.model.resultWorking.fetch({
                    data: $.extend(true, {}, {
                        offset: offset,
                        count: 1,
                        earliest_time: this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                        latest_time: this.model.report.entry.content.get('display.events.timelineLatestTime'),
                        segmentation:  this.model.report.entry.content.get('display.events.list.drilldown'),
                        max_lines: (options.showAllLines) ? 0: this.model.report.getNearestMaxlines(),
                        truncation_mode: 'abstract',
                        search: search
                    }),
                    success: function(model, response) {
                        var matchedModel = model.results.at(0);
                        if (matchedModel) {
                            eventModel.replace(matchedModel.toJSON(), {swap: true});
                        } else {
                            eventModel.trigger('failed-swap');
                        }
                        this.manageState(dispatchState);
                    }.bind(this),
                    error: function(model, response) {
                        eventModel.trigger('failed-swap');
                        this.manageState(dispatchState);
                    }.bind(this)
                });    
            },
            manageState: function(dispatchState) {
                this.model._result.stopReset = false;
                if(this.model.searchJob.entry.content.get('dispatchState') !== dispatchState) {
                    this.model.result.trigger('forcefetch'); 
                }
            },
            _render: function(type) {
                this.$el.append(this.children[type].render().el);
                this.rendered[type] = true;
            },
            render: function() {
                this._render(this.model.report.entry.content.get('display.events.type'));
                this.wake();
                return this;
            }
        });
    }
);
