define(
    [
        'underscore',
        'module',
        'models/Base',
        'splunk.i18n',
        'util/time_utils',
        'views/Base',
        'views/shared/delegates/Modalize',
        'views/shared/delegates/TableDock',
        'views/shared/eventsviewer/shared/TableHead',
        'views/shared/eventsviewer/list/body/Master',
        'splunk.util'
    ],
    function(_, module, BaseModel, i18n, time_utils,  BaseView, Modalize, TableDock, TableHeadView, TableBodyView, util){
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     },
             *     selectableFields: true|false,
             *     allowRowExpand: true|false              
             */
            className: 'scrolling-table-wrapper',
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.model.state = this.model.state || new BaseModel();
                this.children.modalize = new Modalize({el: this.el, tbody: '> table > tbody'});
                this.children.head = new TableHeadView({
                    model: this.model.report,
                    labels: ['Time', 'Event'],
                    allowRowExpand: this.options.allowRowExpand
                });
                this.children.body = new TableBodyView({
                    model: { 
                        result: this.model.result,
                        summary: this.model.summary,
                        state: this.model.state,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        application: this.model.application
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand
                });
                
                if (this.options.headerMode==='dock') {
                    this.children.tableDock = new TableDock({ el: this.el, offset: 36, defaultLayout: 'fixed'});
                }
                
                var modalize = _.debounce(this.modalize);
                this.model.state.on('change:modalize', modalize, this);
                
                this.collection.selectedFields.on('reset add remove', modalize, this);
                
                this.model.report.entry.content.on('change:display.events.rowNumbers', function() {
                     this.$('table:not(.table-embed)').toggleClass('hide-line-num');
                     this.deferUpdateDock();
                }, this);
                
                this.model.report.entry.content.on('change:display.events.list.wrap', function() {
                     this.deferUpdateDock();
                }, this);
                
                this.model.result.results.on('reset', function() {
                    if(!_.isNumber(this.model.state.get('modalize'))) {
                        this.deferUpdateDock();
                    }
                }, this);
                
                /*
                 * The modalize delegate relays clicks on the modazlize mask back to 
                 * its creator.  This is translated into a row specific event
                 * which will set the state model into a collapsed state.
                 */
                this.children.modalize.on('unmodalize', function(idx) {
                    this.model.state.trigger('unmodalize'+idx);
                    this.children.tableDock && this.children.tableDock.enable();
                },this);

                this.model.state.on('change:modalize', function(model) {
                    if(this.children.tableDock) {
                        _.isNumber(model.get('modalize')) ? this.children.tableDock.disable() : this.children.tableDock.enable();
                    }
                },this);
                
                this.children.body.on('rows:pre-remove', function() { this.$el.css('minHeight', this.$el.height()); },this);
                this.children.body.on('rows:added', function() { this.$el.css('minHeight', ''); },this);
            },
            remove: function() {
                BaseView.prototype.remove.apply(this, arguments);
                $(window).off('.'+this.cid);
            },
            modalize: function() {
                this.children.modalize.show(this.model.state.get('modalize'));
                if(this.model.state.get('modalize')) {
                    this.children.tableDock && this.children.tableDock.disable();
                }
            },
            deferUpdateDock: function() {
                if (this.children.tableDock) {
                    _.defer(this.children.tableDock.update.bind(this.children.tableDock));
                }
            },
            render: function() {
                this.$el.html(this.compiledTemplate({
                     hidelinenums: !util.normalizeBoolean(this.model.report.entry.content.get("display.events.rowNumbers")),
                     cid: this.cid
                }));
                this.$('> table.events-results').append(this.children.head.render().el);
                this.$('> table.events-results').append(this.children.body.render().el);
                return this;
            },
            reflow: function() {
                this.children.tableDock && this.children.tableDock.update();
            },
            template: '\
                <table class="table table-chrome <% if(hidelinenums){ %> hide-line-num <% } %> table-row-expanding events-results events-results-table" id="<%= cid %>-table"></table>\
            '
        });
    }
);
