define(
    [
        'module',
        'underscore',
        'models/Base',
        'views/Base',
        'views/shared/eventsviewer/list/body/row/Master',
        'util/console'
    ],
    function(module, _, BaseModel, BaseView, Row, console){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tbody',
            type: 'list',
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         summary: <model.services.search.job.SummaryV2>
             *         state: <models.BaseV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     },
             *     selectableFields: true|false  
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.model.result.results.on('reset', this.debouncedRender, this);
                this.model.state.on('change:sleep', function() {
                    this.model.state.get('sleep') ? this.sleep({lite: true}) : this.wake({lite: true});
                }, this);
            },
            sleep: function(options) {
                (options && options.lite) ? (this.awake = false) : BaseView.prototype.sleep.apply(this);
                return this;
            },
            wake: function(options) {
                if(options && options.lite){
                    this.awake = true;
                    if (this.touch) {
                        this.render();
                    }
                } else {
                    BaseView.prototype.wake.apply(this);
                }
                return this;
            },
            isSelected: function() {
                return (this.model.report.entry.content.get('display.events.type') === this.type);
            },
            cleanup: function() {
                _.chain(this.children).values().invoke('debouncedRemove', {detach: true}).values();
                this.children = {};
            },
            render: function() {
                this.trigger('rows:pre-remove');
                this.cleanup();
                var fragment = document.createDocumentFragment(),
                    isRealTimeSearch = this.model.searchJob.entry.content.get('isRealTimeSearch'),
                    results = isRealTimeSearch ? this.model.result.results.reverse({mutate: false}) : this.model.result.results.models;

                console.debug('Events Lister: rendering', results.length, 'events', isRealTimeSearch ? 'in real-time mode' : 'in historical mode');
                _.each(results, function(event, idx) {
                    var lineNum,
                        id = 'row_' + idx;
                    if (this.model.searchJob.entry.content.get('isRealTimeSearch')) {
                        lineNum = this.model.result.endOffset() - idx;
                    } else {
                        lineNum = this.model.result.get('init_offset') + idx + 1;
                    }
                    this.children[id] = new Row({ 
                        lineNum: lineNum,
                        model: {
                            state: this.model.state,
                            event: event,
                            result: this.model.result,
                            summary: this.model.summary,
                            report: this.model.report,
                            application: this.model.application,
                            searchJob: this.model.searchJob
                        },
                        collection: {
                            selectedFields: this.collection.selectedFields,
                            eventRenderers: this.collection.eventRenderers,
                            workflowActions: this.collection.workflowActions
                        },
                        idx: idx,
                        selectableFields: this.options.selectableFields,
                        allowRowExpand: this.options.allowRowExpand
                    });
                    fragment.appendChild(this.children[id].render().el);
                }, this);
                this.el.appendChild(fragment);
                
                //bulk purge of remove mutex
                _(this.model.state.toJSON()).each(function(value, key) {
                    if(key.indexOf('pendingRemove') === 0) {
                        this.model.state.unset(key);
                    }
                },this);
                
                this.trigger('rows:added');
                return this;
            }
        });
    }
);
