define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/TableHead',
        'views/shared/eventsviewer/list/body/row/SelectedFields',
        'views/shared/eventsviewer/shared/EventFields',
        'views/shared/eventsviewer/shared/RawField',
        'splunk.util'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        TableHeadView,
        SelectedFieldsView,
        EventFieldsView,
        RawField,
        splunkUtil
    ){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            /**
             * @param {Object} options {
             *      model: {
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *         result: <models.services.search.job.ResultsV2>,
             *         state: <models.Base>,
             *         summary: <models.services.searchjob.SummaryV2>,
             *         report: <models.services.SavedSearch>,
             *         searchJob: <models.Job>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     },
             *     selectableFields: true|false
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                
                //suite of namespaced keys
                this.rowExpanded = 'rowExpanded' + this.options.idx;
                this.jsonInteraction = 'interaction' + this.options.idx;
                this.swappingKey = 'swapResult' + this.options.idx;
                this.showAllLines = 'showAllLines' + this.options.idx;
                this.pendingRemove = 'pendingRemove' + this.options.idx;
                
                this.model.renderer = this.collection.eventRenderers.getRenderer(this.model.event.get('eventtype'));
                
                //event 
                this.children.raw = new RawField({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    collection: {
                        eventRenderers: this.collection.eventRenderers
                    },
                    idx: this.options.idx
                });
                //collapsed view
                this.children.selectedFields = new SelectedFieldsView({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        summary: this.model.summary,
                        application: this.model.application,
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        workflowActions: this.collection.workflowActions
                    },
                    idx: this.options.idx,
                    selectableFields: this.options.selectableFields
                });
                this.model.state.on('change:'+this.rowExpanded , this.visibility, this);
                
                //actors that expand the row && modalize
                this.model.state.on(this.jsonInteraction, this.clickToExpandRow, this);
                this.model.state.on('change:'+this.showAllLines, function() {
                    this.clickToExpandRow();
                }, this); 
                this.model.state.on('unmodalize'+this.options.idx, function() {
                    this.clickToExpandRow(true); 
                }, this);
                
                this.model.result.on('tags-updated', function() { 
                    if(this.model.state.get(this.rowExpanded)) { 
                        this.render(); 
                    }
                }, this);
                this.model.report.entry.content.on('change:display.prefs.events.offset', this.collapseState, this);
            },
            events: {
                'click td.expands': function(e) {
                    this.eventFieldsFactory();
                    
                    if (this.model.state.get(this.pendingRemove)) {
                        if(this.model.state.get(this.rowExpanded)) {
                            this.collapseState();
                        } else {
                            this.expandState();
                        }
                        e.preventDefault();
                        return;
                    }
                    this.clickToExpandRow(true);
                    e.preventDefault();
                },
                'keyup ._time': function(e) {
                    e.preventDefault();
                    if(e.which === 13) {
                        this.drilldown($(e.currentTarget).find('span.formated-time'));
                    }
                }, 
                'click .formated-time': function(e) { 
                    e.preventDefault();
                    this.drilldown($(e.currentTarget));
                } 
            },
            drilldown: function($target) {
                var epoch = splunkUtil.getEpochTimeFromISO($target.data().timeIso);
                this.model.report.entry.content.set({
                    'dispatch.earliest_time': epoch,
                    'dispatch.latest_time': '' + (parseFloat(epoch) + 1)
                });
                this.model.report.trigger('eventsviewer:drilldown');
            },
            eventFieldsFactory: function() {
                if (this.children.eventFields) {
                    this.children.eventFields.remove();
                }
                this.children.eventFields = new EventFieldsView({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        summary: this.model.summary,
                        application: this.model.application,
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand,
                    idx: this.options.idx,
                    swappingKey: this.swappingKey,
                    showAllLines: this.showAllLines
                });
                this.$('.event').find(this.children.raw.el).after(this.children.eventFields.render().el);
            },
            clickToExpandRow: function(isRowClick) {
                var options = {
                    'index':  this.options.idx,
                    'rtindex': this.model.result.results.length - this.options.idx - 1,
                    'showAllLines': this.model.state.get(this.showAllLines) 
                };
                if(!this.children.eventFields) {
                    this.eventFieldsFactory();
                }
                // Either a row click or show all lines click 
                if(!this.model.state.get(this.rowExpanded)) {
                    this.expandState();
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                
                // Collapse row click
                } else if(this.model.state.get(this.rowExpanded) && isRowClick) {
                    this.model.state.unset(this.showAllLines, {silent: true}); // force hide all lines
                    options.showAllLines = this.model.state.get(this.showAllLines); //update options
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                    this.collapseState();
                //interaction with show/hide more lines
                } else {
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                }
                this.model.state.unset('timeExpanded', {silent: true});                    
            },
            expandState: function() {
                this.model.state.set(this.rowExpanded, true);
                this.model.state.set('sleep', true);
            },
            collapseState: function() {
                this.model.state.unset('modalize');
                this.model.state.unset(this.rowExpanded);
                this.model.state.unset('sleep');
            },
            debouncedRemove: function() {
                this.model.state.set(this.pendingRemove, true);
                BaseView.prototype.debouncedRemove.apply(this, arguments);
                return this;
            },
            visibility: function() {
                var $arrow =  this.$('td.expands > a > i').removeClass('icon-triangle-right-small icon-triangle-down-small');
                if (this.model.state.get(this.rowExpanded) && this.options.allowRowExpand) {
                    $arrow.addClass('icon-triangle-down-small');
                    this.eventFieldsFactory();
                } else {
                    $arrow.addClass('icon-triangle-right-small');
                    this.children.selectedFields.wake().$el.css('display', '');
                    if (this.children.eventFields) {
                        this.children.eventFields.remove();
                    }
                }
                this.children.raw.visibility();
                this.reflow();
            },
            render: function() {
               var root = this.el;
               //rows are read only (innerHTML) for ie
               this.$el.find('> td').each(function(key, element) {
                   root.removeChild(element);
               });
               this.$el.append(this.compiledTemplate({
                    $: $,
                    event: this.model.event,
                    lineNum: this.options.lineNum,
                    application: this.model.application,
                    expanded: this.model.state.get(this.rowExpanded),
                    formattedTime: this.model.event.formattedTime(),
                    colorClass: (this.model.renderer) ? this.model.renderer.entry.content.get('css_class'): '',
                    allowRowExpand: this.options.allowRowExpand
                }));
                
                this.$('.event').append(this.children.raw.render().el);
                this.$('.event').append(this.children.selectedFields.render().el);

                this.visibility();
                return this;
            },
            reflow: function() {
               (this.model.state.get(this.rowExpanded)) ?
                    this.model.state.set('modalize', this.options.idx):
                    this.model.state.unset('modalize');
                    
                BaseView.prototype.reflow.apply(this, arguments);
            },
            template: '\
                <% if (allowRowExpand) { %>\
                <td class="expands <%- colorClass %>">\
                    <a href="#"><i class="icon-triangle-<%- expanded ? "down" : "right" %>-small"></i></a>\
                </td>\
                <% } %>\
                <td class="line-num"><span><%- lineNum %></span></td>\
                <td class="_time" tabindex="0">\
                    <span class="formated-time" data-time-iso="<%- event.get("_time") %>">\
                    <% if(application.get("locale").indexOf("en") > -1){ %>\
                         <span><%- $.trim(formattedTime.slice(0, formattedTime.indexOf(" "))) %></span>\
                         <br>\
                         <span><%- $.trim(formattedTime.slice(formattedTime.indexOf(" "))) %></span>\
                    <% } else { %>\
                         <%- formattedTime %>\
                    <% } %>\
                    </span>\
                </td>\
                <td class="event"></td>\
            '
        });
    }
);
