define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/delegates/Popdown',
        'views/shared/FieldInfo',
        'views/shared/eventsviewer/shared/WorkflowActions',
        'views/shared/eventsviewer/shared/BaseFields'
    ],
    function($, _, Backbone, module, BaseView, Popdown, FieldInfo, WorkflowActionsView, BaseFieldsView){
        return BaseFieldsView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *      model: {
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *     },
             *     selectableFields: true|false
             */
            initialize: function(){
                BaseFieldsView.prototype.initialize.apply(this, arguments);
                this.rowExpanded = 'rowExpanded' + this.options.idx;
            },
            render: function() {
                var strippedfields = this.model.event.strip(),
                    selectedfields = _.intersection(strippedfields, this.collection.selectedFields.pluck('name')).sort();
                this.$el.html(this.compiledTemplate({
                    selectedfields: selectedfields,
                    selectableFields: this.options.selectableFields,
                    m: this.model.event,
                    slen: this.model.summary.fields.length,
                    r: this.model.result,
                    _partial: this._partial,
                    _condensedSelectedFields: this._condensedSelectedFields,
                    selected: selectedfields.length,
                    _:_
                }));                
                return this;
            },
            _condensedSelectedFields: _.template('\
                <ul class="condensed-selected-fields">\
                <%  _(fields).each(function(field, i) { %>\
                    <% var values = m.get(field) %>\
                    <li>\
                        <% _(values).each(function(value, idx) { %>\
                            <span class="field"><%- field %> =</span>\
                            <span class="field-value"><a class="f-v" data-field-name="<%-field %>" title="<%= value ? value : "&nbsp;"%>"><% if(value) {%><%- value %><% } else { %>&nbsp;<% } %></a></span>\
                            <% var tags = r.getTags(field, value); %>\
                            <% if (tags.length) { %>\
                                  <% _(tags).each(function(tag, idx){ %><a data-tagged-field-name="tag::<%- field %>" class="tag" href="#"><%- tag %>\
                                        <%if(idx!=tags.length-1){%> <%}%>\
                                    </a><% }); %>\
                            <% } %>\
                        <% }) %>\
                    </li>\
                <% }) %>\
              </ul>\
            '),
            _noncondensedSelectedFields: _.template('\
                <table class="table table-condensed table-embed outer-template table-dotted">\
                    <tbody>\
                        <!-- FIXME: this should be inside the row below -->\
                        <tr class="row-more-fields">\
                            <td></td>\
                            <td></td>\
                            <td></td>\
                            <td class="col-more-fields" rowspan="4">\
                                <a href="#" class="more-fields"><%- selected %> of <%- total %> fields</a>\
                            </td>\
                        </tr>\
                        <%= _partial({fields: selectedfields, slen: slen, m: m, r:r, label: void(0), selectableFields: false}) %>\
                    </tbody>\
                </table>\
            '),
            template: '\
                <% if (selected) { %>\
                    <%= _condensedSelectedFields({ fields: selectedfields, m: m, r:r, _:_ }) %>\
                <% } %>\
           '
           // <!-- <%= _noncondensedSelectedFields({fields: selectedfields, _partial: _partial, _:_, slen: slen, m: m, r:r, label: void(0), selectableFields: false}) %> -->\
        });
    }
);
