define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'models/Base',
        'views/Base',
        'views/shared/delegates/Modalize',
        'views/shared/eventsviewer/shared/TableHead',
        'views/shared/eventsviewer/raw/body/Master'
    ],
    function($, _, Backbone, module, BaseModel,  BaseView, Modalize, TableHeadView, TableBodyView){
        return BaseView.extend({
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.jobs.ResultsV2>,
             *         summary: <model.services.search.jobs.SummaryV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     },
             *     selectableFields: true|false,
             *     allowRowExpand: true|false
             * }
             */
            className: 'scrolling-table-wrapper',
            moduleId: module.id,
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.model.state = this.model.state || new BaseModel();
                this.children.modalize = new Modalize({el: this.el, tbody: '> table > tbody'});
                this.children.head = new TableHeadView({
                    labels: ['Event'],
                    allowRowExpand: this.options.allowRowExpand,
                    allowLineNum: false
                });
                this.children.body = new TableBodyView({
                    model: {
                        result: this.model.result,
                        state: this.model.state,
                        summary: this.model.summary,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        application: this.model.application
                    },
                    collection: { 
                        workflowActions: this.collection.workflowActions,
                        eventRenderers: this.collection.eventRenderers,
                        selectedFields: this.collection.selectedFields 
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand
                });

                //Modalize 
                var modalize = _.debounce(this.modalize);
                this.model.state.on('change:modalize', modalize, this);
                this.collection.selectedFields.on('reset add remove', modalize, this);
                
                this.children.modalize.on('unmodalize', function(idx) {
                    this.model.state.trigger('unmodalize'+idx);
                },this);
                
                this.children.body.on('rows:pre-remove', function() { this.$el.css('minHeight', this.$el.height()); },this);
                this.children.body.on('rows:added', function() { this.$el.css('minHeight', ''); },this);
            },
            modalize: function() {
                this.children.modalize.show(this.model.state.get('modalize'));
            },
            render: function() {
                this.$el.html(this.template);
                this.$('> table.events-results').append(this.children.head.render().el);
                this.$('> table.events-results').append(this.children.body.render().el);
                return this;
            },
            template: '\
                <table class="table table-chrome table-row-expanding events-results events-results-table"></table>\
            '
        });
    }
);
