 define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/eventsviewer/shared/EventFields',
        'views/shared/eventsviewer/shared/RawField'
    ],
    function($, _, Backbone, module, BaseView, EventFieldsView, RawField){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>
             *     },             
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions> 
             *     },
             *     selectableFields: true|false,
             *     allowRowExpand: true|false
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                
                //suite of namespaced keys
                this.rowExpanded = 'rowExpanded' + this.options.idx;
                this.jsonInteraction = 'interaction' + this.options.idx;
                this.swappingKey = 'swapResult' + this.options.idx;
                this.showAllLines = 'showAllLines' + this.options.idx;
                this.pendingRemove = 'pendingRemove' + this.options.idx;
                
                this.model.renderer = this.collection.eventRenderers.getRenderer(this.model.event.get('eventtype'));
                
                this.children.raw = new RawField({ 
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    idx: this.options.idx
                });
                this.model.state.on('change:'+this.rowExpanded , this.visibility, this);
                
                //actors that expand the row && modalize
                this.model.state.on(this.jsonInteraction, this.clickToExpandRow, this);
                this.model.state.on('change:'+this.showAllLines, function() {
                    this.clickToExpandRow();
                }, this); 
                this.model.state.on('unmodalize'+this.options.idx, function() { 
                    this.clickToExpandRow(true); 
                }, this);
                
                this.model.result.on('tags-updated', function() { 
                    if(this.model.state.get(this.rowExpanded))  
                        this.render(); 
                }, this);
                
                this.model.report.entry.content.on('change:display.prefs.events.offset', this.collapseState, this);
            },
            events: {
                'click td.expands': function(e) {
                    this.eventFieldsFactory();
                    if (this.model.state.get(this.pendingRemove)) {
                        if(this.model.state.get(this.rowExpanded)) {
                            this.collapseState();
                        } else {
                            this.expandState();
                        }
                        e.preventDefault();
                        return;
                    }
                    this.clickToExpandRow(true);
                    e.preventDefault();
                }
            },
            eventFieldsFactory: function() {
                if (this.children.eventFields) {
                    this.children.eventFields.remove();
                }
                this.children.eventFields = new EventFieldsView({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        summary: this.model.summary,
                        application: this.model.application,
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand,
                    idx: this.options.idx,
                    swappingKey: this.swappingKey,
                    showAllLines: this.showAllLines
                });
                this.$('.event').find(this.children.raw.el).after(this.children.eventFields.render().el);
            },
            clickToExpandRow: function(isRowClick) {
                var options = {
                    'index':  this.options.idx,
                    'rtindex': this.model.result.results.length - this.options.idx - 1,
                    'showAllLines': this.model.state.get(this.showAllLines) 
                };
                
                if(!this.children.eventFields) {
                    this.eventFieldsFactory();
                }
                // Either a row click or show all lines click 
                if(!this.model.state.get(this.rowExpanded)) {
                    this.expandState();
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                
                // Collapse row click
                } else if(this.model.state.get(this.rowExpanded) && isRowClick) {
                    this.model.state.unset(this.showAllLines, {silent: true}); // force hide all lines
                    options.showAllLines = this.model.state.get(this.showAllLines); //update options
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                    this.collapseState();
                //interaction with show/hide more lines
                } else {
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                }
                this.model.state.unset('timeExpanded', {silent: true});                    
            },
            expandState: function() {
                this.model.state.set(this.rowExpanded, true);
                this.model.state.set('sleep', true);
            },
            collapseState: function() {
                this.model.state.unset('modalize');
                this.model.state.unset(this.rowExpanded);
                this.model.state.unset('sleep');
            },
            debouncedRemove: function() {
                this.model.state.set(this.pendingRemove, true);
                BaseView.prototype.debouncedRemove.apply(this, arguments);
                return this;
            },
            visibility: function() {
                var $arrow =  this.$('td.expands > a > i').removeClass('icon-triangle-right-small icon-triangle-down-small');
                if (this.model.state.get(this.rowExpanded) && this.options.allowRowExpand) {
                    $arrow.addClass('icon-triangle-down-small');
                    this.eventFieldsFactory();
                } else {
                    $arrow.addClass('icon-triangle-right-small');
                    if (this.children.eventFields) {
                        this.children.eventFields.remove();
                    }
                }
                this.children.raw.visibility();
                this.reflow();
            },
            render: function() {
                var root = this.el;
                //rows are read only (innerHTML) for ie
                this.$el.find('> td').each(function(key, element) {
                    root.removeChild(element);
                });
                this.$el.append(this.compiledTemplate({ 
                    expanded: this.model.state.get(this.rowExpanded),
                    colorClass: (this.model.renderer) ? this.model.renderer.entry.content.get('css_class'): '',
                    allowRowExpand: this.options.allowRowExpand
                }));
                this.$('.event').append(this.children.raw.render().el);
                this.visibility();
                return this;
            },
            reflow: function() {
               (this.model.state.get(this.rowExpanded)) ?
                    this.model.state.set('modalize', this.options.idx):
                    this.model.state.unset('modalize');
            },
            template: '\
                <% if (allowRowExpand) { %>\
                    <td class="expands <%- colorClass %>">\
                        <a href="#"><i class="icon-triangle-<%- expanded ? "down" : "right" %>-small"></i></a>\
                    </td>\
                <% } %>\
                <td class="event"></td>\
            '
        });
    }
);
