define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/FieldInfo',
        'views/shared/eventsviewer/shared/WorkflowActions',
        'views/shared/eventsviewer/shared/fieldactions/Master',
        'views/shared/eventsviewer/shared/TimeInfo',
        'collections/UIWorkflowActions',
        'models/services/search/IntentionsParser',
        'models/DateInput'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        FieldInfo,
        WorkflowActionsView,
        FieldActions,
        TimeInfoView,
        WorkflowActionsCollection,
        IntentionsParser,
        DateInput
    ){
        return BaseView.extend({
            /**
             * @param {Object} options {
             *      model: {
             *         event: <models.services.search.job.ResultsV2.result[i]>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         application: <model.Application>,
             *         report: <models.services.SavedSearch>,
             *         searchJob: <models.Job>
             *     }
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     },
             *     selectableFields: true|false
             * }
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.model.intentionsParser = new IntentionsParser();
                this.model.intentionsParser.on('change', function() {
                    this.model.report.entry.content.set('search', this.model.intentionsParser.fullSearch());
                    this.model.report.trigger('eventsviewer:drilldown');
                }, this);
                this.model.event.on('change', this.render, this);
             
                this.collection.selectedFields.on('reset add remove', function() {
                    if(this.options.idx === this.model.state.get('modalize')) {
                        this.render(); 
                    }
                }, this);
            },
            events: {
                'click .field-value a.f-v': function(e) {
                    var $value = $(e.target),
                        value = $.trim($value.text()),
                        field = $.trim($value.data().fieldName);
                    this.model.intentionsParser.clear({silent: true});
                    this.model.intentionsParser.fetch({
                        data: {
                            q: this.model.report.entry.content.get('search'),
                            stripReportsSearch: false,
                            action: 'fieldvalue',
                            field: field,
                            value: value,
                            app: this.model.application.get('app'),
                            owner: this.model.application.get('owner')
                        }
                    });
                    e.preventDefault();
                },
                'click .tag': 'tagClick', 
                'click .field-value .tag': 'tagClick',
                'click ._time': function(e) {
                    e.preventDefault();
                },
                'click .field-info': function(e) {
                    e.preventDefault();
                },
                'click .field-actions': function(e) {
                    e.preventDefault();
                },
                'click .event-actions': function(e) {
                    e.preventDefault();
                },
                'mousedown ._time': 'openTimeInfo',
                'mousedown .field-info': 'openFieldInfo',
                'mousedown .field-actions': 'openFieldActions',
                'mousedown .event-actions': 'openEventActions',
                'keydown ._time': function(e) {
                    if (e.keyCode == 13)  {
                        this.openTimeInfo(e);
                    }
                },
                'keydown .field-info': function(e) {
                    if (e.keyCode == 13)  {
                        this.openFieldInfo(e);
                    }
                },
                'keydown .field-actions': function(e) {
                    if (e.keyCode == 13)  {
                        this.openFieldActions(e);
                    }
                },
                'keydown .event-actions': function(e) {
                    if (e.keyCode == 13)  {
                        this.openEventActions(e);
                    }
                }
            },
            tagClick: function(e) {
                var $value = $(e.target),
                    value  = $.trim($value.text()),
                    field  = $.trim($value.data().taggedFieldName);
                this.model.intentionsParser.clear({silent: true});
                this.model.intentionsParser.fetch({
                    data: {
                        q: this.model.searchJob.getStrippedEventSearch(),
                        action: 'fieldvalue',
                        field: field,
                        value: value,
                        app: this.model.application.get('app'),
                        owner: this.model.application.get('owner')
                    }
                });
                e.preventDefault();
            },
            openTimeInfo: function(e) {
                var $target = $(e.currentTarget),
                    time = $target.data().time;
                    
                if (this.children.time && this.children.time.shown) {
                    this.children.time.hide();
                }
                
                this.children.time = new TimeInfoView({
                    model: {
                        report: this.model.report
                    },
                    time: time,
                    onHiddenRemove: true
                });
                $('body').append(this.children.time.render().el);
                this.children.time.show($target);
                e.preventDefault();
            },
            openFieldInfo: function(e) {
                var $target = $(e.currentTarget);
                var field = this.model.summary.fields.findWhere({'name': $target.attr('data-field-name') });
                var fieldName = field.get('name');
                
                if (this.children.fieldInfo && this.children.fieldInfo.shown) {
                    this.children.fieldInfo.hide();
                    if(this.lastMenu == (fieldName+'-field-info'))
                        return true;
                }
                
                if (!field) {
                    alert(_("This event is no longer within the results window.").t());
                }
                
                this.children.fieldInfo = new FieldInfo({
                    model: {
                        field: field,
                        summary: this.model.summary,
                        report: this.model.report,
                        application: this.model.application
                    },
                    collection: {selectedFields: this.collection.selectedFields},
                    onHiddenRemove: true,
                    selectableFields: this.options.selectableFields
                });
                this.lastMenu = fieldName + "-field-info";
                if(!_.isNumber(this.model.state.get('modalize'))){
                    this.model.state.set({
                        'modalize': this.options.idx,
                        'sleep': true
                    });
                }
                $('body').append(this.children.fieldInfo.render().el);
                this.children.fieldInfo.show($target);
                e.preventDefault();
            },
            openFieldActions: function(e) {
                var $target = $(e.currentTarget),
                    fieldName = $target.attr('data-field-name'),
                    fieldValue = $.trim($target.closest('tr').find('.f-v').text());
                
                if (this.children.fieldActions && this.children.fieldActions.shown) {
                    this.children.fieldActions.hide();
                    if(this.lastMenu == (fieldName+'-field-actions'))
                        return true;
                }
                
                this.children.fieldActions = new WorkflowActionsView({
                    model: this.model,
                    collection: this.collection.workflowActions,
                    actions: this.collection.workflowActions.getFieldActions(fieldName, this.model.event),
                    fieldName: fieldName,
                    fieldValue: fieldValue,
                    tags: true,
                    mode: 'menu',
                    onHiddenRemove: true
                });
                this.lastMenu = fieldName + "-field-actions";
                if(!_.isNumber(this.model.state.get('modalize'))){
                    this.model.state.set({
                        'modalize': this.options.idx,
                        'sleep': true
                    });
                }
                $('body').append(this.children.fieldActions.render().el);
                this.children.fieldActions.show($target);
                e.preventDefault();
            },
            openEventActions: function(e) {
                var $target = $(e.currentTarget),
                    index = this.model.result.results.indexOf(this.model.event); 
                if (this.children.eventActions && this.children.eventActions.shown) {
                    this.children.eventActions.hide();
                    return true;
                }
                
                this.children.eventActions = new WorkflowActionsView({
                    model: this.model,
                    collection: this.collection.workflowActions,
                    actions: this.collection.workflowActions.getEventActions(this.model.event),
                    index: index,
                    tags: false,
                    mode: 'menu',
                    onHiddenRemove: true
                });
                $('body').append(this.children.eventActions.render().el);
                this.children.eventActions.show($target);
                e.preventDefault();
            },
            _partial: _.template('\
                <%  _(fields).each(function(field, i) { %>\
                    <% var fieldlist = m.get(field) %>\
                    <% if(fieldlist.length > 1){ %>\
                        <%  _(fieldlist).each(function(mv_field, j) { %>\
                            <tr>\
                               <% if(i==0 && j==0 && label) { %>\
                                   <td rowspan="<%= m.getFieldsLength(fields) %>" class="field-type"><%- label %></td>\
                               <% } %>\
                               <% if(selectableFields && j==0) { %>\
                                   <td rowspan="<%= fieldlist.length %>" class="col-visibility"><label class="checkbox">\
                                   <a href="#" data-name="Everyone.read" class="btn <%- iconVisibility ? "hide" : "show" %>-field">\
                                   <% if(iconVisibility) { %>\
                                    <i class="icon-check"></i>\
                                    <% } %>\
                                   </label></td>\
                               <% } %>\
                               <% if(j==0 && slen>0) {%>\
                                    <td rowspan="<%=fieldlist.length %>" class="field-key">\
                                        <a class="popdown-toggle field-info" href="#" data-field-name="<%- field %>">\
                                            <span><%- field %></span><span class="caret"></span>\
                                        </a>\
                                    </td>\
                               <% } else if(j==0 && slen==0) { %>\
                                    <td rowspan="<%=fieldlist.length %>" class="field-key">\
                                        <span  data-field-name="<%- field %>"><%- field %></span>\
                                    </td>\
                               <% } %>\
                               <td class="field-value">\
                                   <a data-field-name="<%- field %>"  class="f-v" href="#"><%- mv_field %></a>\
                                   <% var tags = r.getTags(field, mv_field); %>\
                                   <% if (tags.length) { %>\
                                       (<% _(tags).each(function(tag, idx){ %><a data-tagged-field-name="tag::<%- field %>" class="tag" href="#"><%- tag %><%if(idx!=tags.length-1){%> <%}%></a><% }); %>)\
                                   <% } %>\
                               </td>\
                               <td  class="actions popdown">\
                                       <a class="popdown-toggle field-actions" href="#" data-field-name="<%- field %>">\
                                           <span class="caret"></span>\
                                       </a>\
                               </td>\
                            </tr>\
                        <% }) %>\
                    <% } else { %>\
                        <tr>\
                           <% if(i==0 && label) { %>\
                               <td rowspan="<%= m.getFieldsLength(fields) %>" class="field-type"><%- label %></td>\
                           <% } %>\
                           <% if(selectableFields) { %>\
                               <td rowspan="<%= fieldlist.length %>" class="col-visibility"><label class="checkbox">\
                               <a href="#" data-name="Everyone.read" class="btn <%- iconVisibility ? "hide" : "show" %>-field">\
                               <% if(iconVisibility) { %>\
                                <i class="icon-check"></i>\
                                <% } %>\
                               </a></label></td>\
                           <% } %>\
                           <% if(slen >0) { %>\
                               <td class="field-key">\
                                   <a class="popdown-toggle field-info" href="#" data-field-name="<%- field %>">\
                                       <span><%- field %></span><span class="caret"></span>\
                                   </a>\
                               </td>\
                           <% } else { %>\
                                <td class="field-key">\
                                    <span  data-field-name="<%- field %>"><%- field %></span>\
                                </td>\
                           <% } %>\
                           <td class="field-value">\
                               <a data-field-name="<%- field %>"  class="f-v" href="#"><%- m.get(field)[0] %></a>\
                               <% var tags = r.getTags(field, m.get(field)); %>\
                               <% if (tags.length > 0) { %>\
                                   (<% _(tags).each(function(tag, idx){ %><a data-field-name="<%- field %>" data-tagged-field-name="tag::<%- field %>" class="tag" href="#"><%- tag %><%if(idx!=tags.length-1){%> <%}%></a><% }); %>)\
                               <% } %>\
                           </td>\
                           <td  class="actions">\
                                   <a class="popdown-toggle field-actions" href="#" data-field-name="<%- field %>">\
                                       <i class="icon-triangle-down-small"></i>\
                                   </a>\
                           </td>\
                       </tr>\
                   <% } %>\
               <% }); %>\
           ')
        });
    }
);
