 define(
    [
        'underscore',
        'module',
        'keyboard/SearchModifier',
        'splunk.util', 
        'views/Base',
        'models/services/search/IntentionsParser',
        'models/services/search/jobs/Result',
        'views/shared/JSONTree'
    ],
    function(_, module, KeyboardSearchModifier, util, BaseView, IntentionsParser, ResultModel, JSONTree){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'div',
            /**
             * @param {Object} options {
             *     model: {
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *         result: <models.services.search.job.ResultsV2,
             *         report: <models.services.SavedSearch>,
             *         searchJob: <models.Job>
             *     },
             *     collection: {
             *         eventRenderers: <collections.services.configs.EventRenderers>
             *     }
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.keyboardSearchModifier = new KeyboardSearchModifier();
                
                this.rowExpanded = 'rowExpanded' + this.options.idx;
                this.jsonInteraction = 'interaction' + this.options.idx;
                this.isSyntaxHighlighted = 'isSyntaxHighlighted' + this.options.idx;
                this.showAllLines = 'showAllLines' + this.options.idx;

                this.children.json = new JSONTree({
                    json: this.model.event.rawToText()
                });
                
                this._isJSON = this.children.json.isJSON();
                
                this.model.state.set(this.isSyntaxHighlighted, this._isJSON); 
                this.model.state.on('change:'+this.isSyntaxHighlighted, this.render, this);

                this.model.intentionsParser = new IntentionsParser();


                this.model.intentionsParser.on('change', function() {
                    this.model.report.entry.content.set('search', this.model.intentionsParser.fullSearch());
                    this.model.report.trigger('eventsviewer:drilldown');
                }, this);

                
                this.model.resultWorking = new ResultModel();
                this.model.resultWorking.results.on('reset', function() {
                    var event = this.model.resultWorking.results.at(0);
                    if (event) {
                        this.model.event.replace(event.toJSON());
                    }
                }, this);
                this.model.event.on('change', function() {
                    this.children.json.setJSON(this.model.event.rawToText(), false);
                    this.render(); 
                }, this);


                this.model.report.entry.content.on('change:display.events.type', function () {
                    var wrap = util.normalizeBoolean(this.model.report.entry.content.get("display.events.list.wrap")),
                        $raw = this.$('.raw-event');
                    $raw.removeClass('wrap');
                    if(this.model.report.entry.content.get('display.events.type') === 'raw' || wrap) {
                       $raw.addClass('wrap');
                    } 
                },this);
                this.model.report.entry.content.on('change:display.events.list.wrap', function () {
                    var wrap = util.normalizeBoolean(this.model.report.entry.content.get("display.events.list.wrap")),
                        $raw = this.$('.raw-event');
                    (wrap && !$raw.hasClass('wrap')) ? $raw.addClass('wrap') : $raw.removeClass('wrap');
                },this);
                
                this.children.json.on('interaction', function() {
                    if(!this.model.state.get(this.rowExpanded)) {
                        this.model.state.trigger(this.jsonInteraction);
                    }
                },this);
            },
            visibility: function() {
                if(this.model.state.get(this.isSyntaxHighlighted)) {
                    this.$('.toggle-raw-json').text(_("Show as raw text").t());
                } else {
                    this.$('.toggle-raw-json').text(_("Show syntax highlighted").t());
                }
            },
            events: {
                'mouseover .t': function(e) {
                    var $elem = this.getSegmentParent(e.target);
                    $elem.addClass('h');
                    e.preventDefault();
                },
                'mouseout .t': function(e) {
                    var $elem = this.getSegmentParent(e.target);
                    $elem.removeClass('h');

                },
                'click .t': function(e) {
                    var $root = this.getSegmentRoot($(e.currentTarget)), data;
                    if(!$root) {
                        $root = this.getSegmentParent(e.currentTarget);
                    }
                    data = {
                        value: $root.text(),
                        app: this.model.application.get('app'),
                        owner: this.model.application.get('owner'),
                        stripReportsSearch: false
                    };

                    this.model.intentionsParser.clear({silent: true});
                    if ($root.hasClass('a')) {
                        data = $.extend(data, {
                            q: this.model.report.entry.content.get('search'),
                            action: 'removeterm'
                        });
                    } else {
                        data = $.extend(data, {
                            q: this.keyboardSearchModifier.isReplacement(e) ? '*' : this.model.report.entry.content.get('search'),
                            action: 'addterm',
                            negate: this.keyboardSearchModifier.isNegation(e)
                        });
                    }
                    this.model.intentionsParser.fetch({data: data});
                    return false;
                },
                'click .hideinline': function(e) {
                    this.model.state.set(this.showAllLines, false);
                    e.preventDefault();
                 },
                'click .showinline': function(e) {
                    this.model.state.set(this.showAllLines, true); 
                    e.preventDefault();
                },
                'click .toggle-raw-json': function(e) {
                    this.model.state.set(this.isSyntaxHighlighted, !this.model.state.get(this.isSyntaxHighlighted)); 
                    e.preventDefault();
                }
            },
            getSegmentParent: function(element){
                var parent = element.parentNode;
                if (parent.childNodes[parent.childNodes.length-1]==element && $(parent).hasClass('t')) {
                    element = parent;
                }
                return $(element);
            },
            getSegmentRoot: function($element) {
                if($element.hasClass('event')) {
                    return void(0);
                } else if($element.hasClass('a')) {
                    return $element;
                } else {
                    return this.getSegmentRoot($element.parent());
                }
            },
            render: function() {
                var wrap = true,
                    content = this.model.report.entry.content,
                    type = content.get('display.events.type'),
                    linecount = parseInt(this.model.event.get('_fulllinecount'), 10),
                    decorations = {
                        'decoration_audit_valid': {"msg": "Valid", "label": "label-success", "icon": "icon-check-circle"} ,
                        'decoration_audit_gap': {"msg": "Gap", "label": "label-warning", "icon": "icon-minus-circle"} ,
                        'decoration_audit_tampered': {"msg": "Tampered!", "label": "label-important", "icon": "icon-alert-circle"} ,
                        'decoration_audit_cantvalidate': {"msg": "Can't validate!", "label": "label-info", "icon": "icon-question-circle" }
                    };
                if(type === 'list') {
                    wrap = util.normalizeBoolean(content.get('display.events.list.wrap'));
                }
                this.el.innerHTML = this.compiledTemplate({
                    _:_,
                    isTable: (this.model.report.entry.content.get('display.events.type') === 'table'),
                    isJSON: this._isJSON,
                    model: this.model.event,
                    linecount: linecount,
                    wrap: wrap,  
                    expanded: this.model.state.get(this.showAllLines),
                    decorations: decorations,
                    rawTextOnly: this.options.noSegmentation
                });
                
                var $rawevent = this.$('.raw-event');
                if(!this.options.noSegmentation && this._isJSON) {
                    if(this.model.state.get(this.isSyntaxHighlighted)){
                        this.$('.json-event').append(this.children.json.render().el);
                    } else {
                        var segmentation = this.model.event.getRaw();

                        $rawevent.removeClass('raw-event').hide();
                        $rawevent[0].innerHTML = segmentation;
                        $rawevent.addClass('raw-event').show();
                    }
                } else if(this.options.noSegmentation) {
                    this.$('.raw-event').append(_.escape(this.model.event.rawToText()));
                } else {
                    this.$('.raw-event').append(this.model.event.getRaw());
                }
                this.visibility();
                return this;
            },
            template: '\
                <% if(model.has("_decoration") && decorations[model.get("_decoration")]) { %>\
                    <% var decoration = decorations[model.get("_decoration")]; %>\
                    <span class="audit label  <%- decoration.label %>"><i class="<%- decoration.icon %>"></i> <%- decoration.msg %></span>\
                <% } %>\
                <pre class="json-event"></pre>\
                <div class="raw-event normal <% if(wrap){ %> wrap <% } %>"></div>\
                <% if(isJSON && !isTable) { %><a href="#" class="toggle-raw-json"><%- _("Show as raw text").t() %></a> <% } %>\
                <% if (expanded) { %>\
                    <a href="#" class="hideinline">Collapse</a>\
                <% } else if (model.isTruncated()) { %>\
                    <a href="#" class="showinline">Show all <%= linecount %> lines</a>\
                <% } %>\
            '
        });
    }
);
