define(
    [
        'underscore',
        'jquery',
        'module',
        'util/time_utils',
        'splunk.util',
        'views/shared/PopTart',
        'strftime'
    ],
    function(_, $, module, timeutils, splunkutils, PopTartView) {
        return PopTartView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *          report: <models.services.SavedSearch>,
             *     }
             * }
             */
            initialize: function() {
                PopTartView.prototype.initialize.apply(this, arguments);
            },
            events: {
                'click td > a.et-lt': function(e) {
                    var $target = $(e.currentTarget),
                        timebounds = $target.data().time;
                    if(timebounds === 'before'){
                        this.model.report.entry.content.set('dispatch.latest_time', this.options.time);
                    } else if (timebounds == 'after') {
                        this.model.report.entry.content.set('dispatch.earliest_time', this.options.time);  
                    } else {
                        var et = parseFloat(splunkutils.getEpochTimeFromISO(this.options.time)), //inclusive
                            lt = et + 0.001; //exclusive
                        this.model.report.entry.content.set({
                            'dispatch.latest_time': lt,
                            'dispatch.earliest_time': et
                        });  
                    }
                    this.model.report.trigger('eventsviewer:drilldown');
                    e.preventDefault();
                },
                'click td > a.time-range': function(e) {
                    var $target = $(e.currentTarget),
                        ranges  = timeutils.rangeFromIsoAndOffset(this.options.time, $target.data().timeUnit);
                    this.model.report.entry.content.set({
                        'dispatch.latest_time': ranges.upperRange.strftime("%s.%Q"),
                        'dispatch.earliest_time': ranges.lowerRange.strftime("%s.%Q")
                    });  
                    this.model.report.trigger('eventsviewer:drilldown');
                    e.preventDefault();
                }
            },
            render: function() {
                this.$el.html(PopTartView.prototype.template);
                this.$('.popdown-dialog-body').html(this.template);
                return this;
            },
            template: '\
                <h3>_time</h3>\
                <table>\
                    <thead>\
                        <tr><th>Events Before or After</th><th>Nearby Events</th></tr>\
                    </thead>\
                    <tbody>\
                        <tr>\
                            <td>\
                                <a class="et-lt" data-time="before" href="#">Before this time</a><br>\
                                <a class="et-lt" data-time="after" href="#">After this time</a><br>\
                                <a class="et-lt" data-time="at" href="#">At this time</a><br>\
                            </td>\
                            <td>\
                                <a class="time-range" data-time-unit="w" href="#">+/- 1 week</a><br>\
                                <a class="time-range" data-time-unit="d" href="#">+/- 1 day</a><br>\
                                <a class="time-range" data-time-unit="h" href="#">+/- 1 hour</a><br>\
                                <a class="time-range" data-time-unit="m" href="#">+/- 1 minute</a><br>\
                                <a class="time-range" data-time-unit="s" href="#">+/- 1 second</a><br>\
                                <a class="time-range" data-time-unit="ms" href="#">+/- 1 milliseconds</a><br>\
                            </td>\
                        </tr>\
                    </tbody>\
                </table>\
            '
        });
    }
);
