define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/shared/PopTart',
        'views/shared/eventsviewer/shared/fieldactions/Master',
        'splunk.util',
        'uri/route'
    ], 
    function($, _, Backbone, module, PopTartView, FieldActions, util, route) {
        return PopTartView.extend({
           /**
            * @param {Object} options {
            *      model: {
            *         application: <models.Application>,
            *         summary: <models.services.search.jobs.SummaryV2>
            *         field: <models.services.search.jobs.SummaryV2.fields[i]
            *     },
            *     collection: <collections.services.data.ui.WorkflowActions>
            * } 
            */
            className: 'dropdown-menu',
            moduleId: module.id,
            initialize: function(){
                PopTartView.prototype.initialize.apply(this, arguments);
                this.children.editTagsActivator = new FieldActions({ 
                    model: this.model,
                    fieldValue: this.options.fieldValue,
                    fieldName: this.options.fieldName
                });
                this.isFieldAction =  (this.options.fieldName && this.options.fieldValue);
                this.children.editTagsActivator.on('hide', this.remove, this);
            },
            events: {
                'click a.actions': function(e) {
                    var m, obj = {}, 
                        index         = $(e.target).data('offset'),
                        model         = this.collection.at(index),
                        isInEventMenu = (model.entry.content.get('display_location') == 'event_menu'),
                        event         = $.extend({}, this.model.event.toJSON()),
                        sid           = this.model.searchJob.get('id'),
                        offset        = this.model.result.offset(this.options.index),
                        namespace     = this.model.application.get('app'),
                        latest_time   = this.model.event.get('_time');

                    _(model.entry.content.toJSON()).each(function(value, k){ 
                        obj[k] = (typeof value == 'string') ?
                            model.fieldSubstitute(model.systemSubstitute(value, sid, offset, namespace, latest_time, this.options.fieldName, this.options.fieldValue), event) :value; 
                    },this);
                    m = new Backbone.Model(obj);
                    this.hide();
                    return (model.entry.content.get('type') === 'link') ? this.link(m): this.search(m);
                }
            },
            uri: function(uri) {
                return uri.indexOf('/') === 0 ? route.encodeRoot(this.model.application.get('root'), this.model.application.get('locale')) + uri : uri;
            },
            link: function(model) {
                var uri = this.uri(model.get('link.uri'));
                if (model.get('link.method').toLowerCase() === 'get') {
                    if(model.get('link.target') === 'self') {
                        window.location.href = uri;
                    } else {
                        window.open(uri, '_blank');
                    }
                    return true;
                }
                var $form = $('<form class="workflow-action"/>');
                $form.attr('target', '_'+model.get('link.target'));
                $form.attr('action', uri);
                $form.attr('method', 'post');
                _(model.get('link.payload')).each(function(v, k) {
                    $form.append($('<input/>').attr({
                        'type': 'hidden',
                        'name': k,
                        'value': v
                    }));        
                }, this);
                $('body').append($form);
                $('form.workflow-action').submit().remove();
                return false;
            },
            search: function(model){
                var options = {data: {}};
                if (util.normalizeBoolean(model.get('search.preserve_timerange'))) {
                    if(this.model.report.entry.content.get('dispatch.earliest_time'))
                        options.data.earliest = this.model.report.entry.content.get('dispatch.earliest_time');
                    if(this.model.report.entry.content.get('dispatchu.latest_time'))
                        options.data.latest = this.model.report.entry.content.get('dispatch.latest_time');
                } 
                if (model.get('search.earliest') || model.get('search.latest')) {
                    options.data.earliest = model.get('search.earliest');
                    options.data.latest = model.get('search.latest');
                }
                options.data.q = model.get('search.search_string');
                var url = route.page(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),  
                    model.get('eai:appName'), 
                    this.model.application.get('page'),
                    options
                );
                if(model.get('search.target') === 'self'){
                    window.location.href = url;
                } else {
                    window.open(url, '_blank');
                }
                return false;
            },    
            render: function() {
                this.$el.html(PopTartView.prototype.template_menu);
                this.$el.append(this.compiledTemplate({  
                    collection: this.collection,
                    actions: this.options.actions,
                    event: $.extend({}, this.model.event.toJSON()),
                    isField: this.isFieldAction,
                    trim: util.smartTrim,
                    _: _
                }));
                
                if(this.options.tags)
                    this.$('ul').prepend(this.children.editTagsActivator.render().el);
                return this;
            },
            template: '\
                <ul>\
                    <% _.each(actions, function(i) { %>\
                        <% var model = collection.at(i); %>\
                        <% if(model) {%>\
                            <li>\
                                <a class="actions" href="#" data-offset="<%-i%>"> <%- trim(_(model.fieldSubstitute(model.entry.content.get("label"), event)).t(), 100)  %></a>\
                            </li>\
                        <% } %>\
                    <% }); %>\
                </ul>\
            '
        });
    }
);

