define(
    [
        'underscore',
        'module',
        'views/Base',
        'models/SplunkDBase',
        'models/services/saved/FVTags',
        'views/shared/eventsviewer/shared/fieldactions/TagDialog'
    ],
    function(_, module, BaseView, SplunkDModel, FVTags, TagDialog) {
        return BaseView.extend({
           moduleId: module.id,
           /**
            * @param {Object} options {
            *      model: {
            *         application: <models.Application>,
            *         summary: <models.services.search.jobs.SummaryV2>
            *     },
            *     collection: <collections.UIWorkflowActions>
            * }
            */
            tagName: 'li',
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.model.tags = new FVTags();
                this.fieldName = this.options.fieldName; 
                this.fieldValue = this.options.fieldValue;
            },
            events: {
                'click .editTag': function(e) {
                    this.trigger('hide');
                    
                    this.model.tags.entry.content.set('name', this.fieldName + '=' + this.fieldValue);
                    this.children.tagDialog = new TagDialog({
                        model: {
                            tags: this.model.tags,
                            application: this.model.application
                        },
                        onHiddenRemove: true
                    });
                    
                    this.children.tagDialog.on('tags_saved', function() {
                        this.model.result.setTags(this.fieldName, this.fieldValue, this.model.tags.entry.content.get('tags'));
                    },this);

                    this.model.tags.setId(
                        this.model.application.get('app'),
                        this.model.application.get('owner'),
                        this.fieldName, this.fieldValue
                    );

                    this.model.clonedTags = new SplunkDModel();
                    this.model.clonedTags.setFromSplunkD(this.model.tags.toSplunkD());
                    this.model.clonedTags.id = this.model.tags.id;
                    this.model.clonedTags.fetch({
                        success: function() {
                            $("body").append(this.children.tagDialog.render().el);
                            this.model.tags.setFromSplunkD(this.model.clonedTags.toSplunkD());
                            this.model.tags.entry.content.set('ui.tags',FVTags.tagArraytoString(this.model.tags.entry.content.get('tags')));
                            this.children.tagDialog.show();
                        }.bind(this),
                        error: function() {
                            $("body").append(this.children.tagDialog.render().el);
                            this.model.tags.setFromSplunkD(this.model.clonedTags.toSplunkD());
                            this.model.tags.unset('id');
                            this.children.tagDialog.show();
                        }.bind(this)
                    });
                    e.preventDefault();
                }
            },
            render: function() {
                this.el.innerHTML = this.compiledTemplate({_:_});
                return this;
            },
            template: '\
                    <a href="#" class="editTag"><%- _("Edit Tags").t() %></a>\
            '
        });
    }
);

