define(
    [
        'module',
        'underscore',
        'splunk.i18n',
        'splunk.util',
        'util/time_utils',
        'jquery.ui.draggable',
        'models/Base',
        'views/Base',
        'views/shared/delegates/Modalize',
        'views/shared/delegates/TableDock',
        'views/shared/eventsviewer/table/TableHead',
        'views/shared/eventsviewer/table/body/Master'
    ],
    function(module, _, i18n, util, time_utils, undefined, BaseModel, BaseView, Modalize, TableDock, TableHeadView, TableBodyView){
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>,
             *         state: <models.BaseV2> (optional)
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions> 
             *     },
             *     selectableFields: true|false,
             *     headerMode: dock|none (eventually this will have static mode),
             *     allowRowExpand: true|false
             */
            className: 'scrolling-table-wrapper',
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                
                this.drag = {};
                
                this.collection.intersectedFields = this.collection.selectedFields.deepClone();                

                this.model.state = this.model.state || new BaseModel();
                this.children.modalize = new Modalize({el: this.el, tbody: '> table > tbody'});

                this.children.head = new TableHeadView({
                    model: {
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: { 
                        intersectedFields: this.collection.intersectedFields,
                        selectedFields: this.collection.selectedFields
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand,
                    isRealTime: this.model.searchJob.entry.content.get('isRealTimeSearch')
                });

                this.children.body = new TableBodyView({
                    model: {
                        result: this.model.result,
                        state: this.model.state,
                        summary: this.model.summary,
                        searchJob: this.model.searchJob,
                        report: this.model.report,
                        application: this.model.application
                    },
                    collection: { 
                        workflowActions: this.collection.workflowActions,
                        intersectedFields: this.collection.intersectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        selectedFields: this.collection.selectedFields
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand
                });
                
                this.model.state.on('init-draggable', this.initDraggable, this);
                
                if (this.options.headerMode==='dock') {
                    this.children.tableDock = new TableDock({ el: this.el, offset: 36, defaultLayout: 'fixed'});
                }
                
                var modalize = _.debounce(function(idx, optArg) { this.modalize(idx, optArg); }.bind(this), 0);
                
                this.model.state.on('change:modalize', function(model) {
                    if(this.children.tableDock) 
                        _.isNumber(model.get('modalize')) ? this.children.tableDock.disable() : this.children.tableDock.enable();
                    modalize(this.model.state.get('modalize'), 'table');
                },this);
                
                this.children.modalize.on('unmodalize', function(idx) {
                    this.model.state.trigger('unmodalize'+idx);
                    this.children.tableDock.enable();
                    this.updateIntersectedFields();
                },this);

                this.collection.selectedFields.on('reset add remove', function(model, collection, index) {
                    if(_.isNumber(this.model.state.get('modalize'))) {
                        return;
                    }
                    if(model.get('name') === this.model.report.entry.content.get('display.events.table.sortColumn')) {
                        this.model.report.entry.content.set({
                            'display.events.table.sortColumn': '',
                            'display.events.table.sortDirection': ''
                        }); 
                    }
                    this.updateIntersectedFields();
                    modalize(this.model.state.get('modalize'), 'table');
                }, this);
                
                this.model.result.results.on('reset', function() {
                    if(!_.isNumber(this.model.state.get('modalize'))) {
                        this.updateIntersectedFields();
                    }
                }, this);

                this.collection.intersectedFields.on('reset', this.updateDock,this); 

                this.model.report.entry.content.on('change:display.events.rowNumbers', function() { 
                    this.$('table:not(.table-embed)').toggleClass('hide-line-num');
                }, this);
                
                this.model.report.entry.content.on('change:display.events.table.wrap', this.updateDock, this);
                
                this.model.report.entry.content.on('change:display.page.search.showFields change:display.prefs.events.offset', this.updateDock, this);
                
                if (this.options.selectableFields && this.children.tableDock) {
                    this.children.tableDock.on('updated', this.initDraggable, this);
                }
                
                this.children.body.on('rows:pre-remove', function() { this.$el.css('minHeight', this.$el.height()); },this);
                this.children.body.on('rows:added', function() { this.$el.css('minHeight', ''); },this);

            },
            updateIntersectedFields: function() {
                var fields = [], models = [];
                if(!!this.model.result.results.length){
                    _.each(_(this.collection.selectedFields.pluck('name')).intersection(this.model.result.fields.pluck('name')), function(value) {
                        models.push({name: value});
                    });
                    this.collection.intersectedFields.reset(models); 
                }
            },
            remove: function() {
                BaseView.prototype.remove.apply(this, arguments);
                $(window).off('.'+this.cid);
            },
            reflow: function() {
                BaseView.prototype.reflow.apply(this, arguments);
                if(this.isAttachedToDocument() && this.children.head.$el.is(':visible')) {
                    this.updateDock();
                }

            },
            style: function() {
                var maxWidth=this.$el.width();
                return "#"+this.cid+"-table .table-expanded{max-width:" + (maxWidth ? maxWidth - 80 : 500) + "px}";
            
            },
            modalize: function(idx, optArg) {
                this.children.modalize.show(idx, optArg);
            },
            render: function() {
                this.$el.html(this.compiledTemplate({
                     hidelinenums: !util.normalizeBoolean(this.model.report.entry.content.get("display.events.rowNumbers")),
                     cid: this.cid
                }));

                this.$('> table.events-results').append(this.children.head.render().el).append(this.children.body.render().el);
                return this;
            },
            initDraggable: function() {
                if (this.options.selectableFields) {
                    this.drag.$theads = this.$el.find('.reorderable');
                    //TO DO!!!!!!!!
                    //1) Needs to add draggables to the dockable header;
                    //2) Needs to get called after table header, body and header dock are fully rendered;
                    this.drag.$theads.draggable({
                        helper: this.dragHelper.bind(this),
                        start: this.startDrag.bind(this),
                        stop: this.stopDrag.bind(this),
                        drag: this.dragged.bind(this), 
                        containment: this.el,
                        distance: 5,
                        scroll: true
                    });
                }
            },
            updateDock: function() {
                if (this.children.tableDock) {
                    this.children.tableDock.update();
                }
            },
            dragHelper: function(e, ui) {
                this.drag.$th = $(e.currentTarget).addClass('moving');
                this.drag.thOffsetTop = this.drag.$th.offset().top;
                this.drag.containerLeft = this.$el.position().left;
                this.drag.$insertionCursor = this.$('.table-insertion-cursor');
                    
                this.drag.$insertionCursor.show();
                this.drag.$helper = $("<div class='reorderable-helper'><span class='reorderable-label'>" + this.drag.$th.text() + "</span></div>").appendTo(this.$el);
                this.drag.$helper.width(this.drag.$th.width());
                this.drag.$helper.css('marginRight', -this.drag.$th.width() + 'px');
                this.findInsertionPoints();
                return this.drag.$helper[0];
            },
            findInsertionPoints: function() {
                this.drag.insertionPoints = [];
                var $headers = this.$('> table > thead > tr > th.reorderable');
                var originalEl = $headers.filter('[data-name=' + this.drag.$th.data('name') + ']')[0]; //this compensates for the possibility of dragging the docked clone
                var originalIndex = this.drag.originalIndex = $headers.index(originalEl);
                
                $headers.each(function(index, el) {
                    var $el = $(el),
                        left = $el.position().left + this.$el.scrollLeft(); 
                        
                    if (index < originalIndex ) { 
                        this.drag.insertionPoints.push({left: left, index: index});
                    } else if (index == originalIndex ) { 
                        this.drag.insertionPoints.push({left: left, index: index});
                        this.drag.insertionPoints.push({left:left + $el.outerWidth(), index: index});
                    } else {
                        this.drag.insertionPoints.push({left:left + $el.outerWidth(), index: index});
                    }
                }.bind(this));
            },
            findClosestInsertion: function(e, ui) {
                if(ui.helper.offset().top - this.drag.thOffsetTop > 100) {
                    return -1;
                } else {
                    var closest = -1,
                        closestDistance = 10000000,
                        cursorLeft = (e.pageX - this.drag.containerLeft) + this.$el.scrollLeft();
                        
                        $.each(this.drag.insertionPoints, function name(index, point) {
                            var distance = Math.abs(point.left - cursorLeft);
                            if (distance < closestDistance) {
                                closest = point;
                                closestDistance = distance;
                            }   
                        });
                   return closest;
                }
            },
            startDrag: function(e, ui){
                    //TO DO!!!!!!!!
                    //need to stop rendering;
            }, 
            stopDrag: function(e, ui){
                var closest = this.findClosestInsertion(e, ui),
                    movingModel = this.collection.selectedFields.findWhere({'name': this.drag.$th.data().name});
                if (closest == -1) {
                    this.collection.selectedFields.remove(movingModel);
                } else if (closest.index !== this.drag.originalIndex) {
                    this.collection.selectedFields.remove(movingModel, {silent: true});
                    this.collection.selectedFields.add(movingModel, {at: closest.index});
                }
                this.drag.$th.removeClass('moving');
                this.drag.$insertionCursor.hide();
            },
            dragged: function(e, ui) {
                var closest = this.findClosestInsertion(e, ui);
                if (closest === -1) { 
                    this.drag.$insertionCursor.hide();
                    ui.helper.addClass("reorderable-remove");
                } else {
                    this.drag.$insertionCursor.show().css('left', closest.left);
                    ui.helper.removeClass("reorderable-remove");
                }
            },
            template: '\
                <table class="table table-chrome table-striped <% if(hidelinenums){ %> hide-line-num <% } %> table-row-expanding events-results events-results-table" id="<%= cid %>-table"></table>\
                <div class="table-insertion-cursor"></div>\
            '
        });
    }
);
