define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'models/Base',
        'views/Base',
        'views/shared/eventsviewer/table/body/PrimaryRow',
        'views/shared/eventsviewer/table/body/SecondaryRow',
        'util/console'
    ],
    function($, _, Backbone, module, BaseModel, BaseView, PrimaryRow, SecondaryRow, console){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tbody',
            type: 'table',
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions> 
             *     },
             *     selectableFields: true|false
             * } 
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.model.result.results.on('reset', this.debouncedRender, this);
                this.model.state.on('change:sleep', function() {
                    this.model.state.get('sleep') ? this.sleep({lite: true}) : this.wake({lite: true});
                }, this);
                this.collection.intersectedFields.on('reset', this.debouncedRender, this); 
            },
            sleep: function(options) {
                (options && options.lite) ? (this.awake = false) : BaseView.prototype.sleep.apply(this);
                return this;
            },
            wake: function(options) {
                if(options && options.lite){
                    this.awake = true;
                    if (this.touch) {
                        this.render();
                    }
                } else {
                    BaseView.prototype.wake.apply(this);
                }
                return this;
            },
            isSelected: function() {
                return (this.model.report.entry.content.get('display.events.type') === this.type);
            },
            cleanup: function() {
                 this.$el.empty();
                 _.chain(this.children).values().invoke('debouncedRemove', {detach: true}).values();
                 this.children = {};
            },
            render: function() {
                this.trigger('rows:pre-remove');
                this.cleanup();
                var fragment = document.createDocumentFragment(),
                    isRealTimeSearch = this.model.searchJob.entry.content.get('isRealTimeSearch'),
                    results = isRealTimeSearch ? this.model.result.results.reverse({mutate: false}) : this.model.result.results.models;

                console.debug('Events Table: rendering', results.length, 'events', isRealTimeSearch ? 'in real-time mode' : 'in historical mode');
                _.each(results, function(event, idx) {
                    var lineNum;

                    if (this.model.searchJob.entry.content.get('isRealTimeSearch')) {
                        lineNum = this.model.result.endOffset() - idx;
                    } else {
                        lineNum = this.model.result.get('init_offset') + idx + 1;
                    }

                    this.children['masterRow_' + idx] = new PrimaryRow({ 
                        model: { 
                            event : event, 
                            report: this.model.report,
                            application: this.model.application,
                            searchJob: this.model.searchJob,
                            result: this.model.result,
                            state: this.model.state
                        }, 
                        collection: {
                            eventRenderers: this.collection.eventRenderers,
                            selectedFields: this.collection.intersectedFields
                        },
                        lineNum: lineNum,
                        idx: idx,
                        allowRowExpand: this.options.allowRowExpand
                    });
                    fragment.appendChild(this.children['masterRow_' + idx].render().el);
                    
                    this.children['fieldRow_' + idx] = new SecondaryRow({
                        model: { 
                            event : event,
                            report: this.model.report,
                            result: this.model.result,
                            summary: this.model.summary,
                            state: this.model.state,
                            application: this.model.application,
                            searchJob: this.model.searchJob
                        }, 
                        collection: {
                            workflowActions: this.collection.workflowActions,
                            selectedFields: this.collection.selectedFields
                        },
                        idx: idx,
                        selectableFields: this.options.selectableFields
                    });
                    fragment.appendChild(this.children['fieldRow_' + idx].render().el);
                },this);
                this.el.appendChild(fragment);
                
                //bulk purge of remove mutex
                _(this.model.state.toJSON()).each(function(value, key) {
                    if(key.indexOf('pendingRemove') === 0) {
                        this.model.state.unset(key);
                    }
                },this);
                
                this.trigger('rows:added');
                this.reflow();
                return this;
            },
            reflow: function() {
                this.model.state.trigger('init-draggable');
            }
        });
    }
);
