define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'models/services/search/IntentionsParser',
        'views/Base',
        'splunk.util'
    ],
    function($, _, Backbone, module, IntentionsParser, BaseView, splunkutil){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            /**
             * @param {Object} options {
             *     model: { 
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *         report: <models.services.SavedSearch>,
             *         state: <models.Base>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *     }
             *     
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.$el.addClass((!!(this.options.idx%2))?'even':'odd');
                this.model.renderer = this.collection.eventRenderers.getRenderer(this.model.event.get('eventtype'));
               
                //suite of namespaced keys
                this.rowExpanded = 'rowExpanded' + this.options.idx;
                this.jsonInteraction = 'interaction' + this.options.idx;
                this.swappingKey = 'swapResult' + this.options.idx;
                this.showAllLines = 'showAllLines' + this.options.idx;
                this.pendingRemove = 'pendingRemove' + this.options.idx;
                
                this.model.intentionsParser = new IntentionsParser(); 
                this.model.intentionsParser.on('change', function() {
                    this.model.report.entry.content.set('search', this.model.intentionsParser.fullSearch());
                    this.model.report.trigger('eventsviewer:drilldown');
                }, this);
                this.model.report.entry.content.on('change:display.events.table.drilldown', this.render, this);
                this.model.state.on('change:'+this.rowExpanded , this.debouncedRender, this);
                
                this.model.state.on(this.jsonInteraction, this.clickToExpandRow, this);
                this.model.state.on('change:'+this.showAllLines, function() {
                    this.clickToExpandRow();
                }, this); 
                this.model.state.on('unmodalize'+this.options.idx, function() {
                    this.clickToExpandRow(true); 
                }, this);
                
                this.collection.selectedFields.on('reset add remove', this.render, this);
                this.model.report.entry.content.on('change:display.prefs.events.offset', this.collapseState, this);
                this.model.report.entry.content.on('change:display.events.list.wrap', function () {
                    var wrap = splunkutil.normalizeBoolean(this.model.report.entry.content.get("display.events.list.wrap")),
                        $cells = this.$('a.field-val').parents('td');
                   (wrap) ? $cells.removeClass('no-wrap'):  $cells.addClass('no-wrap');
                },this);
            },
            events: {
                'click td.expands': function(e) {
                    if (this.model.state.get(this.pendingRemove)) {
                        if(this.model.state.get(this.rowExpanded)) {
                            this.collapseState();
                        } else {
                            this.expandState();
                        }
                        e.preventDefault();
                        return;
                    }
               
                    this.clickToExpandRow(true);
                    e.preventDefault();
                },
                'click ._time-drilldown > a': function(e) {
                    e.preventDefault(); // handled by the cell.
                },
                'click ._time-drilldown': function(e) {
                    e.preventDefault();
                    this.drilldown($(e.currentTarget).find('a'));
                },
                'click .one-value-drilldown > a.field-val': function(e) {
                    e.preventDefault(); // handled by the cell.
                },
                'click .one-value-drilldown': function(e) {
                    e.preventDefault();
                    this.drilldown($(e.currentTarget).find('a'));
                },
                'click .multi-value-drilldown > a.field-val': function(e) {
                    e.preventDefault();
                    this.drilldown($(e.currentTarget));
                },
                'keypress td ': function(e) {
                    e.preventDefault();
                    if(e.which === 13) {
                        this.drilldown($(e.currentTarget).find('a'));
                    }
                } 
            },
            clickToExpandRow: function(isRowClick) {
                var options = {
                    'index':  this.options.idx,
                    'rtindex': this.model.result.results.length - this.options.idx - 1,
                    'showAllLines': this.model.state.get(this.showAllLines) 
                };
                if(!this.model.state.get(this.rowExpanded)) {
                    this.expandState();
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                
                // Collapse row click
                } else if(this.model.state.get(this.rowExpanded) && isRowClick) {
                    this.model.state.unset(this.showAllLines, {silent: true}); // force hide all lines
                    options.showAllLines = this.model.state.get(this.showAllLines); //update options
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                    this.collapseState();
                //interaction with show/hide more lines
                } else {
                    this.model.result.stopReset = true;
                    this.model.result.trigger('swap-single-event ' + this.swappingKey, options);
                }
                this.model.state.unset('timeExpanded', {silent: true});                    
            },
            debouncedRemove: function() {
                this.model.state.set(this.pendingRemove, true);
                BaseView.prototype.debouncedRemove.apply(this, arguments);
                return this;
            },
            drilldown: function($target) {
                var data  = $target.data();
                
                if(data.timeIso) {
                    if(splunkutil.normalizeBoolean(this.model.report.entry.content.get('display.events.table.drilldown'))){
                        var epoch = splunkutil.getEpochTimeFromISO(data.timeIso);
                        this.model.report.entry.content.set({
                            'dispatch.earliest_time': epoch,
                            'dispatch.latest_time': '' + (parseFloat(epoch) + 1)
                        });
                        this.model.report.trigger('eventsviewer:drilldown');
                    }
                  
                } else {

                    var field = data.name,
                        value = $.trim($target.text());
                    this.model.intentionsParser.clear({silent: true});
                    this.model.intentionsParser.fetch({
                        data: {
                            q: this.model.report.entry.content.get('search'),
                            action: 'fieldvalue', 
                            field: field,
                            value: value,
                            app: this.model.application.get('app'),
                            owner: this.model.application.get('owner')
                        }
                    });
                }
            },
            expandState: function() {
                this.model.state.set(this.rowExpanded, true);
                this.model.state.set('sleep', true);
            },
            collapseState: function() {
                this.model.state.unset('modalize');
                this.model.state.unset(this.rowExpanded);
                this.model.state.unset('sleep');
            },
            render: function() {
                var root = this.el;
                //rows are read only (innerHTML) for ie
                this.$el.find('> td').each(function(key, element) {
                    root.removeChild(element);
                });
                this.$el.append(this.compiledTemplate({
                    $: $,
                    _:_,
                    event: this.model.event,
                    lineNum: this.options.lineNum,
                    expanded: this.model.state.get(this.rowExpanded),
                    drilldown: splunkutil.normalizeBoolean(this.model.report.entry.content.get('display.events.table.drilldown')),
                    application: this.model.application,
                    selectedFields: this.collection.selectedFields,
                    colorClass: (this.model.renderer) ? this.model.renderer.entry.content.get('css_class'): '',
                    formattedTime: this.model.event.formattedTime(),
                    allowRowExpand: this.options.allowRowExpand,
                    wrap: splunkutil.normalizeBoolean(this.model.report.entry.content.get('display.events.table.wrap')),
                    splunkutil: splunkutil
                }));
                return this;
            },
            reflow: function() {
               (this.model.state.get(this.rowExpanded)) ?
                    this.model.state.set('modalize', this.options.idx):
                    this.model.state.unset('modalize');
            },
            template: '\
                <% if (allowRowExpand) { %>\
                    <td <%if(expanded) {%>rowspan=2<%}%> class="expands <%- colorClass %>">\
                        <a href="#"><i class="icon-triangle-<%- expanded ? "down" : "right" %>-small"></i></a>\
                    </td>\
                <% } %>\
                <td class="line-num"><span><%- lineNum %></span></td>\
                <td class="_time <%= drilldown ? "_time-drilldown" : ""  %>" tabindex="0">\
                    <% if(drilldown) { %>\
                        <a data-time-iso="<%- event.get("_time") %>">\
                    <% } else { %>\
                        <span data-time-iso="<%- event.get("_time") %>">\
                    <% } %>\
                        <% if(application.get("locale").indexOf("en") > -1){ %>\
                             <span><%- $.trim(formattedTime.slice(0, formattedTime.indexOf(" "))) %></span>\
                             <br>\
                             <span><%- $.trim(formattedTime.slice(formattedTime.indexOf(" "))) %></span>\
                        <% } else { %>\
                             <%- formattedTime %>\
                        <% } %>\
                    <% if(drilldown) { %>  </a>  <% } else { %> </span> <% } %>\
                </td>\
                <% selectedFields.each(function(model) { %>\
                    <% var fields = event.get(model.get("name")); %>\
                    <% if (drilldown) { %>\
                        <td class="<% if(!wrap) { %>no-wrap<% } %> <%= drilldown && fields && fields.length > 1 ? "multi-value-drilldown" : ""  %> <%= drilldown && fields && fields.length == 1 ? "one-value-drilldown" : ""  %>"  tabindex="0"><% _(fields).each(function(field) { %><a class="field-val" data-name="<%- model.get("name") %>"><%- field %></a><% }) %></td>\
                    <% } else { %>\
                        <td class="<% if(!wrap) { %>no-wrap<% } %>"  tabindex="0"><% _(fields).each(function(field) { %><span class="field-val" data-name="<%- model.get("name") %>"><%- field %></span><% }) %></td>\
                    <% } %>\
                <% }) %>\
            '
        });
    }
);  
