 define(
    [
        'module',
        'underscore',
        'views/Base',
        'views/shared/eventsviewer/shared/EventFields',
        'views/shared/eventsviewer/shared/RawField',
        'splunk.util'
    ],
    function(module, _, BaseView, EventFields, RawField,  util){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            className: 'field-row',
            /**
             * @param {Object} options {
             *      model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         event: <models.services.search.job.ResultsV2.result[i]>,
             *         summary: <model.services.search.job.SummaryV2>
             *         state: <models.Base>,
             *         application: <models.Application>
             *     }
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         workflowActions: <collections.services.data.ui.WorkflowActions> 
             *     },
             *     selectableFields: true|false
             * } 
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.$el.addClass((!!(this.options.idx%2))?'even':'odd');
                this.children.eventFields = new EventFields({
                    model: { 
                        event: this.model.event,
                        report: this.model.report,
                        summary: this.model.summary,
                        result: this.model.result,
                        state: this.model.state,
                        application: this.model.application,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        workflowActions: this.collection.workflowActions,
                        selectedFields: this.collection.selectedFields
                    },
                    selectableFields: this.options.selectableFields,
                    idx: this.options.idx
                });
                //event 
                this.children.raw = new RawField({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    noSegmentation: true,
                    idx: this.options.idx
                });
                
                this.rowExpanded = 'rowExpanded' + this.options.idx;
                this.model.state.on('change:'+this.rowExpanded , this.visibility, this);
                
                this.model.result.on('tags-updated', function() { 
                    if(this.model.state.get(this.rowExpanded))  
                        this.render(); 
                }, this);
                
                this.model.state.on('unmodalize'+this.options.idx, this.collapseState, this);
                this.collection.selectedFields.on('reset add remove', function() {
                    if(!_.isNumber(this.model.state.get('modalize')) && this.model.state.get(this.rowExpanded)) {
                        this.$('.event').attr('colspan', this.getCalculatedColSpan());
                    }
                }, this);
            },
            visibility: function() {
                var $arrow =  this.$('td.expands > a > i').removeClass('icon-triangle-right-small icon-triangle-down-small');
                if (this.model.state.get(this.rowExpanded)) {
                    this.$el.css('display', '');
                    $arrow.addClass('icon-triangle-down-small');
                    this.children.raw.wake().$el.show();
                    this.children.eventFields.touch = true;
                    this.children.eventFields.wake({syncRender: true}).$el.show();
                } else {
                    this.$el.hide();
                    $arrow.addClass('icon-triangle-right-small');
                    this.children.raw.sleep().$el.hide();
                    this.children.eventFields.sleep().$el.hide();
                }
                this.reflow();
            },
            getCalculatedColSpan: function() {
                return this.collection.selectedFields.length + 1 +((util.normalizeBoolean(
                    this.model.report.entry.content.get("display.events.rowNumbers"))
                ) ? 1 : 0);
            
            },
            collapseState: function() {
                this.model.state.unset('modalize');
                this.model.state.unset(this.rowExpanded);
                this.model.state.unset('sleep');
            },
            render: function() {
                var root = this.el;
                //rows are read only (innerHTML) for ie
                this.$el.find('> td').each(function(key, element) {
                    root.removeChild(element);
                });
                this.$el.append(this.compiledTemplate({
                    cspan: this.getCalculatedColSpan(), 
                    raw: this.model.event.rawToText()
                }));
                this.$('.event').append(this.children.raw.render().el);
                this.$('.event').append(this.children.eventFields.render().el);
                this.visibility();
                return this;
            },
            reflow: function() {
               (this.model.state.get(this.rowExpanded)) ?
                    this.model.state.set('modalize', this.options.idx):
                    this.model.state.unset('modalize');
                    
                BaseView.prototype.reflow.apply(this, arguments);
            },
            template: '\
                <td class="event" colspan="<%- cspan %>"></td>\
            '
        });
    }
);  
