define(
    [
        'underscore',
        'views/shared/BaseWorkflowAction',
        'models/services/saved/FVTags',
        'views/shared/fieldworkflowaction/TagDialog'
    ], function(_, BaseWorkflowAction, FVTags, TagDialog) {
    return BaseWorkflowAction.extend({
        className: 'field-actions dropdown-menu',
        initialize: function(){
            BaseWorkflowAction.prototype.initialize.apply(this, arguments);
            this.collection.on('reset', this.render, this);
            this.tagsModel = new FVTags();
        },
        events: {
            'click .editTag': function(e) {
                this.tagsModel.id = FVTags.buildId(
                    this.model.application.get('app'),
                    this.model.application.get('owner'),
                    this.model.workflowActions.get('fieldName'),
                    this.model.workflowActions.get('fieldValue')
                );

                this.inmem = this.model.workflowActions.clone();

                this.children.tagDialog = new TagDialog({
                    model: {
                        inmem: this.inmem,
                        tags: this.tagsModel,
                        application: this.model.application
                    },
                    onHiddenRemove: true
                });

                this.tagsModel.fetch({
                    success: function() {
                        this.inmem.set({
                            tags: FVTags.tagArraytoString(this.tagsModel.entry.content.get('tags'))
                        });

                        $("body").append(this.children.tagDialog.render().el);
                        this.children.tagDialog.show();

                    }.bind(this),
                    error: function() {
                        this.tagsModel.unset('id');

                        $("body").append(this.children.tagDialog.render().el);
                        this.children.tagDialog.show();

                    }.bind(this)
                });

                e.preventDefault();
            }
        },
        render: function() {
            var $arrow = null;
            if (this.el.innerHTML) {
                 $arrow = this.$('.arrow').detach();
            }
            var template = _(this.template).template({
                rows: this.collection.models,
                fieldName: this.model.workflowActions.get('fieldName'),
                fieldValue: this.model.workflowActions.get('fieldValue')
            });
            this.$el.html(template);

            if ($arrow) {
                this.$('.arrow').replaceWith($arrow);
            }

            return this;
        },
        template: '\
                <div class="arrow"></div>\
                <ul>\
                    <li>\
                        <a href="#" class="editTag">Edit Tags</a>\
                    </li>\
                    <% _(rows).each(function(row) { %>\
                        <li>\
                            <a href="<%- row.get("link.uri")%>" target="<%- row.get("link.target") %>" data-type="<%- row.get("type")%>"> <%- row.get("label") %></a>\
                        </li>\
                    <% }); %>\
                </ul>\
        '
    });
});
