define(
    [
        'underscore',
        'module',
        'models/services/saved/FVTags',
        'views/shared/Modal',
        'views/shared/controls/ControlGroup',
        'views/shared/FlashMessages',
        'bootstrap.modal'
    ],
    function(_, module, FVTags, Modal, ControlGroup, FlashMessage, undefined) {
        return Modal.extend({
            moduleId: module.id,
            initialize: function() {
                Modal.prototype.initialize.apply(this, arguments);

                this.children.flashMessage = new FlashMessage({ model: this.model.tags });

                this.children.field = new ControlGroup({
                    className: 'field-value form-horizontal',
                    controlType: 'Textarea',
                    controlOptions: {
                        modelAttribute: 'nameAndValue',
                        model: this.model.inmem,
                        save: false,
                        placeholder: 'Optional',
                        enabled: false
                    },
                    label: 'Field Value'
                });

                this.children.tags = new ControlGroup({
                    className: 'tags form-horizontal',
                    controlType: 'Textarea',
                    controlOptions: {
                        modelAttribute: 'tags',
                        model: this.model.inmem,
                        save: false,
                        placeholder: ''
                    },
                    label: 'Tag(s)',
                    help: 'Comma or space separated list of tags.'
                });
            },
            events: $.extend({}, Modal.prototype.events, {
                'click .btn-primary': function(e) {
                    var data = {};
                    var tagsArray = FVTags.tagStringtoArray(this.model.inmem.get('tags'));
                    if(this.model.tags.id){
                        //tag already exist for field value pair
                        if(tagsArray.length === 0) {
                            this.model.tags.destroy();
                            this.hide();
                            return;
                        }
                        this.model.tags.resetTags(tagsArray);
                    } else {
                        //tag does not already exist for field value pair
                        this.model.tags.entry.content.set({
                            name: this.model.inmem.get('nameAndValue'),
                            'tag.': tagsArray
                        });

                        data = {
                            app: this.model.application.get('app'),
                            owner: this.model.application.get('owner')
                        };
                    }

                    this.model.tags.save({}, {
                        data: data,
                        success: function(model, response) {
                            this.hide();
                        }.bind(this)
                    });
                }

            }),
            render: function() {
                this.$el.html(Modal.TEMPLATE);

                this.$(Modal.HEADER_TITLE_SELECTOR).html("Edit Tags");

                this.$(Modal.BODY_SELECTOR).prepend(this.children.flashMessage.render().el);

                this.$(Modal.BODY_SELECTOR).append(Modal.FORM_HORIZONTAL);

                this.$(Modal.BODY_FORM_SELECTOR).append(this.children.field.render().el);
                this.$(Modal.BODY_FORM_SELECTOR).append(this.children.tags.render().el);

                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);
                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_SAVE);

                return this;
            }
        }
    );
});
