define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/jobstatus/controls/Progress',
        'views/shared/jobstatus/controls/Cancel',
        'views/shared/jobstatus/controls/Stop',
        'views/shared/jobstatus/controls/PlayPause',
        'views/shared/jobstatus/controls/Reload',
        'views/shared/jobstatus/controls/menu/Master',
        'util/console'
    ],
    function(_, module, BaseView, Progress, Cancel, Stop, PlayPause, Reload, Menu, console) {
        return BaseView.extend({
            moduleId: module.id,
            className: 'pull-right btn-group includes-job-menu',
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);

                if (this.options.showJobMenu !== false) {
                    this.children.menu = new Menu({
                        model: {
                            searchJob: this.model.searchJob,
                            application: this.model.application,
                            appLocal: this.model.appLocal
                        },
                        allowDelete: this.options.allowDelete
                    });
                } else {
                    this.$el.removeClass('includes-job-menu');
                }

                this.children.progress = new Progress({model: this.model.searchJob});
                this.children.playPause = new PlayPause({model: this.model.searchJob});
                //TODO: hiding cancel for now because it doesn't show up in the prototype
                //this.children.cancel = new Cancel({model: this.model});
                this.children.stop = new Stop({model: this.model.searchJob});
                if(this.options.enableReload) {
                    this.children.reload = new Reload({model: this.model.searchJob});
                }

                this.model.searchJob.entry.content.on('change', this.render, this);
            },
            render: function() {
                if (!this.el.innerHTML) {
                    _.each(this.children, function(child) {
                        this.$el.append(child.render().$el);
                    }, this);
                }
                this.model.searchJob.isNew() ? this.$el.hide() : this.$el.show();

                var dynamicChildren = [
                    this.children.playPause,
                    //this.children.cancel,
                    this.children.stop
                ];
                if(this.options.enableReload) {
                    dynamicChildren.push(this.children.reload);
                }
                _.each(dynamicChildren, function(child) {
                        if(child.isActive()) {
                            var wasFocused = child.$el.is(':focus');
                            this.$el.append(child.$el);
                            if(wasFocused) {
                                child.$el.focus();
                            }
                        } else {
                            child.$el.detach();
                        }
                }, this);

                return this;
            }
        });
    }
);
