define(
    [
         'underscore',
         'module',
         'views/Base',
         'views/shared/controls/ControlGroup',
         'views/shared/FlashMessages',
         'views/shared/Modal'
     ],
     function(
         _,
         module,
         Base,
         ControlGroup,
         FlashMessage,
         Modal
     ){
        return Base.extend({
            moduleId: module.id,
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                
                var viz_type = this.model.inmem.getVizType();

                this.model.inmem.setTimeRangeWarnings(this.model.reportPristine);
                this.children.flashMessage = new FlashMessage({ model: this.model.inmem });

                //name
                this.children.title = new ControlGroup({
                    label: _("Title").t(),
                    controlType:'Label',
                    controlOptions: {
                        model: this.model.inmem.entry,
                        modelAttribute: 'name'
                    }
                });

                //viz toggle
                this.children.visualization = new ControlGroup({
                    label: _("Visualization").t(),
                    controlType:'SyntheticRadio',
                    controlOptions: {
                        className: "btn-group btn-group-2 HUHWTF",
                        items: [
                            { value: '1', label: viz_type.label, icon: viz_type.icon, iconSize: '' },
                            { value: '0', label: _('None').t()}
                        ],
                        model: this.model.inmem.entry.content,
                        modelAttribute: 'display.visualizations.show'
                    }
                });
                
                //timeRangePicker toggle
                this.children.timeRangePickerToggle = new ControlGroup({
                    label: _("Time Range Picker").t(),
                    controlType:'SyntheticRadio',
                    controlOptions: {
                        className: "btn-group btn-group-2",
                        items: [
                            { value: '1', label: _('Yes').t() },
                            { value: '0', label: _('No').t() }
                        ],
                        model: this.model.inmem.entry.content,
                        modelAttribute: 'display.general.timeRangePicker.show'
                    }
                });
            },
            events: {
                "click .modal-btn-primary" : function(e) {
                    this.model.inmem.entry.content.set({
                        'request.ui_dispatch_view': this.model.application.get('page')
                    });
                    this.model.inmem.save({}, {
                        success: function(model, response) {
                            this.model.inmem.trigger('saveSuccess');
                        }.bind(this)
                    });

                    e.preventDefault();
                }
            },
            render: function() {
                this.$el.html(Modal.TEMPLATE);

                this.$(Modal.HEADER_TITLE_SELECTOR).html(this.options.title);

                this.$(Modal.BODY_SELECTOR).prepend(this.children.flashMessage.render().el);

                this.$(Modal.BODY_SELECTOR).append(Modal.FORM_HORIZONTAL);

                this.$(Modal.BODY_FORM_SELECTOR).append(this.children.title.render().el);

                if (!this.model.reportPristine.isAlert()) {
                    if (this.options.chooseVisualizationType) {
                        if (this.model.searchJob.isReportSearch()) {
                            this.$(Modal.BODY_FORM_SELECTOR).append(this.children.visualization.render().el);
                        } else {
                            this.model.inmem.entry.content.set('display.visualizations.show', '0');
                        }
                    }

                    this.$(Modal.BODY_FORM_SELECTOR).append(this.children.timeRangePickerToggle.render().el);
                }

                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CANCEL);
                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_SAVE);

                return this;
            }
        });
    }
);
