/**
 * @author sfishel
 *
 * View that encapsulates the header of the results table.
 *
 * Meant to be lightweight so that it can be re-instantiated and rendered each time the table re-renders.
 */

define([
            'jquery',
            'underscore',
            'module',
            'views/Base'
        ],
        function(
            $,
            _,
            module,
            Base
        ) {

    return Base.extend({

        moduleId: module.id,

        tagName: 'thead',

        /**
         * @constructor
         *
         * @param options {Object} {
         *     rowNumbers {Boolean} whether or not to leave an empty column at the left for the row numbers in the table body
         *     fields {Array<String>} the fields to render as the column headers
         *     sortKeyAttribute {String} the HTML attribute to use when encoding the sort-key into <th> tag
         * }
         */

        // constructor not overridden

        render: function() {
            var columnClasses = {};
            _(this.options.columnTypes).each(function(columnType, field) {
                columnClasses[field] = this.columnTypeToClassName(columnType);
            }, this);
            var html = this.compiledTemplate({
                rowNumbers: this.options.rowNumbers,
                fields: this.options.fields,
                sortKeyAttribute: this.options.sortKeyAttribute,
                columnClasses: columnClasses
            });

            this.$el.html(html);
            return this;
        },

        highlightColumn: function(index) {
            if(this.options.rowNumbers) {
                index++;
            }
            this.$('th').eq(index).addClass('highlighted');
        },

        unHighlightColumn: function(index) {
            if(this.options.rowNumbers) {
                index++;
            }
            this.$('th').eq(index).removeClass('highlighted');
        },

        columnTypeToClassName: function(columnType) {
            if(columnType === 'number') {
                return 'numeric';
            }
            if(columnType === 'timestamp') {
                return 'timestamp';
            }
            return '';
        },

        template: '\
            <tr class="">\
                <% if(rowNumbers) { %>\
                    <th class="row-number"></th>\
                <% } %>\
                <% _(fields).each(function(field) { %>\
                    <th class="sorts <%- columnClasses[field] %>" <%- sortKeyAttribute %>="<%- field %>">\
                    <a href="#"><%- field %><i class="icon-sorts "></i></a>\
                    </th>\
                <% }) %>\
            </tr>\
        '

    });

});
