define([
            'jquery',
            'underscore',
            'module',
            'views/Base',
            './renderers/BaseCellRenderer'
        ],
        function(
            $,
            _,
            module,
            Base,
            BaseCellRenderer
        ) {

    var ResultsTableRow = Base.extend({

        moduleId: module.id,

        tagName: 'tr',

        render: function() {
            this.$el.attr(ResultsTableRow.ROW_INDEX_ATTR, this.options.rowIndex);
            if(this.options.hasOwnProperty('rowNumber')) {
                this.$el.append($('<td class="row-number"></td>').text(this.options.rowNumber));
            }
            this.generateCells();
            return this;
        },

        remove: function() {
            this.disposeCells();
            return Base.prototype.remove.call(this);
        },

        generateCells: function() {
            this.disposeCells();

            var renderedCells = this._renderedCells = [];
            var renderedCell;
            var cellRenderers = this.getCellRenderers();
            var cellRenderer;

            _(this.options.cells).each(function(cellData) {
                var $td = $("<td></td>");
                $td.attr(ResultsTableRow.CELL_INDEX_ATTR, cellData.index);

                renderedCell = { $td: $td };

                for (var i = 0, l = cellRenderers.length; i < l; i++) {
                    cellRenderer = cellRenderers[i];
                    if (cellRenderer instanceof BaseCellRenderer) {
                        if (cellRenderer.canRender(cellData)) {
                            cellRenderer.setup($td, cellData);
                            cellRenderer.render($td, cellData);
                            renderedCell.cellData = cellData;
                            renderedCell.cellRenderer = cellRenderer;
                            break;
                        }
                    } else if (typeof cellRenderer === "function") {
                        if (cellRenderer($td, cellData)) {
                            break;
                        }
                    }
                }

                this.$el.append($td);
                renderedCells.push(renderedCell);
            }, this);
        },

        disposeCells: function() {
            if (!this._renderedCells)
                return;

            _(this._renderedCells).each(function(renderedCell) {
                if (renderedCell.cellRenderer) {
                    renderedCell.cellRenderer.teardown(renderedCell.$td, renderedCell.cellData);
                }
                renderedCell.$td.remove();
            }, this);

            this._renderedCells = null;
        },

        getCellRenderers: function() {
            if (this.options.table) {
                return this.options.table.getCellRenderers();
            }
            return [];
        },

        highlightRow: function() {
            this.$el.addClass('highlighted');
        },

        unHighlightRow: function() {
            this.$el.removeClass('highlighted');
        },

        // $target could be a cell or a multivalue sub-cell
        highlightCell: function($target, index) {
            var rowNumbersEnabled = this.options.hasOwnProperty('rowNumber'),
                $tds = this.$('td'),
                numRowSplits = this.options.numRowSplits,
                $rowSplitCells = rowNumbersEnabled ? $tds.slice(0, numRowSplits + 1) : $tds.slice(0, numRowSplits);

            $rowSplitCells.addClass('highlighted');
            $target.addClass('highlighted');
        },

        // $target could be a cell or a multivalue sub-cell
        unHighlightCell: function($target, index) {
            var rowNumbersEnabled = this.options.hasOwnProperty('rowNumber'),
                $tds = this.$('td'),
                numRowSplits = this.options.numRowSplits,
                $rowSplitCells = rowNumbersEnabled ? $tds.slice(0, numRowSplits + 1) : $tds.slice(0, numRowSplits);

            $rowSplitCells.removeClass('highlighted');
            $target.removeClass('highlighted');
        }

    },
    {
        ROW_INDEX_ATTR: 'data-row-index',
        CELL_INDEX_ATTR: 'data-cell-index',
        MV_SUBCELL_CLASSNAME: 'multivalue-subcell',
        MV_INDEX_ATTR: 'data-mv-index'
    });

    return ResultsTableRow;

});