define(
    [
        'jquery',
        'underscore',
        'module',
        'views/Base',
        'views/shared/delegates/TextareaResize',
        'models/SHelper',
        'splunk.util',
        'util/dom_utils',
        'jquery.bgiframe',
        'splunk.jquery.csrf'
    ],
    function($, _, module, BaseView, TextareaResize, SHelperModel, splunk_util, dom_utils /* remaining dependencies do not export */) {
        var View = BaseView.extend({
           keys: {
               ENTER : 13,
               UP_ARROW: 38,
               DOWN_ARROW: 40,
               LEFT_ARROW: 37,
               RIGHT_ARROW: 39,
               PAGE_DOWN: 34,
               PAGE_UP: 33,
               SPACE_BAR: 32,
               TAB: 9,
               ESCAPE: 27
           },
           moduleId: module.id,
           id: 'search',
           /**
            * @param {Object} options {
            *     model: <models.Report.entry.content>
            * }
            */
           initialize: function(options) {
               if (!this.children) {
                   BaseView.prototype.initialize.apply(this, arguments);
                   this.debouncedFillAssistant = _.debounce(this.fillAssistant, 250).bind(this);
               }
               
               this.options = $.extend(true, {}, this.options, (options || {}));

               var defaults = {
                   useTypeahead: true,
                   useAssistant: true,
                   useAutoFocus: true,
                   autoOpenAssistant: splunk_util.normalizeBoolean(this.model.get('display.prefs.autoOpenSearchAssistant')),
                   showCommandHelp: true,
                   showCommandHistory: true,
                   showFieldInfo: false,
                   maxSearchBarLines: 80,
                   minWithForTwoColumns: 560,
                   singleton: false,
                   disableOnSubmit: false,
                   assistantDelay: 200
               };
               _.defaults(this.options, defaults);

               this.assistant = {
                   enabled: false,
                   rolloverEnabled: true, //required to override mouseenter function during keyboard scrolling.
                   cursor: 0,
                   rolloverTimer: 0
               };
               this.multiline = false;
               
               var reportContentModel = this.model;
               this.model = {
                    content: reportContentModel,
                    sHelper: new SHelperModel()     
               };
               
               // None Contained Event Binding
               this.model.content.on('change:search', function(){
                   this.setSearchString(this.model.content.get('search') || "");
                   if (this.hasOwnProperty("resize")) {
                       this.children.resize.resizeTextarea();
                       this.updateMultiline();
                   }
               }, this);
               
               this.model.content.on('applied', function(options){
                   this._onFormSubmit(options);
               }, this);
               
               if (this.options.singleton) {
                   this.model.content.on('enableSearchInput', function() {
                       this.$('.search-field').attr('disabled', false);
                   }, this);                   
               }
               
               if (!this.$el.html()) {
                   $(window).on('resize.' + this.uniqueNS(), this.setAssistantWidth.bind(this));
    
                   $(document).on('keyup.' + this.uniqueNS(), function(e) {
                       if (e.keyCode == this.keys['ESCAPE']) {
                           this.closeAssistant();
                       }
                   }.bind(this));
                   
                   $(document).on('click.' + this.uniqueNS(), function(e) {
                       if ((e.target === this.$el[0]) ||($.contains(this.$el[0], e.target))) {
                           return;
                       }
                       this.closeAssistant();
                  }.bind(this));

              }

           },
           events: {
               'click     .search-assistant-autoopen-toggle' :    'toggleAutoOpen',
               'click     .search-assistant-activator' :   'toggleAssistant',
               'focus     .search-field' :                 'onSearchFieldFocus',
               'keydown   .search-field' :                 'onSearchFieldKeyDown',
               'keyup     .search-field' :                 'onSearchFieldKeyUp',
               'keydown   a.sakeyword' :                   'onKeywordKeyDown',
               'keyup     a.sakeyword' :                   'onKeywordKeyUp',
               'keyup     .salink' :                       'onKeywordKeyUp',
               'click     .search-assistant-container a' : 'onSuggestionSelect',
               'mousedown .search-assistant-resize' :       'resizeAssistant',
               //this is a port of the generated javascript from within the shelper mako template
               'click .saMoreLink': function(evt) {
                   var $element = $(evt.target);
                   if ($element.hasClass('saMoreLinkOpen')) {
                       $element.removeClass('saMoreLinkOpen')
                              .html(_.t('More &raquo;'));
                       $($element.attr('divToShow')).css('display','none');   
                   } else {
                       $element.addClass('saMoreLinkOpen')
                              .html(_.t('&laquo; Less'));
                       $($element.attr('divToShow')).css('display', 'block');
                   }
               }
           }, 
           toggleAutoOpen: function(e) {
               this.setAutoOpen(!this.options.autoOpenAssistant);
               e.preventDefault();
           },
           setAutoOpen: function(isEnabled) {
               if (typeof(isEnabled) == 'undefined') {
                   isEnabled = this.options.autoOpenAssistant;
               } else {
                   this.options.autoOpenAssistant = isEnabled;
               }

               if (splunk_util.normalizeBoolean(this.model.content.get('display.prefs.autoOpenSearchAssistant')) !== isEnabled) {
                   this.model.content.set({'display.prefs.autoOpenSearchAssistant': isEnabled});
               }

               this.$assistantAutoOpenToggle.find("i").toggleClass('icon-check', this.options.autoOpenAssistant);
           },
           onSearchFieldFocus: function(e) {
               this.assistant.cursor = - 1;
               this._highlightKeyword(this.assistant.cursor);
               return true;
           },
           onSearchFieldKeyUp: function(e) {
               if (e.metaKey || e.ctrlKey) {
                   return true;
               }
               this.updateMultiline();
               switch (e.keyCode) {
                   case this.keys['ESCAPE']:
                       return true; //don't open the search assistant.
                   case this.keys['ENTER']:
                       return false;
                   default:
                       break;
               }

               if (!this.assistant.enabled) {
                   if (this.options.autoOpenAssistant) {
                       this.openAssistant();
                   }
               } else {
                   this.debouncedFillAssistant();
               }
               this.onSearchFieldChange();

               var searchInput = this.getSearchFieldValue();
               this.model.content.set({search: searchInput}, {silent: true});

               return true;
           },
           onSearchFieldKeyDown: function(e) {
               if (e.metaKey || e.ctrlKey) {
                   return true;
               }
               switch (e.keyCode) {
                   case this.keys['TAB']:
                       this.closeAssistant();
                       break;
                   case this.keys['DOWN_ARROW']:
                       // Left bracket and down arrow register as 40. If the shift key is down, then it must be a bracket.
                       if (!e.shiftKey) return this.onSearchFieldDownArrow(e);
                       break;
                   case this.keys['ENTER']:
                       return this.onSearchFieldEnter(e);
                   default:
                       break;
               }
               return true;
           },
           onSearchFieldDownArrow: function(e) {
               if (this.assistant.enabled && this.$assistantKeywordCount>0 && (!this.children.resize.isMultiline() || this.children.resize.caretIsLast())) {
                   this.selectNextKeyword();
                   return false;
               } else if (!this.assistant.enabled && (this.getSearchFieldValue() =='')) {
                   this.openAssistant();
               }
           },
           updateMultiline: function() {
               // add mulitline class for state style
               var multiline = this.children.resize.isMultiline();

               if (multiline &&( multiline != this.multiline)) {
                 this.$el.addClass('multiline');
                 this.multiline = true;
               } else if (!multiline && (multiline != this.multiline)) {
                 this.$el.removeClass('multiline');
                 this.multiline = false;
               }
           },
           resizeAssistant: function(e) {
               var startY = e.pageY;
               var startHeight = this.$assistantContainer.height();
               e.preventDefault();
               e.stopPropagation();

               this.$assistantResize.on("click.assistantResizeActive",
                   function(e){
                       e.preventDefault();
                       e.stopPropagation();
                       return false;
                   }.bind(this)
               );

               $(document).on("mousemove.assistantResizeActive",
                   function(e){
                       var newHeight = startHeight - (startY - e.pageY);
                       newHeight = newHeight < 75 ? 0 : newHeight;
                       newHeight = Math.min(newHeight, 500);
                       this.setAssistantHeight(newHeight);
                       e.preventDefault();
                       e.stopPropagation();
                       return false;
                   }.bind(this)
               );

               $(document).on("mouseup.assistantResizeActive",
                   function(e){
                       var newHeight = startHeight - (startY - e.pageY);
                       if (newHeight < 75) {
                           this.closeAssistant();
                           this.setAssistantHeight(startHeight);
                       }
                       $(document).off(".assistantResizeActive");
                   }.bind(this)
               );
               return false;
           },
           toggleAssistant: function(e) {
               this.searchFieldfocus();
               if (this.assistant.enabled) {
                   this.closeAssistant();
               } else {
                   this.openAssistant();
               }
               e.preventDefault();
           },
           closeAssistant: function() {
               this.model.sHelper.fetchAbort();
               // Exit early if closeAssistant has been called before render has created all of the views
               if (!this.$assistantContainer)
                    return false;

               this.$assistantContainer.hide();
               this.$assistantAutoOpenToggle.hide();
               this.assistant.enabled = false;
               this.model.sHelper.fetchAbort();
               this.$assistantActivator.addClass("icon-triangle-down-small").removeClass("icon-triangle-up-small");
               this.$assistantResize.removeClass("search-assistant-resize-active");
               this.$el.removeClass('search-assistant-open');
               return true;
           },
           openAssistant: function() {
               if (!this.options.useAssistant) {
                   return false;
               }
               this.assistant.enabled = true;
               this.$assistantActivator.addClass("icon-triangle-up-small").removeClass("icon-triangle-down-small");
               this.$assistantResize.addClass("search-assistant-resize-active");
               this.$el.addClass('search-assistant-open');
               this.fillAssistant();
               return true;
           },
           _getUserEnteredSearch: function() {
               var q = this.$searchField.attr('value') || '*';
               q = splunk_util.addLeadingSearchCommand(q, true);
               return q;
           },
           fillAssistant: function() {
               if (!this.assistant.enabled) return false;
               var searchString = this._getUserEnteredSearch();
               
               //TODO: revisit multi app namespace support
               var namespace    = 'search';//Splunk.util.getCurrentApp();
               
               this.model.sHelper.safeFetch({
                   data: {
                       'snippet': 'true',
                       'snippetEmbedJS': 'false',
                       'namespace': namespace,
                       'search': searchString,
                       'useTypeahead': this.options.useTypeahead,
                       'useAssistant': this.options.useAssistant,
                       'showCommandHelp': this.options.showCommandHelp,
                       'showCommandHistory': this.options.showCommandHistory,
                       'showFieldInfo': this.options.showFieldInfo
                   },
                   success: function() {
                       this.$assistantContainer.html(this.model.sHelper.get('raw') || '');
                       this.fillAssistantCompleteCallback();
                   }.bind(this)
               });
               return true;
           },
           fillAssistantCompleteCallback: function() {
               if (!this.assistant.enabled) {
                   return false;
               }
               this.setAssistantWidth();
               this.$assistantContainer.show().bgiframe().scrollTop(0);
               this.$assistantAutoOpenToggle.show();
               this.setAssistantHeight(this.$assistantContainer.height() || 250);

               this.$assistantKeywordCount = this.$('.sakeyword').length;
               this.assistant.cursor = -1;
               this.searchFieldfocus();

               return true;
           },
           setAssistantHeight: function(newHeight) {
               if (newHeight > 500) newHeight = 500; // make sure we don't go over 500px
               this.$assistantContainer.height(newHeight);
               this.$('.saHelpWrapper').css('min-height', newHeight);
           },
           setAssistantWidth: function() {
               var assistantInner = this.$('.assistantInner');

               if(assistantInner.length && (this.$searchField.width() < this.options.minWithForTwoColumns)) {
                   assistantInner.addClass('assistant-inner-narrow');
               } else {
                   assistantInner.removeClass('assistant-inner-narrow');
               }
               return true;
           },
           onSearchFieldChange: function() {
                if (this.searchFieldIsEmpty()) {
                   this.$label.show();
                   return true;
                } else {
                   this.$label.hide();
                   return false;
                }
           },
           searchFieldIsEmpty: function() {
               return (this.getSearchFieldValue().length == 0);
           },
           getSearchFieldValue: function(){
               return $.trim(this.$searchField.attr('value'));
           },
           onKeywordKeyUp: function(e) {
               if (e.metaKey || e.ctrlKey || e.shiftKey) {
                   return true;
               }
               switch (e.keyCode) {
                   case this.keys['DOWN_ARROW']:
                   case this.keys['UP_ARROW']:
                   case this.keys['TAB']:
                   case this.keys['ENTER']:
                   case this.keys['LEFT_ARROW']:
                   case this.keys['RIGHT_ARROW']:
                   case this.keys['SPACE_BAR']:
                       return false;
                   default:
                       break;
               }

               if (!this.assistant.enabled) {
                   if (this.options.autoOpenAssistant) {
                       this.openAssistant();
                   }
               } else {
                   this.fillAssistant();
               }
               return true;
           },
           selectNextKeyword: function() {
               if (this.assistant.cursor >= (this.$assistantKeywordCount-1)) {
                   this.assistant.cursor = this.$assistantKeywordCount - 1;
               } else {
                   this.assistant.cursor += 1;
                   this._highlightKeyword(this.assistant.cursor);
               }
               return true;
           },
           onSearchFieldEnter: function(e) {

               //ctrl-enter|shift-enter adds line to the textarea
               if (e.ctrlKey || e.shiftKey) {
                   return true;
               }

               if (this.assistant.enabled) {
                   this.closeAssistant();
               }
               this._onFormSubmit();

               return false;
           },

           _highlightKeyword: function(keywordPosition) {
               // set the CSS style for selected
               var el = $('.sakeyword', this.$assistantContainer
                   ).removeClass('saKeywordSelected'
                   ).slice(keywordPosition, keywordPosition + 1
                   ).addClass('saKeywordSelected');

               if (el.length) {
                   // keep selected item in view
                   var win = this.$assistantContainer;
                   var visibleWindowTop = win.scrollTop();
                   var visibleWindowBottom = win.scrollTop() + win.height();
                   var elementTop = el.position().top + visibleWindowTop;

                   var elementHeight = el.outerHeight();
                   if (elementTop < visibleWindowTop) {
                       win.scrollTop(elementTop);
                   } else if (elementTop + elementHeight > visibleWindowBottom) {
                       win.scrollTop(elementTop + elementHeight - win.height());
                   }

                   el.focus();
               }
               return true;
           },
           _onFormSubmit: function(options) {
               options = options || {};
               // don't do anything if there's nothing in the search box
               if (this.searchFieldIsEmpty()) {
                   return false;
               } else {
                   if (this.options.disableOnSubmit) {
                       this.$('.search-field').attr('disabled', true);
                   }
                   var currentSearch = this.model.content.get('search'),
                       searchInput = this.getSearchFieldValue();

                   if (currentSearch !== searchInput){
                       this.model.content.set({ search: searchInput }, options);
                   } else {
                       if (!options.silent) {
                           this.model.content.unset("search", {silent: true});
                           this.model.content.set({search: searchInput});
                       }
                   }

                   return true;
               }
           },
           onKeywordKeyDown: function(e) {
               if (e.metaKey || e.ctrlKey) {
                   return true;
               }
               switch (e.keyCode) {
               case this.keys['DOWN_ARROW']:
                   // Left bracket and down arrow register as 40. If the shift key is down, then it must be a bracket.
                   if (!e.shiftKey) return this.onSearchFieldDownArrow(e);
                   break;
               case this.keys['UP_ARROW']:
                   return this.onKeywordUpArrow(e);
               case this.keys['TAB']:
               case this.keys['ENTER']:
               case this.keys['RIGHT_ARROW']:
               case this.keys['SPACE_BAR']:
                   return this.onSuggestionSelect(e);
               default:
                   break;
               }
               return true;
           },
           onKeywordUpArrow: function(e) {
               if (this.assistant.enabled && (this.$assistantKeywordCount>0)) {
                   this.selectPreviousKeyword();
                   return false;
               }
           },
           selectPreviousKeyword: function() {
               if (this.assistant.cursor <= 0) {
                   this.assistant.cursor = -1;
                   this.searchFieldfocus();
               } else {
                   this.assistant.cursor -= 1;
               }
               this._highlightKeyword(this.assistant.cursor);
               return true;
           },
           onSuggestionSelect: function(e) {
               var newval = $.trim($(e.currentTarget).attr('replacement'));

               if (!this.assistant.enabled || !newval) {
                   return true;
               }

               if (newval.substr(-1) != '=') {
                      newval += ' '; // don't add space after =
               }
               this.setSearchString(newval);
               this.fillAssistant();

               //Set the cursor position to the end
               dom_utils.setCaretPosition(this.$('.search-field').get(0), this.$searchField.attr('value').length);

               e.preventDefault();
           },
           /**
            * Sometimes, like when we're resurrecting a search, we will
            * write our own input value.
            */
           setSearchString: function(terms) {
               this.$searchField.attr('value', terms);
               this.onSearchFieldChange();
               this.searchFieldfocus();
               if (this.children.resize) {
                   this.children.resize.resizeTextarea();
                   this.updateMultiline();
               }
           },
           searchFieldfocus: function() {
               if (!this.$('.search-field').attr('disabled')) {
                   this.$('.search-field').focus();
               }
           },
           render: function() {
               var html = this.$el.html();
               if ((this.options.singleton && !html) || !this.options.singleton) {
                   var self = this,
                       inputValue = this.model.content.get('search') || "";

                   var template = _.template(this.template, {
                       showButton: true,
                       label: 'Search',
                       id: Math.random() * 600000,
                       inputValue: inputValue,
                       _: _
                   });

                   this.$el.html(template);

                   // Setup shortcuts
                   this.$label = this.$('label');
                   this.$assistantWrapper = this.$('.search-assistant-wrapper');
                   this.$assistantContainer = this.$('.search-assistant-container');
                   this.$assistantAutoOpenToggle = this.$('.search-assistant-autoopen-toggle');
                   this.$assistantActivator = this.$('.search-assistant-activator');
                   this.$searchField = this.$('.search-field');
                   this.$assistantResize = this.$('.search-assistant-resize');

                   this.setAutoOpen();
                   this.paste = function() {
                       this.onSearchFieldChange();
                   };
                    
                    this.$('.search-field').bind('input propertychange', function() { this.paste(); }.bind(this));

                   _.defer(function(){
                       if (self.options.useAutoFocus) {
                           self.searchFieldfocus();
                       }
                       var maxLines = Math.floor(($(window).height() - 100) / parseInt(this.$('.search-field').css('lineHeight'), 10));
                       self.children.resize = new TextareaResize({el: self.$searchField[0], maxLines: maxLines, minHeight: 20});
                       self.updateMultiline();
                   });

                   if (!this.options.useAssistant) {
                       this.$assistantWrapper.hide();
                   }

                   this.onSearchFieldChange();
               } else {
                   this.$('.search-field').val(this.model.content.get('search') || '');
               }
               
               if (this.options.disableOnSubmit) {
                   this.$('.search-field').attr('disabled', false);
               }
               
               return this;
           },
           reflow: function() {
               var el = this.$('.search-field').get(0),
                   inputValue = this.model.content.get('search') || "",
                   currentCaretPos = dom_utils.getCaretPosition(el);
               dom_utils.setCaretPosition(el, (currentCaretPos || inputValue.length));
               BaseView.prototype.reflow.apply(this, arguments);
           },
           remove: function() {
               if (!this.options.singleton) {
                   $(window).off('resize.' + this.uniqueNS());
                   $(document).off('keyup.' + this.uniqueNS());
                   $(document).off('click.' + this.uniqueNS());
                   $(document).on('mousemove.assistantResizeActive');
                   $(document).on('mouseup.assistantResizeActive'); 
                   this.$('.search-field').off('input propertychange', this.paste);
                   return BaseView.prototype.remove.apply(this, arguments);
               }
               this.model.sHelper.fetchAbort();
               return this;
           },
           template: '\
                <div class="search-field-background"> \
                </div> \
                <div class="search-field-wrapper"> \
                    <label for="<%- id %>" class="placeholder-text"><%- _("enter search here...").t() %></label> \
                    <textarea rows="1" name="q" spellcheck="false" class="search-field" id="<%- id %>" autocorrect="off" autocapitalize="off"><%- inputValue %></textarea> \
                </div> \
                <div class="search-assistant-wrapper"> \
                    <div class="search-assistant-container-wrapper"><div class="search-assistant-container"></div></div> \
                    <a class="search-assistant-autoopen-toggle" href="" style="display:none;"><i></i><%= _("Auto Open").t() %></a> \
                       <div class="search-assistant-resize"></div> \
                    <a href="#" class="search-assistant-activator icon-triangle-down-small"></a>\
                </div> \
             '
        },
        {
            instance: function(options) {
                if (!options || !options.model) {
                    throw new Error("You must define a state model to use this instance static method");
                }
                if (!View._instance) {
                    View._instance = new View(options);
                } else {
                    View._instance.stopListening();
                    View._instance._configure(options);
                    View._instance.initialize(options);
                }
                return View._instance;
            }
        });
        return View;
    }
);

