define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/searchbar/Apps',
        'views/shared/searchbar/Input',
        'views/shared/timerangepicker/Master',
        'views/shared/searchbar/Submit'
    ],
    function($, _, Backbone, module, BaseView, Apps, Input, TimeRangePicker, Submit) {
        return BaseView.extend({
            moduleId: module.id,
            className: 'search-bar-wrapper',
            /**
             * @param {Object} options {
             *     model: {
             *         state: <models.Report.entry.content>,
             *         timeRange: <models.TimeRange>,
             *         appLocal: <models.services.AppLocal>,
             *         user: <models.services.authentication.User>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         times (Optional): <collections.services.data.ui.TimesV2>,
             *         apps (Optional): <collections.services.AppsLocals>
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                _.defaults(this.options, {
                    showTimeRangePicker:true,
                    singletonInput: false,
                    disableOnSubmit: false,
                    primaryAction: false
                });
                this.showTimeRangePicker = this.options.showTimeRangePicker;

                if (this.collection && this.collection.apps) {
                    this.children.apps = new Apps({
                        collection: this.collection.apps,
                        model: this.model.state,
                        applicationModel: this.model.application
                    });
                }

                if (this.options.singletonInput) {
                    this.children.searchInput = Input.instance({
                        model: this.model.state,
                        singleton: this.options.singletonInput,
                        disableOnSubmit: this.options.disableOnSubmit
                    });
                } else {
                    this.children.searchInput = new Input({
                        model: this.model.state
                    });                    
                }


                if (this.showTimeRangePicker) {
                    this.children.timeRangePicker = new TimeRangePicker({
                        model: {
                            state: this.model.state,
                            timeRange: this.model.timeRange,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            application: this.model.application
                        },
                        collection: this.collection.times,
                        timerangeClassName: 'btn'
                    });
                }

                this.children.submit = new Submit({
                    model: this.model.state
                });

            },
            render: function() {
                var template = _.template(this.template, {showTimeRangePicker: this.showTimeRangePicker, primary: this.options.primaryAction});
                this.$el.html(template);

                if (this.children.apps) {
                    this.$('.search-apps').append(this.children.apps.render().el);
                }
                this.$('.search-input').append(this.children.searchInput.render().el);
                if (this.showTimeRangePicker) {
                    this.$('.search-timerange').append(this.children.timeRangePicker.render().el);
                }
                this.$('.search-button').append(this.children.submit.render().el);
                return this;
            },
            template: '\
                <form method="get" action="" class="search-form">\
                    <table class="search-bar <%- primary ? "search-bar-primary" : "" %>">\
                        <tbody>\
                            <tr>\
                                <td class="search-input" width="100%"></td>\
                                <td class="search-apps"></td>\
                                <% if (showTimeRangePicker) { %>\
                                    <td class="search-timerange"></td>\
                                <% } %>\
                                <td class="search-button"></td>\
                            </tr>\
                        </tbody>\
                    </table>\
                </form>\
            '
        });
    }
);
