define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/CollectionPaginator',
        'views/shared/tablecaption/Count',
        'views/shared/controls/ControlGroup',
        'views/shared/tablecaption/Input',
        'views/shared/delegates/Dock'
    ],
    function(
        _,
        module,
        BaseView,
        PaginatorView,
        CountView,
        ControlGroup,
        InputView,
        Dock
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'div',
            className: 'table-caption',
            /**
             * @param {Object} options {
             *     countLabel: <String> The count vanity label
             *     model: {
             *         state: <models.State>,
             *         uiPrefs: <models.services.admin.UIPrefs>
             *     }, 
             *     collection: <collections.services.SavedSearches>
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.children.count = new CountView({
                    countLabel: this.options.countLabel,
                    model: this.model.state,
                    collection: this.collection
                });

                this.children.filterButtons = new ControlGroup({
                    controlType: 'SyntheticRadio',
                    controlOptions: {
                        modelAttribute: 'display.prefs.aclFilter',
                        model: this.model.uiPrefs.entry.content,
                        items: [
                            { label: _('All').t(), value: 'none' },
                            { label: _('Yours').t(), value: 'owner' },
                            { label: _('This App\'s').t(), value: 'app' }
                        ]
                     }
                });

                this.children.paginatorView = new PaginatorView({
                    collection: this.collection,
                    model: this.model.state
                });

                if(!this.options.noFilter){
                    this.children.input = new InputView({
                        model: {
                            state: this.model.state
                        },
                        key: this.options.filterKey
                    });
                }
            },
            render: function() {
                this.$el.append(this.template);
                this.$('.table-caption-inner').append(this.children.count.render().el);
                this.$('.table-caption-inner').append(this.children.paginatorView.render().el);

                this.$('.table-caption-inner').append(this.children.filterButtons.render().el);
                if(!this.options.noFilter){
                     this.$('.table-caption-inner').append(this.children.input.render().el);
                }

                this.children.tableDock = new Dock({ el: this.el, affix: '.table-caption-inner' });

                return this;
            },
            template: '\
                <div class="table-caption-inner"></div>\
            '
        });
    }
);
