define(
    [
        'jquery',
        'underscore',
        'backbone',
        "module",
        "views/Base",
        "models/services/search/TimeParser",
        "models/TimeRange",
        "views/shared/timerangepicker/dialog/advanced/timeinput/Master",
        "views/shared/FlashMessages",
        'uri/route',
        'util/console',
        "util/time_utils"
    ],
    function($, _, Backbone, module, BaseView, TimeParserModel, TimeRangeModel, TimeInput, FlashMessages, route, console, time_utils) {
        return BaseView.extend({
            moduleId: module.id,
            className: 'accordion-inner',
            initialize: function() {
                BaseView.prototype.initialize.call(this, arguments);
                
                this.label = _("Advanced").t();
                
                var workingTimeRangeAttributes = $.extend(true, {
                    enableRealTime: this.options.enableCustomAdvancedRealTime
                }, this.model.timeRange.toJSON());

                this.model = {
                    appLocal: this.model.appLocal,
                    application: this.model.application,
                    range: this.model.timeRange,
                    workingRange: new TimeRangeModel(workingTimeRangeAttributes),
                    earliestTimeParser: new TimeParserModel({
                        key: time_utils.stripRTSafe(this.model.timeRange.get("earliest"), false),
                        value: this.model.timeRange.get("earliest_iso")
                    }),
                    latestTimeParser: new TimeParserModel({
                        key: time_utils.stripRTSafe(this.model.timeRange.get("latest"), true),
                        value: this.model.timeRange.get("latest_iso")
                    })
                };

                this.children.earliestTimeInput = new TimeInput({
                    model: {
                        working: this.model.workingRange,
                        timeParser: this.model.earliestTimeParser
                    },
                    label: _("Earliest").t(),
                    blankValue: '0',
                    modelAttribute: "earliest",
                    isLatest: false
                });

                this.children.latestTimeInput = new TimeInput({
                    model: {
                        working: this.model.workingRange,
                        timeParser: this.model.latestTimeParser
                    },
                    label: _("Latest").t(),
                    blankValue: '',
                    modelAttribute: "latest",
                    isLatest: true
                });

                this.children.flashMessages = new FlashMessages({
                    model: {
                        workingRange: this.model.workingRange,
                        earliestTimeParser: this.model.earliestTimeParser,
                        latestTimeParser: this.model.latestTimeParser
                    }
                });

                this.model.range.on('change:earliest change:latest prepopulate', _.debounce(function() {
                   this.model.workingRange.set({
                       'earliest': this.model.range.get('earliest'), 
                       'latest': this.model.range.get('latest')
                   });
                }, 0), this);
                
                this.model.workingRange.on('validated', function(validated, model, error_payload){
                    if (!validated) {
                        this.enableApply();
                    }
                }, this);
            },
            remove: function() {
                BaseView.prototype.remove.apply(this, arguments);
                _.chain(this.model).omit(['appLocal', 'application', 'range']).each(function(model){
                    model.deepOff();
                    model.fetchAbort();
                });
             },
            supportsRange: function() {
                return true; //supports anything
            },
            events: {
                "click .apply": function(event) {
                    if ($(event.currentTarget).hasClass("disabled")) {
                        event.preventDefault();
                        return;
                    }

                    var that = this,
                        earliest = this.children.earliestTimeInput.$('input').val() || '0',
                        latest = this.children.latestTimeInput.$('input').val() || '';

                    this.disableApply();
                    
                    this.model.workingRange.save(
                            {
                                earliest: earliest,
                                latest: latest
                            },
                            {
                                success: function(model) {
                                    that.enableApply();
                                    var latest_date = model.get('latest_date'),
                                        latest_js_date = latest_date ? Date(latest_date.getTime()) : latest_date;
                                    that.model.range.set({
                                        'earliest': model.get('earliest'),
                                        'latest': model.get('latest'),
                                        'earliest_epoch': model.get('earliest_epoch'),
                                        'latest_epoch': model.get('latest_epoch'),
                                        'earliest_iso': model.get('earliest_iso'),
                                        'latest_iso': model.get('latest_iso'),
                                        'earliest_date': new Date(model.get('earliest_date').getTime()),
                                        'latest_date': latest_js_date
                                    });
                                    that.model.range.trigger("applied");
                                },
                                error: function(model, error) {
                                    that.enableApply();
                                }
                            }
                    );
                    event.preventDefault();
                }
            },
            enableApply: function() {
                this.$("#apply_" + this.cid).removeClass('disabled');
            },
            disableApply: function() {
                this.$("#apply_" + this.cid).addClass('disabled');
            },
            render: function() {
                // TODO: 'Remove if statement after all consumers pass the appLocal and application model'
                var docRoute = "http://docs.splunk.com/";
                if (this.model.appLocal && this.model.application) {
                    docRoute = route.docHelp(
                        this.model.application.get("root"),
                        this.model.application.get("locale"),
                        'learnmore.timerange.picker'
                    );
                } else {
                    console.warn("The timerangepicker advanced view needs the AppLocal and Application model passed to it");
                }
                var template = _.template(this.template, {
                    _: _,
                    cid: this.cid,
                    docRoute: docRoute
                });
                this.$el.html(template);
                this.children.flashMessages.render().$el.insertBefore(this.$(".apply"));
                this.children.earliestTimeInput.render().$el.insertBefore(this.$(".apply"));
                this.children.latestTimeInput.render().$el.insertBefore(this.$(".apply"));

                return this;
            },
            template: '\
                    <button class="apply btn" id="apply_<%- cid %>"><%- _("Apply").t() %></button>\
                    <a href="<%- docRoute %>" target="_blank" title="<%- _("Splunk help").t() %>" class="btn-documentation" style="line-height:22px;"><%- _("Documentation").t() %> <i class="icon-external"></i></a>\
            '
    });
}
);
