define([
    'jquery',
    'underscore',
    'backbone',
    "module",
    "views/Base",
    "views/shared/timerangepicker/dialog/advanced/timeinput/Hint",
    "util/time_utils"
],
function(
    $,
    _,
    Backbone,
    module,
    BaseView,
    Hint,
    time_utils
){
        return BaseView.extend({
            tagName: 'table',
            className: 'form form-inline pull-left',
            moduleId: module.id,
            initialize: function() {
                BaseView.prototype.initialize.call(this, arguments);
                
                this.children.hint = new Hint({
                    model: {
                        timeParser: this.model.timeParser
                    }
                });
                
                this.model.working.on('change:' + this.options.modelAttribute, function() {
                    this.update_value();
                    this.update_hint();
                }, this);
            },
            events: {
                'keyup input[type="text"]': 'update_hint',
                'focusout input[type="text"]': 'update_hint'
            },
            update_hint: function(event) {
                var time = time_utils.stripRTSafe((this.$('input').val() || this.options.blankValue), this.options.isLatest) || 'now';

                this.model.timeParser.fetch({
                    data: {
                        time: time
                    }
                });                   
            },
            update_value: function() {
                var time = this.model.working.get(this.options.modelAttribute) || "";
                if (time !== this.options.blankValue){
                    this.$('input').val(time);
                }
            },
            render: function() {                
                var template = _.template(this.template, {
                    _: _,
                    cid: this.cid,
                    label: this.options.label,
                    time: this.model.working.get(this.options.modelAttribute) || ""
                });
                this.$el.html(template);
            
                //hint
                this.$("#hint_" + this.cid).html(this.children.hint.render().el);
                this.update_hint();
                
                return this;
            },
            template: '\
                <tbody>\
                    <tr>\
                        <td>\
                            <label class="control-label" for="<%- cid %>"><%- label %><%- _(":").t() %></label>\
                        </td>\
                        <td>\
                            <input type="text" size="20" value="<%- time %>" id="<%- cid %>"/>\
                            <span id="hint_<%- cid %>" class="help-block help-block-timestamp"></span>\
                        </td>\
                    </tr>\
                </tbody>\
            '
        });
    }
);
