define(
    [
        "jquery",
        "underscore",
        "backbone",
        "module",
        "models/DateInput",
        "models/TimeRange",
        'views/Base',
        "views/shared/controls/DateControl",
        "views/shared/FlashMessages"
    ],
    function($, _, Backbone, module, DateInputModel, TimeRangeModel, Base, DateControl, FlashMessages) {
        return Base.extend({
            tagName: 'tbody',
            moduleId: module.id,
            initialize: function() {
                Base.prototype.initialize.call(this, arguments);
            
                this.model = {
                    range: this.model,
                    workingRange: new TimeRangeModel(),
                    earliestDateInput: new DateInputModel()
                };
                    
                this.yesterday = new Date();
                this.yesterday.setDate(this.yesterday.getDate() - 1);
                this.yesterday.setHours(0, 0, 0, 0);
                this.model.earliestDateInput.setFromJSDate(this.yesterday, {silent:true});
            
                this.children.earliestTimeInput = new DateControl({
                    model: this.model.earliestDateInput,
                    validate: true
                });
                
                this.children.flashMessages = new FlashMessages({
                    model: {
                        workingRange: this.model.workingRange,
                        earliestDateInput: this.model.earliestDateInput
                    }
                });
                    
                this.model.range.on('change:earliest_date prepopulate', _.debounce(this.prepopulate, 0), this);
                
                this.model.earliestDateInput.on("validated:invalid", this.disableApply, this);
                this.model.earliestDateInput.on("validated:valid", this.enableApply, this);
                
                this.model.earliestDateInput.on("change", function(){
                    this.model.workingRange.trigger("validated", true, this.model.workingRange, {});
                    this.model.workingRange.trigger("serverValidated", true, this.model.workingRange, []);
                }, this);
            },
            prepopulate: function() {
                var earliest = this.model.range.get('earliest_date');
                
                if (earliest && !this.model.range.hasNoEarliest()) {
                    this.model.earliestDateInput.setFromJSDate(earliest, {validate: true});
                    this.model.earliestDateInput.setHoursMinSecFromStr("00:00:00.000", {silent:true});
                }
            },
            events: {
                "click .apply": function(event) {
                    if ($(event.currentTarget).hasClass("disabled")) {
                        event.preventDefault();
                        return;
                    }
                
                    var that = this,
                    earliestIsoWithoutTZ = this.model.earliestDateInput.isoWithoutTZ();
                    this.disableApply();
            
                    this.model.workingRange.save(
                            {
                                earliest: earliestIsoWithoutTZ,
                                latest: "now"
                            }, 
                            {
                                success: function(model) {
                                    that.enableApply();
                                    that.model.range.set({
                                        'earliest': model.get('earliest_epoch'),
                                        'latest': 'now',
                                        'earliest_epoch': model.get('earliest_epoch'),
                                        'latest_epoch': model.get('latest_epoch'),
                                        'earliest_iso': model.get('earliest_iso'),
                                        'latest_iso': model.get('latest_iso'),
                                        'earliest_date': new Date(model.get('earliest_date').getTime()),
                                        'latest_date': new Date(model.get('latest_date').getTime())
                                    });
                                    that.model.range.trigger("applied");
                                },
                                error: function(model, error) {
                                    that.enableApply();
                                }
                            }
                    );
                    event.preventDefault();
                }
            },
            enableApply: function() {
                this.$("#apply_" + this.cid).removeClass('disabled');
            },
            disableApply: function() {
                this.$("#apply_" + this.cid).addClass('disabled');
            },
            render: function() {
                var template = _.template(this.template, {
                    _: _,
                    cid: this.cid
                });
                this.$el.html(template);
                this.$(".flash_messages").append(this.children.flashMessages.render().el);
                this.$(".earliest_time").prepend(this.children.earliestTimeInput.render().el); 
                return this;
            },
            template: '\
                <tr>\
                <td class="earliest_time"><span class="help-block"><%- _("00:00:00").t() %></span></td>\
                <td><button class="apply btn" id="apply_<%- cid %>"><%- _("Apply").t() %></button></td>\
                </tr>\
            '
        });
    }
);
