define([
        "jquery",
        "underscore",
        "backbone",
        "module",
        "models/DateInput",
        "models/TimeRange",
        'views/Base',
        "views/shared/controls/DateControl",
        "views/shared/FlashMessages"
      ],
     function($, _, Backbone, module, DateInputModel, TimeRangeModel, Base, DateControl, FlashMessages) {
        return Base.extend({
            tagName: 'tbody',
            moduleId: module.id,
            initialize: function() {
                Base.prototype.initialize.call(this, arguments);
        
                this.model = {
                    range: this.model,
                    workingRange: new TimeRangeModel(),
                    latestDateInput: new DateInputModel()
                };
            
                this.today = new Date();
                this.today.setHours(0, 0, 0, 0);
                this.model.latestDateInput.setFromJSDate(this.today, {silent:true});
        
                this.children.latestTimeInput = new DateControl({
                    model:  this.model.latestDateInput,
                    validate: true
                });
                
                this.children.flashMessages = new FlashMessages({
                    model: {
                        workingRange: this.model.workingRange,
                        latestDateInput: this.model.latestDateInput
                    }
                });
            
                this.model.range.on('change:latest_date prepopulate', _.debounce(this.prepopulate, 0), this);
                
                this.model.latestDateInput.on("validated:invalid", this.disableApply, this);
                this.model.latestDateInput.on("validated:valid", this.enableApply, this);
                
                this.model.latestDateInput.on("change", function(){
                    this.model.workingRange.trigger("validated", true, this.model.workingRange, {});
                    this.model.workingRange.trigger("serverValidated", true, this.model.workingRange, []);
                }, this);
            },
            prepopulate: function() {
                var latest = this.model.range.get('latest_date');
                
                if (latest) {
                    this.model.latestDateInput.setFromJSDate(latest, {validate: true});
                    this.model.latestDateInput.setHoursMinSecFromStr("00:00:00.000", {silent:true});
                }
            },
            events: {
                "click .apply": function(event) {
                    if ($(event.currentTarget).hasClass("disabled")) {
                        event.preventDefault();
                        return;
                    }
                
                    this.model.latestDateInput.setHoursMinSecFromStr("00:00:00.000", {silent:true, validate:true});
            
                    var that = this,
                    latestIsoWithoutTZ = this.model.latestDateInput.isoWithoutTZ();
            
                    this.disableApply();
            
                    this.model.workingRange.save(
                            {
                                earliest: "",
                                latest: latestIsoWithoutTZ
                            }, 
                            {
                                success: function(model) {
                                    that.enableApply();
                                    that.model.range.set({
                                        'earliest': "",
                                        'latest': model.get('latest_epoch'),
                                        'earliest_epoch': model.get('earliest_epoch'),
                                        'latest_epoch': model.get('latest_epoch'),
                                        'earliest_iso': model.get('earliest_iso'),
                                        'latest_iso': model.get('latest_iso'),
                                        'earliest_date': new Date(0),
                                        'latest_date': new Date(model.get('latest_date').getTime())
                                    });
                                    that.model.range.trigger("applied");
                                },
                                error: function(model, error) {
                                    that.enableApply();
                                }
                            }
                    );
                    event.preventDefault();
                }
            },
            enableApply: function() {
                this.$("#apply_" + this.cid).removeClass('disabled');
            },
            disableApply: function() {
                this.$("#apply_" + this.cid).addClass('disabled');
            },
            render: function() {
                var template = _.template(this.template, {
                    _: _,
                    cid: this.cid
                });
                this.$el.html(template);

                this.$(".flash_messages").append(this.children.flashMessages.render().el);
                this.$(".latest_time").prepend(this.children.latestTimeInput.render().el); 
                return this;
            },
            template: '\
                <tr>\
                <td class="flash_messages"></td>\
                <td class="latest_time"><span class="help-block"><%- _("00:00:00").t() %></span></td>\
                <td><button class="apply btn" id="apply_<%- cid %>"><%- _("Apply").t() %></button></td>\
                </tr>\
            '
        });
    }
);
