define([
        "jquery",
        "underscore",
        "backbone",
        "module",
        "models/DateInput",
        "models/TimeRange",
        'views/Base',
        "views/shared/controls/DateControl",
        "views/shared/FlashMessages"
      ],
     function($, _, Backbone, module, DateInputModel, TimeRangeModel, Base, DateControl, FlashMessages) {
        return Base.extend({
            tagName: 'tbody',
            moduleId: module.id,
            initialize: function() {
                Base.prototype.initialize.call(this, arguments);
            
                this.model = {range: this.model, workingRange: new TimeRangeModel()};
                    
                this.today = new Date();
                this.today.setHours(0, 0, 0, 0);
            
                this.model.latestDateInput = new DateInputModel();
                this.model.latestDateInput.setFromJSDate(this.today, {silent:true});
            
                this.yesterday = new Date();
                this.yesterday.setDate(this.yesterday.getDate() - 1);
                this.yesterday.setHours(0, 0, 0, 0);
            
                this.model.earliestDateInput = new DateInputModel();
                this.model.earliestDateInput.setFromJSDate(this.yesterday, {silent:true});
            
                this.children.earliestTimeInput = new DateControl({
                    model: this.model.earliestDateInput,
                    inputClassName: "timerangepicker-earliest-date",
                    validate: true
                });
            
                this.children.latestTimeInput = new DateControl({
                    model: this.model.latestDateInput,
                    inputClassName: "timerangepicker-latest-date",
                    validate: true
                });
            
                this.children.flashMessages = new FlashMessages({
                    model: {
                        workingRange: this.model.workingRange,
                        earliestDateInput: this.model.earliestDateInput,
                        latestDateInput: this.model.latestDateInput
                    }
                });
                
                this.model.range.on('change:earliest_date change:latest_date prepopulate', _.debounce(this.prepopulate, 0), this);
                
                this.model.earliestDateInput.on("validated:invalid", this.disableApply, this);
                this.model.earliestDateInput.on("validated:valid", this.enableApply, this);
                
                this.model.earliestDateInput.on("change", function(){
                    this.model.workingRange.trigger("validated", true, this.model.workingRange, {});
                    this.model.workingRange.trigger("serverValidated", true, this.model.workingRange, []);
                }, this);
                
                this.model.latestDateInput.on("validated:invalid", this.disableApply, this);
                this.model.latestDateInput.on("validated:valid", this.enableApply, this);
                
                this.model.latestDateInput.on("change", function(){
                    this.model.workingRange.trigger("validated", true, this.model.workingRange, {});
                    this.model.workingRange.trigger("serverValidated", true, this.model.workingRange, []);
                }, this);
            },
            prepopulate: function() {
                var earliest = this.model.range.get('earliest_date'),
                    latest =  this.model.range.get('latest_date');
                
                if (!this.model.range.hasNoEarliest() && earliest) {
                    this.model.earliestDateInput.setFromJSDate(earliest, {validate: true});
                }
                if (latest) {
                    var latestDate= new Date(latest.getTime());
                    
                    if (latestDate.getHours() == 0) {
                        latestDate.setDate((latestDate.getDate() - 1), {validate: true});
                    }
                    this.model.latestDateInput.setFromJSDate(latestDate, {validate: true});
                }
            },
            events: {
                "click .apply": function(event) {
                    if ($(event.currentTarget).hasClass("disabled")) {
                        event.preventDefault();
                        return;
                    }
                    
                    this.model.earliestDateInput.setHoursMinSecFromStr("00:00:00.000", {silent:true, validate: true});
                    this.model.latestDateInput.setHoursMinSecFromStr("24:00:00.000", {silent:true, validate: true});

                    var that = this,
                    earliestIsoWithoutTZ = this.model.earliestDateInput.isoWithoutTZ(),
                    latestIsoWithoutTZ = this.model.latestDateInput.isoWithoutTZ();

                    this.disableApply();
                    
                    this.model.workingRange.save(
                            {
                                earliest: earliestIsoWithoutTZ,
                                latest: latestIsoWithoutTZ
                                
                            }, 
                            {
                                success: function(model) {
                                    that.enableApply();
                                    that.model.range.set({
                                        'earliest': model.get('earliest_epoch'),
                                        'latest': model.get('latest_epoch'),
                                        'earliest_epoch': model.get('earliest_epoch'),
                                        'latest_epoch': model.get('latest_epoch'),
                                        'earliest_iso': model.get('earliest_iso'),
                                        'latest_iso': model.get('latest_iso'),
                                        'earliest_date': new Date(model.get('earliest_date').getTime()),
                                        'latest_date': new Date(model.get('latest_date').getTime())
                                    });
                                    that.model.range.trigger("applied");
                                },
                                error: function(model, error) {
                                    that.enableApply();
                                }
                            }
                    );
                    event.preventDefault();
                }
            },
            generateLabel: function() {
                //This is a temporary generator. Should move this into the model?
                var earliest = this.model.earliestDateInput.jsDate();
                var latest = this.model.latestDateInput.jsDate();
                latest.setDate(latest.getDate() -1);

                return earliest.toDateString().slice(4, -5) + " - " + latest.toDateString().slice(4, -5);

            },
            enableApply: function() {
                this.$("#apply_" + this.cid).removeClass('disabled');
            },
            disableApply: function() {
                this.$("#apply_" + this.cid).addClass('disabled');
            },
            render: function() {
                var template = _.template(this.template, {
                    _: _,
                    cid: this.cid
                });
                this.$el.html(template);

                this.$(".flash_messages").append(this.children.flashMessages.render().el);
                $(this.children.earliestTimeInput.render().el).prependTo(this.$('.earliest-date'));
                $(this.children.latestTimeInput.render().el).prependTo(this.$('.latest'));

                return this;
            },
            template: '\
                <tr><td class="flash_messages" colspan="4"></td></tr>\
                <tr>\
                    <td class="earliest-date"><span class="help-block"><%- _("00:00:00").t() %></span></td>\
                    <td><label class="control-label"><%- _("and").t() %></label></td>\
                    <td class="latest"><span class="help-block"><%- _("24:00:00").t() %></span></td>\
                    <td><button class="apply btn" id="apply_<%- cid %>"><%- _("Apply").t() %></button></td>\
                </tr>\
            '
        });
    }
);
