define([
        "jquery",
        "underscore",
        "backbone",
        "module",
        "views/Base",
        "views/shared/controls/SyntheticSelectControl",
        "views/shared/timerangepicker/dialog/daterange/BetweenDates",
        "views/shared/timerangepicker/dialog/daterange/BeforeDate",
        "views/shared/timerangepicker/dialog/daterange/AfterDate",
        "views/shared/FlashMessages"
        ],
        function($, _, Backbone, module, BaseView, SyntheticSelectControl, BetweenDates, BeforeDate, AfterDate, FlashMessages) {
            return BaseView.extend({
                className: 'accordion-inner',
                moduleId: module.id,
                initialize: function(options) {
                    BaseView.prototype.initialize.call(this, arguments);

                    this.label = _("Date Range").t();

                    this.children.rangeTypeSelect = new SyntheticSelectControl({
                        model: this.model,
                        items: [
                            {value: 'between_dates', label: _('Between').t()},
                            {value: 'before_date', label: _('Before').t()},
                            {value: 'after_date', label: _('Since').t()}
                        ],
                        modelAttribute: 'range_type',
                        className: 'timerangepicker-range-type',
                        toggleClassName: 'btn',
                        menuWidth: 'narrow',
                        popdownOptions: {attachDialogTo:'body'},
                        save: false
                    });

                    var rangeTypeOptions = {
                        model: this.model,
                        parentCid: this.cid
                    };
                    this.children.betweenDates = new BetweenDates(rangeTypeOptions);
                    this.children.beforeDate = new BeforeDate(rangeTypeOptions);
                    this.children.afterDate = new AfterDate(rangeTypeOptions);

                    this.model.on("change:range_type", function() {
                        var range_type = this.model.get('range_type') || "";
                        this.children.betweenDates.$el.hide();
                        this.children.beforeDate.$el.hide();
                        this.children.afterDate.$el.hide();
                        switch(range_type){
                            case "before_date":
                                this.children.beforeDate.$el.show();
                                break;
                            case "after_date":
                                this.children.afterDate.$el.show();
                                break;
                            default:
                                this.children.betweenDates.$el.show();
                            break;
                        }
                    }, this);

                    this.model.on('change:earliest change:latest prepopulate', _.debounce(this.prepopulate, 0), this);

                },
                prepopulate: function() {
                    var earliestIsAbsolute = this.model.isAbsolute('earliest'),
                        hasNoEarliest = this.model.hasNoEarliest(),
                        latestIsAbsolute =  this.model.isAbsolute('latest');

                    if (!hasNoEarliest && earliestIsAbsolute && latestIsAbsolute) {
                        this.model.set('range_type', 'between_dates');
                    } else if (hasNoEarliest && latestIsAbsolute) {
                        this.model.set('range_type', 'before_date');
                    } else if (earliestIsAbsolute && this.model.latestIsNow()) {
                        this.model.set('range_type', 'after_date');
                    }
                },
                supportsRange: function() {
                    var supportsEarliest = false,
                        supportsLatest = false,
                        hasNoEarliest = this.model.hasNoEarliest(),
                        latestIsNow = this.model.latestIsNow(),
                        earliestIsWholeDay = this.model.isWholeDay('earliest'),
                        latestIsWholeDay = this.model.isWholeDay('latest');

                    if (hasNoEarliest && latestIsNow) {
                        return false;
                    }

                    return (earliestIsWholeDay && latestIsWholeDay)
                            || (earliestIsWholeDay && latestIsNow)
                            || (hasNoEarliest && latestIsWholeDay);
                },
                render: function() {
                    this.$el.append( this.children.rangeTypeSelect.render().el);

                    var template = _.template(this.template, {
                        cid: this.cid
                    });
                    this.$el.append(template);

                    this.$('.date-range-container').append(this.children.betweenDates.render().el);
                    this.$('.date-range-container').append(this.children.beforeDate.render().el);
                    this.children.beforeDate.$el.hide();
                    this.$('.date-range-container').append(this.children.afterDate.render().el);
                    this.children.afterDate.$el.hide();

                    return this;
                },
                template: '\
                    <table class="form form-inline date-range-container">\
                    </table>\
                '
        });
    }
);
