define(
    [
        'underscore',
        'jquery',
        'module',
        'views/shared/PopTart',
        'views/shared/vizcontrols/components/Master'
    ],
    function(_, $, module, PopTart, Component){
        return PopTart.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         report: <models.Report>,
             *         state: <models.SplunkDBaseV2>
             *     },
             *     saveOnApply: <Boolean>
             *   }
             */

            options: {
                saveOnApply: false
            },

            initialize: function(options) {
                PopTart.prototype.initialize.apply(this, arguments);
                this.children.visualizationControls && this.children.visualizationControls.remove();
                this.children.visualizationControls = new Component({
                    model: this.model.visualization
                });
            },
            events: {
                'click .viz-editor-apply': function(e) {
                    e.preventDefault();
                    this.applyChanges(); 
                },
                'click .viz-editor-cancel': function(e) {
                    this.hide();
                    e.preventDefault();
                },
                'keypress .viz-editor-apply': function(e) {
                    e.preventDefault();
                    this.applyChanges(); 
                },
                'keypress .viz-editor-cancel': function(e) {
                    this.hide();
                    e.preventDefault();
                },
                'keypress input:text': function(e) {
                    var enterKeyCode = 13,
                        $target = $(e.target);

                    if(e.keyCode == enterKeyCode) {
                        $target.blur();
                        e.preventDefault();
                        this.applyChanges();
                    }
                }
            },
            applyChanges: function() {
                this.model.visualization.validate();
                if(this.model.visualization.isValid()) {
                    this.model.report.entry.content.set($.extend({}, this.model.visualization.toJSON()));
                    if(this.options.saveOnApply) {
                        this.model.report.save();
                    }
                    this.hide();
                }
            },
            render: function() {
                this.$el.html(PopTart.prototype.template);
                this.$el.append(this.template);
                this.$('.popdown-dialog-body').append(this.children.visualizationControls.render().el);
                // ghetto hack to override default padding on poptart ;_;
                this.$('.popdown-dialog-body').removeClass('popdown-dialog-padded');
                return this;
            },
            template: '\
                    <a class="viz-editor-cancel btn pull-left" tabindex="0">'+_("Cancel").t()+'</a>\
                    <a class="viz-editor-apply btn btn-primary pull-right" tabindex="0"> '+_("Apply").t()+'</a>\
            '
        });
    }
);
