define(
    [
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/vizcontrols/Format',
        'models/services/search/IntentionsParser',
        'models/Visualization'
    ],
    function(_, Backbone, module, BaseView, SyntheticSelectControl, Format, IntentionsParser, VisualizationModel) {
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         report: <models.Report>
             *     },
             *     visualizationTypes: [events &| statistics &| visualization],
             *     saveOnApply: <Boolean>
             * }
             */
            options: {
                saveOnApply: false
            },
            vizToAttrs: {
                line: {
                    'display.visualizations.charting.chart': 'line',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                area: {
                    'display.visualizations.charting.chart': 'area',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                column: {
                    'display.visualizations.charting.chart': 'column',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                bar: {
                    'display.visualizations.charting.chart': 'bar',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                pie: {
                    'display.visualizations.charting.chart': 'pie',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                scatter: {
                    'display.visualizations.charting.chart': 'scatter',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                radialGauge: {
                    'display.visualizations.charting.chart': 'radialGauge',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                fillerGauge: {
                    'display.visualizations.charting.chart': 'fillerGauge',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                markerGauge: {
                    'display.visualizations.charting.chart': 'markerGauge',
                    'display.visualizations.type': 'charting',
                    'display.general.type': 'visualizations'
                },
                single: {
                    'display.visualizations.type': 'singlevalue',
                    'display.general.type': 'visualizations'
                },
                events: {
                    'display.general.type': 'events'
                },
                statistics: {
                    'display.general.type': 'statistics'
                }
            },
            visualizationGroupings: {
                "events": {
                    label: 'g1',
                    items: [
                        { value: 'events', label: _("Events").t(), icon: 'list', description: _(" ").t() }
                    ]
                },
                 "statistics": {
                    label: 'g2',
                    items: [
                        {value: 'statistics', label: _("Statistics Table").t(), icon: 'table', description: _(" ").t() }
                    ]
                },
                "visualizations": {
                    label: 'g3',
                    items: [
                        { value: 'line', label: _("Line").t(), icon: 'chart-line', description: _(" ").t() },
                        { value: 'area', label: _("Area").t(), icon: 'chart-area', description: _(" ").t() },
                        { value: 'column', label: _("Column").t(), icon: 'chart-column', description: _(" ").t() },
                        { value: 'bar', label: _("Bar").t(), icon: 'chart-bar', description: _(" ").t() },
                        { value: 'pie', label: _("Pie").t(), icon: 'chart-pie', description: _(" ").t() },
                        { value: 'scatter', label: _("Scatter").t(), icon: 'chart-scatter', description: _(" ").t() },
                        { value: 'single', label: _("Single Value").t(), icon: 'single-value', description: _(" ").t() },
                        { value: 'radialGauge', label: _("Radial Gauge").t(), icon: 'gauge-radial', description: _(" ").t() },
                        { value: 'fillerGauge', label: _("Filler Gauge").t(), icon: 'gauge-filler', description: _(" ").t() },
                        { value: 'markerGauge', label: _("Marker Gauge").t(), icon: 'gauge-marker', description: _(" ").t() }
                    ]
                }
            },
            reportTree: {
                'match': {
                    'display.general.type': {
                        'visualizations' : {
                            'match': {
                                'display.visualizations.type': {
                                    'singlevalue': {
                                        'view': 'single'
                                    },
                                    'charting': {
                                        'match': {'display.visualizations.charting.chart': {
                                                'line': {  'view': 'line' },
                                                'area': { 'view': 'area' },
                                                'column': { 'view': 'column' },
                                                'bar': { 'view': 'bar' },
                                                'pie': { 'view': 'pie' },
                                                'scatter': { 'view': 'scatter' },
                                                'radialGauge': { 'view': 'radialGauge' },
                                                'fillerGauge': { 'view': 'fillerGauge' },
                                                'markerGauge': { 'view': 'markerGauge' }
                                            }
                                        }
                                    }
                                }
                            }
                        },
                        'statistics': {
                            'view': 'statistics'
                        },
                        'events': {
                            'view': 'events'
                        }
                    }
                }
            },
            commandToChartType: { 
                "timechart" : ["line", "area", "column"],
                //"chart" : ["column", "line", "area", "bar", "pie", "scatter", "radialGauge", "fillerGauge", "markerGauge"],
                "top" : ["column", "bar", "pie"],
                "rare" : ["column", "bar", "pie"]
            },
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                
                var defaults = {
                        bindToChangeOfSearch: true
                };
                this.options = $.extend(true, defaults, this.options);
                
                this.model.intentionsParser = new IntentionsParser();
                this.model.intentionsParser.on('change', this.render, this); 
                
                this.model.visualization = new VisualizationModel();
                this.model.visualization.set({dashboard: this.options.dashboard});
                this.setVisualizationFromReport();
                this.setSyntheticSelectGroups();

                this.model.report.entry.content.on('change:display.general.type', function(model, value, options) {
                    if(_.indexOf(this.options.vizTypes, value)>-1){
                        this.setVisualizationFromReport();
                        this.setSyntheticSelectGroups();
                        this.render();
                    }
                },this);

                this.model.visualization.on('change:viz_type', function() {
                    var viz = this.model.visualization.get('viz_type');
                    this.model.report.entry.content.set('viz_type', viz);
                    this.model.report.entry.content.set(this.vizToAttrs[viz]);
                    this.model.visualization.set(this.vizToAttrs[viz]);
                    if(this.options.saveOnApply) {
                        this.model.report.save();
                    }
                },this);

                this.model.report.entry.content.on('change:search change:id', this.updateVizTypeVisability, this);
               
                if(this.options.bindToChangeOfSearch) { 
                    this.model.report.entry.content.on('change:search', this.intentionsFetch, this); //simplexml
                }
                
                this.intentionsFetch();
            },
            intentionsFetch: function() {
                this.model.intentionsParser.fetch({
                    data: {
                        q: this.model.report.entry.content.get('search'),
                        app: this.model.application.get('app'),
                        owner: this.model.application.get('owner'),
                        parse_only: true
                    }
                });
            },
            updateVizTypeVisability: function() {
                if (!this.children.visualizationType || !this.model.visualization.get('dashboard')){
                    return;
                }

                if (this.model.report.isPivotReport() && !this.model.report.isNew()){
                    this.children.visualizationType.disable();
                    this.children.visualizationType.tooltip({animation:false, title:_('Edit visualization with the pivot tool.').t(), container: 'body'});
                } else {
                    this.children.visualizationType.enable();
                    this.children.visualizationType.tooltip('disable');
                }
            },
            setRecommendations: function() {
                if(this.model.intentionsParser.has('reportsSearch')) {
                    var reportsSearch = this.model.intentionsParser.get('reportsSearch'),
                        recommended = false;

                    if(reportsSearch) {
                        var reportingCommand = reportsSearch.replace(/\s{2,}/g, ':::').split(':::')[0];
                        if(reportingCommand) {
                            this.recommenedTypes = this.commandToChartType[reportingCommand];
                            if(this.recommenedTypes && !!this.recommenedTypes.length) {
                                _.chain(this.visualizationGroupings.visualizations.items).filter(function(item) {
                                    return this.recommenedTypes.indexOf(item.value) > -1;
                                },this)
                                .each(function(item) {
                                   this.children.visualizationType.$el.find("li a[data-value='"+item.value+"'] > span.link-description").text(_("Recommended").t());
                                   recommended = true;
                                },this)
                                .value();
                            }
                        }
                        recommended && this.children.visualizationType.$el.find("li a[data-value='statistics'] > span.link-description").text(_("Recommended").t());
                    } else {
                        this.children.visualizationType.$el.find("li a[data-value='events'] > span.link-description").text(_("Recommended").t());
                    }
                }
            },
            setVisualizationFromReport: function() {
                this.model.report.entry.content.unset('viz_type');
                this.model.visualization.set($.extend({}, this.model.report.entry.content.toJSON()));
                var viewObj = this.setVizType(this.reportTree), view;
                view = (viewObj && viewObj.view) ? viewObj.view : 'column';
                this.model.visualization.set('viz_type', view);
            },
            setSyntheticSelectGroups: function() {
                var vizDropdown = [], content = this.model.report.entry.content;
                this.children.visualizationType && this.children.visualizationType.remove();

                if(this.options.vizTypes){
                    _(this.options.vizTypes).each(function(value, idx) {
                        vizDropdown.push(this.visualizationGroupings[value]);
                    },this);
                }

                if(vizDropdown.length){
                    this.children.visualizationType = new SyntheticSelectControl({
                        model: this.model.visualization,
                        groupedItems: vizDropdown,
                        className: "btn-combo pull-left",
                        toggleClassName: 'btn-pill',
                        modelAttribute: 'viz_type',
                        menuClassName: 'viz-dropdown',
                        popdownOptions: {
                            attachDialogTo: 'body'
                        }
                    });
                }
            },
            setVizType: function(reportTree) {
                if (reportTree && reportTree.view){
                    return reportTree;
                } else if (reportTree && reportTree.match){
                    var match;
                    _(reportTree.match).each(function(v, k){
                       match = v[this.model.report.entry.content.get(k)];
                    }, this);
                    if (match) return this.setVizType(match);
                }
            },
            events: {
                'click .format': function(e) {
                    var $target = $(e.currentTarget);
                    this.setVisualizationFromReport(); 
                    this.children.format = new Format({
                        model: {
                            report: this.model.report,
                            visualization: this.model.visualization
                        },
                        onHiddenRemove: true,
                        saveOnApply: this.options.saveOnApply,
                        ignoreClasses: ['color-picker-container']
                    });
                    $('body').append(this.children.format.render().el);
                    this.children.format.show($target);
                    e.preventDefault();
                }
            },
            render: function() {
                this.$el.empty();
                this.setSyntheticSelectGroups();
                this.children.visualizationType && this.$el.append(this.children.visualizationType.render().el);
                this.setRecommendations();
                this.updateVizTypeVisability();
                this.$el.append(this.template);
                return this;
            },
            template: '\
                <div class="btn-group pull-left">\
                    <a class="btn-pill popdown-toggle format" href="#">\
                        <i class="icon-paintbrush"/><span class="link-label">'+_("Format").t()+'</span><span class="caret"></span>\
                    </a>\
                </div>\
            '
        });
    }
);
