define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/controls/ControlGroup'
    ],
    function(_, module, Base, ControlGroup) {
        return Base.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: <models.services.SavedSearch.entry.content>
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                this.showEventType = this.options.showEventType === false ? false : true;
                if (this.showEventType) {
                    this.children.eventType = new ControlGroup({
                        label: _("Display").t(),
                        controlType:'SyntheticRadio',
                        controlClass: 'controls-thirdblock',
                        controlOptions: {
                            className: "btn-group",
                            items: [
                                { label: _("Raw").t(),  value: 'raw'  },
                                { label: _("List").t(), value: 'list' },
                                { label: _("Table").t(),value: 'table'}
                            ],
                            model: this.model,
                            modelAttribute: 'display.events.type'
                        }
                    });
                }
                this.children.rowNumbers = new ControlGroup({
                    label: _("Row Numbers").t(),
                    controlType:'SyntheticRadio',
                    controlClass: 'controls-halfblock',
                    controlOptions: {
                        className: "btn-group",
                        items: [
                            {value:'1', label:_("Yes").t()},
                            {value:'0', label:_("No").t()}
                        ],
                        model: this.model,
                        modelAttribute: 'display.events.rowNumbers'
                    }
                });
                this.children.wrapResultsList = new ControlGroup({
                    label: _("Wrap Results").t(),
                    controlType:'SyntheticRadio',
                    controlClass: 'controls-halfblock',
                    controlOptions: {
                        className: "btn-group",
                        items: [
                            {value:'1', label:_("Yes").t()},
                            {value:'0', label:_("No").t()}
                        ],
                        model: this.model,
                        modelAttribute: 'display.events.list.wrap'
                    }
                });
                this.children.wrapResultsTable = new ControlGroup({
                    label: _("Wrap Results").t(),
                    controlType:'SyntheticRadio',
                    controlClass: 'controls-halfblock',
                    controlOptions: {
                        className: "btn-group",
                        items: [
                            {value:'1', label:_("Yes").t()},
                            {value:'0', label:_("No").t()}
                        ],
                        model: this.model,
                        modelAttribute: 'display.events.table.wrap'
                    }
                });

                this.children.maxlines = new ControlGroup({
                    label: _("Max Lines").t(),
                    controlType:'SyntheticSelect',
                    controlClass: 'controls-block',
                    controlOptions: {
                        model: this.model,
                        menuWidth: "narrow",
                        items: [
                            {value: '5', label: _("5 lines").t()},
                            {value: '10', label: _("10 lines").t()},
                            {value: '20', label: _("20 lines").t()},
                            {value: '50', label: _("50 lines").t()},
                            {value: '100', label: _("100 lines").t()},
                            {value: '200', label: _("200 lines").t()},
                            {value: '0', label: _("All lines").t()}
                        ],
                        modelAttribute: 'display.events.maxLines',
                        toggleClassName: "btn",
                        nearestValue: true
                    }
                });
                this.children.drilldownRaw = new ControlGroup({
                    label: _("Drilldown").t(),
                    controlType:'SyntheticSelect',
                    controlClass: 'controls-block',
                    controlOptions: {
                        menuWidth: 'narrow',
                        toggleClassName: 'btn',
                        items: [
                            {value:"none", label:_("None").t()},
                            {value:"inner", label:_("Inner").t()},
                            {value:"outer", label:_("Outer").t()},
                            {value:"full", label:_("Full").t()}
                        ],
                        model: this.model,
                        modelAttribute: 'display.events.raw.drilldown'
                    }
                });
                this.children.drilldownList = new ControlGroup({
                    label: _("Drilldown").t(),
                    controlType:'SyntheticSelect',
                    controlClass: 'controls-block',
                    controlOptions: {
                        menuWidth: 'narrow',
                        toggleClassName: 'btn',
                        items: [
                            {value:"none", label:_("None").t()},
                            {value:"inner", label:_("Inner").t()},
                            {value:"outer", label:_("Outer").t()},
                            {value:"full", label:_("Full").t()}
                        ],
                        model: this.model,
                        modelAttribute: 'display.events.list.drilldown'
                    }
                });
                this.children.drilldownTable = new ControlGroup({
                    label: _("Drilldown").t(),
                    controlType:'SyntheticRadio',
                    controlClass: 'controls-halfblock',
                    controlOptions: {
                        className: "btn-group",
                        items: [
                            {value:'1', label:_("On").t()},
                            {value:'0', label:_("Off").t()}
                        ],
                        model: this.model,
                        modelAttribute: 'display.events.table.drilldown'
                    }
                });
                if (this.model.get('dashboard')){
                    this.children.count = new ControlGroup({
                        label: _("Rows Per Page").t(),
                        controlType:'Text',
                        controlClass: 'controls-block',
                        controlOptions: {
                            model: this.model,
                            menuWidth: "narrow",
                            modelAttribute: 'display.prefs.events.count'
                        }
                    });
                }
                this.model.on('change:display.events.type', this.visibility, this);
                this.model.on('change:display.events.list.drilldown change:display.events.raw.drilldown', this.mediateDrilldown, this);
                this.model.on('change:display.events.list.wrap change:display.events.table.wrap', this.mediateWrap, this);
            },
            mediateDrilldown: function() {
                (this.model.get('display.events.type') === 'list') ?
                    this.model.set('display.events.raw.drilldown', this.model.get('display.events.list.drilldown')):
                    this.model.set('display.events.list.drilldown', this.model.get('display.events.raw.drilldown'));
            },
            mediateWrap: function() {
                (this.model.get('display.events.type') === 'list') ?
                    this.model.set('display.events.table.wrap', this.model.get('display.events.list.wrap')):
                    this.model.set('display.events.list.wrap', this.model.get('display.events.table.wrap'));
            },
            visibility: function() {
                switch(this.model.get('display.events.type')){
                    case 'list':
                        this.children.wrapResultsList.$el.show();
                        this.children.wrapResultsTable.$el.hide();
                        this.children.rowNumbers.$el.show();
                        this.children.drilldownList.$el.show();
                        this.children.drilldownRaw.$el.hide();
                        this.children.drilldownTable.$el.hide();
                        break;
                    case 'raw':
                        this.children.wrapResultsList.$el.hide();
                        this.children.wrapResultsTable.$el.hide();
                        this.children.rowNumbers.$el.hide();
                        this.children.drilldownList.$el.hide();
                        this.children.drilldownRaw.$el.show();
                        this.children.drilldownTable.$el.hide();
                        break;
                    case 'table':
                        this.children.wrapResultsList.$el.hide();
                        this.children.wrapResultsTable.$el.show();
                        this.children.rowNumbers.$el.show();
                        this.children.drilldownList.$el.hide();
                        this.children.drilldownRaw.$el.hide();
                        this.children.drilldownTable.$el.show();
                        break;
                    default:
                        break;
                }
            },
            render: function() {
                if (this.showEventType) {
                    this.$el.append(this.children.eventType.render().el);
                }
                this.$el.append(this.children.rowNumbers.render().el);
                this.$el.append(this.children.wrapResultsList.render().el);
                this.$el.append(this.children.wrapResultsTable.render().el);
                this.$el.append(this.children.maxlines.render().el);
                this.$el.append(this.children.drilldownList.render().el);
                this.$el.append(this.children.drilldownRaw.render().el);
                this.$el.append(this.children.drilldownTable.render().el);
                if (this.children.count){
                    this.$el.append(this.children.count.render().el);
                }
                this.visibility();
                return this;
            }
        });
    }
);
