define(
    [
        'underscore',
        'jquery',
        'module',
        'views/Base',
        'views/shared/FlashMessages',
        'views/shared/vizcontrols/components/General',
        'views/shared/vizcontrols/components/XAxis',
        'views/shared/vizcontrols/components/YAxis',
        'views/shared/vizcontrols/components/Legend',
        'views/shared/vizcontrols/components/color/Master'
    ],
    function(_, $, module, Base, FlashMessages, General, XAxis, YAxis, Legend, Ranges){
        return Base.extend({
            moduleId: module.id,
            className: 'tabbable tabs-left',
            initialize: function(options) {
                Base.prototype.initialize.apply(this, arguments);
                this.children.flashMessages = new FlashMessages({ model: this.model });
                this.type = this.model.get('viz_type');
                if(_.indexOf(this.model.components[this.type], 'gen')>-1)
                    this.children.general = new General({ model: this.model });
                if(_.indexOf(this.model.components[this.type], 'x')>-1)
                    this.children.xaxis = new XAxis({ model: this.model });
                if(_.indexOf(this.model.components[this.type], 'y')>-1)
                    this.children.yaxis = new YAxis({ model: this.model });
                if(_.indexOf(this.model.components[this.type], 'leg')>-1)
                    this.children.legend = new Legend({ model: this.model });
                if(_.indexOf(this.model.components[this.type], 'ranges')>-1)
                    this.children.ranges = new Ranges({ model: this.model });
            },
            events: {
                'click a[data-toggle]': function(e) {
                    var $target = $(e.currentTarget),
                        type = $target.data().type;
                    
                    _(this.children).each(function(child) { 
                        child.$el.hide(); 
                    },this);
                    this.children[type].$el.show(); 
                    this.$el.find('.nav > li').removeClass('active');
                    $target.parent().addClass('active');
                    
                    e.preventDefault();
                }
            },
            render: function() {
                this.$el.html(_(this.template).template({
                    _: _,
                    tabs: this.model.components[this.type]
                }));
                this.$('.tab-content').prepend(this.children.flashMessages.render().el);
                _(this.children).each(function(child) { 
                    this.$('.tab-content').append(child.render().el);
                    child.$el.hide();
                },this);
                this.children.general.$el.show();
                return this;
            },
            template: '\
                <ul class="nav nav-tabs">\
                    <% if (_.indexOf(tabs, "gen")>-1) { %>\
                        <li class="active">\
                            <a href="#" data-toggle="tab" data-type="general"><%- _("General").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if (_.indexOf(tabs, "x")>-1) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="xaxis"><%- _("X-Axis").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if (_.indexOf(tabs, "y")>-1) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="yaxis"><%- _("Y-Axis").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if (_.indexOf(tabs, "leg")>-1) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="legend"><%- _("Legend").t() %></a>\
                        </li>\
                    <% } %>\
                    <% if (_.indexOf(tabs, "ranges")>-1) { %>\
                        <li>\
                            <a href="#" data-toggle="tab" data-type="ranges"><%- _("Color Ranges").t() %></a>\
                        </li>\
                    <% } %>\
                </ul>\
                <div class="tab-content"></div>\
            '
        });
    }
);
