define([
            'jquery',
            'underscore',
            'module',
            'views/shared/controls/TextControl',
            'views/shared/controls/ControlGroup'
        ],
        function(
            $,
            _,
            module,
            TextControl,
            ControlGroup
        ) {

    var SliceMinimumSizeControl = TextControl.extend({

        initialize: function() {
            this.options = $.extend({
                modelAttribute: 'display.visualizations.charting.chart.sliceCollapsingThreshold',
                inputClassName: 'input-mini',
                append: _('<span class="add-on"><%- _("%").t() %></span>').template({})
            }, this.options);
            TextControl.prototype.initialize.call(this, this.options);
        },

        // the next two methods normalize the model attribute from a decimal in the model to a percent when displayed to the user
        setValueFromModel: function(render) {
            var rawValue = this.model.get(this.getModelAttribute()),
                // SPL-72065, need to do this to avoid floating point errors
                normalizedValue = parseFloat((parseFloat(rawValue) * 100).toPrecision(12));

            this._setValue(_.isNaN(normalizedValue) ? rawValue : normalizedValue, render);
            return this;
        },

        getUpdatedModelAttributes: function() {
            var updateAttrs = TextControl.prototype.getUpdatedModelAttributes.apply(this, arguments),
                rawValue = updateAttrs[this.getModelAttribute()],
                normalizedValue = parseFloat(updateAttrs[this.getModelAttribute()]) / 100;

            updateAttrs[this.getModelAttribute()] = _.isNaN(normalizedValue) ? rawValue : normalizedValue;
            return updateAttrs;
        }

    });

    return ControlGroup.extend({

        moduleId: module.id,

        initialize: function() {
            this.options.label = _("Minimum Size").t();
            this.options.controls = [
                new SliceMinimumSizeControl({ model: this.model })
            ];
            this.options.help = _('Minimum Size is applied when there are more than 10 slices.').t();
            ControlGroup.prototype.initialize.call(this, this.options);
        }

    });

});
