define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/timeline/Title',
        'views/shared/searchbar/Master',
        'views/shared/FlashMessages',
        'views/timeline/results/Master',
        'views/timeline/initialhelp/Master',
        'util/splunkd_utils'
    ],
    function(
        _,
        module,
        BaseView,
        TitleView,
        SearchBarView,
        FlashMessageView,
        ResultsContainerView,
        InitialHelpView,
        splunkd_utils
    ){
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         timeline: <model.services.search.job.TimelineV2>,
             *         searchJob: <models.services.search.Job>,
             *         report: <models.Report>,
             *         reportPristine: <models.Report>,
             *         chartWorking: <models.Charting>,
             *         resultJsonCols: <models.services.search.jobs.ResultJsonCols>,
             *         resultJsonRows: <models.services.search.jobs.ResultJsonRows>,
             *         searchResultsPreview: <models.services.search.jobs.searchResultsPreview>,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>,
             *         user: <models.services.admin.User>
             *     },
             *     collection: {
             *         times (Optional): <collections.services.data.ui.TimesV2>,
             *         selectedFields: <collections.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     }
             * }
             *
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.errorTypes = [splunkd_utils.FATAL, splunkd_utils.ERROR];

                this.isError = splunkd_utils.messagesContainsOneOfTypes(this.model.report.error.get("messages"), this.errorTypes) ||
                                splunkd_utils.messagesContainsOneOfTypes(this.model.searchJob.error.get("messages"), this.errorTypes);

                this.hasResultsContainer = this.model.report.entry.content.get('search') || this.isError;

                //children views
                this.children.searchBarView = new SearchBarView({
                    model: {
                        state: this.model.report.entry.content,
                        timeRange: this.model.timeRange,
                        user: this.model.user,
                        appLocal: this.model.appLocal,
                        application: this.model.application
                    },
                    collection: {
                        times: this.collection.times
                    },
                    singletonInput: true,
                    disableOnSubmit: true,
                    primaryAction: true
                });

                this.children.titleView = new TitleView({
                    model: {
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    }
                });

                this.children.flashMessageView = new FlashMessageView({
                    model: {
                        searchJob: this.model.searchJob,
                        searchJobControl: this.model.searchJob.control,
                        report: this.model.report
                    },
                    whitelist: this.errorTypes
                });

                this.children.initialHelpView = new InitialHelpView({
                    model: {
                        report: this.model.report,
                        metaDataResult: this.model.metaDataResult,
                        application: this.model.application,
                        appLocal: this.model.appLocal
                    }
                });

                if (this.hasResultsContainer){
                    this.children.resultsContainerView = new ResultsContainerView({
                        model: {
                            result: this.model.result,
                            summary: this.model.summary,
                            timeline: this.model.timeline,
                            searchJob: this.model.searchJob,
                            report: this.model.report,
                            chartWorking: this.model.chartWorking,
                            resultJsonCols: this.model.resultJsonCols,
                            resultJsonRows: this.model.resultJsonRows,
                            application: this.model.application,
                            appLocal: this.model.appLocal
                        },
                        collection:  {
                            selectedFields: this.collection.selectedFields,
                            eventRenderers: this.collection.eventRenderers,
                            workflowActions: this.collection.workflowActions
                        },
                        showAutoPause: this.options.showAutoPause
                    });
                }
            },
            render: function() {
                this.$el.html(this.template);
                var $inner = this.$('.section-header');

                if (this.hasResultsContainer){
                    $inner.append(this.children.titleView.render().el);
                    $inner.append(this.children.searchBarView.render().el);
                    $inner.append(this.children.flashMessageView.render().el);
                    this.$el.append(this.children.resultsContainerView.render().el);
                } else {
                    $inner.append(this.children.titleView.render().el);
                    $inner.append(this.children.searchBarView.render().el);
                    $inner.append(this.children.flashMessageView.render().el);
                    this.$el.append(this.children.initialHelpView.render().el);
                }
                                
                return this;
            },
            reflow: function() {
                BaseView.prototype.reflow.apply(this, arguments);
                
                if (this.hasResultsContainer) {
                    var containerTop = this.$el.offset().top,
                        currentScrollPos = $(document).scrollTop();
                    
                    if (currentScrollPos < (containerTop - 1)) {
                        $(document).scrollTop(containerTop);
                    }
                }
            },
            template: '<div class="section-padded section-header"></div>'
        });
    }
);
