define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/timeline/actions/Master'
    ],
    function(_, module, Base, Actions){
        return Base.extend({
            tagName: "div",
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         searchJob: <helpers.ModelProxy>,
             *         report: <models.services.SavedSearch>,
             *         appLocal: <models.services.AppLocal>,
             *         user: <models.services.admin.User>
             *     }
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.children.actions = new Actions({
                    model: {
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        application: this.model.application,
                        searchJob: this.model.searchJob,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    }
                });
            },
            render: function() {
                var activeSearchName = this.model.report.entry.get('name');
                var searchName = _('Search').t();
                var iconClass = 'icon-search-thin';

                if (activeSearchName && activeSearchName !== "_new") {
                    searchName = activeSearchName;
                    if (this.model.reportPristine.isAlert()) {
                        iconClass = 'icon-bell';
                    } else {
                        iconClass = 'icon-report';
                    }
                } else if (!this.model.searchJob.isNew() || (this.model.report.entry.content.get("search"))) {
                    searchName = _('New Search').t();
                }

                var template = this.compiledTemplate({
                    searchName: searchName,
                    iconClass: iconClass
                });
                this.$el.html(template);
                
                this.$el.prepend(this.children.actions.render().el);
            },
            template: '\
                    <h2 class="search-name section-title" title="<%- searchName %>"><i class="<%= iconClass %>"></i> <%- searchName %></h2>\
                '
        });
    }
);