define(
    [
        'module',
        'views/Base',
        'views/timeline/actions/SaveReport',
        'views/timeline/actions/savemenu/Master',
        'views/timeline/actions/View',
        'views/timeline/actions/Close',
        'util/splunkd_utils'
    ],
    function(module, Base, SaveReport, SaveControls, View, Close, splunkd_utils) {
        /**
         * View Hierarchy:
         * Save
         *
         * Save As Controls +
         *      As Report
         *      As Dashboard Panel
         *      As Alert
         *      As EventType
         *
         * View
         * Close
         */
        return Base.extend({
            moduleId: module.id,
            className: 'document-controls',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.errorTypes = [splunkd_utils.FATAL, splunkd_utils.ERROR];

                //save report
                this.children.saveReport = new SaveReport({
                    model:  {
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        application: this.model.application,
                        searchJob: this.model.searchJob,
                        user: this.model.user
                    }
                });

                //Save As Controls
                this.children.saveControls = new SaveControls({
                    model: {
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        application: this.model.application,
                        searchJob: this.model.searchJob,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    }
                });

                this.children.view = new View({
                    model: {
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        application: this.model.application,
                        searchJob: this.model.searchJob,
                        user: this.model.user
                    }
                });

                this.children.close = new Close({
                    model: {
                        report: this.model.report,
                        application: this.model.application
                    }
                });
            },
            render: function() {
                this.$el.html(this.template);

                var isError = splunkd_utils.messagesContainsOneOfTypes(this.model.report.error.get("messages"), this.errorTypes) ||
                        splunkd_utils.messagesContainsOneOfTypes(this.model.searchJob.error.get("messages"), this.errorTypes),
                        $btnGroup = this.$('.btn-group');

                if (!this.model.report.isNew() && !isError) {
                    this.$el.prepend(this.children.saveReport.render().el);
                }

                if ((!this.model.report.isNew() || (this.model.report.entry.content.get("search")))  && !isError) {
                    $btnGroup.append(this.children.saveControls.render().el);
                }

                if (!this.model.report.isNew() && !isError) {
                    $btnGroup.append(this.children.view.render().el);
                }

                if (!this.model.report.isNew() || (this.model.report.entry.content.get("search")) || isError) {
                    $btnGroup.append(this.children.close.render().el);
                }
                
                return this;
            },
            template: '<div class="btn-group"></div>'

        });
    }
);
