define(
    [
         'underscore',
         'module',
         'views/Base',
         'views/shared/reportcontrols/dialogs/savereport/Master'
     ],
     function(_, module, Base, SaveDialog) {
        return Base.extend({
            tagName: "span",
            moduleId: module.id,
            className: 'save-report',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                
                this.model.report.entry.content.on("change", this.render, this);
                this.model.report.on("sync", this.render, this);
                this.model.searchJob.on("prepared", this.render, this);
            },
            events: {
                'click a.btn-primary': function(e) {
                    this.children.saveDialog = new SaveDialog({
                        model: {
                            report: this.model.report,
                            reportPristine: this.model.reportPristine,
                            application: this.model.application,
                            searchJob: this.model.searchJob,
                            user: this.model.user
                        },
                        onHiddenRemove: true
                    });

                    $("body").append(this.children.saveDialog.render().el);
                    this.children.saveDialog.show();

                    e.preventDefault();
                },
                'click a.disabled': function(e) {
                    e.preventDefault();
                }
            },
            render: function() {
                var canWrite = this.model.report.canWrite(this.model.user.canScheduleSearch(), this.model.user.canRTSearch()),
                    blackList = [
                        "^disable_timerange_trigger",
                        '^display\.events\.timelineEarliestTime',
                        '^display\.events\.timelineLatestTime',
                        '^display\.events\.table\.sortColumn',
                        '^display\.events\.table\.sortDirection',
                        '^display\.statistics\.sortColumn',
                        '^display\.statistics\.sortDirection',
                        "^display\.prefs\..*"
                    ],
                    hasChanged, template;
                
                if (canWrite && !this.model.searchJob.isPreparing()) {
                    hasChanged = !this.model.report.contentEqualsFromBlacklist(this.model.reportPristine, blackList);
                    template = this.compiledTemplate({
                        _: _,
                        report: this.model.report,
                        hasChanged: hasChanged
                    });                    
                } else {
                    template = "";
                }
                
                this.$el.html(template);
                return this;
            },
            template: '\
                <% if (!report.isNew() && hasChanged) { %>\
                    <a class="btn btn-primary" href="#"><%- _("Save").t() %></a>\
                <% } else {%>\
                    <a class="btn disabled" href="#"><%- _("Save").t() %></a>\
                <% } %>\
            '
        });
    }
);
