define(
    [
         'jquery',
         'underscore',
         'module',
         'views/Base',
         'views/timeline/actions/savemenu/SaveReportAs',
         'views/timeline/actions/savemenu/CreateAlert',
         'views/timeline/actions/savemenu/CreateEventType',
         'views/timeline/actions/savemenu/CreateDashboardPanel',
         'views/shared/delegates/Popdown'
    ],
    function(
         $,
         _,
         module,
         Base,
         SaveReportAs,
         CreateAlert,
         CreateEventType,
         CreateDashboardPanel,
         Popdown
    ){
        return Base.extend({
            moduleId: module.id,
            className: 'save btn-combo',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                //save report as
                this.children.saveReportAs = new SaveReportAs({
                    model:  {
                        report: this.model.report,
                        application: this.model.application,
                        searchJob: this.model.searchJob,
                        user: this.model.user
                    }
                });

                //create alert
                this.children.createAlert = new CreateAlert({
                    model: {
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    }
                });

                //create dashboard panel
                this.children.createDashboardPanel = new CreateDashboardPanel({
                    model: {
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    }
                });

                //create event type
                this.children.createEventType = new CreateEventType({
                    model: {
                        report: this.model.report,
                        application: this.model.application
                    }
                });

                this.model.report.entry.fields.on('change', function(){
                    var optional = this.model.report.entry.fields.get('optional');
                    (_.isArray(optional) && _.indexOf(optional, 'cron_schedule') > -1) ?
                        this.children.createAlert.$el.show():
                        this.children.createAlert.$el.hide();
                }, this);
                
                this.model.searchJob.on("prepared", this.render, this);
            },
            render: function() {
                if (this.model.searchJob.isPreparing()) {
                    this.$el.html("");
                } else {
                    this.$el.html(this.compiledTemplate({
                        _: _
                    }));
                    this.children.popdown = new Popdown({el: this.el, attachDialogTo:'body'});
    
                    this.$('.save_reports_actions').append(this.children.saveReportAs.render().el);
    
                    this.$('.save_reports_actions').append(this.children.createDashboardPanel.render().el);
    
                    if (this.model.user.canScheduleSearch() && !this.model.user.isFree()) {
                        this.$('.save_reports_actions').append(this.children.createAlert.render().el);
                    }
    
                    this.$('.report_actions').append(this.children.createEventType.render().el);
                }
                return this;
            },
            template: '\
                <a class="dropdown-toggle btn" href="#">\
                    <%- _("Save As").t() %><span class="caret"></span>\
                </a>\
                <div class="dropdown-menu dropdown-menu-narrow">\
                    <div class="arrow"></div>\
                    <ul class="save_reports_actions"></ul>\
                    <ul class="report_actions"></ul>\
                </div>\
            '
        });
    }
);
