define(
    [
        'jquery',
        'underscore',
        'module',
        'models/Job',
        'models/services/search/jobs/Result',
        'views/Base',
        'views/timeline/initialhelp/datasummary/Master',
        'util/time_utils'
    ],
    function($, _, module, JobModel, ResultModel, Base, DataSummaryModal, time_utils) {
        return Base.extend({
            moduleId: module.id,
            className:'panel panel-what',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                
                this.model.metaDataResult.on("sync", this.render, this);
            },
            events: {
                'click a.btn-data-summary': function(e) {
                    e.preventDefault();
                    
                    if (this.$("a.btn-data-summary").hasClass("disabled")) {
                        return;
                    } else {
                        this.$("a.btn-data-summary").addClass("disabled");
                    }
                    
                    var sourceTypesJobDeferred = $.Deferred(),
                        hostsJobDeferred = $.Deferred(),
                        sourcesJobDeferred = $.Deferred();
                    
                    this.model.sourceTypesResult = new ResultModel();
                    this.model.sourceTypesResult.fetchData.set({count: 50}, {silent: true});
                    this.model.hostsResult = new ResultModel();
                    this.model.hostsResult.fetchData.set({count: 50}, {silent: true});
                    this.model.sourcesResult = new ResultModel();
                    this.model.sourcesResult.fetchData.set({count: 50}, {silent: true});
                    
                    this.model.sourceTypesJob = JobModel.createMetaDataSearch(
                        '| metadata type=sourcetypes | search totalCount > 0 | table sourcetype totalCount recentTime | fieldformat totalCount=tostring(totalCount, "commas") | fieldformat recentTime=strftime(recentTime, "' + time_utils.ISO_PATTERN + '")',
                        sourceTypesJobDeferred,
                        this.model.application
                    );
                    
                    $.when(sourceTypesJobDeferred).then(function() {
                        var resultsPreviewId = this.model.sourceTypesJob.entry.links.get('results_preview');
                        
                        if (resultsPreviewId) {
                            this.model.sourceTypesResult.set("id", resultsPreviewId);
                            this.model.sourceTypesResult.safeFetch();
                            this.model.sourceTypesJob.on("jobProgress", function() {
                                var resultPreviewCount = this.model.sourceTypesJob.entry.content.get("resultPreviewCount");
                                if (_.isNumber(resultPreviewCount) && (resultPreviewCount > 0)) {
                                    this.model.sourceTypesResult.safeFetch();
                                }
                            }, this);
                        }
                    }.bind(this));
                    
                    this.model.hostsJob = JobModel.createMetaDataSearch(
                        '| metadata type=hosts | search totalCount > 0 | table host totalCount recentTime | fieldformat totalCount=tostring(totalCount, "commas") | fieldformat recentTime=strftime(recentTime, "' + time_utils.ISO_PATTERN + '")',
                        hostsJobDeferred,
                        this.model.application
                    );
                    
                    $.when(hostsJobDeferred).then(function() {
                        var resultsPreviewId = this.model.hostsJob.entry.links.get('results_preview');
                        
                        if (resultsPreviewId) {
                            this.model.hostsResult.set("id", resultsPreviewId);
                            this.model.hostsResult.safeFetch();
                            this.model.hostsJob.on("jobProgress", function(){
                                var resultPreviewCount = this.model.hostsJob.entry.content.get("resultPreviewCount");
                                if (_.isNumber(resultPreviewCount) && (resultPreviewCount > 0)) {
                                    this.model.hostsResult.safeFetch();
                                }                               
                            }, this);
                        }
                    }.bind(this));
                    
                    this.model.sourcesJob = JobModel.createMetaDataSearch(
                        '| metadata type=sources | search totalCount > 0 | table source totalCount recentTime | fieldformat totalCount=tostring(totalCount, "commas") | fieldformat recentTime=strftime(recentTime, "' + time_utils.ISO_PATTERN + '")',
                        sourcesJobDeferred,
                        this.model.application
                    );
                    
                    $.when(sourcesJobDeferred).then(function() {
                        var resultsPreviewId = this.model.sourcesJob.entry.links.get('results_preview');
                        
                        if (resultsPreviewId) {
                            this.model.sourcesResult.set("id", resultsPreviewId);
                            this.model.sourcesResult.safeFetch();
                            this.model.sourcesJob.on("jobProgress", function(){
                                var resultPreviewCount = this.model.sourcesJob.entry.content.get("resultPreviewCount");
                                if (_.isNumber(resultPreviewCount) && (resultPreviewCount > 0)) {
                                    this.model.sourcesResult.safeFetch();
                                }                               
                            }, this);
                        }
                    }.bind(this));
                    
                    $.when(sourceTypesJobDeferred, hostsJobDeferred, sourcesJobDeferred).then(function(){
                        this.model.sourceTypesJob.startPolling();
                        this.model.hostsJob.startPolling();
                        this.model.sourcesJob.startPolling();
                        
                        this.children.dataSummaryModal = new DataSummaryModal({
                            model: {
                                report: this.model.report,
                                sourceTypesJob: this.model.sourceTypesJob,
                                hostsJob: this.model.hostsJob,
                                sourcesJob: this.model.sourcesJob,
                                sourceTypesResult: this.model.sourceTypesResult,
                                hostsResult: this.model.hostsResult,
                                sourcesResult: this.model.sourcesResult,
                                application: this.model.application
                            },
                            onHiddenRemove: true
                        });

                        $("body").append(this.children.dataSummaryModal.render().el);
                        this.children.dataSummaryModal.show();
                        
                        this.children.dataSummaryModal.on("hidden", function(){                            
                            this.model.sourceTypesJob.destroy();
                            this.model.hostsJob.destroy();
                            this.model.sourcesJob.destroy();
                            this.$("a.btn-data-summary").removeClass("disabled");
                        }, this);
                    }.bind(this));
                }
            },
            render: function() {
                var result = this.model.metaDataResult.results.at(0),
                    hasResult = !!result,
                    cntEvents, earliestRel, latestRel, latestTimeRaw, now;
                
                if (hasResult) {
                    cntEvents = result.get('cnt')[0] || '0';
                    earliestRel = time_utils.convertToRelativeTime(parseInt(result.get('min')[0], 10)) || _('N/A').t();
                    latestTimeRaw = parseInt(result.get('max')[0], 10);
                    now = (new Date()).getTime()/1000;
                    if (latestTimeRaw >= now) {
                        latestRel = _('Now').t();
                    } else {
                        latestRel = time_utils.convertToRelativeTime(latestTimeRaw) || _('N/A').t();
                    }
                }
                
                this.el.innerHTML = this.compiledTemplate({
                    _: _,
                    hasResult: hasResult,
                    cntEvents: cntEvents,
                    earliestRel: earliestRel,
                    latestRel: latestRel
                });

                return this;
            },
            template: '\
                <h3><%- _("What to Search").t() %></h3>\
                <div class="what-columns">\
                    <% if (hasResult) { %>\
                        <div style="float:left; width: 30%; text-align:center">\
                            <%- format_decimal(cntEvents) %> <%- _("Events").t() %><br>\
                            <span style="text-transform: uppercase; color: #999; font-size: 11px;"><%- _("Indexed").t() %></span>\
                        </div>\
                        <div style="float:left;  width: 30%; text-align:center">\
                            <%= earliestRel %> <br>\
                            <span style="text-transform: uppercase; color: #999; font-size: 11px;"><%- _("Earliest Event").t() %></span>\
                        </div>\
                        <div style="float:left; width: 30%; text-align:center">\
                            <%= latestRel %> <br>\
                            <span style="text-transform: uppercase; color: #999; font-size: 11px;"><%- _("Latest Event").t() %></span>\
                        </div>\
                    <% } else { %>\
                        <%- _("Waiting for data...").t() %>\
                    <% } %>\
                </div>\
                <a href="#" class="btn btn-data-summary" style="margin-bottom: 10px"><%- _("Data Summary").t() %></a>\
            '
        });
    }
);
