define(
    [
        'jquery',
        'module',
        'underscore',
        'views/Base',
        'views/timeline/results/EventsTab',
        'views/timeline/results/eventspane/Master',
        'views/timeline/results/StatisticsTab',
        'views/timeline/results/statisticspane/Master',
        'views/timeline/results/VisualizationsTab',
        'views/timeline/results/VisualizationsPane',
        'views/shared/delegates/Tabs',
        'views/shared/JobDispatchState',
        'views/shared/jobstatus/Master',
        'util/splunkd_utils'
    ],
    function(
        $,
        module,
        _,
        BaseView,
        EventsTab,
        EventsPane,
        StatisticsTab,
        StatisticsPane,
        VisualizationsTab,
        VisualizationsPane,
        TabsDelegate,
        JobDispatchState,
        JobStatusView,
        splunkd_utils
    ) {
        return BaseView.extend({
            moduleId: module.id,
             /**
             * @param {Object} options {
             *     model: {
             *         result: <models.services.search.job.ResultsV2>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         timeline: <model.services.search.job.TimelineV2>,
             *         searchJobProxy: <helpers.ModelProxy>,
             *         resultJsonCols: <models.services.search.jobs.ResultJsonCols>,
             *         resultJsonRows: <models.services.search.jobs.ResultJsonRows>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>
             *     },
             *     collections: {
             *         workflowActions: <collections.services.data.ui.WorkflowActions>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         selectedFields: <collections.SelectedFields>,
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                
                this.errorTypes = [splunkd_utils.FATAL, splunkd_utils.ERROR, splunkd_utils.NOT_FOUND];

                this.permanentError = splunkd_utils.messagesContainsOneOfTypes(this.model.report.error.get("messages"), this.errorTypes) ||
                                splunkd_utils.messagesContainsOneOfTypes(this.model.searchJob.error.get("messages"), this.errorTypes);
                
                this.isGoingToBeRemoved = false;
                
                //blank state messages if the job is not yet reached running or beyond
                this.children.jobDispatchState = new JobDispatchState({
                    model: this.model.searchJob
                });
                
                this.children.jobStatusView = new JobStatusView({
                    model: {
                        searchJob: this.model.searchJob,
                        state: this.model.report.entry.content,
                        application: this.model.application,
                        appLocal: this.model.appLocal
                    },
                    enableSearchMode: true,
                    showAutoPause: this.options.showAutoPause
                });
                
                //event tab
                this.children.eventTab = new EventsTab({
                    model: {
                        report: this.model.report,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob
                    }
                });

                this.children.eventsPane = new EventsPane({
                    model: {
                        result: this.model.result,
                        summary: this.model.summary,
                        timeline: this.model.timeline,
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                   }
                });

                //statistics tab
                this.children.statisticsTab = new StatisticsTab({
                    model: {
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    }
                });

                //statistics tab pane
                this.children.statisticsPane = new StatisticsPane({
                    id: 'statistics_' + this.cid,
                    model: {
                        report: this.model.report,
                        resultJsonRows: this.model.resultJsonRows,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        appLocal: this.model.appLocal
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields
                    }
                });

                //viz tab
                this.children.visualizationsTab = new VisualizationsTab({
                    model: {
                        report: this.model.report
                    }
                });

                //viz tab pane
                this.children.visualizationsPane = new VisualizationsPane({
                    model: {
                        report: this.model.report,
                        chartWorking: this.model.chartWorking,
                        resultJsonCols: this.model.resultJsonCols,
                        resultJsonRows: this.model.resultJsonRows,
                        searchJob: this.model.searchJob,
                        appLocal: this.model.appLocal,
                        application: this.model.application
                    }
                });
                
                this.model.report.entry.content.on('change:display.general.type', this.wake, this);
                
                this.model.searchJob.on("prepared", this.render, this);
                
                this.model.searchJob.entry.content.on("change:dispatchState", function() {
                    var dispatchState = this.model.searchJob.entry.content.get('dispatchState');
                    this.setSearchErrorState(_.isUndefined(dispatchState));
                }, this);

                this.model.searchJob.on("sync error", function() {
                    var isError = this.model.searchJob.entry.content.get("isFailed") || splunkd_utils.messagesContainsOneOfTypes(this.model.searchJob.error.get("messages"), [splunkd_utils.NOT_FOUND]);
                    this.permanentError = isError;
                    this.setSearchErrorState(isError);
                }, this);
                
                this.model.searchJob.control.on("sync error", function() {
                    var isError = splunkd_utils.messagesContainsOneOfTypes(this.model.searchJob.control.error.get("messages"), [splunkd_utils.NOT_FOUND, splunkd_utils.FATAL]);
                    this.permanentError = isError;
                    this.setSearchErrorState(isError);
                }, this);
                
                this.model.report.entry.content.on("change:search change:dispatch.earliest_time change:dispatch.latest_time change:display.page.search.mode", _.debounce(function(){
                    this.isGoingToBeRemoved = true;
                    this.setSearchErrorState(true);
                }.bind(this), 0), this);
                
                this.model.report.on("request close", function() {
                    this.isGoingToBeRemoved = true;
                    this.setSearchErrorState(true);                    
                }, this);
            },
            setSearchErrorState: function(isError) {
                var sid = this.model.searchJob.id;

                if (this.permanentError || isError || !sid || this.isGoingToBeRemoved) {
                    if (!sid || this.isGoingToBeRemoved) {
                        this.children.jobStatusView.sleep().$el.css('visibility', 'hidden');
                    } else {
                        this.children.jobStatusView.wake().$el.css('visibility', 'visible');
                    }
                    this.children.eventTab.$el.addClass('timeline-disabled');
                    this.children.statisticsTab.$el.addClass('timeline-disabled');
                    this.children.visualizationsTab.$el.addClass('timeline-disabled');
                    this.children.statisticsPane.$el.addClass('timeline-disabled');
                    this.children.visualizationsPane.$el.addClass('timeline-disabled');
                    this.children.eventsPane.$el.addClass('timeline-disabled');
                } else {
                    this.children.jobStatusView.wake().$el.css('visibility', 'visible');
                    this.children.eventTab.$el.removeClass('timeline-disabled');
                    this.children.statisticsTab.$el.removeClass('timeline-disabled');
                    this.children.visualizationsTab.$el.removeClass('timeline-disabled');
                    this.children.statisticsPane.$el.removeClass('timeline-disabled');
                    this.children.visualizationsPane.$el.removeClass('timeline-disabled');
                    this.children.eventsPane.$el.removeClass('timeline-disabled');
                }
            },
            wake: function() {
                switch(this.model.report.entry.content.get('display.general.type')) {
                    case 'events':
                        this.children.eventsPane.wake().$el.show();
                        this.children.statisticsPane.sleep().$el.hide();
                        this.children.visualizationsPane.sleep().$el.hide();
                        break;
                    case 'statistics':
                        this.children.eventsPane.sleep().$el.hide();
                        this.children.statisticsPane.wake().$el.show();
                        this.children.visualizationsPane.sleep().$el.hide();
                        break;
                    case 'visualizations':
                        this.children.eventsPane.sleep().$el.hide();
                        this.children.statisticsPane.sleep().$el.hide();
                        this.children.visualizationsPane.wake().$el.show();
                        break;
                }
                return this;
            },
            render: function() {
                if (!this.el.innerHTML) {
                    this.$el.html(this.template);
                    this.$el.prepend(this.children.jobStatusView.render().el);
                }
                
                if (this.model.searchJob.isPreparing()) {
                    this.$('.tab-content').html(this.children.jobDispatchState.render().el);
                } else {
                    this.$('.nav-tabs').append(this.children.eventTab.render().el);
                    this.$('.nav-tabs').append(this.children.statisticsTab.render().el);
                    this.$('.nav-tabs').append(this.children.visualizationsTab.render().el);
                    this.$('> .tab-content').html(this.children.statisticsPane.render().el);
                    this.$('> .tab-content').append(this.children.visualizationsPane.render().el);
                    this.$('> .tab-content').append(this.children.eventsPane.render().el);
                    this.wake();
                    this.setSearchErrorState(this.permanentError);
                }
                
                return this;
            },
            template: '\
                <ul class="nav nav-tabs main-tabs"></ul>\
                <div class="tab-content" style="overflow:visible;"></div>\
            '
        });
    }
);
