define(
    [
        'underscore',
        'jquery',
        'module',
        'jquery.ui.draggable',
        'models/services/search/IntentionsParser',
        'views/Base',
        'views/shared/vizcontrols/Master',
        'views/shared/JSChart',
        'views/shared/SingleValue',
        'views/shared/results_table/ResultsTableMaster',
        'views/timeline/results/shared/NoStatistics',
        'views/shared/JobDispatchState'
    ],
    function(_, $, module, jqueryDraggable, IntentionsParser,  Base, VizEditor, JSChart, SingleValue, ResultsTable, NoStatistics, JobDispatchState) {
        return Base.extend({
            moduleId: module.id,
            className: 'tab-pane',
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                
                this.model.intentionsParser = new IntentionsParser();
                this.model.intentionsParser.on('change', function() {
                    var attrs = $.extend({}, {
                        'search': this.model.intentionsParser.get('fullSearch'),
                        'dispatch.earliest_time': this.model.intentionsParser.get('dispatch.earliest_time'),
                        'dispatch.latest_time': this.model.intentionsParser.get('dispatch.latest_time')
                    });
                    this.model.report.entry.content.set(attrs);
                }, this);
                
                this.children.noStatistics = new NoStatistics({
                    model: {
                        report: this.model.report,
                        appLocal: this.model.appLocal,
                        application: this.model.application
                    }
                });
                
                this.children.jobDispatchState = new JobDispatchState({
                    model: this.model.searchJob,
                    mode: 'auto'
                });
                
                this.children.vizEditor = new VizEditor({
                    model: {
                        report: this.model.report,
                        application: this.model.application
                    },
                    vizTypes: ['visualizations'],
                    bindToChangeOfSearch: false
                });

                this.children.jsChart = new JSChart({
                    model: {
                        searchResultsColumn: this.model.resultJsonCols,
                        state: this.model.report.entry.content
                    },
                    width: '100%',
                    height: this.model.report.entry.content.get('display.visualizations.chartHeight')
                });

                this.children.singleValue = new SingleValue({
                    model: {
                        searchResultsColumn: this.model.resultJsonCols,
                        state: this.model.report.entry.content
                    },
                    width: 800,
                    height: 300
                });
                
                this.children.resultsTable = new ResultsTable({
                    model: {
                        results: this.model.resultJsonRows,
                        metadata: this.model.report.entry.content,
                        format: this.model.report.entry.content
                    },
                    sortKeyAttr: 'display.statistics.sortColumn',
                    sortDirAttr: 'display.statistics.sortDirection',
                    countAttr: 'display.prefs.statistics.count',
                    offsetAttr: 'display.prefs.statistics.offset',
                    disable: true
                });

                //event handlers
                this.model.searchJob.entry.content.on("change:dispatchState change:resultCount change:resultPreviewCount", this.visibility, this);
                this.model.report.entry.content.on('change:display.visualizations.type', this.visibility, this);

                this.children.jsChart.on('chartClick', function(eventInfo) {
                        var field, value;
                        this.model.intentionsParser.clear({silent: true});
                        if(eventInfo.type === 'legendClick') {
                            // if the legend click has a name and value (this happens for scatter charts), use them
                            if(eventInfo.hasOwnProperty('name') && eventInfo.hasOwnProperty('value')) {
                                this.model.intentionsParser.fetch({
                                    data: {
                                        q: this.model.report.entry.content.get('search'),
                                        action: 'fieldvalue',
                                        field: eventInfo.name,
                                        value: eventInfo.value,
                                        fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                        app: this.model.application.get('app'),
                                        owner: this.model.application.get('owner')
                                    }
                                });
                            }
                            // otherwise do a keyword search based on name2
                            else {
                                this.model.intentionsParser.fetch({
                                    data: {
                                        q: this.model.report.entry.content.get('search'),
                                        action: 'keyword',
                                        field: eventInfo.name2,
                                        fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                        app: this.model.application.get('app'),
                                        owner: this.model.application.get('owner')
                                    }
                                });
                            }
                            return;
                        }
                        
                        if(eventInfo._span) {
                            this.model.intentionsParser.set({
                                'dispatch.earliest_time': eventInfo.value,
                                'dispatch.latest_time': JSON.stringify(parseFloat(eventInfo.value) + eventInfo._span)
                            }, {silent: true});
                        }
                        if(eventInfo.name === '_time') {
                            field = eventInfo.name2;
                            value = eventInfo.value2;
                        } else {
                            field = [eventInfo.name, eventInfo.name2];
                            value = [eventInfo.value, eventInfo.value2];
                        }

                        this.model.intentionsParser.fetch({
                            data: {
                                q: this.model.report.entry.content.get('search'),
                                action: 'fieldvalue',
                                field: field,
                                value: value,
                                fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                                app: this.model.application.get('app'),
                                owner: this.model.application.get('owner')
                            }
                        });
                },this);
                
                this.debouncedWindowResize =  _(_.debounce(this.children.jsChart.reflow, 50)).bind(this);
                $(window).on('resize.' + this.uniqueNS(), this.debouncedWindowResize);

                this.model.chartWorking.on('change:editing', function() {
                    if (this.model.chartWorking.get('editing')) {
                        this.children.jsChart.sleep();
                        this.children.singleValue.sleep();
                    } else {
                        this.children.jsChart.wake();
                        this.children.singleValue.wake();
                    }
                }, this);
            },
            updateChartHeight: function() {
                var heightDelta = parseInt(this.$('.chart-resize').css('top').replace('px',''), 10),
                    currentHeight = parseInt(this.model.report.entry.content.get('display.visualizations.chartHeight'), 10) ||
                                    this.children.jsChart.$el.height(),
                    height = currentHeight + heightDelta;
                
                if(height < 26) {
                    height = 26;
                }   
                                    
                this.children.jsChart.setHeight(height);
                this.model.report.entry.content.set('display.visualizations.chartHeight', height);
                this.children.jsChart.reflow(); 
                this.$('.chart-resize').css('top', 0);
            },
            remove: function() {
                Base.prototype.remove.apply(this, arguments);
                $(window).off('resize.' + this.uniqueNS());
            },
            visibility: function() {
                var resultCount = this.model.searchJob.entry.content.get('isPreviewEnabled') ? this.model.searchJob.entry.content.get('resultPreviewCount') : this.model.searchJob.entry.content.get('resultCount'),
                        isSingleValue = (this.model.report.entry.content.get('display.visualizations.type') === 'singlevalue');
                if (!this.model.searchJob.isReportSearch()) {
                    this.children.jobDispatchState.$el.hide();
                    this.children.noStatistics.$el.show();
                    this.children.singleValue.$el.hide();
                    this.children.jsChart.$el.hide();
                    this.children.vizEditor.$el.hide();
                    this.children.resultsTable.$el.hide();
                    this.$('.chart-resize').hide();
                    return;
                }
                if (resultCount === 0) {
                    if (this.model.searchJob.entry.content.get('dispatchState') == 'DONE') {
                        this.children.noStatistics.$el.hide();
                        this.children.singleValue.$el.hide();
                        this.children.jsChart.$el.hide();
                        this.children.jobDispatchState.$el.show();
                        this.children.vizEditor.$el.show();
                        this.children.resultsTable.$el.hide();
                        this.$('.chart-resize').hide();
                    } else {
                        this.children.noStatistics.$el.hide();
                        this.children.singleValue.$el.hide();
                        this.children.jobDispatchState.$el.show();
                        if (isSingleValue) {
                            this.children.jsChart.$el.hide();
                            this.$('.chart-resize').hide();
                        } else {
                            this.children.jsChart.$el.show();
                            this.$('.chart-resize').show();
                        }
                        this.children.vizEditor.$el.show();
                        this.children.resultsTable.$el.hide();
                    }
                } else {
                    this.children.jobDispatchState.$el.hide();
                    this.children.noStatistics.$el.hide();
                    if (isSingleValue) {
                        this.children.singleValue.$el.show();
                        this.children.jsChart.$el.hide();
                        this.$('.chart-resize').hide();
                    } else {
                        this.children.singleValue.$el.hide();
                        this.children.jsChart.$el.show();
                        this.$('.chart-resize').show();
                    }
                    this.children.vizEditor.$el.show();
                    this.children.resultsTable.$el.show();
                }
            },            
            render: function() {
                var that = this;
                this.$el.html(this.template);
                this.$el.append(this.children.noStatistics.render().el);
                this.$(".visualization-controls").append(this.children.vizEditor.render().el);
                this.$el.append(this.children.jsChart.render().el);
                this.$el.append(this.children.singleValue.render().el);
                this.$el.append($('<div class="chart-resize"></div>'));
                this.$('.chart-resize').draggable({ 
                    axis: 'y',
                    stop: function() {
                        that.updateChartHeight();
                    }
                });
                this.$el.append(this.children.jobDispatchState.render().el);
                this.$el.append(this.children.resultsTable.render().el);
                this.visibility();
                return this;
            },
            template: '\
                <div class="visualization-controls"></div>\
            '
        });
    }
);
