define(
    [
        'jquery',
        'underscore',
        'module',
        'collections/FlashMessages',
        'models/Base',
        'views/Base',
        'views/timeline/results/eventspane/fieldsviewer/Master',
        'views/shared/eventsviewer/Master',
        'views/timeline/results/eventspane/controls/Master',
        'views/shared/CanvasTimeline',
        'views/shared/controls/ControlGroup',
        'views/timeline/results/eventspane/VerboseWarning',
        'views/shared/JobDispatchState',
        'views/shared/FlashMessagesLegacy',
        'util/splunkd_utils',
        'splunk.util'
    ],
    function($, _, module, FlashMessagesCollection, BaseModel, BaseView, FieldsViewer, EventsViewer, EventControls, Timeline, ControlGroup, VerboseWarning, JobDispatchState, FlashMessagesView,  splunkd_utils, splunkUtil){
        return BaseView.extend({
            moduleId: module.id,
            className: 'events-fields-container',
            /**
             * @param {Object} options {
             *     model: {
             *         event: <models.services.search.job.ResultsV2>,
             *         summary: <model.services.search.job.SummaryV2>,
             *         timeline: <model.services.search.job.TimelineV2>,
             *         searchJob: <models.Job>,
             *         report: <models.services.SavedSearch>,
             *         application: <models.Application>,
             *     }
             *     collection: {
             *         selectedFields: <collections.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     }
             * }
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);

                //models
                this.model.state = new BaseModel();
                
                this.collection.flashMessages = new FlashMessagesCollection();
                
                this.children.timelineState = new FlashMessagesView({
                    collection: this.collection.flashMessages
                });
                
                //views
                this.children.timeline = new Timeline({
                    className: 'timeline-container',
                    model: {
                        searchJob: this.model.searchJob,
                        report: this.model.report
                    }
                });
                //TODO: we don't currently have this.model.savedSearch....
                this.children.timeline.savedSearchModel = this.model.savedSearch;

                this.children.eventControls = new EventControls({
                     model: {
                        content: this.model.report.entry.content,
                        searchJob: this.model.searchJob,
                        timeline: this.model.timeline,
                        state: this.model.state
                    }
                });

                this.children.fieldsViewer = new FieldsViewer({
                    model: {
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        summary: this.model.summary,	
                        state: this.model.state
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields
                    }
                });

                this.children.eventsViewer = new EventsViewer({
                    model: {
                        result: this.model.result,
                        summary: this.model.summary,
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        state: this.model.state
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions
                    },
                    scrollToTopOnPagination: true
                });
                
                this.children.jobDispatchState = new JobDispatchState({
                    model: this.model.searchJob,
                    mode: 'events'
                });
                
                this.children.verboseWarning = new VerboseWarning({
                    model: {
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    }
                });
                 
                this.model.state.on('change:modalize', function(model) {
                    (model.get('modalize')) ?
                        this.children.timeline.sleep():
                        this.children.timeline.wake();
                },this);
                this.model.state.on('change:fieldpicker', function(model) {
                    if(model.get('fieldpicker')){
                        this.children.eventsViewer.sleep();
                        this.children.timeline.sleep();
                    } else {
                        this.children.eventsViewer.wake();
                        this.children.timeline.wake();
                    }
                },this);

                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime', function() { 
                    var tl_et = this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                        tl_lt = this.model.report.entry.content.get('display.events.timelineLatestTime');
                    
                    if(!(!!(tl_et && tl_lt))){
                        this.children.timelineState.$el.hide();
                        this.visibility();
                    }                    
                },this);
                this.model.timeline.buckets.on('reset', function() {
                    var tl_et = this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                        tl_lt = this.model.report.entry.content.get('display.events.timelineLatestTime');
                    
                    if(!!(tl_et && tl_lt) && (this.model.timeline.availableCount(tl_et, tl_lt).length === 0)){
                        _.chain(this.children).omit(['timeline']).each(function(child) {
                            child.$el.hide();
                        }); 
                        this.children.timelineState.$el.show();
                        
                        this.collection.flashMessages.reset([{
                            key: 'waiting',
                            type: 'info',
                            html: _('No results in current time range.').t()
                        }]);
                    } else {
                        this.children.timelineState.$el.hide();
                        this.visibility();
                    }
                }, this); 

                this.model.report.entry.content.on('change:display.page.search.showFields', this.visibility, this);
                this.model.searchJob.entry.content.on('change:eventCount change:dispatchState', this.visibility, this);
            },
            visibility: function() {
                if (this.model.searchJob.entry.content.get('eventCount') === 0) {
                    this.needsReflow = true;
                    this.children.verboseWarning.$el.hide();
                    this.children.fieldsViewer.$el.hide();
                    this.children.eventControls.$el.hide();
                    this.children.timeline.hide();
                    this.children.eventsViewer.$el.hide();
                    this.children.jobDispatchState.$el.show();
                } else {
                    if (this.model.searchJob.entry.content.get('eventAvailableCount') === 0) {
                        this.needsReflow = true;
                        this.children.fieldsViewer.$el.hide();
                        this.children.eventControls.$el.hide();
                        this.children.timeline.hide();
                        this.children.eventsViewer.$el.hide();
                        if (this.model.searchJob.isReportSearch() && (this.model.searchJob.getAdhocSearchMode() !== splunkd_utils.VERBOSE)) {
                            this.children.verboseWarning.$el.show();
                            this.children.jobDispatchState.$el.hide();
                        } else {
                            this.children.verboseWarning.$el.hide();
                            this.children.jobDispatchState.$el.show();
                        }
                    } else {
                        this.children.verboseWarning.$el.hide();
                        this.children.eventControls.$el.show();
                        this.children.eventsViewer.$el.show();
                        this.children.jobDispatchState.$el.hide();
                        if (this.model.searchJob.entry.content.get('statusBuckets') > 0) {
                            this.children.timeline.show();
                            if (splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.page.search.showFields'))) {
                                this.$el.addClass('show-fields');
                                this.children.fieldsViewer.$el.show();
                            } else {
                                this.$el.removeClass('show-fields');
                                this.children.fieldsViewer.$el.hide();
                            }                                
                        } else {
                            this.children.timeline.hide();
                            this.$el.removeClass('show-fields');
                            this.children.fieldsViewer.$el.hide();
                        }
                        this._reflow();
                    }
                }
            },
            _reflow: function() {
                if(this.needsReflow) {
                    this.needsReflow = !this.needsReflow;
                    this.reflow();
                }
            },
            render: function() {
                this.$el.append(this.children.timeline.render().el);
                this.$el.append(this.children.timelineState.render().el);
                this.$el.append(this.children.jobDispatchState.render().el);
                this.$el.append(this.children.verboseWarning.render().el);
                this.$el.append(this.children.eventControls.render().el);
                this.$el.append('<div class="timeline-results-wrapper"></div>');
                this.$('.timeline-results-wrapper').append(this.children.fieldsViewer.render().el);
                this.$('.timeline-results-wrapper').append(this.children.eventsViewer.render().el);
                this.visibility();
                return this;
            }
        });
    }
);
