define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/timeline/results/eventspane/controls/ShowField',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/controls/ControlGroup',
        'views/shared/SearchResultsPaginator',
        'views/shared/delegates/Popdown',
        'views/shared/delegates/Dock',
        'views/shared/vizcontrols/components/Events',
        'splunk.util',
        'helpers/user_agent'
    ],
    function(_, module, Base, ShowField, SyntheticSelectControl, ControlGroup, SearchResultsPaginator, Popdown, Dock, EventsFormat, splunkUtil, userAgent) {
        return Base.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         content: <models.services.SavedSearch.Content>,
             *         timeline: <model.services.search.job.TimelineV2>,
             *         searchJob: <models.Job>,
             *     }
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                this.children.showField = new ShowField({
                    model: {
                        content: this.model.content,
                        state: this.model.state 
                    }
                });
                this.children.count = new SyntheticSelectControl({
                        modelAttribute: 'display.prefs.events.count',
                        model: this.model.content,
                        items: [
                            { value: '10',  label: _('10 Per Page').t()  },
                            { value: '20',  label: _('20 Per Page').t()  },
                            { value: '50',  label: _('50 Per Page').t()  }
                        ],
                        save: false,
                        elastic: true,
                        menuWidth: "narrow",
                        toggleClassName: 'btn-pill'
                });
                this.children.tabletype = new SyntheticSelectControl({
                        modelAttribute: 'display.events.type',
                        model: this.model.content,
                        items: [
                            { label: _('Raw').t(),  value: 'raw'  },
                            { label: _('List').t(), value: 'list' },
                            { label: _('Table').t(), value: 'table' }
                        ],
                        menuWidth: "narrow",
                        save: false,
                        elastic: true,
                        className: 'btn-group',
                        toggleClassName: 'btn-pill'
                });

                this.children.eventsResultsPaginator = new SearchResultsPaginator({
                    model: {
                        state: this.model.content,
                        searchJob: this.model.searchJob,
                        timeline: this.model.timeline
                    },
                    offsetKey: 'display.prefs.events.offset',
                    countKey: 'display.prefs.events.count'
                });

                this.children.eventsFormat = new EventsFormat({
                    model: this.model.content,
                    showEventType: false
                });

                this.model.state.on('change:modalize', function() {
                    var tableDock = this.children.tableDock;
                    if (tableDock) {
                        (this.model.state.get('modalize') >= 0) ?
                            tableDock.disable():
                            tableDock.enable();
                    }
                },this);

                this.model.searchJob.entry.content.on('change:statusBuckets', this.visibility, this);
                this.model.content.on('change:display.page.search.showFields', this.visibility, this);
            },
            events: {
                'click .events-controls-inner.disabled': function(e) {
                    this.model.state.trigger('unmodalize', this.model.state.get('modalize'));
                    this.model.state.unset('modalize');
                    e.preventDefault();
                }
            },
            visibility: function() {
                var $showField = this.children.showField.$el;
                if (this.model.searchJob.entry.content.get('statusBuckets') > 0 && !splunkUtil.normalizeBoolean(this.model.content.get('display.page.search.showFields'))) {
                    $showField.show();
                } else {
                    $showField.hide();
                }
            },
            render: function() {
                this.$el.html(this.compiledTemplate({
                    _: _
                }));
                this.visibility();
                this.$('.events-controls-inner').prepend(this.children.tabletype.render().el);
                this.$('.events-controls-inner').prepend(this.children.showField.render().el);
                this.$('.events-controls-inner').append(this.children.count.render().el);
                this.children.popdown = new Popdown({el: this.$(".format-events"), toggle:'> .format.btn-pill', mode: "dialog", attachDialogTo: (userAgent.isIE7() ? 'body': false)});
                this.$('.popdown-dialog-body').append(this.children.eventsFormat.render().el);
                this.$('.events-controls-inner').append(this.children.eventsResultsPaginator.render().el);
                // added element for overlay
                this.$('.events-controls-inner').append('<div class="events-controls-inner-cover"/>');

                this.children.tableDock = new Dock({ el: this.el, affix: '.events-controls-inner' });
            },
            template: '\
                <div class="events-controls-inner">\
                    <div class="btn-group format-events">\
                        <a class="format btn-pill" href="#"><%- _("Format").t() %><span class="caret"></span></a>\
                        <div class="popdown-dialog  dialog-format">\
                            <div class="arrow"></div>\
                            <div class="popdown-dialog-body popdown-dialog-padded form form-horizontal form-format"></div>\
                        </div>\
                    </div>\
                </div>\
            '
        });
    }
);
