define(
    [
        'underscore',
        'backbone',
        'jquery',
        'module',
        'models/services/search/jobs/Result',
        'models/services/search/jobs/Summary',
        'views/Base',
        'views/timeline/results/eventspane/fieldsviewer/List',
        'views/shared/fieldpicker/Master',
        'views/shared/delegates/Dock'
    ],
    function(_, Backbone, $, module, ResultV2Model, SummaryModel, BaseView, ListView, PickerView, Dock) {
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *          summary: <model.services.search.job.SummaryV2>,
             *          searchJob: <models.Job>,
             *          report: <models.services.SavedSearch>,
             *          application: <models.Application>
             *     },
             *     collections: {
             *         selectedFields: <collections.SelectedFields>
             *     }
             * }
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.children.list = new ListView({
                    model: {
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application,
                        summary: this.model.summary
                    },
                    collection: {   
                        selectedFields: this.collection.selectedFields
                    }
                });
             
                this.model.summary.fields.on('reset', function() {
                    var $toggle = this.$('.all');
                    this.model.summary.fields.length ? $toggle.show() : $toggle.hide();
                }, this);
                
                this.model.summary.on('change:fieldPickerOpen', function() {
                    if(!this.model.summary.get('fieldPickerOpen')){
                        this.children.picker && this.children.picker.remove();
                    }
                },this);
            },
            events: {
                'click a.remove': function(e) {
                    this.model.report.entry.content.set('display.page.search.showFields', "0");
                    e.preventDefault();
                },
                'click a.all, a.additional-fields': function(e) {
                    this.children.picker && this.children.picker.remove();
                    
                    //TODO: consolidate 
                    this.model.summary.set({ 'fieldPickerOpen': true });
                    this.model.state.set('fieldpicker', true);
                    
                    var fieldPickerSummary = new SummaryModel({id: this.model.summary.id});
                    var clonedSelectedFields = this.collection.selectedFields.deepClone();
                    var fetchFieldPickerSummary = function() {
                        fieldPickerSummary.safeFetch({
                            data: {
                                min_freq: this.model.report.entry.content.get('display.prefs.fieldCoverage'),
                                earliest_time: this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                                latest_time: this.model.report.entry.content.get('display.events.timelineLatestTime'),
                                search: this.model.report.entry.content.get('display.prefs.fieldFilter')
                            }
                        });                        
                    }.bind(this);
                    
                    this.model.searchJob.on('jobProgress', fetchFieldPickerSummary, this);
                    this.model.report.entry.content.on('change:display.prefs.fieldFilter change:display.prefs.fieldCoverage', fetchFieldPickerSummary, this);
                    
                    fetchFieldPickerSummary();
                    
                    this.children.picker = new PickerView({
                        model: {
                            report: this.model.report,
                            searchJob: this.model.searchJob,
                            application: this.model.application,
                            summary: fieldPickerSummary
                        },
                        collection: {
                            selectedFields: clonedSelectedFields
                        }
                    });

                    $('body').append(this.children.picker.render().el);
                    this.children.picker.show();
                    this.children.list.sleep();                    

                    this.children.picker.on('hidden', function() {
                        this.children.list.wake();
                        this.collection.selectedFields.reset(clonedSelectedFields.toJSON());
                        
                        //TODO: consolidate 
                        this.model.summary.set({ 'fieldPickerOpen': false });
                        this.model.state.unset('fieldpicker');
                        
                        this.model.searchJob.off('jobProgress', fetchFieldPickerSummary, this);
                        this.model.report.entry.content.off('change:display.prefs.fieldFilter change:display.prefs.fieldCoverage', fetchFieldPickerSummary, this);
                    }, this);

                    e.preventDefault();
                }
            },
            render: function() {
                this.children.tableDock && this.children.tableDock.dispose();
                this.el.innerHTML = this.compiledTemplate({
                    _: _
                });
                this.$el.append(this.children.list.render().el);
                this.children.tableDock = new Dock({ el: this.$('.fields-controls')[0], affix: '.fields-controls-inner' });
                return this;
            },
            template: '\
                    <div class="fields-controls">\
                    <div class="fields-controls-inner">\
                    <a href="#" class="remove btn-pill"><i class="icon-collapse-left icon-no-underline"></i><span><%- _("Hide Fields").t() %></span></a>\
                    <a href="#" class="all btn-pill"><i class="icon-list icon-no-underline icon-large"></i><span><%- _("All Fields").t() %></span></a>\
                    </div>\
                    </div>\
            '
        });
    }
);
