define(
    [
        'jquery',
        'underscore',
        'module',
        'models/services/search/IntentionsParser',
        'views/Base',
        'views/timeline/results/statisticspane/StatisticsControls',
        'views/shared/results_table/ResultsTableMaster',
        'views/shared/JobDispatchState',
        'views/timeline/results/shared/NoStatistics'
    ],
    function($, _, module, IntentionsParser, BaseView, StatisticsControls, ResultsTable, JobDispatchState, NoStatistics){
        return BaseView.extend({
            moduleId: module.id,
            className: 'tab-pane',
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                
                this.model.intentionsParser = new IntentionsParser();
                this.model.intentionsParser.on('change', function() {
                    var attrs = $.extend({}, {
                        'search': this.model.intentionsParser.get('fullSearch'),
                        'dispatch.earliest_time': this.model.intentionsParser.get('dispatch.earliest_time'),
                        'dispatch.latest_time': this.model.intentionsParser.get('dispatch.latest_time')
                    });
                    this.model.report.entry.content.set(attrs);
                }, this);

                //child views
                this.children.jobDispatchState = new JobDispatchState({
                    model: this.model.searchJob,
                    mode: this.model.searchJob.entry.content.get('isPreviewEnabled') ? 'results_preview' : 'results'
                });
                
                this.children.noStatistics = new NoStatistics({
                    model: {
                        report: this.model.report,
                        appLocal: this.model.appLocal,
                        application: this.model.application
                    }
                });

                this.children.statisticsControls = new StatisticsControls({
                    model: {
                        report: this.model.report,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    }
                });

                this.children.resultsTable = new ResultsTable({
                    model: {
                        results: this.model.resultJsonRows,
                        metadata: this.model.report.entry.content,
                        format: this.model.report.entry.content
                    },
                    sortKeyAttr: 'display.statistics.sortColumn',
                    sortDirAttr: 'display.statistics.sortDirection',
                    countAttr: 'display.prefs.statistics.count',
                    offsetAttr: 'display.prefs.statistics.offset',
                    tableDockOffset: 36
                });
                
                this.children.resultsTable.on('cellClick', function(clickMeta) {
                    var field, value;
                    this.model.intentionsParser.clear({silent: true});
                    
                    if(clickMeta._span) {
                        this.model.intentionsParser.set({
                            'dispatch.earliest_time': clickMeta.value,
                            'dispatch.latest_time': JSON.stringify(parseFloat(clickMeta.value) + clickMeta._span)
                        }, {silent: true});
                        delete clickMeta.name;
                        delete clickMeta.value;
                        delete clickMeta.rowContext['row._time'];
                    }

                    if(this.model.report.entry.content.get('display.statistics.drilldown') === 'row') {
                        field = _(clickMeta.rowContext).keys().map(function(f) { return f.replace('row.', ''); });
                        value = _(clickMeta.rowContext).values();
                    } else if(this.model.report.entry.content.get('display.statistics.drilldown') === 'cell') {
                        if(clickMeta.name && clickMeta.name2){
                            field = [clickMeta.name, clickMeta.name2];
                            value = [clickMeta.value, clickMeta.value2];
                        } else if(clickMeta.name) {
                            field = clickMeta.name;
                            value = clickMeta.value;
                        } else {
                            field = clickMeta.name2;
                            value = clickMeta.value2;
                        }
                    }

                    this.model.intentionsParser.fetch({
                         data: {
                            q: this.model.report.entry.content.get('search'),
                            action: 'fieldvalue',
                            field: field || '',
                            value: value || '',
                            fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                            app: this.model.application.get('app'),
                            owner: this.model.application.get('owner')
                        }
                    });
                },this);

                this.model.report.entry.content.on('change:display.visualizations.type', this.visibility, this);
                this.model.searchJob.entry.content.on('change:resultPreviewCount change:resultCount change:dispatchState change:isPreviewEnabled', this.visibility, this);
            },
            visibility: function() {
                var resultCount = this.model.searchJob.entry.content.get('isPreviewEnabled') ? this.model.searchJob.entry.content.get('resultPreviewCount') : this.model.searchJob.entry.content.get('resultCount');
                if (!this.model.searchJob.isReportSearch()) {
                    this.children.jobDispatchState.$el.hide();
                    this.children.noStatistics.$el.show();
                    this.children.statisticsControls.$el.hide();
                    this.children.resultsTable.$el.hide();
                    return;
                }
                if (resultCount === 0) {
                    this.children.jobDispatchState.$el.show();
                    this.children.noStatistics.$el.hide();
                    this.children.statisticsControls.$el.show();
                    this.children.resultsTable.$el.hide();
                } else {
                    this.children.jobDispatchState.$el.hide();
                    this.children.noStatistics.$el.hide();
                    this.children.statisticsControls.$el.show();
                    this.children.resultsTable.$el.show();
                }
            },
            render: function() {
                this.$el.append(this.children.noStatistics.render().el);
                this.$el.append(this.children.statisticsControls.render().el);
                this.$el.append(this.children.jobDispatchState.render().el);
                this.$el.append(this.children.resultsTable.render().el);
                this.visibility();
                return this;
            }
    });
});
