define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/SyntheticSelectControl',
        'views/shared/SearchResultsPaginator',
        'views/shared/delegates/Popdown',
        'views/shared/delegates/Dock',
        'views/shared/vizcontrols/components/Statistics'
    ],
    function(_, module, Base, ControlGroup, SyntheticSelectControl, SearchResultsPaginator, Popdown, Dock, StatisticsFormat) {
        /**
         * View Hierarchy:
         *
         * SyntheticSelect (Count)
         * 
         * ControlGroup (Drilldown)
         * ControlGroup (Row Numbers)
         * ControlGroup (Wrap Results)
         * ControlGroup (Data Overlay)
         * 
         * SyntheticSelect (Preview)
         * 
         * SearchResultsPaginator
         */
        return Base.extend({
            moduleId: module.id,
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.canWriteToJob = this.model.searchJob.entry.acl.canWrite();

                //child views
                this.children.count = new SyntheticSelectControl({
                    menuWidth: "narrow",
                    className: "btn-group pull-left",
                    items: [
                        {value: '10', label: _('10 Per Page').t()},
                        {value: '20', label: _('20 Per Page').t()},
                        {value: '50', label: _('50 Per Page').t()},
                        {value: '100', label: _('100 Per Page').t()}
                    ],
                    model: this.model.report.entry.content,
                    modelAttribute: 'display.prefs.statistics.count',
                    toggleClassName: 'btn-pill'
                });

                if (this.canWriteToJob) {
                    this.children.preview = new SyntheticSelectControl({
                        menuWidth: "narrow",
                        className: "btn-group pull-left",
                        items: [
                            {value: '1', label: _('Preview').t()},
                            {value: '0', label: _('No Preview').t()}
                        ],
                        model: this.model.report.entry.content,
                        modelAttribute: 'display.general.enablePreview',
                        toggleClassName: 'btn-pill'
                    });
                }

                this.children.searchResultsPaginator = new SearchResultsPaginator({
                    mode: 'results_preview',
                    model: {
                        state: this.model.report.entry.content,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob
                    },
                    offsetKey: 'display.prefs.statistics.offset',
                    countKey: 'display.prefs.statistics.count'
                });

                this.children.format = new StatisticsFormat({
                    model: this.model.report.entry.content
                });
            },
            show: function() {
                Base.prototype.show.call(this, arguments);
                this.children.searchResultsPaginator.show({render: true});
            },
            render: function() {
                this.$el.html(this.compiledTemplate({
                    _: _
                }));
                this.children.popdown = new Popdown({el:this.$('.popdown'), mode: "dialog" });

                this.$('.statistics–controls–inner').prepend(this.children.count.render().el);

                this.$('.popdown-dialog-body').append(this.children.format.render().el);
                
                if (this.canWriteToJob) {
                    this.$('.statistics–controls–inner').append(this.children.preview.render().el);
                }
                
                this.$('.statistics–controls–inner').append(this.children.searchResultsPaginator.render().el);

                this.children.tableDock = new Dock({ el: this.el, affix: '.statistics–controls–inner' });

                return this;
            },
            template: '\
                <div class="statistics–controls–inner">\
                <div class="btn-group popdown pull-left">\
                    <a class="dropdown-toggle btn-pill" href="#"><%- _("Format").t() %><span class="caret"></span></a>\
                    <div class="popdown-dialog dialog-format dialog-format-statistics">\
                        <div class="arrow"></div>\
                        <div class="popdown-dialog-body popdown-dialog-padded form form-horizontal form-format">\
                        </div>\
                    </div>\
                </div>\
                </div>\
            '
        });
    }
);
