define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/Section',
        'views/shared/FlashMessages',
        'views/shared/DynamicFields',
        'contrib/text!views/virtual_indexes/config/ProviderSetup.html',
        'uri/route'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        ControlGroup,
        Section,
        FlashMessagesView,
        DynamicFieldsView,
        Template,
        route
        ){
        return BaseView.extend({
            moduleId: module.id,
            template: Template,
            /**
             * @param {Object} options {
             *
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.formModel = new Backbone.Model(); // event bus
                this.sections = {};
                this.isEdit = false;
                this.formLoadDfd = $.Deferred();

                this.children.flashMessages = new FlashMessagesView({ model: this.model.provider });

                this.children.name = new ControlGroup({
                    className: 'prov-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'name',
                        model: this.model.provider.entry,
                        save: false
                    },
                    required: true,
                    label: _('Name').t()
                });

                this.collection.indexesConf.on('change reset', function() {
                    this.familyNames = this.collection.indexesConf.map(function(model) {
                        var name = model.entry.get('name').match(/provider-family:(.*)/)[1];
                        return {label: name, value: name};
                    });
                    this.children.family = new ControlGroup({
                        className: 'prov-family control-group',
                        controlType: 'SyntheticSelect',
                        controlClass: 'controls-block',
                        controlOptions: {
                            modelAttribute: 'vix.family',
                            model: this.model.provider.entry.content,
                            items: this.familyNames,
                            className: 'btn-group view-count',
                            menuWidth: 'narrow',
                            toggleClassName: 'btn'
                        },
                        label: _("Provider family").t()
                    });
                    // notify the section of loading complete
                    this.formModel.trigger('update', 'family');

                    // make the first element submitted by default
                    if (!this.model.provider.entry.content.get('vix.family') && this.familyNames && this.familyNames.length>0) {
                        this.model.provider.entry.content.set('vix.family', this.familyNames[0].value);
                    }

                    if (this.familyNames.length <= 1) {
                        this.children.family.disable();
                    } else {
                        this.children.family.enable();
                    }
                }, this);

                this.children.hadoopVersion = new ControlGroup({
                        className: 'prov-hadoop-version control-group',
                        controlType: 'SyntheticSelect',
                        controlOptions: {
                               modelAttribute: 'vix.command.arg.3',
                               model: this.model.provider.entry.content,
                               items:[
                                     {label: _('Hadoop 1.x, (MRv1)').t(), value: '$SPLUNK_HOME/bin/jars/SplunkMR-s6.0-h1.0.jar' }, 
                                     {label: _('Hadoop 2.x, (MRv1)').t(), value: '$SPLUNK_HOME/bin/jars/SplunkMR-s6.0-h2.0.jar' }, 
                                     {label: _('Hadoop 2.x, (Yarn)').t(), value: '$SPLUNK_HOME/bin/jars/SplunkMR-s6.0-hy2.0.jar'} 
                               ],
                               className: 'btn-group view-count',
                               menuWidth: 'narrow',
                               toggleClassName: 'btn'
                        },
                        label: _("Hadoop version").t()
                });

                this.children.javaHome = new ControlGroup({
                    className: 'prov-java-home control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.env.JAVA_HOME',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Java home').t(),
                    help: _('Example: /usr/jdk').t()
                });

                this.children.hadoopHome = new ControlGroup({
                    className: 'prov-hadoop-home control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.env.HADOOP_HOME',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Hadoop home').t(),
                    help: _('Example: /usr/hadoop').t()
                });

                this.children.jobTracker = new ControlGroup({
                    className: 'prov-job-tracker control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.mapred.job.tracker',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Job tracker').t(),
                    help: _('Example: jobtracker.example.com:8021').t()
                });

                this.children.fileSystem = new ControlGroup({
                    className: 'prov-file-system control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.fs.default.name',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('File system').t(),
                    help: _('Example: hdfs://namenode.example.com:8020').t()
                });

                this.children.homeHdfs = new ControlGroup({
                    className: 'prov-home-hdfs control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.splunk.home.hdfs',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('HDFS working directory').t(),
                    help: _('Example: /user/splunk/hunk-01.example.com/').t(),
                    tooltip: _('Path in HDFS that you want this provider to use as it\'s workspace').t()
                });

                this.children.jobQueue = new ControlGroup({
                    className: 'prov-job-queue control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.mapred.job.queue.name',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Job queue').t(),
                    help: _('Example: highPriorityQ').t(),
                    tooltip: _('Job queue where MapReduce jobs of this provider get submitted to').t()
                });


                // wait for all the controls to finish loading before getting the list of model attributes bound for exclusion
                this.formModel.on('loadComplete', function() {
                    this.children.dynamicFields = new DynamicFieldsView({
                        model: this.model.provider.entry.content,
                        prefix: 'vix.',
                        exclude: this._getChildrenAttributes(),
                        hideEmpty: true
                    });
                    this.formModel.trigger('update', 'dynamicFields');
                    this.formLoadDfd.resolve();
                }, this);

                /*
                Sections defintion
                 */

                this.sections.basic = new Section({
                    label: '',
                    id: 'basic',
                    children: this.children,
                    model: {
                        form: this.formModel,
                        data: this.model.provider
                    },
                    order: ['name', 'family']
                });
                this.sections.envVars = new Section({
                    label: _('Environment variables').t(),
                    id: 'env-vars',
                    children: this.children,
                    model: {
                        data: this.model.provider
                    },
                    order: ['javaHome', 'hadoopHome']
                });
                this.sections.providerPaths = new Section({
                    label: _('Hadoop cluster information').t(),
                    id: 'provider-paths',
                    children: this.children,
                    model: {
                        data: this.model.provider
                    },
                    order: ['hadoopVersion', 'jobTracker', 'fileSystem']
                });
                this.sections.splunkSettings = new Section({
                    label: _('Splunk settings').t(),
                    id: 'splunk-settings',
                    children: this.children,
                    model: {
                        data: this.model.provider
                    },
                    order: ['homeHdfs', 'jobQueue']
                });
                
                this.sections.additionalSettings = new Section({
                    label: _('Additional settings').t(),
                    id: 'additional-settings',
                    description: _('Additional settings for hadoop configuration. Refer to Hadoop mapred documentation for setting name and description.').t(),
                    children: this.children,
                    order: ['dynamicFields']
                });
                // avoiding circular dependency
                this.sections.additionalSettings.loadStatusDfd.resolve();

                // this is for consistency since sections are child views too
                $.extend(this.children, this.sections);

                // trigger 'loadComplete' when all the sections reportedly finish loading
                var sectionDfds = _.map(this.sections, function(section) {
                    return section.getLoadStatusDfd();
                }, this);
                $.when.apply($, sectionDfds).then(function(schemas) {
                    this.formModel.trigger('loadComplete');
                }.bind(this));

                // the dropdown associated with this model attribute toggles visibility of several sections
                this.model.provider.entry.content.on('change:vix.family', function() {
                    var family = this.model.provider.entry.content.get('vix.family');
                    if (family !== 'hadoop') {
                        this.children.envVars.hide();
                        this.children.providerPaths.hide();
                        this.children.splunkSettings.hide();
                        $('#additional-settings p').hide(); // hide additional settings description for non-hadoop
                    } else {
                        this.children.envVars.show();
                        this.children.providerPaths.show();
                        this.children.splunkSettings.show();
                        $('#additional-settings p').show();
                    }
                }, this);

                this.model.application.on('change reset', function() {
                    this.returnUrl = route.manager(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        'system',
                        'virtual_indexes',
                        {
                            data:
                            {
                                t: 'providers'
                            }
                        }
                    );
                }, this);

                this.model.provider.on('change reset', function() {
                    if (!this.isEdit) {
                        this.model.provider.entry.set('name','');
                    }
                    // preserve events in subviews while rerendering
                    var adminContent = this.$('.adminContent').detach();
                    this.render();
                    this.$('.adminContent').replaceWith(adminContent);
                }, this);
            },
            events: {
                'click .btn-primary': function(e) {
                    e.preventDefault();
                    this.model.provider.entry.content.set('name', this.model.provider.entry.get('name'));

                    this.model.provider.save({}, {silent: true}).done(function(){
                        window.location.href = this.returnUrl;
                    }.bind(this));
                    $('html, body').animate({ scrollTop: 0 }, 'fast');
                },
                'click .cancel': function(e) {
                    e.preventDefault();
                    window.location.href = this.returnUrl;
                }
            },
            /*
            For 'Additional settings' we want to get all the model attributes that are not included into any of the ControlGroups.
            This method returns the list of attributes already used that should be excluded.
             */
            _getChildrenAttributes: function() {
                return _.map(this.children, function(child) {
                    if (child.hasOwnProperty('childList')) {
                        return child.childList[0].getModelAttribute();
                    }
                    return '';
                });
            },
            managerRoute: function(page, options) {
                return route.manager(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'system',
                    page,
                    options
                );
            },
            render: function() {
                this.isEdit = (this.model.provider.get('mode') == 'edit');
                var html = this.compiledTemplate({
                    isEdit: this.isEdit,
                    managerRoute: this.managerRoute.bind(this)
                });

                var $html = $(html);
                if (!this.el.innerHTML) {
                    $html.find('.adminContent').append(this.children.flashMessages.render().el);

                    $html.find('.adminContent').append(this.children.basic.render().el);
                    $html.find('.adminContent').append(this.children.envVars.render().el);
                    $html.find('.adminContent').append(this.children.providerPaths.render().el);
                    $html.find('.adminContent').append(this.children.splunkSettings.render().el);
                    $html.find('.adminContent').append(this.children.additionalSettings.render().el);

                    $html.find('.adminContent').append('<div class="button-wrapper" />');
                    $html.find('.adminContent .button-wrapper').append('<a href="#" class="btn cancel pull-left">' + _('Cancel').t() + '</a>');
                    $html.find('.adminContent .button-wrapper').append('<a href="#" class="btn btn-primary pull-right">' + _('Save').t() + '</a>');
                }
                this.$el.html($html);

                if (this.isEdit) {
                    this.children.name.childList[0].disable();
                    this.formLoadDfd.done(function() {
                        this.children.family.childList[0].disable();
                    }.bind(this));
                }
                return this;
            }

        });
    }
);
